#include <string.h>
#include "os.h"
#include "texture.h"

void SwapBytes(WORD * wData);
void SwapBytes(DWORD * dwData);
void FileReadBytes(FILE * file, BYTE * buffer, int count = 1);
void FileReadWords(FILE * file, WORD * buffer, int count = 1);
void FileReadDWords(FILE * file, DWORD * buffer, int count = 1);
void FileWriteBytes(FILE * file, BYTE * buffer, int count = 1);
void FileWriteWords(FILE * file, WORD * buffer, int count = 1);
void FileWriteDWords(FILE * file, DWORD * buffer, int count = 1);
void MacSwapBytes(WORD * wData);
void MacSwapBytes(DWORD * dwData);

int IsTransparent(BYTE * sourceRow, BYTE * alphaRow);
int IsOpaque(BYTE * sourceRow, BYTE * alphaRow);
int IsAlpha(BYTE * sourceRow, BYTE * alphaRow);
int IsRLE(BYTE * sourceRow, BYTE * alphaRow, BYTE * endOfRow);
int fsize(char *filename);
void TextureReadLoop(BYTE ** stuff, LOOP ** loops, int theLoop);
void TextureReadCel(BYTE ** stuff, LOOP ** loops, int theLoop, int theCel);

BOOL estimateAlpha = TRUE;

#ifdef _STAND_ALONE
void vDiagOut(char *lpszFormat,...)
{
	va_list args;
	va_start(args, lpszFormat);

	if (gFileLog) {
		vfprintf(gFileLog, lpszFormat, args);
	}
	vprintf(lpszFormat, args);

	va_end(args);
}

#endif

#define ALPHA_FLOOR_LIMIT 31
#define ALPHA_CEILING_LIMIT 248
int IsTransparent(BYTE * sourceRow, BYTE * alphaRow)
{
	if (
		((alphaRow == NULL) && (*sourceRow == 0)) ||
		((alphaRow != NULL) && (*alphaRow <= ALPHA_FLOOR_LIMIT))
	)
		return TRUE;
	else
		return FALSE;
}

int IsOpaque(BYTE * sourceRow, BYTE * alphaRow)
{
	if (
		((alphaRow == NULL) && (*sourceRow != 0)) ||
		((alphaRow != NULL) && (*alphaRow >= ALPHA_CEILING_LIMIT))
	)
		return TRUE;
	else
		return FALSE;
}

int IsAlpha(BYTE *, BYTE * alphaRow)
{
	if ((alphaRow != NULL) && (*alphaRow > ALPHA_FLOOR_LIMIT) && (*alphaRow < ALPHA_CEILING_LIMIT))
		return TRUE;
	else
		return FALSE;
}

int IsRLE(BYTE * sourceRow, BYTE * alphaRow, BYTE * endOfRow)
{
	BOOL sourceMeetsMinRunLengthRequirement = TRUE;
	BOOL alphaMeetsMinRunLengthRequirement = TRUE;
	BOOL sourceOpaqueForMinRunLength = TRUE;
	int i;

	if ((sourceRow + MIN_RUN_LENGTH - 1) > endOfRow)
		sourceMeetsMinRunLengthRequirement = FALSE;
	else
		for (i = 1; i < MIN_RUN_LENGTH; i++) {
			if (*sourceRow != (*(sourceRow + i)))
				sourceMeetsMinRunLengthRequirement = FALSE;
		}

	if (sourceMeetsMinRunLengthRequirement) {
		if (alphaRow == NULL)
			alphaMeetsMinRunLengthRequirement = FALSE;
		else
			for (i = 0; i < MIN_RUN_LENGTH; i++) {
				if (!(IsOpaque(sourceRow + i, alphaRow + i)))
					alphaMeetsMinRunLengthRequirement = FALSE;
			}

		if (alphaMeetsMinRunLengthRequirement)
			return TRUE;
		else {
			if (alphaRow != NULL)
				sourceOpaqueForMinRunLength = FALSE;
			else
				for (i = 0; i < MIN_RUN_LENGTH; i++) {
					if (!(IsOpaque(sourceRow, NULL)))
						sourceOpaqueForMinRunLength = FALSE;
				}

			if (sourceOpaqueForMinRunLength)
				return TRUE;
		}
	}
	return FALSE;
}

BYTE *TextureCompressRow(BYTE * sourceRow, int length, int *lengthOfRow, BYTE * alphaRow)
{
	static BYTE tempRow[1024];
	BYTE *endOfSourceRow;
	BYTE *outRow;
	BYTE *tempOut;
	BYTE *compressedRow;
	int i;

	*lengthOfRow = 0;
	endOfSourceRow = sourceRow + length;
	outRow = &(tempRow[0]);

	while (sourceRow < endOfSourceRow) {
		i = 1;
		if (IsTransparent(sourceRow, alphaRow)) {
			++sourceRow;
			if (alphaRow)
				++alphaRow;
			while (IsTransparent(sourceRow, alphaRow) && (sourceRow < endOfSourceRow)) {
				++sourceRow;
				if (alphaRow)
					++alphaRow;
				++i;
			}
			*outRow = TRANSPARENT_FLAG;
			++outRow;
			*(short *) outRow = i;
			MacSwapBytes((WORD *) outRow);
			outRow += sizeof(short);
		} else if (IsAlpha(sourceRow, alphaRow)) {
			*outRow = ALPHA_FLAG;
			++outRow;
			tempOut = outRow;
			outRow += sizeof(short);
			*outRow = *sourceRow;
			++outRow;
			++sourceRow;
			*outRow = *alphaRow;
			++outRow;
			++alphaRow;
			while (IsAlpha(sourceRow, alphaRow) && (sourceRow < endOfSourceRow)) {
				*outRow = *sourceRow;
				++outRow;
				++sourceRow;
				*outRow = *alphaRow;
				++outRow;
				++alphaRow;
				++i;
			}
			*tempOut = i;
			*(short *) tempOut = i;
			MacSwapBytes((WORD *) tempOut);
			tempOut += sizeof(short);
		} else if (IsOpaque(sourceRow, alphaRow) && !IsRLE(sourceRow, alphaRow, endOfSourceRow)) {
			*outRow = OPAQUE_FLAG;
			++outRow;
			tempOut = outRow;
			outRow += sizeof(short);
			*outRow = *sourceRow;
			++outRow;
			++sourceRow;
			if (alphaRow)
				++alphaRow;
			while (IsOpaque(sourceRow, alphaRow) && (sourceRow < endOfSourceRow)
				   && !IsRLE(sourceRow, alphaRow, endOfSourceRow)) {
				*outRow = *sourceRow;
				++outRow;
				++sourceRow;
				if (alphaRow)
					++alphaRow;
				++i;
			}
			*(short *) tempOut = i;
			MacSwapBytes((WORD *) tempOut);
			tempOut += sizeof(short);
		} else {
			// RLE
			*outRow = RLE_FLAG;
			++outRow;
			tempOut = sourceRow;
			++sourceRow;
			if (alphaRow)
				++alphaRow;
			while ((*sourceRow == *tempOut) && (sourceRow < endOfSourceRow) && IsOpaque(sourceRow, alphaRow)) {
				++sourceRow;
				if (alphaRow)
					++alphaRow;
				++i;
			}
			*(short *) outRow = i;
			MacSwapBytes((WORD *) outRow);
			outRow += sizeof(short);
			*outRow = *tempOut;
			++outRow;
		}
	}
	*lengthOfRow = outRow - &(tempRow[0]);
	compressedRow = new BYTE[*lengthOfRow];
	memcpy(compressedRow, tempRow, *lengthOfRow);
	return compressedRow;
}

BYTE maxAlpha = 255;

void TextureRenderRow(BYTE * destRow, BYTE * compressedRow, int start, int length, RGBQUAD * colors, int inverseRes, BYTE * inverseTable, int destBitsPerPixel)
{
	int size;
	int flags;
	int i;
	BYTE a;
	BYTE f,
	 b;
	int bytesPerPixel;
	RGBQUAD *tempColor;

	bytesPerPixel = destBitsPerPixel >> 3;
	flags = *compressedRow;
	++compressedRow;
	size = *(short *) compressedRow;
	compressedRow += sizeof(short);
	while ((start >= size) && (size > 0)) {
		start -= size;
		switch (flags) {
		case TRANSPARENT_FLAG:
			break;
		case OPAQUE_FLAG:
			compressedRow += size;
			break;
		case ALPHA_FLAG:
			compressedRow += size * 2;
			break;
		case RLE_FLAG:
			++compressedRow;
			break;
		default:
#ifdef OS_WINDOWS
			MessageBox(NULL, "Texture decompression failed!", "Error", MB_OK | MB_TASKMODAL);
#endif // OS_WINDOWS
				break;
		}
		flags = *compressedRow;
		++compressedRow;
		size = *(short *) compressedRow;
		compressedRow += sizeof(short);
	}
	while ((length > 0) && (size > 0)) {
		size -= start;
		switch (flags) {
		case TRANSPARENT_FLAG:
			if (length > size) {
				// This run of pixels isn't enough.
				start = 0;
				destRow += size * bytesPerPixel;
				length -= size;
				flags = *compressedRow;
				++compressedRow;
				size = *(short *) compressedRow;
				compressedRow += sizeof(short);
			} else {
				// This run of pixels is enough.
				length = 0;
			}
			break;
		case OPAQUE_FLAG:
			compressedRow += start;
			if (length > size) {
				// This run of pixels isn't enough.
				start = 0;
				// Adjust for true color.
				switch (bytesPerPixel) {
				case 1:
					memcpy(destRow, compressedRow, size);
					destRow += size;
					compressedRow += size;
					break;
				case 3:
					if (maxAlpha != 255) {
						// do alpha
						a = maxAlpha;
						for (i = 0; i < size; ++i) {
							tempColor = &(colors[*compressedRow]);
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbBlue) * a) >> 8)));
							++destRow;
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbGreen) * a) >> 8)));
							++destRow;
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbRed) * a) >> 8)));
							++destRow;
							++compressedRow;
						}
					} else {
						for (i = 0; i < size; ++i) {
							tempColor = &(colors[*compressedRow]);
							*destRow = tempColor->rgbBlue;
							++destRow;
							*destRow = tempColor->rgbGreen;
							++destRow;
							*destRow = tempColor->rgbRed;
							++destRow;
							++compressedRow;
						}
					}
					break;
				}
				length -= size;
				flags = *compressedRow;
				++compressedRow;
				size = *(short *) compressedRow;
				compressedRow += sizeof(short);
			} else {
				// This run of pixels is enough.
				// Adjust for true color.
				switch (bytesPerPixel) {
				case 1:
					memcpy(destRow, compressedRow, length);
					break;
				case 3:
					if (maxAlpha != 255) {
						// do alpha
						a = maxAlpha;
						for (i = 0; i < length; ++i) {
							tempColor = &(colors[*compressedRow]);
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbBlue) * a) >> 8)));
							++destRow;
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbGreen) * a) >> 8)));
							++destRow;
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbRed) * a) >> 8)));
							++destRow;
							++compressedRow;
						}
					} else {
						for (i = 0; i < length; ++i) {
							tempColor = &(colors[*compressedRow]);
							*destRow = tempColor->rgbBlue;
							++destRow;
							*destRow = tempColor->rgbGreen;
							++destRow;
							*destRow = tempColor->rgbRed;
							++destRow;
							++compressedRow;
						}
					}
					break;
				}
				length = 0;
			}
			break;
		case ALPHA_FLAG:
			compressedRow += start * 2;
			if (length > size) {
				// This run of pixels isn't enough.
				start = 0;
				// memcpy(destRow, compressedRow, size);
				for (i = 0; i < size; ++i) {
					f = *compressedRow;
					++compressedRow;
					a = *compressedRow;
					++compressedRow;
					// Adjust for true color.
					switch (bytesPerPixel) {
					case 1:
						b = *destRow;
						f = PaletteCalculateAlpha(colors, f, b, a, inverseRes, inverseTable);
						*destRow = f;
						destRow += bytesPerPixel;
						break;
					case 3:
						if (maxAlpha != 255) {
							// do alpha
							a = a * maxAlpha / 255;
						}
						tempColor = &(colors[f]);
						*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbBlue) * a) >> 8)));
						++destRow;
						*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbGreen) * a) >> 8)));
						++destRow;
						*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbRed) * a) >> 8)));
						++destRow;
						break;
					}
				}
				length -= size;
				flags = *compressedRow;
				++compressedRow;
				size = *(short *) compressedRow;
				compressedRow += sizeof(short);
			} else {
				// This run of pixels is enough.
				// memcpy(destRow, compressedRow, length);
				for (i = 0; i < length; ++i) {
					f = *compressedRow;
					++compressedRow;
					a = *compressedRow;
					++compressedRow;
					// Adjust for true color.
					switch (bytesPerPixel) {
					case 1:
						b = *destRow;
						f = PaletteCalculateAlpha(colors, f, b, a, inverseRes, inverseTable);
						*destRow = f;
						destRow += bytesPerPixel;
						break;
					case 3:
						if (maxAlpha != 255) {
							// do alpha
							a = a * maxAlpha / 255;
						}
						tempColor = &colors[f];
						*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbBlue) * a) >> 8)));
						++destRow;
						*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbGreen) * a) >> 8)));
						++destRow;
						*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbRed) * a) >> 8)));
						++destRow;
						break;
					}
				}
				length = 0;
			}
			break;
		case RLE_FLAG:
			if (length > size) {
				// This run of pixels isn't enough.
				start = 0;
				// Adjust for true color.
				// memset(destRow, *compressedRow, size);
				switch (bytesPerPixel) {
				case 1:
					memset(destRow, *compressedRow, size);
					destRow += size;
					break;
				case 3:
					tempColor = &(colors[*compressedRow]);
					if (maxAlpha != 255) {
						// do alpha
						a = maxAlpha;
						for (i = 0; i < size; ++i) {
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbBlue) * a) >> 8)));
							++destRow;
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbGreen) * a) >> 8)));
							++destRow;
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbRed) * a) >> 8)));
							++destRow;
						}
					} else {
						for (i = 0; i < size; ++i) {
							*destRow = tempColor->rgbBlue;
							++destRow;
							*destRow = tempColor->rgbGreen;
							++destRow;
							*destRow = tempColor->rgbRed;
							++destRow;
						}
					}
					break;
				}
				++compressedRow;
				length -= size;
				flags = *compressedRow;
				++compressedRow;
				size = *(short *) compressedRow;
				compressedRow += sizeof(short);
			} else {
				// This run of pixels is enough.
				// Adjust for true color.
				// memset(destRow, *compressedRow, length);
				switch (bytesPerPixel) {
				case 1:
					memset(destRow, *compressedRow, length);
					break;
				case 3:
					tempColor = &(colors[*compressedRow]);
					if (maxAlpha != 255) {
						// do alpha
						a = maxAlpha;
						for (i = 0; i < length; ++i) {
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbBlue) * a) >> 8)));
							++destRow;
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbGreen) * a) >> 8)));
							++destRow;
							*destRow = (BYTE) ((*destRow - (((*destRow - tempColor->rgbRed) * a) >> 8)));
							++destRow;
						}
					} else {
						for (i = 0; i < length; ++i) {
							*destRow = tempColor->rgbBlue;
							++destRow;
							*destRow = tempColor->rgbGreen;
							++destRow;
							*destRow = tempColor->rgbRed;
							++destRow;
						}
					}
					break;
				}
				length = 0;
			}
			break;
		default:
#ifdef OS_WINDOWS
			MessageBox(NULL, "Texture decompression failed!", "Error", MB_OK | MB_TASKMODAL);
#endif // OS_WINDOWS
				break;
		}
	}
}


#ifdef OS_WINDOWS
int fsize(char *filename)
{
	FILE *file;
	int old,
	 r;

	file = fopen(filename, "rb");
	old = ftell(file);
	fseek(file, 0, SEEK_END);
	r = ftell(file);
	fseek(file, old, SEEK_SET);
	fclose(file);
	return r;
}

#else
#include <unix.h>
#include <unistd.h>
int fsize(char *filename)
{
	long dataLength;
	OSErr myErr;
	FSSpec mySpec;
	short myVolRefNum;

	myErr = FSMakeFSSpec(0, 0, c2pstr(filename), &mySpec);
	if (myErr == noErr) {
		myErr = FSpOpenDF(&mySpec, 0, &myVolRefNum);
		myErr = GetEOF(myVolRefNum, &dataLength);
		myErr = FSClose(myVolRefNum);
	}
	return (int) dataLength;
}

#endif // OS_WINDOWS

char *szGetSeriesFileName(char *szRoot, char *szExt, int nCelDigits, int nCel, int nLoopDigits, int nLoop)
{
	static char szBuf[256];

	if (!nCelDigits) {
		nCel = 0;
	}
	if (!nLoopDigits) {
		nLoop = 0;
	}

	sprintf(szBuf, "%s%.*d%.*d.%s", szRoot, nLoopDigits, nLoop, nCelDigits, nCel, szExt);

	return szBuf;
}

void ConvertSeriesBMPsToTEX(char *rootFilename, int nStartIndex, int nEndIndex, 
							BOOL bTrim, 
							int nCelDigits,
							int nLoopDigits,
							int nLoops,
							ORG_MODE eOrgMode, 
							ORG_POS eHPos, 
							ORG_POS eVPos, 
							int nHOrg, int nVOrg)
// nEndIndex of -1 means keep incrementing index until animation is exhausted
// All BMPs must share the same palette, but this
// constraint is not explicitly error-checked by the program.
{
	int i, j, h;							// , nLength;
	int nCel, nLoop, nLoopCels, nTotalCels;

	char inColorFilename[512];
	char szFirstFound[512];
	FILE *inColorFile = NULL;
	int inColorFileSize;
	BYTE *inColorData = NULL;
	BYTE *inColorPixels = NULL;

	char inAlphaFilename[512];
	FILE *inAlphaFile = NULL;
	int inAlphaFileSize;
	BYTE *inAlphaData = NULL;
	BYTE *inAlphaPixels = NULL;

	char outFilename[512];
	FILE *outFile = NULL;

	char szDebug[128];
	int inWidth, inHeight, inWidthInBytes, colorsUsed;
	RGBQUAD inColorPalette[256];
	short celInfo[4];			// width, height, xOrigin, yOrigin
	short celHeight, celWidth;
	BYTE **rows = NULL;
	int *lengthOfRows = NULL;

	int trimLeft, trimRight, trimTop, trimBottom, nOffset;

	int *pnLoopCels = new int[nLoops];
	int *pnLoopEnds = new int[nLoops];
	if (!pnLoopCels || !pnLoopEnds) {
		vDiagOut("Out of memory allocating loop info. Exiting.\n");
		goto exit1;
	}

	int nOrgX, nOrgY;

	// Open output file
	strcpy(outFilename, szGetSeriesFileName(rootFilename, "tex", 0, 0, 0, 0));

	vDiagOut("Opening output file %s.\n", outFilename);

	outFile = fopen(outFilename, "wb");
	if (outFile == NULL) {
		vDiagOut("Error opening output file %s. Exiting.\n", outFilename);
		goto exit2;
	}
	// Count the number of cels we will be converting, and write the header

	vDiagOut("Counting cels.\n");

	nTotalCels = 0;
	szFirstFound[0] = NULL;
	for (nLoop = 0; nLoop < nLoops; ++nLoop) {
		nLoopCels = 0;
		nCel = nStartIndex;

		vDiagOut("Loop %*d:", nLoopDigits, nLoop);

		for (nCel = nStartIndex; nEndIndex == -1 || nCel <= nEndIndex; ++nCel) {
			// Start by setting up filename for bmp files

			strcpy(inColorFilename, 
				szGetSeriesFileName(rootFilename, "bmp", 
					nCelDigits, nCel, nLoopDigits, nLoop));

			wsprintf(szDebug, "Opening bitmap: %s\n", inColorFilename);
			OutputDebugString(szDebug);

			//vDiagOut("Checking for file %s.\n", inColorFilename);

			// Open bmp files for the current index
			inColorFile = fopen(inColorFilename, "rb");

			if (inColorFile == NULL) {
				if (nEndIndex == -1) {
					break;				// We've hit the last bitmap so we know
										// the number of Cels
				}
				vDiagOut(" ");
			} else {
				fclose(inColorFile);
				inColorFile = NULL;

				// save off the first found to use for palette
				if (szFirstFound[0] == 0) {
					strcpy(szFirstFound, inColorFilename);
				}

				nLoopCels++;
				nTotalCels++;
				vDiagOut(".");
			}
		}
		vDiagOut("(%d)\n", nLoopCels);
		pnLoopCels[nLoop] = nLoopCels;
		pnLoopEnds[nLoop] = nCel - 1;
	}	

	// Done counting number of cels
	if (nTotalCels <= 0 || szFirstFound[0] == 0) {
		vDiagOut("No input cels found. Exiting.\n");
		if (outFile) {
			fclose(outFile);
			outFile = NULL;
			remove(outFilename);
		}
		goto exit2;
	}

	vDiagOut("Total cels: %d\n", nTotalCels);

	// write the header
	// Open bmp files for the current index
	vDiagOut("Getting palette from %s.\n", szFirstFound);
	inColorFile = fopen(szFirstFound, "rb");
	if (!inColorFile) {
		vDiagOut("Couldn't open first BMP for palette. Exiting.\n");
		goto exit2;
	}

	// Read in bitmap header information
	inColorFileSize = fsize(szFirstFound);
	if (!inColorFileSize) {
		vDiagOut("First BMP is empty. Exiting.\n");
		goto exit2;
	}

	inColorData = new BYTE[inColorFileSize];
	if (inColorData) {
		FileReadWords(inColorFile, (WORD *) (inColorData + 0));
		FileReadDWords(inColorFile, (DWORD *) (inColorData + 2));
		FileReadWords(inColorFile, (WORD *) (inColorData + 6), 2);
		FileReadDWords(inColorFile, (DWORD *) (inColorData + 10), 4);
		FileReadWords(inColorFile, (WORD *) (inColorData + 26), 2);
		FileReadDWords(inColorFile, (DWORD *) (inColorData + 30), 6);
		FileReadBytes(inColorFile, inColorData + 54, inColorFileSize - 54);
	} else {
		vDiagOut("Out of memory allocating BMP file space. Exiting.\n");
		goto exit2;
	}

	// Get palette for the texture header
	colorsUsed = *(DWORD *) (inColorData + 46);

	if (colorsUsed) {
		for (i = 0; i < colorsUsed; ++i) {
			inColorPalette[i].rgbBlue = *(inColorData + 54 + i * 4);
			inColorPalette[i].rgbGreen = *(inColorData + 54 + i * 4 + 1);
			inColorPalette[i].rgbRed = *(inColorData + 54 + i * 4 + 2);
			inColorPalette[i].rgbReserved = *(inColorData + 54 + i * 4 + 3);
		}
		for (i = (int) colorsUsed; i < 256; ++i) {
			inColorPalette[i].rgbBlue = *(inColorData + 54);
			inColorPalette[i].rgbGreen = *(inColorData + 54 + 1);
			inColorPalette[i].rgbRed = *(inColorData + 54 + 2);
			inColorPalette[i].rgbReserved = *(inColorData + 54 + 3);
		}
	} else {
		for (i = 0; i < 256; ++i) {
			inColorPalette[i].rgbBlue = *(inColorData + 54 + i * 4);
			inColorPalette[i].rgbGreen = *(inColorData + 54 + i * 4 + 1);
			inColorPalette[i].rgbRed = *(inColorData + 54 + i * 4 + 2);
			inColorPalette[i].rgbReserved = *(inColorData + 54 + i * 4 + 3);
		}
	}

	WriteTEXHeader(outFile, inColorPalette, nLoops);
	delete [] inColorData;
	inColorData = NULL;
	fclose(inColorFile);
	inColorFile = NULL;

	//
	// Now actually convert the cels
	//
	for (nLoop = 0; nLoop < nLoops; ++nLoop) {
		vDiagOut("\nConverting loop %d: %d cels from %d to %d.\n", nLoop, pnLoopCels[nLoop], nStartIndex, pnLoopEnds[nLoop]);

		// Now we're ready to start reading in the cels for real.

		WriteTEXLoopHeader(outFile, pnLoopCels[nLoop]);
		nLoopCels = 0;

		if (pnLoopCels[nLoop] == 0) {
			continue;
		}

		for (nCel = nStartIndex; nCel <= pnLoopEnds[nLoop]; ++nCel) {
			// Start by setting up filename for bmp files
			strcpy(inColorFilename, 
				szGetSeriesFileName(rootFilename, "bmp", 
					nCelDigits, nCel, nLoopDigits, nLoop));

			wsprintf(szDebug, "Reading bitmap: %s\n", inColorFilename);
			OutputDebugString(szDebug);

			// Done setting up color filename, now setup alpha filename
			strcpy(inAlphaFilename, 
				szGetSeriesFileName(rootFilename, "a", 
					nCelDigits, nCel, nLoopDigits, nLoop));

			wsprintf(szDebug, "Reading alpha: %s\n", inAlphaFilename);
			OutputDebugString(szDebug);

			// Open bmp files for the current index
			inColorFile = fopen(inColorFilename, "rb");
			inAlphaFile = fopen(inAlphaFilename, "rb");

			if ((inColorFile == NULL)) {
				if (inAlphaFile) {
					fclose(inAlphaFile);
					inAlphaFile = NULL;
				}
				//vDiagOut("Bitmap %s not found. Skipping.\n", inColorFilename);
				continue; // skip over missing cels
									
			}
			++nLoopCels;

			//vDiagOut("Opening input file %s.\n", inColorFilename);
			if (!inAlphaFile) {
				vDiagOut("No alpha file %s, destination cel will not have alpha.\n", inAlphaFilename);
			}

			// Read in bitmap header information
			inColorFileSize = fsize(inColorFilename);
			if (inColorFileSize) {
				inColorData = new BYTE[inColorFileSize];
				if (inColorData) {
					FileReadWords(inColorFile, (WORD *) (inColorData + 0));
					FileReadDWords(inColorFile, (DWORD *) (inColorData + 2));
					FileReadWords(inColorFile, (WORD *) (inColorData + 6), 2);
					FileReadDWords(inColorFile, (DWORD *) (inColorData + 10), 4);
					FileReadWords(inColorFile, (WORD *) (inColorData + 26), 2);
					FileReadDWords(inColorFile, (DWORD *) (inColorData + 30), 6);
					FileReadBytes(inColorFile, inColorData + 54, inColorFileSize - 54);
				} else {
					vDiagOut("Out of memory allocating BMP file space.\n");
					goto exit2;
				}
			} else {
				vDiagOut("This bitmap appears to be empty. This TEX file cannot be finished. Exiting.\n");
				goto exit2;
			}
		
			// Read in Alpha file header information
			if (inAlphaFile) {
				inAlphaFileSize = fsize(inAlphaFilename);
				if (inAlphaFileSize) {
					inAlphaData = new BYTE[inAlphaFileSize];
					if (inAlphaData) {
						FileReadWords(inAlphaFile, (WORD *) (inAlphaData + 0));
						FileReadDWords(inAlphaFile, (DWORD *) (inAlphaData + 2));
						FileReadWords(inAlphaFile, (WORD *) (inAlphaData + 6), 2);
						FileReadDWords(inAlphaFile, (DWORD *) (inAlphaData + 10), 4);
						FileReadWords(inAlphaFile, (WORD *) (inAlphaData + 26), 2);
						FileReadDWords(inAlphaFile, (DWORD *) (inAlphaData + 30), 6);
						FileReadBytes(inAlphaFile, inAlphaData + 54, inAlphaFileSize - 54);
					} else {
						vDiagOut("Out of memory allocating alpha file space for file %s.\n",
							inAlphaFilename);
						goto exit2;
					}
				} else {
					vDiagOut("Alpha file %s appears to be empty. Destination will not have alpha.\n",
						inAlphaFilename);
					if (inAlphaFile) {
						fclose(inAlphaFile);
						inAlphaFile = NULL;
					}
				}
			}
		
		
			inWidth = *(DWORD *) (inColorData + 18);
			inHeight = *(DWORD *) (inColorData + 22);
			inWidthInBytes = inWidth + 3 - ((inWidth - 1) % 4);
			inColorPixels = new BYTE[inWidthInBytes * inHeight];
		
			if (!inColorPixels) {
				vDiagOut("Out of memory allocating %d bytes bitmap pixel space.\n",
					inWidthInBytes * inHeight);
				goto exit2;
			}
			
			memset(inColorPixels, 0, inWidthInBytes * inHeight);

			inAlphaPixels = new BYTE[inWidthInBytes * inHeight];
			if (!inAlphaPixels) {
				vDiagOut("Out of memory allocating %d bytes alpha pixel space.\n",
					inWidthInBytes * inHeight);
				goto exit2;
			}
			

			// Read bitmap data into memory and uncompress if necessary
			// Check for RLE compression
			switch (*(DWORD *) (inColorData + 30)) {
			case BI_RLE8:
				// Compressed
				BitmapCopyRLEPixels(inWidthInBytes, inColorPixels, inColorData + *(DWORD *) (inColorData + 10));
				break;
			case BI_RGB:
				// Uncompressed
				BitmapCopyPixels(inWidthInBytes, inColorPixels, inColorData + *(DWORD *) (inColorData + 10), inWidthInBytes, inHeight);
				break;
			default:
				// Unknown format, don't use it.
				break;
			}

			if (inAlphaFile) {
				switch (*(DWORD *) (inAlphaData + 30)) {
				case BI_RLE8:
					// Compressed
					BitmapCopyRLEPixels(inWidthInBytes, inAlphaPixels, inAlphaData + *(DWORD *) (inAlphaData + 10));
					break;
				case BI_RGB:
					// Uncompressed
					BitmapCopyPixels(inWidthInBytes, inAlphaPixels, inAlphaData + *(DWORD *) (inAlphaData + 10), inWidthInBytes, inHeight);
					break;
				default:
					// Unknown format, don't use it.
					break;
				}
			} else {
				// generate a quick alpha mask
				for (i = 0; i < inHeight; ++i) {
					for (j = 0; j < inWidth; ++j) {
						nOffset = i * inWidthInBytes + j;
						if (inColorPixels[nOffset]) {
							inAlphaPixels[nOffset] = 255;
						} else {
							inAlphaPixels[nOffset] = 0;
						}
					}
				}
			}
			
			if (bTrim) {
				// Scan pixel data to see if we can trim transparency.
				// keep in mind that the data is bottom-up
				trimBottom = -1;
				for (i = 0; i < inHeight && trimBottom == -1; i++) {
					for (j = 0; j < inWidth && trimBottom == -1; j++) {
						nOffset = i * inWidthInBytes + j;
						if (!(IsTransparent(inColorPixels + nOffset, inAlphaPixels + nOffset))) {
							// Encountered first nonTransparent pixel
							trimBottom = (inHeight - 1) - i;
						}
					}
				}
				if (trimBottom == -1) {
					trimBottom = inHeight - 1;
				}
				trimTop = -1;
				for (i = inHeight - 1; i >= 0 && trimTop == -1; i--) {
					for (j = 0; j < inWidth && trimTop == -1; j++) {
						nOffset = i * inWidthInBytes + j;
						if (!(IsTransparent(inColorPixels + nOffset, inAlphaPixels + nOffset))) {
							// Encountered first nonTransparent pixel
							trimTop = (inHeight - 1) - i;
						}
					}
				}
				if (trimTop == -1) {
					trimTop = 0;
				}
				trimLeft = -1;
				for (j = 0; j < inWidth && trimLeft == -1; j++) {
					for (i = 0; i < inHeight && trimLeft == -1; i++) {
						nOffset = i * inWidthInBytes + j;
						if (!(IsTransparent(inColorPixels + nOffset, inAlphaPixels + nOffset))) {
							// Encountered first nonTransparent pixel
							trimLeft = j;
						}
					}
				}
				if (trimLeft == -1) {
					trimLeft = 0;
				}
				trimRight = -1;
				for (j = inWidth - 1; j >= 0 && trimRight == -1; j--) {
					for (i = 0; i < inHeight && trimRight == -1; i++) {
						nOffset = i * inWidthInBytes + j;
						if (!(IsTransparent(inColorPixels + nOffset, inAlphaPixels + nOffset))) {
							// Encountered first nonTransparent pixel
							trimRight = j;
						}
					}
				}
				if (trimRight == -1) {
					trimRight = inWidth - 1;
				}
			} else {
				// no trimming
				trimLeft = 0;
				trimTop = 0;
				trimRight = inWidth - 1;
				trimBottom = inHeight - 1;
			}

			celWidth = trimRight - trimLeft + 1;
			celHeight = trimBottom - trimTop + 1;

			// origin point calculation
			//
			switch (eHPos) {
			case OP_CENTER:
				if (eOrgMode == OM_PRE_TRIM) {
					nOrgX = inWidth / 2 - trimLeft;
				} else {
					nOrgX = celWidth / 2;
				}
				break;

			case OP_TOP_LEFT:
				if (eOrgMode == OM_PRE_TRIM) {
					nOrgX = - trimLeft;
				} else {
					nOrgX = 0;
				}
				break;

			case OP_BOTTOM_RIGHT:
				if (eOrgMode == OM_PRE_TRIM) {
					nOrgX = (inWidth - 1) - trimLeft;
				} else {
					nOrgX = celWidth - 1;
				}
				break;

			case OP_EXACT:
				if (eOrgMode == OM_PRE_TRIM) {
					nOrgX = nHOrg - trimLeft;
				} else {
					nOrgX = nHOrg;
				}
				break;
			}

			switch (eVPos) {
			case OP_CENTER:
				if (eOrgMode == OM_PRE_TRIM) {
					nOrgY = inHeight / 2 - trimTop;
				} else {
					nOrgY = celHeight / 2;
				}
				break;

			case OP_TOP_LEFT:
				if (eOrgMode == OM_PRE_TRIM) {
					nOrgY = - trimTop;
				} else {
					nOrgY = 0;
				}
				break;

			case OP_BOTTOM_RIGHT:
				if (eOrgMode == OM_PRE_TRIM) {
					nOrgY = (inHeight - 1) - trimTop;
				} else {
					nOrgY = celHeight - 1;
				}
				break;

			case OP_EXACT:
				if (eOrgMode == OM_PRE_TRIM) {
					nOrgY = nVOrg - trimTop;
				} else {
					nOrgY = nVOrg;
				}
				break;
			}

			// Store cel info
			celInfo[0] = celWidth;
			celInfo[1] = celHeight;
			celInfo[2] = nOrgX;
			celInfo[3] = nOrgY;

			vDiagOut("Loop %d, Cel %d (File %d): Bounding Rect: %d %d %d %d; W, H: %d, %d; Origin: %d, %d.\n", 
				nLoop, nLoopCels - 1, nCel,
				trimLeft, trimTop, trimRight, trimBottom, 
				celWidth, celHeight, nOrgX, nOrgY);

			// Compress the row data from bottom up
			// because that's the way BMPs are stored
			rows = NULL;
			lengthOfRows = new int[celHeight];
			rows = new PBYTE[celHeight];

			for (j = 0; j < celHeight; ++j) {
				h = (inHeight - 1) - (trimTop + j);
				rows[j] = TextureCompressRow(inColorPixels + (h * inWidthInBytes) + trimLeft, celWidth, &lengthOfRows[j], inAlphaPixels + (h * inWidthInBytes) + trimLeft);
			}

			WriteTEXCel(outFile, celInfo, rows, lengthOfRows);
			//nCel++;

			// Cleanup
			if (rows) {
				delete[] rows;
				rows = NULL;
			}
			if (lengthOfRows) {
				delete[] lengthOfRows;
				lengthOfRows = NULL;
			}
			if (inColorPixels) {
				delete[] inColorPixels;
				inColorPixels = NULL;
			}
			if (inColorData) {
				delete[] inColorData;
				inColorData = NULL;
			}
			if (inAlphaPixels) {
				delete[] inAlphaPixels;
				inAlphaPixels = NULL;
			}
			if (inColorFile) {
				fclose(inColorFile);
				inColorFile = NULL;
			}
			if (inAlphaData) {
				delete[] inAlphaData;
				inAlphaData = NULL;
			}
			if (inAlphaFile) {
				fclose(inAlphaFile);
				inAlphaFile = NULL;
			}
		} // cels

		if (nLoopCels != pnLoopCels[nLoop]) {
			vDiagOut("We didn't find the same set of bitmaps on the second pass. Exiting.");
			goto exit2;
		}
	} // loops
		
exit2:

	if (rows) {
		delete[] rows;
		rows = NULL;
	}
	if (lengthOfRows) {
		delete[] lengthOfRows;
		lengthOfRows = NULL;
	}
	if (inColorPixels) {
		delete[] inColorPixels;
		inColorPixels = NULL;
	}
	if (inColorData) {
		delete[] inColorData;
		inColorData = NULL;
	}
	if (inAlphaPixels) {
		delete[] inAlphaPixels;
		inAlphaPixels = NULL;
	}
	if (inColorFile) {
		fclose(inColorFile);
		inColorFile = NULL;
	}
	if (inAlphaData) {
		delete[] inAlphaData;
		inAlphaData = NULL;
	}
	if (inAlphaFile) {
		fclose(inAlphaFile);
		inAlphaFile = NULL;
	}
	if (outFile) {
		fclose(outFile);
		outFile = NULL;
	}
	if (pnLoopCels) {
		delete [] pnLoopCels;
		pnLoopCels = NULL;
	}
	if (pnLoopEnds) {
		delete [] pnLoopEnds;
		pnLoopCels = NULL;
	}

exit1:
	vDiagOut("Finished.\n\n");

	return;
}

void Convert2BMPsToTEX(char *inColorFilename, char *inAlphaFilename, char *outFilename, int height)
{
	int i;
	FILE *inColorFile = NULL;
	FILE *inAlphaFile = NULL;
	FILE *outFile = NULL;
	BYTE *inColorData = NULL;
	BYTE *inAlphaData = NULL;
	int inColorFileSize;
	int inAlphaFileSize;
	int inWidth,
	 inHeight,
	 inWidthInBytes;
	BYTE *inColorPixels = NULL;
	BYTE *inAlphaPixels = NULL;
	int colorsUsed;
	RGBQUAD inColorPalette[256];

	inColorFile = fopen(inColorFilename, "rb");
	inAlphaFile = fopen(inAlphaFilename, "rb");
	outFile = fopen(outFilename, "wb");
	if (!outFile) {
		vDiagOut("Error opening output file %s. Exiting.\n", outFilename);
	} else if (!inColorFile) {
		vDiagOut("Error opening input file %s. Exiting.\n", inColorFilename);
	} else if (!inAlphaFile) {
		vDiagOut("No alpha file %s. Destination will not have alpha.\n", inAlphaFilename);
	}
	if ((inColorFile == NULL) ||
		(outFile == NULL)) {
		if (inColorFile)
			fclose(inColorFile);
		if (inAlphaFile)
			fclose(inAlphaFile);
		if (outFile)
			fclose(outFile);
		return;
	}
	inColorFileSize = fsize(inColorFilename);

	if (inColorFileSize) {
		inColorData = new BYTE[inColorFileSize];
		if (inColorData) {
			FileReadWords(inColorFile, (WORD *) (inColorData + 0));
			FileReadDWords(inColorFile, (DWORD *) (inColorData + 2));
			FileReadWords(inColorFile, (WORD *) (inColorData + 6), 2);
			FileReadDWords(inColorFile, (DWORD *) (inColorData + 10), 4);
			FileReadWords(inColorFile, (WORD *) (inColorData + 26), 2);
			FileReadDWords(inColorFile, (DWORD *) (inColorData + 30), 6);
			FileReadBytes(inColorFile, inColorData + 54, inColorFileSize - 54);
		} else {
			if (gFileLog) {
				vDiagOut("Out of memory allocating BMP file space.\n");
			}
			if (inColorFile)
				fclose(inColorFile);
			if (inAlphaFile)
				fclose(inAlphaFile);
			if (outFile)
				fclose(outFile);
			return;
		}
	}
	if (inAlphaFile) {
		inAlphaFileSize = fsize(inAlphaFilename);
		if (inAlphaFileSize) {
			inAlphaData = new BYTE[inAlphaFileSize];
			if (inAlphaData) {
				FileReadWords(inAlphaFile, (WORD *) (inAlphaData + 0));
				FileReadDWords(inAlphaFile, (DWORD *) (inAlphaData + 2));
				FileReadWords(inAlphaFile, (WORD *) (inAlphaData + 6), 2);
				FileReadDWords(inAlphaFile, (DWORD *) (inAlphaData + 10), 4);
				FileReadWords(inAlphaFile, (WORD *) (inAlphaData + 26), 2);
				FileReadDWords(inAlphaFile, (DWORD *) (inAlphaData + 30), 6);
				FileReadBytes(inAlphaFile, inAlphaData + 54, inAlphaFileSize - 54);
			} else {
				vDiagOut("Out of memory allocating Alpha file space.\n");
				if (inColorFile)
					fclose(inColorFile);
				if (inAlphaFile)
					fclose(inAlphaFile);
				if (outFile)
					fclose(outFile);
				return;
			}
		} else {
			vDiagOut("Alpha file seems to be empty. No alpha will be used\n");
			if (inAlphaFile)
				fclose(inAlphaFile);
			inAlphaFile = NULL;
		}
	}
	inWidth = *(DWORD *) (inColorData + 18);
	inHeight = *(DWORD *) (inColorData + 22);
	inWidthInBytes = inWidth + 3 - ((inWidth - 1) % 4);
	inColorPixels = new BYTE[inWidthInBytes * inHeight];
	if (!inColorPixels) {
		vDiagOut("Out of memory allocating bitmap pixel space.\n");
		exit(3);
	}
	memset(inColorPixels, 0, inWidthInBytes * inHeight);

	if (inAlphaFile) {
		inAlphaPixels = new BYTE[inWidthInBytes * inHeight];
		if (!inAlphaPixels) {
			vDiagOut("Out of memory allocating alpha pixel space.\n");
			exit(3);
		}
		memset(inAlphaPixels, 0, inWidthInBytes * inHeight);
	}
	// Get the palette from the color file.
	colorsUsed = *(DWORD *) (inColorData + 46);

	if (colorsUsed) {
		for (i = 0; i < colorsUsed; ++i) {
			inColorPalette[i].rgbBlue = *(inColorData + 54 + i * 4);
			inColorPalette[i].rgbGreen = *(inColorData + 54 + i * 4 + 1);
			inColorPalette[i].rgbRed = *(inColorData + 54 + i * 4 + 2);
			inColorPalette[i].rgbReserved = *(inColorData + 54 + i * 4 + 3);
		}
		for (i = (int) colorsUsed; i < 256; ++i) {
			inColorPalette[i].rgbBlue = *(inColorData + 54);
			inColorPalette[i].rgbGreen = *(inColorData + 54 + 1);
			inColorPalette[i].rgbRed = *(inColorData + 54 + 2);
			inColorPalette[i].rgbReserved = *(inColorData + 54 + 3);
		}
	} else {
		for (i = 0; i < 256; ++i) {
			inColorPalette[i].rgbBlue = *(inColorData + 54 + i * 4);
			inColorPalette[i].rgbGreen = *(inColorData + 54 + i * 4 + 1);
			inColorPalette[i].rgbRed = *(inColorData + 54 + i * 4 + 2);
			inColorPalette[i].rgbReserved = *(inColorData + 54 + i * 4 + 3);
		}
	}

	// Get the palette from the alpha file.
	// Do later if the palettes aren't in the right order.

	// Check for RLE compression
	switch (*(DWORD *) (inColorData + 30)) {
	case BI_RLE8:
		// Compressed
		BitmapCopyRLEPixels(inWidthInBytes, inColorPixels, inColorData + *(DWORD *) (inColorData + 10));
		break;
	case BI_RGB:
		// Uncompressed
		BitmapCopyPixels(inWidthInBytes, inColorPixels, inColorData + *(DWORD *) (inColorData + 10), inWidthInBytes, inHeight);
		break;
	default:
		// Unknown format, don't use it.
		break;
	}

	if (inAlphaFile) {
		switch (*(DWORD *) (inAlphaData + 30)) {
		case BI_RLE8:
			// Compressed
			BitmapCopyRLEPixels(inWidthInBytes, inAlphaPixels, inAlphaData + *(DWORD *) (inAlphaData + 10));
			break;
		case BI_RGB:
			// Uncompressed
			BitmapCopyPixels(inWidthInBytes, inAlphaPixels, inAlphaData + *(DWORD *) (inAlphaData + 10), inWidthInBytes, inHeight);
			break;
		default:
			// Unknown format, don't use it.
			break;
		}
	}
	WriteTEX(outFile, inColorPixels, inColorPalette, inAlphaPixels, inWidth, inHeight, height, inWidthInBytes);

	// Cleanup
	if (inColorPixels) {
		delete[] inColorPixels;
	}
	if (inColorData) {
		delete[] inColorData;
	}
	fclose(inColorFile);
	if (inAlphaFile) {
		if (inAlphaPixels) {
			delete[] inAlphaPixels;
		}
		if (inAlphaData) {
			delete[] inAlphaData;
		}
		fclose(inAlphaFile);
	}
	vDiagOut("Finished.\n\n");

	fclose(outFile);
}

void WriteTEXHeader(FILE * outFile, RGBQUAD * inColorPalette, int nLoops)
{
	int tempInt,
	 i;
	short tempShort;

	// 'tex '
	tempInt = (' ' << 24) | ('x' << 16) | ('e' << 8) | 't';
	FileWriteDWords(outFile, (DWORD *) & tempInt);

	// Version
	tempInt = ('1' << 24) | ('0' << 16) | ('0' << 8) | '0';
	FileWriteDWords(outFile, (DWORD *) & tempInt);

	// palette
	for (i = 0; i < 256; ++i) {
		FileWriteBytes(outFile, (BYTE *) & (inColorPalette[i].rgbRed));
		FileWriteBytes(outFile, (BYTE *) & (inColorPalette[i].rgbGreen));
		FileWriteBytes(outFile, (BYTE *) & (inColorPalette[i].rgbBlue));
	}

	// numLoops
	tempShort = (short)nLoops;
	FileWriteWords(outFile, (WORD *) & tempShort);

}

void WriteTEXLoopHeader(FILE *outFile, int nCels)
{
	short tempShort;
	// numCels
	tempShort = nCels;
	FileWriteWords(outFile, (WORD *) & tempShort);
}

void WriteTEXCel(FILE * outFile, short *celInfo, PBYTE * rows, int *lengthOfRows)
{
	short i,
	 tempShort,
	 celHeight;
	char szDebug[128];

	wsprintf(szDebug, " Width %hd\n Height %hd\n xOrigin %hd\n yOrigin %hd\n",
			 celInfo[0], celInfo[1], celInfo[2], celInfo[3]);
	OutputDebugString(szDebug);

	// cel data
	// width, height, xOrigin, yOrigin, x, y
	tempShort = celInfo[0];		// width
	FileWriteWords(outFile, (WORD *) & tempShort);
	celHeight = celInfo[1];		// height
	FileWriteWords(outFile, (WORD *) & celHeight);
	tempShort = celInfo[2];		// xOrigin
	FileWriteWords(outFile, (WORD *) & tempShort);
	tempShort = celInfo[3];		// yOrigin
	FileWriteWords(outFile, (WORD *) & tempShort);
	tempShort = 0;
	FileWriteWords(outFile, (WORD *) & tempShort);
	FileWriteWords(outFile, (WORD *) & tempShort);


	for (i = 0; i < celHeight; ++i) {
		tempShort = lengthOfRows[i];
		FileWriteWords(outFile, (WORD *) & tempShort);
		FileWriteBytes(outFile, rows[i], lengthOfRows[i]);
	}
}


void WriteTEX(FILE * outFile, BYTE * inColorPixels, RGBQUAD * inColorPalette, BYTE * inAlphaPixels, int inWidth, int inHeight, int height, int inWidthInBytes)
{
	int i,
	 j,
	 k;
	int found;
	int right;
	BYTE c;
	BYTE *aRow;
	int celHeight;
	int numCels;
	BYTE **rows;
	int *lengthOfRows;

	found = FALSE;

	if (height == 0) {
		// Try to calculate height based on a row of black/white
		// alternating.

		i = 0;
		while ((i < inHeight) && !found) {
			aRow = inColorPixels + ((inHeight - i - 1) * inWidthInBytes);
			j = 0;
			right = TRUE;
			while ((j < inWidth) && right) {
				c = aRow[j];
				if (j % 2) {
					if ((inColorPalette[c].rgbRed >= 32) ||
						(inColorPalette[c].rgbGreen >= 32) ||
						(inColorPalette[c].rgbBlue >= 32))
						right = FALSE;
				} else {
					if ((inColorPalette[c].rgbRed <= 224) ||
						(inColorPalette[c].rgbGreen <= 224) ||
						(inColorPalette[c].rgbBlue <= 224))
						right = FALSE;
				}
				++j;
			}
			if (right) {
				found = TRUE;
			} else {
				++i;
			}
		}

		if (found) {
			celHeight = i;
			numCels = (inHeight + 1) / (celHeight + 1);
		} else {
			celHeight = inHeight;
			numCels = 1;
		}
	} else {
		celHeight = height;
		numCels = inHeight / celHeight;
	}

	// Bitmap has been loading into inColorPixels, in uncompressed format.
	// Compress it.
	int h;

	rows = NULL;
	// Pixels from disk were in bottom-up order.
	// Rows will be in top-down order.
	lengthOfRows = new int[numCels * celHeight];
	rows = new PBYTE[numCels * celHeight];

	BYTE *tempRow;
	int foundAlpha;

	// Scan image alpha for an empty alpha channel.
	if (inAlphaPixels) {
		foundAlpha = FALSE;
		for (i = 0; i < numCels; ++i) {
			for (j = 0; j < celHeight; ++j) {
				if (found)
					h = (inHeight - i * (celHeight + 1) - 1) - j;
				else
					h = (inHeight - i * celHeight - 1) - j;
				tempRow = inAlphaPixels + (h * inWidthInBytes);
				for (k = 0; k < inWidth; ++k) {
					if (tempRow[k] != 0) {
						foundAlpha = TRUE;
						// break out of loops
						k = inWidth;
						j = celHeight;
						i = numCels;
					}
				}
			}
		}
		if (foundAlpha == FALSE) {
			// Don't use the alpha channel, its invalid.
			inAlphaPixels = NULL;
		}
	}
	for (i = 0; i < numCels; ++i) {
		for (j = 0; j < celHeight; ++j) {
			if (found)
				h = (inHeight - i * (celHeight + 1) - 1) - j;
			else
				h = (inHeight - i * celHeight - 1) - j;
			if (inAlphaPixels)
				rows[i * celHeight + j] = TextureCompressRow(inColorPixels + (h * inWidthInBytes), inWidth, lengthOfRows + (i * celHeight + j), inAlphaPixels + (h * inWidthInBytes));
			else
				rows[i * celHeight + j] = TextureCompressRow(inColorPixels + (h * inWidthInBytes), inWidth, lengthOfRows + (i * celHeight + j), 0);
		}
	}

	int tempInt;
	short tempShort;

	// 'tex '
	tempInt = (' ' << 24) | ('x' << 16) | ('e' << 8) | 't';
	FileWriteDWords(outFile, (DWORD *) & tempInt);

	// Version
	tempInt = ('1' << 24) | ('0' << 16) | ('0' << 8) | '0';
	FileWriteDWords(outFile, (DWORD *) & tempInt);

	// palette
	for (i = 0; i < 256; ++i) {
		FileWriteBytes(outFile, (BYTE *) & (inColorPalette[i].rgbRed));
		FileWriteBytes(outFile, (BYTE *) & (inColorPalette[i].rgbGreen));
		FileWriteBytes(outFile, (BYTE *) & (inColorPalette[i].rgbBlue));
	}

	// numLoops
	tempShort = 1;
	FileWriteWords(outFile, (WORD *) & tempShort);

	// numCels
	tempShort = numCels;
	FileWriteWords(outFile, (WORD *) & tempShort);

	// cel data
	for (j = 0; j < numCels; ++j) {
		// width, height, xOrigin, yOrigin, x, y
		tempShort = inWidth;
		FileWriteWords(outFile, (WORD *) & tempShort);
		tempShort = celHeight;
		FileWriteWords(outFile, (WORD *) & tempShort);
		tempShort = 0;
		FileWriteWords(outFile, (WORD *) & tempShort);
		FileWriteWords(outFile, (WORD *) & tempShort);
		FileWriteWords(outFile, (WORD *) & tempShort);
		FileWriteWords(outFile, (WORD *) & tempShort);

		for (i = j * celHeight; i < j * celHeight + celHeight; ++i) {
			tempShort = lengthOfRows[i];
			FileWriteWords(outFile, (WORD *) & tempShort);
			FileWriteBytes(outFile, rows[i], lengthOfRows[i]);
		}
	}

	// Cleanup
	delete[] lengthOfRows;
	for (i = 0; i < numCels * celHeight; ++i)
		delete[] rows[i];
	delete[] rows;
}

#define RLE_ESCAPE  0
#define RLE_EOL     0
#define RLE_EOF     1
#define RLE_JMP     2
#define RLE_RUN     3

void BitmapCopyRLEPixels(WORD wWidthBytes, BYTE * pb, BYTE * prle)
{
	BYTE cnt;
	BYTE b;
	WORD x;
	WORD dx,
	 dy;

	x = 0;

	for (;;) {
		cnt = *prle++;
		b = *prle++;

		if (cnt == RLE_ESCAPE) {
			switch (b) {
			case RLE_EOF:
				return;
			case RLE_EOL:
				pb += wWidthBytes - x;
				x = 0;
				break;
			case RLE_JMP:
				dx = (WORD) * prle++;
				dy = (WORD) * prle++;
				pb += ((DWORD) wWidthBytes * dy) + dx;
				x += dx;
				break;
			default:
				cnt = b;
				x += (WORD) cnt;
				while (cnt-- > 0)
					*pb++ = (*prle++);
				if (b & 1)
					prle++;
				break;
			}
		} else {
			x += (WORD) cnt;

			while (cnt-- > 0)
				*pb++ = (b);
		}
	}
}

void BitmapCopyPixels(WORD, BYTE * pb, BYTE * prle, int wWidthInBytes, int wHeight)
{
	int i,
	 j;

	j = wWidthInBytes * wHeight;
	for (i = 0; i < j; ++i) {
		*pb++ = (*prle++);
	}
}

BYTE PaletteCalculateAlpha(RGBQUAD colors[256], BYTE f, BYTE b, BYTE alpha, int inverseRes, BYTE * inverseTable)
{
	int da;
	BYTE dred,
	 dgreen,
	 dblue;
	RGBQUAD fColor,
	 bColor;
	int FP,
	 BP;

	if (f == b)
		return f;
	da = alpha;
	fColor = colors[f];
	bColor = colors[b];
	FP = fColor.rgbRed;
	BP = bColor.rgbRed;
	dred = (BYTE) ((BP - (BP - FP) * da / 255));
	FP = fColor.rgbGreen;
	BP = bColor.rgbGreen;
	dgreen = (BYTE) ((BP - (BP - FP) * da / 255));
	FP = fColor.rgbBlue;
	BP = bColor.rgbBlue;
	dblue = (BYTE) ((BP - (BP - FP) * da / 255));
	if (estimateAlpha) {
		return inverseTable[(dred >> (8 - inverseRes)) * (1 << inverseRes) * (1 << inverseRes) + (dgreen >> (8 - inverseRes)) * (1 << inverseRes) + (dblue >> (8 - inverseRes))];
	} else {
		return PaletteFindNearest(colors, dred, dgreen, dblue);
	}
}

#define SLACK 50
#define CLOSE_ENOUGH 100

BYTE PaletteFindNearest(RGBQUAD colors[256], BYTE red, BYTE green, BYTE blue)
{
	int i;
	int best;
	int bestDelta;
	int delta;
	int dr,
	 dg,
	 db;
	RGBQUAD *theColor;

	best = 0;
	bestDelta = 256 * 256 * 256;
	theColor = &(colors[0]);
	for (i = 0; i < 256; ++i) {
		dr = theColor->rgbRed - red;
		dg = theColor->rgbGreen - green;
		db = theColor->rgbBlue - blue;
		++theColor;
		if ((dr < -SLACK) || (dr > SLACK))
			continue;
		if ((dg < -SLACK) || (dg > SLACK))
			continue;
		if ((db < -SLACK) || (db > SLACK))
			continue;
		delta = abs((int) (dr * dr + dg * dg + db * db));
		if (delta < bestDelta) {
			best = i;
			bestDelta = delta;
		}
		if (delta < CLOSE_ENOUGH) {
			i = 256;
		}
	}
	return best;
}

void FileReadBytes(FILE * file, BYTE * buffer, int count)
{
	fread(buffer, 1, count, file);
}

void FileReadWords(FILE * file, WORD * buffer, int count)
{
	fread(buffer, 2, count, file);
	int i;

	for (i = 0; i < count; ++i)
		MacSwapBytes(&(buffer[i]));
}

void FileReadDWords(FILE * file, DWORD * buffer, int count)
{
	fread(buffer, 4, count, file);
	int i;

	for (i = 0; i < count; ++i)
		MacSwapBytes(&(buffer[i]));
}

void FileWriteBytes(FILE * file, BYTE * buffer, int count)
{
	fwrite(buffer, 1, count, file);
}

void FileWriteWords(FILE * file, WORD * buffer, int count)
{
	int i;

	for (i = 0; i < count; ++i)
		MacSwapBytes(&(buffer[i]));
	fwrite(buffer, 2, count, file);
}

void FileWriteDWords(FILE * file, DWORD * buffer, int count)
{
	int i;

	for (i = 0; i < count; ++i)
		MacSwapBytes(&(buffer[i]));
	fwrite(buffer, 4, count, file);
}

void SwapBytes(WORD * wData)
{
	*wData = (*wData << 8) | (*wData >> 8);
}

void SwapBytes(DWORD * dwData)
{
	WORD hiWORD = (WORD) (*dwData >> 16);
	WORD loWORD = (WORD) (*dwData);
	SwapBytes(&hiWORD);
	SwapBytes(&loWORD);
	*dwData = loWORD << 16 | hiWORD;
}

void TextureUnFlatten(BYTE * stuff,	// input params
					   short *numLoops, LOOP ** loops, RGB ** colors)
{								// output params
	int i;

	i = *(int *) stuff;			// tag = 'tex '
	stuff += sizeof(int);
	i = *(int *) stuff;			// version
	stuff += sizeof(int);
	*colors = (RGB *) stuff;	// palette
	stuff += 768;
	*numLoops = *(short *) stuff;
	stuff += sizeof(short);

	*loops = new LOOP[*numLoops];
	for (i = 0; i < *numLoops; ++i) {
		TextureReadLoop(&stuff, loops, i);
	}
}

void TextureFlatten(BYTE ** fileData, int *lengthOfFileData,	// output params
			    BYTE ** rows, short numLoops, LOOP * loops, RGB * colors)
{								// input params
}

void TextureReadLoop(BYTE ** stuff, LOOP ** loops, int theLoop)
{
	int i;
	int numCels;

	numCels = *(short *) *stuff;
	*stuff += sizeof(short);
	loops[theLoop]->numCels = numCels;
	loops[theLoop]->cels = new CEL[numCels];

	for (i = 0; i < numCels; ++i) {
		TextureReadCel(stuff, loops, theLoop, i);
	}
}

void TextureReadCel(BYTE ** stuff, LOOP ** loops, int theLoop, int theCel)
{
	int i,
	 j;
	int height;

	loops[theLoop]->cels[theCel].width = *(short *) *stuff;
	*stuff += sizeof(short);
	height = *(short *) *stuff;
	*stuff += sizeof(short);
	loops[theLoop]->cels[theCel].height = height;
	loops[theLoop]->cels[theCel].xOrigin = *(short *) *stuff;
	*stuff += sizeof(short);
	loops[theLoop]->cels[theCel].yOrigin = *(short *) *stuff;
	*stuff += sizeof(short);
	loops[theLoop]->cels[theCel].x = *(short *) *stuff;
	*stuff += sizeof(short);
	loops[theLoop]->cels[theCel].y = *(short *) *stuff;
	*stuff += sizeof(short);

	// Remap the bitmap to the current palette.

	loops[theLoop]->cels[theCel].rows = new PBYTE[height];

	for (i = 0; i < height; ++i) {
		j = *(short *) *stuff;
		*stuff += sizeof(short);
		loops[theLoop]->cels[theCel].rows[i] = *stuff;
		*stuff += j;
	}
}

BYTE TextureColorOfPoint(int loop, int cel, LOOP * loops, int x, int y)
{
	BYTE *compressedRow;
	int size;
	int flags;
	int start;

	if (x >= loops[loop].cels[cel].width)
		return 0;
	if (y >= loops[loop].cels[cel].height)
		return 0;
	start = x;
	compressedRow = loops[loop].cels[cel].rows[y];
	flags = *compressedRow;
	++compressedRow;
	size = *(short *) compressedRow;
	compressedRow += sizeof(short);
	while ((start >= size) && (size > 0)) {
		start -= size;
		switch (flags) {
		case TRANSPARENT_FLAG:
			break;
		case OPAQUE_FLAG:
			compressedRow += size;
			break;
		case ALPHA_FLAG:
			compressedRow += size * 2;
			break;
		case RLE_FLAG:
			++compressedRow;
			break;
		default:
			break;
		}
		flags = *compressedRow;
		++compressedRow;
		size = *(short *) compressedRow;
		compressedRow += sizeof(short);
	}
	if (size > 0) {
		size -= start;
		switch (flags) {
		case TRANSPARENT_FLAG:
			return 0;
		case OPAQUE_FLAG:
			compressedRow += start;
			return *compressedRow;
		case ALPHA_FLAG:
			return 0;
		case RLE_FLAG:
			return *compressedRow;
		default:
			return 0;
		}
	}
	return 0;
}

void MacSwapBytes(WORD * wData)
{
#ifdef OS_WINDOWS
#else
	SwapBytes(wData);
#endif // OS_WINDOWS
}

void MacSwapBytes(DWORD * dwData) {
#ifdef OS_WINDOWS
#else
	SwapBytes(dwData);
#endif // OS_WINDOWS
}
