#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#define TRANSPARENT_FLAG	0x01
#define OPAQUE_FLAG			0x02
#define ALPHA_FLAG			0x04
#define RLE_FLAG			0x08

#define MIN_RUN_LENGTH 8

#ifdef _STAND_ALONE
extern FILE *gFileLog;
void vDiagOut(char *lpszFormat,...);
#endif


typedef struct
	{
	short	numPixels;
	} TRANSPARENT_BLOCK;

typedef struct
	{
	short	numPixels;
	BYTE	*pixels;
	} OPAQUE_BLOCK;

typedef struct
	{
	BYTE	alpha;
	BYTE	pixel;
	} ALPHA_PIXEL;

typedef struct
	{
	short	numPixels;
	ALPHA_PIXEL	*pixels;
	} ALPHA_BLOCK;

typedef struct
	{
	short	numPixels;
	BYTE	pixel;
	} RLE_BLOCK;

typedef struct
	{
	BYTE	flags;
	union
		{
		TRANSPARENT_BLOCK	tb;
		OPAQUE_BLOCK		ob;
		ALPHA_BLOCK			ab;
		RLE_BLOCK			rb;
		} SOME_BLOCK;
	} BLOCK;

typedef struct
	{
	short length;
	BLOCK *blocks;	// variable length blocks
	} ROW;

typedef struct
	{
	short	width;
	short	height;
	short	xOrigin;
	short	yOrigin;
	short	x;
	short	y;
//	ROW		*rows;
	BYTE	**rows;
	} CEL;

typedef struct
	{
	short	numCels;
	CEL		*cels;
	} LOOP;

typedef struct
	{
	BYTE	red;
	BYTE	green;
	BYTE	blue;
	} RGB;

typedef struct
	{
	int		sig;		// 'tex ', i.e. low byte = 't'
	int		version;	// High Byte = major version, etc.
	RGB		colors[256];
	short	numLoops;
	LOOP	*loops;
	} TEXTURE;

typedef enum {
	OM_PRE_TRIM,
	OM_POST_TRIM
} ORG_MODE;

typedef enum {
	OP_TOP_LEFT,
	OP_CENTER,
	OP_BOTTOM_RIGHT,
	OP_EXACT,
} ORG_POS;

void ConvertSeriesBMPsToTEX(char *rootFilename, int nStartIndex, int nEndIndex, 
		BOOL bTrim = TRUE, 
		int nSeriesDigits = 4, 
		int nLoopDigits = 0,
		int nLoops = 0,
		ORG_MODE eOrgMode = OM_PRE_TRIM, 
		ORG_POS eHPos = OP_CENTER, 
		ORG_POS eVPos = OP_CENTER, 
		int nHOrg = 0, 
		int nVOrg = 0);

char *szGetSeriesFileName(char *szRoot, char *szExt, int nCelDigits, int nCel, int nLoopDigits, int nLoop);

void TextureRenderRow(BYTE *destRow, BYTE *compressedRow, int start, int length, RGBQUAD *colors, int inverseRes, BYTE *inverseTable, int destBitsPerPixel = 8);
BYTE *TextureCompressRow(BYTE *sourceRow, int length, int *lengthOfRow, BYTE *alphaRow=NULL);
void Convert2BMPsToTEX(char *inColorFilename, char *inAlphaFilename, char *outFilename, int height);
void BitmapCopyRLEPixels(WORD wWidthBytes, BYTE *pb, BYTE *prle);
void BitmapCopyPixels(WORD wWidthBytes, BYTE *pb, BYTE *prle, int wWidthInBytes, int wHeight);
BYTE PaletteCalculateAlpha(RGBQUAD colors[256], BYTE f, BYTE b, BYTE alpha, int inverseRes, BYTE *inverseTable);
BYTE PaletteFindNearest(RGBQUAD colors[256], BYTE red, BYTE green, BYTE blue);
void WriteTEX(FILE *outFile, BYTE *inColorPixels, RGBQUAD *inColorPalette, BYTE *inAlphaPixels, int inWidth, int inHeight, int height, int inWidthInBytes);
void WriteTEXHeader(FILE *outfile, RGBQUAD *inColorPalette, int nLoops);
void WriteTEXLoopHeader(FILE *outfile, int nCels);
void WriteTEXCel(FILE* outFile, short *celInfo, PBYTE* rows, int* lengthOfRows);
BYTE TextureColorOfPoint(int loop, int cel, LOOP *loops, int x, int y);

// TextureUnFlatten: Convert a byte array into a tex in memory.
// This is the last step after reading a tex from disk.
void TextureUnFlatten(BYTE *stuff,									// input params
		short *numLoops, LOOP **loops, RGB **colors);				// output params

// TEXFlatten: Convert a tex in memory into a byte array.
// This is the first step before writing a text to disk.
void TextureFlatten(BYTE **fileData, int *lengthOfFileData,			// output params
		BYTE **rows, short numLoops, LOOP *loops, RGB *colors);		// input params
