// ADDPLANE.H -- the the plane for adding or moving objects during map editing
//
// TW

#include "addplane.h"
#include "intplane.h"
#include "mapplane.h"
#include "bitmap.h"
#include "obstaclepoint.h"
#include "procs.h"
#include "building.h"
#include "item.h"
#include "mapedit.h"
//#include "walker.h"
#include "cycler.h"
#include "missiongoal.h"

AddPlane::AddPlane(int nPri) 
: sciPlane(pApp->pWindow, FALSE, nPri) {

//	vSetPri(gpoInterface->nGetPri() + 5);
	vSetRect(0,0, gpoMap->mpoPlane->nWidth(), gpoMap->mpoPlane->nHeight());
	Posn(gpoMap->mpoPlane->GetX(),gpoMap->mpoPlane->GetY());
//	vSetBackgroundColor(11,211,11);

//	mpoBitmap = NULL;
	mpoTemplate = NULL;
	mpoNode = NULL;
	mpoMoveObj = NULL;
	mpoStartNode = NULL;
	
	mnTemplateID = 0;
	mnRegBitmapID = 0;
	mnIntBitmapID = 0;
	mbMouseDown = FALSE;		//for moving only
	mbMouseMoved = FALSE;		//for moving only
	
	mpoObject = NULL;
	
	mpoNoWay = new sciView();
	mpoNoWay->vSetPlane(this);
	mpoNoWay->SetView(444);
	mpoNoWay->Init();
	mpoNoWay->Posn(0,0);
	mpoNoWay->SetPri(10);
	mpoNoWay->Hide();

	meMode = AM_NONE;
	meLimiter = OL_DRAW;

}

AddPlane::~AddPlane() 
{
	vClearObject();

	if (mpoObject) {
		delete mpoObject;
		mpoObject = NULL;
	}

	if (mpoNoWay) {
		delete mpoNoWay;
		mpoNoWay = NULL;
	}

}

void AddPlane::Render (RECT &updateRect) {
	//Move, position, and show object(and NoWay if needed) or hide everything
	if (mpoObject && OnMe(GetCursorX(),GetCursorY()))
	{
		mpoNode = gpoMap->poNodeFromMouse();
		if (mpoNode)
		{
//			vDebugOut("x:%d y:%d\n",mpoNode->mnXPos,mpoNode->mnYPos);
//			mpoBitmap->vDraw(mpoNode->nPixelX() - gpoMap->nOrgPixelX(), mpoNode->nPixelY() - gpoMap->nOrgPixelY(), this);
			mpoObject->Posn(mpoNode->nPixelX() - gpoMap->nOrgPixelX(), mpoNode->nPixelY() - gpoMap->nOrgPixelY());
			mpoObject->Show();

			if (!bValidNode())
			{	//not a valid node so position and show NoWay sign
				mpoNoWay->Posn(mpoNode->nPixelX() - gpoMap->nOrgPixelX(), mpoNode->nPixelY() - gpoMap->nOrgPixelY());
				mpoNoWay->Show();
			}
			else
			{	//valid node so hide NoWay Sign
				mpoNoWay->Hide();
			}
		} else {	//on plane but no legal node
			mpoNoWay->Hide();
			if (mpoObject)
				mpoObject->Hide();
		}
	} else {
		// we're not moving/adding an object and the mouse isn't on the plane 
		mpoNode = NULL;
		mpoNoWay->Hide();	//hide just in case
	}
	
	if (mpoObject)
	{
		if (meMode == AM_ADD) 
		{
			mpoObject->SetLoop(0);
			mpoObject->SetCel(0);
		} else if (meMode == AM_MOVE) {
			if (mpoMoveObj) 
			{
				mpoObject->SetLoop(mpoMoveObj->nGetLoop());
				mpoObject->SetCel(mpoMoveObj->nGetCel());
			}
		}
	}

	sciPlane::Render(mrClip);

}

void AddPlane::Doit() {
	sciPlane::Doit();
}

//Returns TRUE if object can be put in node, FALSE otherwise. Depends on type of object and its limitations
Boolean AddPlane::bValidNode()
{
	if (mpoNode == NULL)
		return FALSE;

	switch (meLimiter)
	{
	case OL_DRAW:
		//no limitiations
		return TRUE;
	case OL_UNIT:
		//can't add/move unit into a node that already contains a unit 
		//EXCEPTION - moving a unit onto 'itself'
		if (meMode == AM_MOVE)
		{
			if (mpoNode->mpoUnit && mpoNode->mpoUnit != mpoMoveObj)	//if a unit is in a node and it's not the one we're moving
				return FALSE;
		} else {
			if (mpoNode->mpoUnit)
				return FALSE;
		}
		break;
	case OL_BUILDING:
		//can't add a building if it's interior would overlap a node that's already marked as an interior
		{
			int nWidth = gpoMap->nGetWidth();
			int nHeight = gpoMap->nGetHeight();

			int nSize = moIntPointList.GetSize();
			if (nSize) {
				for (int i = 0; i < nSize; ++i) 
				{
					ObstaclePoint *poPnt= moIntPointList.GetItem(i);
					int nNewNodeX = mpoNode->mnXPos + poPnt->mnXPos;
					int nNewNodeY = mpoNode->mnYPos + poPnt->mnYPos;
					
	//				vDebugOut ("%d %d %d %d\n", nNewNodeX, nNewNodeY, nWidth, nHeight);
					if ((nNewNodeX < 0) || (nNewNodeX > nWidth  - 1) ||
						(nNewNodeY < 0) || (nNewNodeY > nHeight - 1) ) 
					{
						continue; //point would be off of game map so continue with loop
					}

					MapNode *poOffsetNode = gpoMap->poGetNode(nNewNodeX, nNewNodeY);
					assert(poOffsetNode);
					if (poOffsetNode->eTerrainType() == TT_INTERIOR)
						return FALSE;
				}
			}
		}
		break;
	case OL_SPAWN:
		break;
	}

	return TRUE;
}

Boolean AddPlane::HandleEvent(sciEvent &oEvent) {
	if (!OnMe(oEvent.x, oEvent.y) || meMode == AM_NONE )
	{
		mpoNoWay->Hide();
		if (mpoObject)
			mpoObject->Hide();
		return oEvent.claimed;
	}


	//1st check keypresses for any mode
	switch (oEvent.type) {
	case KEY_PRESS:
		switch (oEvent.c)
		{
		case 'V':
			if (mnIntBitmapID != 0 && mpoObject)
			{
				if (mpoObject->GetView() == mnRegBitmapID)
					mpoObject->SetView(mnIntBitmapID);
				else
					mpoObject->SetView(mnRegBitmapID);
				oEvent.claimed = TRUE;	//comment out if we want to pass 'v' to MapEditPlane so all houses would also change
			}
			break;
		case VK_ESCAPE:	// 'ESC' key
			if (meMode == AM_MOVE && mbMouseMoved && mpoMoveObj)
				mpoMoveObj->vSetAlpha(255);

			gpoMapEdit->vSetMode(EM_NONE);
			vClearObject();
			mpoNoWay->Hide();
			gpoMap->vUpdateObstacles();
			oEvent.claimed = TRUE;
			break;
		} // oEvent.c
		break; //KEY_PRESS
	} //oEvent.type

	
	if (oEvent.claimed == TRUE)
		return oEvent.claimed;


	
	//now check mouse events depending on the mode we're in
	switch (meMode) {
	case AM_MOVE:
		{
			switch (oEvent.type)
			{
			case KEY_PRESS:
				switch (oEvent.c)
				{
				case VK_LEFT:
					oEvent.claimed = gpoMap->bKeyScrollLeft();
					break;
				case VK_RIGHT:
					oEvent.claimed = gpoMap->bKeyScrollRight();
					break;
				case VK_UP:
					oEvent.claimed = gpoMap->bKeyScrollUp();
					break;
				case VK_DOWN:
					oEvent.claimed = gpoMap->bKeyScrollDown();
					break;
				}
				break; //KEY_PRESS
			case MOUSE_UP:
				{
					if (mbMouseMoved == FALSE) 
					{
						gpoMap->vClearAllSelections();
						gpoMap->vAddSelection(mpoMoveObj);
						//mpoDrawObj->vSetSelected(TRUE);
					} else {
						//now actually move object
						vMoveObject();	//bValid node is called within vMoveObject to validate a legal move in order to reset alpha
					}
					gpoMapEdit->vSetMode(EM_NONE);
					vClearObject();
					pApp->pWindow->vRemoveEventHog(this);
					gpoMap->vUpdateObstacles();
					gpoMap->vDraw();
					oEvent.claimed = TRUE;
				}
				break;

			case MOUSE_MOVE:
				if (mbMouseDown && !mbMouseMoved) 
					vMoveCheck();
				break;

			} //oEvent.type
		}
		break; // AM_MOVE

	case AM_ADD:
		{
			switch (oEvent.type)
			{
			case MOUSE_DOWN:
				{
					switch (oEvent.modifiers)
					{
					case LEFT_MOUSE:
						//if (mpoNode)
						//	vDebugOut("x:%d y:%d\n",mpoNode->mnXPos,mpoNode->mnYPos);
						if (bValidNode()) 
						{
							vAddObject();
							gpoMap->vUpdateObstacles();
						}
						gpoMapEdit->vSetMode(EM_NONE);
						vClearObject();
						mpoNoWay->Hide();
						oEvent.claimed = TRUE;
						break;
					case RIGHT_MOUSE:
						if (bValidNode()) 
						{
							vAddObject();
							gpoMap->vUpdateObstacles();
						}
						//vClearObject();
						//mpoNoWay->Hide();
						oEvent.claimed = TRUE;
						break;
					} // oEvent.modifers
				
				}
				break; //MOUSE_DOWN

			} //oEvent.type
		}


	} //oEvent.type

	
	if (mbMouseDown)
		oEvent.claimed = TRUE; ///we're 'hogging' events until we release the button. NOTE - mbMouseDown only set to T when we attempt a vMove

	return oEvent.claimed;

}

//Sets up and initializes mpoObject and mpoNoWay
//Params -
//	nTemplateID - the template number of the object. Used to get Interior lists, interior bitmap, and normal bitmap ID(for adding)
//
//	NOT USED ANYMORE(nBitID)
//	nBitID		- for moving only, the actual bitmap ID since it can be 1 of 5 different ones, the 'Bnor' in the
//				  template won't give us the number we need.
void AddPlane::vSetupObject(int nTemplateID, int nBitID)
{
	mnTemplateID = nTemplateID;

	mpoTemplate = (Template*)pApp->pResourceManager->LoadResource(RES_TEMPLATE, nTemplateID);
	assert(mpoTemplate);

//	int nBnor = mpoTemplate->lGetValue(TAG_Bnor, 0);

//	if (nBitID)
//		mnRegBitmapID = nBitID;
//	else 
//		mnRegBitmapID = nBnor;

	mnRegBitmapID = mpoTemplate->lGetValue(TAG_Bnor, 0);
	mnIntBitmapID = mpoTemplate->lGetValue(TAG_Bint, 0);

//	mpoBitmap = (Bitmap*)pApp->pResourceManager->LoadResource(RES_BITMAP, nBitmapID);
	mpoObject = new sciView();
	mpoObject->vSetPlane(this);
	mpoObject->SetView(mnRegBitmapID);
	mpoObject->Init();
	mpoObject->Posn(0,0);
	mpoObject->SetPri(1); //this pri must be lower than NoWay sign
	mpoObject->Hide();
	
	vSetupIntList();
	
	int nClass = mpoTemplate->lGetValue(TAG_clas, 0);
	switch (nClass)
	{
	case OC_UNIT:
		meLimiter = OL_UNIT;
		break;
	case OC_BUILDING:
		meLimiter = OL_BUILDING;
		break;
	default:
		meLimiter = OL_DRAW;
		break;
	}
}


//Add a draw object
void AddPlane::vAdd(int nTemplateID)
{
	vClearObject();
	vSetupObject(nTemplateID);
	meMode = AM_ADD;
}

//clears out all data relevant to object just moved or added
void AddPlane::vClearObject()
{
	if (mpoTemplate) {
		pApp->pResourceManager->FreeResource(mpoTemplate);
		mpoTemplate = NULL;
	}
//	if (mpoBitmap) {
//		pApp->pResourceManager->FreeResource(mpoBitmap);
//		mpoBitmap = NULL;
//	}
	if (mpoObject) {
		delete mpoObject;
		mpoObject = NULL;
	}
	
	mpoNode = NULL;
	moIntPointList.DisposeAll();
	moIntPointList.RemoveAll();
	
	meMode = AM_NONE;
	meLimiter = OL_DRAW;
	mnTemplateID = 0;
	mnRegBitmapID = 0;
	mnIntBitmapID = 0;

	mpoMoveObj = NULL;
	mpoStartNode = NULL;
	mbMouseDown = FALSE;		//for moving only
	mbMouseMoved = FALSE;		//for moving only
}

void AddPlane::vSetupIntList()
{
	long plListSize;
	long *plObstList;
	int i;

	//interior obstalces
	plObstList = mpoTemplate->plGetList(TAG_Libs, &plListSize);
	if (plListSize % 2 == 0) {
		// we have data in Libs tag thats an even multiple of 2
		for (i=0; i < plListSize; i+=2) {
			ObstaclePoint *poPnt = new ObstaclePoint(plObstList[i], plObstList[i+1]);
			moIntPointList.AddToEnd(poPnt);
		}
	}
}


void AddPlane::vAddObject()
{
	if (mpoNode == NULL || mpoObject == NULL)
		return;

	int nClass = mpoTemplate->lGetValue(TAG_clas, OC_DRAW);
	switch (nClass) {
	case OC_OBSTACLE: 
		{ 
			vDebugOut ("Obstacle Object\n");
			DrawObj *poNewObstObj = new DrawObj(gpoMap->mpoPlane, mnTemplateID);
			poNewObstObj->vSetLayer(DP_OBSTACLES);
			mpoNode->vAddObj(poNewObstObj);
			vMarkUpdate();
		}
		break;

	case OC_FLOOR: 
		{ 
			vDebugOut ("Floor Object\n");
			DrawObj *poNewFloorObj = new DrawObj(gpoMap->mpoPlane, mnTemplateID);
			poNewFloorObj->vSetLayer(DP_FLOOR);
			mpoNode->vAddObj(poNewFloorObj);
			vMarkUpdate();
		}
		break;

	case OC_UNIT:
		{
			vDebugOut ("Unit Object\n");
			long lUnit = lCheckTemplate(mnTemplateID, TAG_unit, UT_VAMPIRE);
			if (lUnit == UT_VAMPIRE)
			{
				Vampire *poNewVampObj = new Vampire(game->poLocalPlayer(), gpoMap->mpoPlane, mpoNode, mnTemplateID);	//put on clipboard
				if (mnTemplateID >=20210 && mnTemplateID <= 20243)
					poNewVampObj->mjPlayerLink = 1;
				else
					poNewVampObj->mjPlayerLink = 2;
			}
			else
			{
				Unit * poNewUnitObj = NULL;
				Unit::Create(&poNewUnitObj, NULL, gpoMap->mpoPlane, mpoNode, mnTemplateID);
				poNewUnitObj->mjPlayerLink = 0;
			}

			vMarkUpdate();
		}
		break;

	case OC_BUILDING:
		{
			vDebugOut ("Building Object\n");
			Building *poNewBuilding = new Building(gpoMap->mpoPlane, mnTemplateID);
			poNewBuilding->vSetLayer(DP_OBSTACLES);
			mpoNode->vAddObj(poNewBuilding);
			vMarkUpdate();
		}
		break;
	case OC_GOAL:
		{
			vDebugOut ("Goal Object\n");
			MissionGoal * poNewGoal = new MissionGoal(gpoMap->mpoPlane, mnTemplateID);
			poNewGoal->vSetLayer(DP_DEBUG);
			mpoNode->vAddObj(poNewGoal);
			vMarkUpdate();
		}
		break;
	case OC_ITEM:
		{
			vDebugOut ("Item Object\n");
			Item * poNewItem = new Item(gpoMap->mpoPlane, mnTemplateID);
			poNewItem->vSetLayer(DP_ITEMS);
			mpoNode->vAddObj(poNewItem);
			vMarkUpdate();
		}
		break;
	}

}

//checks to see if we're attemping a move, if so, init objects
void AddPlane::vMoveCheck()
{
	MapNode *poNode = gpoMap->poNodeFromMouse();
	if (poNode && mpoMoveObj)
	{
		if (poNode != mpoStartNode)
		{
			mbMouseMoved = TRUE;
			mpoMoveObj->vSetSelected(FALSE);
			mpoMoveObj->vSetAlpha(128);

//			if (mpoMoveObj->bIsUnit())
//				vSetupObject(mpoMoveObj->poTemplate()->resourceID, mpoMoveObj->nBitmapID());
//			else
				vSetupObject(mpoMoveObj->poTemplate()->resourceID);
			mpoObject->vSetFlip(mpoMoveObj->bFlip());
			if (mpoMoveObj->bIsOpen() && mnIntBitmapID)
				mpoObject->SetView(mnIntBitmapID);
			//now need to temporarily clear out interior nodes
			gpoMap->vUpdateObstacles(mpoMoveObj);
			//mpoMoveObj->vClearObstacles(gpoMap, gpoMap->nGetWidth(), gpoMap->nGetHeight());
			//gpoMap->vUpdateObstacles();
		}
	}

}

void AddPlane::vMove(DrawObj *poObj)
{
	vClearObject(); 
	mpoStartNode = gpoMap->poNodeFromMouse();
	meMode = AM_MOVE;
	mpoMoveObj = poObj;
	mbMouseDown = TRUE;
	mbMouseMoved = FALSE;
	pApp->pWindow->vAddEventHog(this);	//make a hog until a mouse UP occurs
}

//do the actual move
void AddPlane::vMoveObject()
{
	mpoMoveObj->vSetAlpha(255);
	
	if (!bValidNode())
		return;

	//TBD - might need to add a switch statement for spawn moves
	if (mpoMoveObj->bIsUnit())
	{
		mpoMoveObj->poNode()->mpoUnit = NULL;
		mpoNode->mpoUnit = (Unit*)mpoMoveObj;
		mpoMoveObj->vSetPos(mpoNode->mnXPos, mpoNode->mnYPos); 
	} else {
		Boolean bUpdateSimilarParents;

		//see if we're moving a child around
		DrawObj *poParent = mpoMoveObj->poParent();
		if (poParent)
			bUpdateSimilarParents = bAdjustParentOffsetList(poParent);
		else
			bUpdateSimilarParents = FALSE;	//no need to write list since we're not moving a child

		mpoMoveObj->poNode()->vRemoveObj(mpoMoveObj);
		mpoNode->vAddObj(mpoMoveObj);

		if (bUpdateSimilarParents)
			gpoMap->vResetChildren(poParent);

	}
}


//Adjust ctno List of poParent based on the node difference of mpoMoveObj.
//Returns TRUE if it's OK to write out new ctno list. If F is returned, don't write it
Boolean AddPlane::bAdjustParentOffsetList(DrawObj *poParent) {
	MapNode *poOldNode = mpoMoveObj->poNode();
	MapNode *poNewNode = mpoNode;
	Template *poParentTemplate = poParent->poTemplate();
	assert(poOldNode);
	assert(poNewNode);
	assert(poParentTemplate);

	//set up lists from parent
	List<IntegerPoint>	oChildren;
	List<ObstaclePoint> oOffsets;
	poParent->vReadInList(TAG_ctem,&oChildren);
	poParent->vReadInList(TAG_ctno,&oOffsets);
	
	int nCtemSize = oChildren.GetSize();
	int nCtnoSize = oOffsets.GetSize();
	
	//abort if either list is empty
	if (nCtemSize == 0 || nCtemSize == 0)
	{
		assert(0);	//somebody messed with the ctno/ctem lists
		return FALSE;
	}

	//make sure lists are the same size
	if (nCtnoSize != nCtemSize) 
	{
		assert(0);	//somebody messed with the ctno/ctem lists
		return FALSE;
	}

	//set up original child to parent offset
	int nOrigXOff = poOldNode->mnXPos - poParent->poNode()->mnXPos;
	int nOrigYOff = poOldNode->mnYPos - poParent->poNode()->mnYPos;

	//setup offset that the child moved
	int nOffX = poNewNode->mnXPos - poOldNode->mnXPos;
	int nOffY = poNewNode->mnYPos - poOldNode->mnYPos;
	

	//now go through ctno and ctem lists and match up the templateID(in ctem) and
	//its node offset(in ctno) since I suppose its possible to have something look like:
	//ctem=128, 200, 128
	//ctno=5,5, 2,-3, 6,0
	//
	//if we search for templateID 128, which offset are we referencing?
	//This will get the correct node offset and make the changes

	for (int i=0; i < nCtemSize; ++i) {
		IntegerPoint *poTemplateID = oChildren.GetItem(i);
		ObstaclePoint *poPointList = oOffsets.GetItem(i);
		//now make sure original offsets and templateID match
		if ( (poTemplateID->mnNumber == mpoMoveObj->nTemplateID()) &&
			(poPointList->mnXPos == nOrigXOff && poPointList->mnYPos == nOrigYOff) )
		{
			poPointList->mnXPos += nOffX;
			poPointList->mnYPos += nOffY;
			poParent->vWriteList(TAG_ctno, &oOffsets);
			return TRUE;
		}

	}

	return FALSE;
}
