// ADDPLANE.H -- the the plane for adding or moving objects during map editing
//
// TW

#ifndef _addplane
#define _addplane

#include "sciplane.h"
#include "mapnode.h"
#include "drawobj.h"

class Bitmap;
class ObstaclePoint;
class MapNode;

//Used to determine the limitations as to where the objects can be moved/added
typedef enum _OBJ_LIMIT {
	OL_DRAW,	// 0	no limitations
	OL_UNIT,	// 1	units can't be in another units node
	OL_BUILDING,// 2	building interiors can't overlap another buildings interior
	OL_SPAWN,	// 3	all nodes must be clear for a spawn point
} OBJ_LIMIT;

typedef enum _ADD_MODE {
	AM_NONE,	// 0	no mode
	AM_ADD,		// 1	adding an object
	AM_MOVE,	// 2	moving an object
} ADD_MODE;

class AddPlane : public sciPlane {
public:

	AddPlane(int nPri);
	~AddPlane();

	virtual void Render(RECT &updateRect);
	virtual void Doit();
	Boolean HandleEvent(sciEvent & oEvent);

	void vAdd(int nTemplateID);
	void vMove(DrawObj *poObj);

protected:
	void vSetupObject(int nTemplateID, int nBitID = 0);		//sets up and initializes mpoObject and mpoNoWay
	void vClearObject();					//clears out all data relevant to object just moved or added
	void vSetupIntList();					//sets up interior list of object(if any)
	void vAddObject();						//do the actual adding
	void vMoveObject();						//do the actual move
	void vMoveCheck();						//checks to see if we're attemping a move, if so, init objects
	Boolean bValidNode();					//returns TRUE if object can be put in node, F otherwise
	Boolean bAdjustParentOffsetList(DrawObj *poParent);	//adjust ctno List of poParent, return T if list written

	OBJ_LIMIT			meLimiter;			//limitations of placement of object
	ADD_MODE			meMode;				//what are we doing, adding, moving, or nothing

	List<ObstaclePoint> moIntPointList;		//list of interior points(if any)
	Bitmap				*mpoBitmap;			//the 'Bitmap' of the object we're adding/moving
	Template			*mpoTemplate;		//template of the object we want to add/move
	MapNode				*mpoNode;			//node the mouse is over
	MapNode				*mpoStartNode;		//the node the mouse was over we 1st attempt a move
	sciView				*mpoObject;			//the view of the object we're adding/moving
	sciView				*mpoNoWay;			//can't move/add here
	DrawObj				*mpoMoveObj;		//the actual Draw Object we want to move
	int					mnTemplateID;		//resource ID number
	int					mnRegBitmapID;		//normal(exterior for buildings) bitmap ID
	int					mnIntBitmapID;		//interior bitmap ID for buildings only

	//specifically for moving
	Boolean				mbMouseDown;		//TRUE if mouse button is pressed down
	Boolean				mbMouseMoved;		//TRUE if mouse button is down and the mouse is moving(ie, we're dragging an object)
};

#endif