// CYCLER.H -- our cyclers
//
// TW
//



#ifndef _cyclers_
#define _cyclers_

#include "scimain.h"

class DrawObj;
class Bitmap;

typedef enum _CYCLE_TYPE {		//for TAG_Atyp
	CT_NONE,			//0
	CT_FORWARD,			//1
	CT_REVERSE,			//2
	CT_OSCILLATE,		//3
	CT_RANDOM,			//4
	CT_MOVER,			//5		//special case. Units only. Signifies a 'walker'
} CYCLE_TYPE;

typedef enum _CYCLE_SPAN {		//for TAG_Aspn
	CS_CONTINUOUS,		//0
	CS_INTERMITTENT,	//1
} CYCLE_SPAN;


typedef enum _CYCLE_DIR {
	CD_FORWARD,
	CD_REVERSE,
} CYCLE_DIR;


class Cycler 
{
public:
	Cycler(DrawObj *poClient);
	virtual ~Cycler();
	virtual void vAnimationCycle();
	virtual void vPause() {}
	virtual void vReset();
	CYCLE_TYPE	eCycleType() {return meType;}
protected:
	void			vResetDelay();		//set up a new delay(if any) between animations
	virtual Boolean	bTimeToCycle();		//returns TRUE if we should be cycling, FALSE if we need to wait
	virtual Boolean	bChangeCel();		//returns TRUE if it's ok to change to the next cel

	CYCLE_DIR		meDirection;
	CYCLE_TYPE		meType;				//forward, reverse, etc
	CYCLE_SPAN		meSpan;				//continuous or intermittent
	DrawObj			*mpoClient;
	Boolean			mbCycling;			//T when cycling has started
	int				mnNumCels;			//number of cels the client bitmap has in its animation loop
	int				mnLastCel;			//the last cel in the animation loop

	int				mnGameCyclesPerSecond;
	int				mnCyclesPerFrame;	//1=20 cycles per frame, 2=10, 3=6.66, etc 
	int				mnCycleCtr;			//animation cycle counter

	//For Intermittent cyclers
	double			mfMinWaitTime;		//the minimum and maximum wait time between animations
	double			mfMaxWaitTime;
	double			mfLoopsThru;		//loops thru animation: -.5 means a half a second of animation, 2 means 2 full times through the animation

	int				mnCelsToAnimate;	//based on mfLoopsThru, this is the calculated number of cels to animate
	int				mnCelsAnimated;		//a counter that counts up to mnCelsToAnimate

	int				mnCyclesToDelay;	//based on mnMin/MaxWaitTime, how many cycles to wait before starting(or restarting) the animation loop
	int				mnCyclesDelayed;	// a counter that counts up to mnCyclesToDelay
};

//Cycles forwards incrementing the cels
class ForwardCycler : public Cycler 
{
public:
	ForwardCycler(DrawObj *poClient);
	~ForwardCycler();
	void vAnimationCycle();
protected:
};


// Cycles backwards decrementing the cels
class ReverseCycler : public Cycler 
{
public:
	ReverseCycler(DrawObj *poClient);
	~ReverseCycler();
	void vAnimationCycle();
protected:
};


// Cycles forwards then backwards, etc.
class OscillateCycler : public Cycler 
{
public:
	OscillateCycler(DrawObj *poClient);
	~OscillateCycler();
	void vAnimationCycle();
protected:
	Boolean			mbWaitCycle;
};


// Picks a random cel excluding the current cel
class RandomCycler : public Cycler 
{
public:
	RandomCycler(DrawObj *poClient);
	~RandomCycler();
	void vAnimationCycle();
protected:
};

// Cycles the cels of a walker
class MotionCycler : public Cycler 
{
public:
	MotionCycler(DrawObj *poClient);
	~MotionCycler();
	void vAnimationCycle();
	void vReset();
protected:
	Boolean			bTimeToCycle();		//returns TRUE if we should be cycling, FALSE if we need to wait
	Boolean			bChangeCel();		//returns TRUE if it's ok to change to the next cel
	Unit			*mpoUnit;			//backpointer to the owner

};

#endif 
