// DEBUG.CPP -- the global debugger event handler
//
// TW

#include "debug.h"
#include "mapnode.h"
#include "gamemap.h"
#include "drawobj.h"
#include "unit.h"
#include "obstaclepoint.h"
#include "procs.h"
#include "viewbmp.h"
#include "intplane.h"
#include "colordef.h"
#include "mapedit.h"
#include "player.h"
#include "vampire.h"
#include "soundlist.h"
#include "ztimer.h"


Boolean gbToggleObstacleGrid = FALSE;
Boolean gbMapEditing = FALSE;	//TRUE when we're in MAP EDITING mode
Boolean gbToggleInteriors = FALSE;	//TRUE for interior view, FALSE for exterior view
Boolean bDebugOn = FALSE;

char strFileName[33];	//includes null terminator

extern int numBytesAllocated;
extern int maxMemoryAllocated;


DebugHandler::DebugHandler() {
	mpoMousePosPlane = NULL;
}

Boolean DebugHandler::HandleEvent (sciEvent &evt) {
//vDebugOut ("ty:%d mo:%d ch:%d\n", evt.type, evt.modifiers, (char) evt.c);

	if (bDebugOn) {
		return evt.claimed;
	}
	
	switch (evt.type) {
	case KEY_PRESS:
		bDebugOn = TRUE;
		if (evt.modifiers & ALT_DOWN) 
		{
			switch (evt.c) {
			
			//Adding visibility to the map
			case 'A': // ALT-A
/*				{
					int nResID;
					int nPlayerLink;	//will be cast as a BYTE later
					ViewBitmap *poObject; 
					DrawObj *poDrawObj;
					if (bGetNumber ("Type in the resource number you want to Add to Map.\n\nENTER to continue\nESC to cancel", nResID, 8)) {

						//check to see if valid resource just quick and dirty
						poObject = new ViewBitmap(nResID);
						if (poObject->GetSize() == 0) {
							vMessage ("Invalid Resource.");
						}
						else {
							poDrawObj = new DrawObj(gpoMap->mpoPlane, nResID);
							if (poDrawObj->bIsUnit()) {
								//we're adding a unit to the map, so get the player # it will be for
								if (bGetNumber("Enter Player Number:\n\nENTER to continue\nESC to cancel",nPlayerLink, 3)) {
																	
									if ( (nPlayerLink >=  0) && (nPlayerLink <= 255) ) {
										vAddObj(nResID, (BYTE)nPlayerLink);
									}
									else {
										vMessage ("Player Number Too Large");
									}
								}
							}
							else {
								// we're adding a non-unit object to the map
								vAddObj(nResID);
							}
							delete poDrawObj;
						}
						delete poObject;
					}
				}
*/
				game->ToggleClearGameMap();
				gpoMap->vVisibilitySetAll(255);
				evt.claimed = TRUE;
				break;
				
			//Check for duplicate objects on a map node and highlight if present
			//TBD - will need to check for overlapping once we get code in for how many nodes
			//      a given object covers, etc.
			case 'C': // ALT-C
				{
					//1st clear any selected objects
					gpoMap->vClearAllSelections();

					int nX, nY;
					int nSize;	//# of objects in current Node(nX,nY)
					int nFound = 0;
					Boolean nFoundOne = 0;
					int nMapWidth = gpoMap->nGetWidth();
					int nMapHeight = gpoMap->nGetHeight();
					int i,j;		// counters to go through & search moDrawn list for duplicates
					int nKey;	  // the layer we search for to find any duplicates
					DrawObj *poObj, *poObj1; // need to compare 2 items at a time and these are it
					MapNode *poNode;

					//Now loop through each map node and its lists and see if more than one
					//item exists in them
					for (nX = 0; nX < nMapWidth; nX++) {
						for (nY = 0; nY < nMapHeight; nY++) {
							poNode = gpoMap->poGetNode(nX, nY);
							assert (poNode);
							nSize = poNode->moDrawn.GetSize();
							nFoundOne = FALSE;
							//search the list from beginning to 2nd to last item
							for (i=0; i < (nSize-1); ++i) {
								poObj = poNode->moDrawn.GetItem(i);
								nKey = poObj->nLayer();
								for (j = (i+1); j < nSize; ++j) {
									poObj1 = poNode->moDrawn.GetItem(j);
									if (nKey == poObj1->nLayer()) {
										//we found 2 items with the same draw layer in it
										nFoundOne = TRUE;
										if ( (poObj->bIsSelectable()) && (!(poObj->bIsSelected())) ){ 
											gpoMap->vAddSelection(poObj);
										}

										if ( (poObj1->bIsSelectable()) && (!(poObj1->bIsSelected())) ) {
											gpoMap->vAddSelection(poObj1);
										}
										//TBD  - non selectable things like ground won't highlight so will need
										//     - to do something to show where (if any) these things are
									}
								}
							}
							nFound += nFoundOne;
						}
					}
					gpoMap->vDraw();
					if (nFound)
						game->sound2->Play(SOUND_OhMyGod); //oh my god
					else
						game->sound2->Play(SOUND_Nope); //nope
					vDebugOut ("%d Nodes w/2 or more objects found", nFound);
					evt.claimed = TRUE;
					break;
				}

			//Delete highlighted objects from map
			case 'D': // ALT-D
/*				{
					DrawObj *poItem = (DrawObj *)gpoMap->moSelection.GetLast();
					MapNode *poNode;
					if (poItem)
						game->sound2->Play(SOUND_HomerScream);
					while (poItem) {
						poNode = poItem->poNode();//gpoMap->poGetNode(poItem->mpoLocation->mnXPos, poItem->mpoLocation->mnYPos);
						assert (poNode);
						poNode->vRemoveObj(poItem);
						poNode->meType = TT_CLEAR;
						gpoMap->moSelection.Remove(poItem);
						delete poItem;
						poItem = (DrawObj *)gpoMap->moSelection.GetLast();
					}
					//gpoMap->moSelection.RemoveAll();
					
					// redraw the new screen
					gpoMap->vUpdateObstacles();
					gpoMap->vDraw();
				}
				evt.claimed = TRUE;
*/				break;
			
			// Toggles Mouse position coordinate box on/off
			case 'M': // ALT-M
				if (mpoMousePosPlane) {
					delete mpoMousePosPlane;
					mpoMousePosPlane = NULL;
				} else{
					mpoMousePosPlane = new DebugMousePlane();
				}

				break;
			
			case 'O': // ALT-O
				break;

			
			case 'S':
				break; // ALT-S

			case 'V':
				{
					int i;
					int nSize = gpoMap->moSelection.GetSize();
					for (i = 0; i < nSize; ++i) {
						Unit * poItem = (Unit *)gpoMap->moSelection.GetItem(i);
						if (poItem->bIsUnit()) 
						{
							//poUnit->vMarkSeen(game->poLocalPlayer()->mnPlayer);
							int nHealth = poItem->GetHealth();
							nHealth = max(0, nHealth - 50 * poItem->GetMaxHealth() / 100);
							Unit::SetAttribute(poItem, ATTR_HEALTH, nHealth);
							vDebugOut("lowering health from %d to %d\n",poItem->GetHealth(), nHealth);
						}
					}
				}
				break;

			case 'X': // ALT-X
				game->QuitGame(TRUE);
				evt.claimed = TRUE;
				break;

			case 'Z': // ALT-Z
				FrameBuffer *poBuf = pApp->poFrameBuf();
				if (poBuf->meFBType == FB_WINDOW) {
					poBuf->vSetMode(FB_FULLSCREEN);
				} else {
					poBuf->vSetMode(FB_WINDOW);
				}
				evt.claimed = TRUE;
				break;

			
			} //evt.c
		}
		else if (evt.modifiers & CTRL_DOWN) 
		{
			switch (evt.c) {
			
			case 'A': //CTRL-A
				char str[64];
				wsprintf(str, "Max Memory used = %d.\n", maxMemoryAllocated);
				MessageBox(NULL, str, "Memory", MB_OK | MB_TASKMODAL);
				wsprintf(str, "Current Memory used = %d.\n", numBytesAllocated);
				MessageBox(NULL, str, "Memory", MB_OK | MB_TASKMODAL);
				break;
			
			//center screen in middle of map
			case 'C': //CTRL-C
				{
					MapNode *poNode = gpoMap->poGetNode(gpoMap->nGetWidth() /2, gpoMap->nGetHeight() /2);
					gpoMap->vCenterNode(poNode);
					break;
				}

			//loading a map
			case 'L': //CTRL-L
				if (bGetString("Enter the name of the MAP file to LOAD:\n\nENTER to continue\nESC to cancel", strFileName, 32)) {
					HRESULT hr = gpoMap->Load(strFileName);
					if (FAILED(hr))
					{
						char strMsg[255];
						switch(hr)
						{
						case STG_E_FILENOTFOUND:
							wsprintf(strMsg, "Map \"%s\" not found.\n\nENTER to continue", strFileName);
							vMessage(strMsg);
							break;
						case STG_E_READFAULT:
							wsprintf(strMsg, "Problem with Map \"%s\".\n\nENTER to continue", strFileName);
							vMessage(strMsg);
							break;
						}
					}
					if (pApp->pSoundManager)
					{
						pApp->pSoundManager->SetWorldCoordinates(gpoMap->nGetWidth() - gpoMap->nGetScreenWidth(), gpoMap->nGetHeight() - gpoMap->nGetHeight());
					}
					gpoMap->vUpdateObstacles();
					if (gpoInterface)
					{
						gpoInterface->vReAdjustScroller();
					}
				}
				evt.claimed = TRUE;
				break;
			
			//map editing
			case 'M': // CTRL-M
				gbMapEditing = abs(gbMapEditing - 1);
				if (gbMapEditing) 
				{
					gbMapEditing = FALSE;	//temporary so we can clear out interface/portraits
					gpoMap->vClearAllSelections();
					gpoMap->vCommitSelection();
					gpoMap->vVisibilitySetAll(255);

					gbMapEditing = TRUE;	//put back to TRUE
					game->vSetPause(TRUE);
					DebugString ("Editing map\n");
					gpoMapEdit = new MapEditPlane();
					
				}
				else 
				{
					delete gpoMapEdit;
					gpoMapEdit = NULL;
					DebugString ("Done Editing map\n");
					gpoMap->vVisibilitySetAll(0);
					gpoMap->vClearAllSelections();
					gpoMap->vDraw();
					game->vSetPause(FALSE);
				}
				evt.claimed = TRUE;
				break;

			case 'O': //CTRL-O
				gbToggleObstacleGrid = abs(gbToggleObstacleGrid - 1);
				gpoMap->vDraw();
				vDebugOut ("%d", gbToggleObstacleGrid);
				evt.claimed = TRUE;
				break;

			// saving a map
			case 'S': //CTRL-S
				if (bGetString("Enter the name of the MAP file to SAVE:\n\nENTER to continue\nESC to cancel", strFileName, 32)) 
				{
					DebugString ("Saving Map...\n");
					gpoMap->vSaveMap(strFileName, MAP_VERSION);
				}
				evt.claimed = TRUE;
				break;
			
			// a quick and dirty way to switch between interior and exterior views for those
			// objects that support that
			case 'V': // CTRL-V
				gbToggleInteriors = abs(gbToggleInteriors - 1);
				gpoMap->vDraw();
				evt.claimed = TRUE;
				break;
			}	// evt.c
		} // evt.modifiers

		break; // case KEY_PRESS

	} //evt.type
	
	bDebugOn = FALSE;
	return evt.claimed;
};



//Boolean MapEditHandler::HandleEvent (sciEvent &oEvent) {
//return TRUE;
//}

DebugMousePlane::DebugMousePlane() 
: sciPlane(pApp->pWindow, FALSE, 100) {

	bMouseDown = FALSE;
	//vSetPri(gpoInterface->nGetPri() + 10);
	vSetPri(window->nGetHighPriority() + 50);

	mpoMousePos = new sciText;
	mpoMousePos->vSetPlane(this);
//	mpoMousePos->SetText("XXXXXXXXXXXXX\nXXXXXXXXXXXXX");	//place holder
//	mpoMousePos->vSetSize();
	mpoMousePos->SetBackgroundColor(C_BLACK);
	mpoMousePos->Posn(0,0);
	mpoMousePos->Init();

	vSetRect(0,0, mpoMousePos->GetWidth(), mpoMousePos->GetHeight());
	Posn(0,324);
	nNewX = left;
	nNewY = top;
}

DebugMousePlane::~DebugMousePlane() {
	if (mpoMousePos) {
		delete mpoMousePos;
		mpoMousePos = NULL;
	}
}

void DebugMousePlane::Render (RECT &updateRect) {
	//mpoMousePos->vDraw();
	sciPlane::Render(updateRect);
}

void DebugMousePlane::Doit() {
	MapNode *poNode = gpoMap->poNodeFromMouse();
	if (gunCalls) {
		if (poNode) {
			//sprintf (mstr, "x%d y%d fps%3.1f", pApp->nMouseX, pApp->nMouseY, pApp->mfFPS);
			sprintf (mstr, "x: %d, y: %d\nfps %3.1f; tm %.2f\nnX: %d, nY: %d", pApp->nMouseX, pApp->nMouseY, pApp->mfFPS, fGetTimerRatio(), poNode->mnXPos, poNode->mnYPos);
		} else {
			sprintf (mstr, "x: %d y: %d\nfps %3.1f; tm %.2f", pApp->nMouseX, pApp->nMouseY,pApp->mfFPS, fGetTimerRatio());
		}
	} else {
		if (poNode) {
			//sprintf (mstr, "x%d y%d fps%3.1f", pApp->nMouseX, pApp->nMouseY, pApp->mfFPS);
			sprintf (mstr, "x: %d, y: %d\nfps %3.1f\nnX: %d, nY: %d", pApp->nMouseX, pApp->nMouseY, pApp->mfFPS, poNode->mnXPos, poNode->mnYPos);
		} else {
			sprintf (mstr, "x: %d y: %d\nfps %3.1f", pApp->nMouseX, pApp->nMouseY,pApp->mfFPS);
		}
	}
	assert (strlen(mstr) < 256); //mstr is too small
	mpoMousePos->SetText(mstr);
	mpoMousePos->vSetSize();
//	if (bMouseDown)
//		vSetRect(nNewX,nNewY, nNewX + mpoMousePos->GetWidth(), nNewY + mpoMousePos->GetHeight());
//	else
	vSetRect(left,top, left + mpoMousePos->GetWidth(), top + mpoMousePos->GetHeight());
	//window->Update(mrClip);
}

Boolean DebugMousePlane::HandleEvent(sciEvent &oEvent) {
	if ( bOnMe (oEvent) || bMouseDown) {

		switch (oEvent.type) {
		case MOUSE_DOWN:
			if ((oEvent.modifiers & LEFT_MOUSE) || (oEvent.modifiers & RIGHT_MOUSE)) 
			{
				vLocalToGlobal(&oEvent.x, &oEvent.y);
				nDragDiffX = abs(oEvent.x - left);	//difference of mouse position to the upper
				nDragDiffY = abs(oEvent.y - top);	//left of plane
				nNewX = left;						
				nNewY = top;
				bMouseDown = TRUE;
				vGlobalToLocal(&oEvent.x, &oEvent.y);	
				oEvent.claimed = TRUE;
			}	// oEvent.modifiers
			
			break; //MOUSE_DOWN

		case MOUSE_UP:
			if (oEvent.modifiers & RIGHT_MOUSE) 
			{
				bMouseDown = FALSE;
				oEvent.claimed = TRUE;
			}
			else if (oEvent.modifiers & LEFT_MOUSE) 
			{
				bMouseDown = FALSE;
				oEvent.claimed = TRUE;
			}			
			break; //MOUSE_UP

		case MOUSE_MOVE:
			if (bMouseDown) {
				vLocalToGlobal(&oEvent.x, &oEvent.y);
				oEvent.claimed = TRUE;

				nNewX = oEvent.x - nDragDiffX;
				nNewY = oEvent.y - nDragDiffY;

				if ((left != nNewX) || (top != nNewY)) {
					left = nNewX;
					top = nNewY;
					bottom = top + mnHeight;
					right = left + mnWidth;
					vCalcClip();
				}
				vGlobalToLocal(&oEvent.x, &oEvent.y);	
				window->Update(mrClip);
			}
			
			break; //MOUSE_MOVE

		}
		
	}	//bOnMe
	
	
	return oEvent.claimed;

}

Boolean DebugMousePlane::bOnMe(sciEvent &oEvent)
{
	vLocalToGlobal(&oEvent.x, &oEvent.y);	
	if ((oEvent.x >= mrClip.left) &&
		(oEvent.x <= mrClip.right) &&
		(oEvent.y >= mrClip.top) &&
		(oEvent.y <= mrClip.bottom)) {

		vGlobalToLocal(&oEvent.x, &oEvent.y);	
		return TRUE;

	} else {
		vGlobalToLocal(&oEvent.x, &oEvent.y);	
		return FALSE;
	}
}

