// DRAWOBJ.H	 The basic object type on the game map
//
// DJM

#ifndef _DRAWOBJ_
#define _DRAWOBJ_

#include "scimain.h"
#include "object.h"
#include "mapnode.h"
#include "command.h"
#include "template.h"
#include "list.h"
#include "SoundEmitter.h"

class MapPlane;
//class GameMap;
class Bitmap;
class Player;
class Cycler;


typedef enum _OBJ_CLASS {
	OC_DRAW,	// 0
	OC_UNIT,	// 1
	OC_BUILDING,// 2
	OC_ITEM,	// 3
	OC_MISSILE,	// 4
	OC_OBSTACLE,// 5
	OC_FLOOR,	// 6	walkable things that go over grass like road, floor, meadow, etc
	OC_GOAL,	// 7    EVERYONE SHOULD HAVE GOALS 
} OBJ_CLASS;


//If DrawObject has a parent, then it has to have one of these 3 states signifying when it
//should draw itself depending on its parent's OPEN/CLOSED state.
//(really applies to parents that are buildings, nothing else)
typedef enum _DRAW_STATE {
	DS_ALWAYS,		//0		//for TAG_drst
	DS_INTERIOR,	//1
	DS_EXTERIROR,	//2
} DRAW_STATE;


class Building;
class Unit;
class Item;
class ObstaclePoint;
class IntegerPoint;

class DrawObj : public SoundEmitter
{
	friend class DebugHandler;
	friend class MapEditPlane;
public:
	DrawObj(MapPlane *poPlane, int nTemplateID = 0);
	~DrawObj();

	FLOAT		mfHealth;
	FLOAT		mfMaxHealth;

	void DestroyFromMap();

	sciPlane *poGetPlane() {return (sciPlane*)mpoPlane;}

	int GetHealth() { return (int)mfHealth; }
	int GetMaxHealth() { return (int)mfMaxHealth; }
	virtual BOOL bTakesDamage() { return FALSE; }
	virtual FLOAT vDamage(FLOAT fDamage, Unit * poUnitFrom);

	void GetPlayPosition(unsigned int * unX, unsigned int *unY);
	// debugging stuff -- checksum and text dumping
	virtual long lGetChecksum(void);
	virtual void vDumpSyncText(FILE *fileDump);
	
	virtual void vSaveMap(FILE *file, WORD wVersion);	
	virtual void vLoadMap(FILE *file, WORD wVersion);
	
	virtual Boolean bOnMe(sciEvent &oEvent);
	virtual Boolean HandleEvent(sciEvent & oEvent);
	void vDraw(int nX, int nY); // quick and dirty draw
	virtual void vDraw();

	void vHighlight();
	void vMarkUpdate();
	void vPlayRandomSound(long lTag, BOOL bWithObject = TRUE);

	virtual void vCommand(Command &oWhat, BOOL bInternal = FALSE);
	virtual void vDoit(void) {return;} // while on-screen

	//cycling/animation related
	virtual void	vGameCycle(void) {}
	virtual int		nGetCel() { return mnCel; }
	virtual int		nGetLoop() { return mnLoop; }
	int				nGetNumCels() {mpoBitmap->SetLoop(mnLoop); return mpoBitmap->GetNumCels();}
	int				nGetNumLoops() {return mpoBitmap->GetNumLoops();}
	virtual void	vCue();
	void			vIncrementCel();
	void			vDecrementCel();
	void			vIncrementLoop();
	void			vDecrementLoop();
	void			vStartCycling() {mbDoneCycling = FALSE;}
	void			vStopCycling() {mbDoneCycling = TRUE;}	//stops or pauses cycling
	Boolean			bDoneCycling() {return mbDoneCycling;}
	Boolean			bFlip() {return mbFlip;}
	void			vSetCel(int nCel) {mnCel = nCel;}
	void			vSetLoop(int nLoop) {mnLoop = nLoop;}
	void			vSetFlip(Boolean bFlip) {mbFlip = bFlip;}

	
	virtual BOOL bAlive(void) {return TRUE;}
	virtual Player *poOwner(void) {return NULL;}

	int  nGetID(void) {return mnObjID;};
	void vSetID(int nNewID) {mnObjID = nNewID;};

	OBJ_CLASS eClass(void) {return meClass;}

	virtual void vSetNode(MapNode *poNode);
	void vMoveToNode(MapNode *poNode);
	MapNode *poNode(void) {return mpoCurNode;}
	void vSetLayer(int nLayer) {mnLayer = nLayer;}
	int nLayer(void) {return mnLayer;}

	virtual int nPixelX(void) {return mpoCurNode->nPixelX();}
	virtual int nPixelY(void) {return mpoCurNode->nPixelY();}

	void vSetSelected(Boolean bSelect) {mbSelected = bSelect;}; // set status with no side effects

	void vSetSelectable(Boolean bSelectable) {mbSelectable = bSelectable;};
	Boolean bIsSelectable(void) {return mbSelectable;};
	Boolean bIsSelected(void) {return mbSelected;}
	//Boolean bIsUnit(void) {return (mpoTemplate->lGetValue(TAG_clas, 0) == 1);};

	int nTemplateID(void) {return mpoTemplate ? mpoTemplate->resourceID : 0;}
	int nBitmapID(void) {return mpoBitmap ? mpoBitmap->resourceID : 0;}
	Template *poTemplate(void) {return mpoTemplate;}

	int nBitmapWidth(void)  {return mpoBitmap ? mpoBitmap->Width()  : 0;}
	int nBitmapHeight(void) {return mpoBitmap ? mpoBitmap->Height() : 0;}

	void vSetBitmap(int nBitmap);

//	List<Object> moObstacle;		// the obstacle list for this object/floor
	void vUpdateObstacles();
	void vClearObstacles(GameMap *poMap, int nWidth, int nHeight);

	Boolean bIsVisible(void) {return mbVisible;}
	void vSetVisibility (Boolean bVisible) {mbVisible = bVisible;}

	// class-related stuff
	Boolean bIsBuilding(void) {return meClass == OC_BUILDING;}
	Boolean bIsUnit(void) {return meClass == OC_UNIT;}
	Boolean bIsItem(void) {return meClass == OC_ITEM;}
	virtual Boolean bIsVampire(void) {return FALSE;}
	
	virtual void	vSetPos(WORD wX, WORD wY) {}	//needed this virtual for editing purposes
	virtual Boolean bIsOpen(void) {return FALSE;}
	virtual int		nInsideBitmapID(void) {return NULL;}
	virtual void	vChangeToInside(void);  
	virtual void	vChangeToOutside(void); 
	virtual void	vSetForcing(Boolean bForce);
	virtual void	vSetAlpha(BYTE jAlpha) {mjAlpha = jAlpha;}

	//map editing mode specific stuff
	virtual void vMapEditHandleEvent(sciEvent &oEvent);	
	virtual void vDrag(sciEvent &oEvent);		// dragging around
 	virtual void vCopySelf(sciEvent &oEvent);	//makes copy of self and inserts into node nearest event
	virtual void vInsertSelf(sciEvent &oEvent);	//inserts self into node nearest event
	void vDrawObstacles(void);					//draws the obstacles for editing purposes
	void vReadInList(long lTag, List<ObstaclePoint> *poPointList);	//reads in a x,y type list
	void vReadInList(long lTag, List<IntegerPoint> *poPointList);	//reads in an integer list
	void vWriteList(long lTag, List<ObstaclePoint> *poPointList);	//save profile string
	void vWriteList(long lTag, List<IntegerPoint> *poPointList);	//save profile string
	virtual void vSaveBitmap(void);				//saves the bitmap as a TEX with any changed origin point info
	void vSetParent(DrawObj *poDrawObj);

	//Object grouping related stuff
	void	vInitChildren();					//Initialize any children this object may own
	void	vRemoveChild(DrawObj *poObj);		//Removes child from list
	void	vSetParentOpen(Boolean bOpen);		//sets mbParentOpen to bOpen for every child
	Boolean	bChildren();						//returns F if mpoChildren is NULL or if size of list is 0. Returns True otherwise.
	void	vSetChildNodeNew(MapNode *poNode);	//sets up nodes of children for the 1st time using node offsets in 'ctno' list
	void	vSetChildNodeOff(MapNode *poNewNode);	//changes nodes of children based on node passed
	Boolean bOnChildren(sciEvent &oEvent);		//returns T if event was on child and that child bubbles up events
	DrawObj	*poParent()		{return mpoParent;}
	Boolean	bEventBubble()	{return mbEventBubbling;}

protected:
	virtual		Bitmap * GetHighlightBitmap() { return NULL; }
	Bitmap		*mpoBitmap;				// the bitmap
	Cycler		*mpoCycler;				//the basic cycler

	OBJ_CLASS	meClass;
	
	RECT			mrCurrent;				// local bounding rectangle
	MapPlane		*mpoPlane;
	int			mnObjID;					// for network games; everything needs an ID
	int			mnCel;
	int			mnLoop;
	Boolean		mbSelected;				// am I selected?
	Boolean		mbSelectable;			// can I be selected?
	Boolean		mbDoneCycling;			
	MapNode		*mpoCurNode;			// the one we're on
	int			mnLayer;					// the layer we're on
	Template		*mpoTemplate;			// the template itself
	Boolean		mbVisible;				// am I visible to other players
	Boolean		mbFlip;

	BYTE		mjAlpha;

	WORD		*mpwSpecialPalette;		// do I have a special palette for drawing?

	//Object grouping related stuff
	RefList<DrawObj>	*mpoChildren;
	DrawObj				*mpoParent;
	DRAW_STATE			meDrawState;
	Boolean				mbEventBubbling;	//if TRUE, the object will pass any events if gets up to its parent
	Boolean				mbParentOpen;		//whether or not it's parent object is open. Buildings will be T or F,
											//all other non-building objects will always be FALSE

};

inline void DrawObj::vIncrementCel()
{
	if (mnCel + 1 >= nGetNumCels())
		mnCel = 0;
	else
		++mnCel;

}

inline void DrawObj::vDecrementCel()
{
	if (mnCel - 1 < 0)
		mnCel = nGetNumCels() - 1;
	else
		--mnCel;

}


#endif
