#include "os.h"
#include "editbox.h"
#include "scitext.h"
#include "sciprop.h"
#include "sciscrip.h"
#include <stdio.h>

#define CARET_SPEED 30

class CaretScript : public sciScript
{
 public:
	void ChangeState(int newState) {
		sciScript::ChangeState(newState);
		switch (state) {
		case 0:
			// Show caret.
			((sciProp *) client)->Show();
			SetTicks(CARET_SPEED);
			SetNextState(1);
			break;
		case 1:
			// Hide caret.
			((sciProp *) client)->Hide();
			SetTicks(CARET_SPEED);
			SetNextState(0);
			break;
		}
	}
};


EditBox::EditBox()
{
	isUp = TRUE;
	enabled = TRUE;
	mouseDown = FALSE;
	SetPri(1000);
	caret = NULL;
}

EditBox::~EditBox()
{
	if (caret)
		delete caret;
}

void EditBox::DoLeftClick()
{
	//if (isUp) {
	//} else {
	//}
	isUp = !isUp;
}

Boolean EditBox::HandleEvent(sciEvent & event)
{
	char textBuffer[256];

	if (!enabled)
		return FALSE;
	switch (event.type) {
	case KEY_PRESS:
		if (IsEnabled() && IsNotHidden()) {
			if (isUp) {
				switch (event.c) {
				case 8:			// Backspace
					if (strlen(GetText()) > 0) {
						sprintf(textBuffer, "%s", GetText());
						textBuffer[strlen(GetText()) - 1] = 0;
						SetText(textBuffer);
					}
					break;
				case 13:		// Enter
					// if (IsEnabled() && IsNotHidden())
					// DoClick();
					// Pass <enter>s on.
					return event.claimed;
				default:
					// Make sure the buffer doesn't overflow.
					if (
						(strlen(GetText()) < sizeof(textBuffer) - 2) &&
						(strlen(GetText()) < (unsigned int) maxLength)
						) {
						sprintf(textBuffer, "%s%c", GetText(), event.c);
						SetText(textBuffer);
					}
					break;
				}
				event.claimed = TRUE;
			}
		}
		break;
	}
	return event.claimed;
}

Boolean EditBox::IsEnabled()
{
	return enabled;
}

void EditBox::Posn(int posnX, int posnY)
{
	sciView::Posn(posnX, posnY);
	PositionCaret();
}

void EditBox::Init()
{
	sciText::Init();
	caret = new sciProp;
	caret->plane = plane;
	caret->SetView(2);
	PositionCaret();
	caret->Init();
	caret->Show();
	caret->SetScript(new CaretScript);
	caret->script->SetTicks(0);
}

void EditBox::RenderText()
{
	sciText::RenderText();
	PositionCaret();
}

void EditBox::PositionCaret()
{
	if (caret) {
		caret->Posn(GetX() + GetCaretX(), GetY() + GetCaretY());
		if (caret->script) {
			caret->script->SetNextState(0);
			caret->script->SetTicks(0);
		}
	}
}

void EditBox::SetMaxLength(int l)
{
	maxLength = l;
}
