// FRAMEBUF.H -- The main frame buffer and interface to the screen
//
// DJM

#ifndef _FRAMEBUF_
#define _FRAMEBUF_

#include "palette.h"
#include "ddraw.h"
//#include "scimain.h"
//#include "mgraph.h"

typedef struct _BUFFER_INFO {
	int		nLineOffset; // offset to next line
	int		nWidth;
	int		nHeight;
} BUFFER_INFO;

typedef DWORD	COLOR16;
typedef BYTE	COLOR5;
typedef	BYTE	ALPHA5; // packed alpha (currently 5 bits)

typedef enum _FB_TYPE {
	FB_WINDOW,
	FB_FULLSCREEN,
} FB_TYPE;

typedef struct _MY_BM_INFO {
	BITMAPINFOHEADER bi;
	union 
	{
		RGBQUAD colors[256];
		DWORD fields[256];
	};
} MY_BM_INFO;

//
// NOTES:
//


// stuff for the alpha table
#define AM_COLOR_BITS 5
#define AM_ALPHA_BITS 5
#define AM_ALPHA_MASK 0x1F
#define AM_COLOR_MASK 0x1F
#define AM_COLOR_MAX	(1 << AM_COLOR_BITS)
#define AM_ALPHA_MAX	(1 << AM_ALPHA_BITS)
#define AM_ALPHA_DIVISOR 32.0

#define AT_SIZE 32768		// 15 bits: 5 fore, 5 back, 5 alpha
#define AT_FORE_LOWBIT 10
#define AT_BACK_LOWBIT 5

#define AT_FORE_MASK 0x07C00
#define AT_BACK_MASK 0x003E0
#define AT_2_FORE_MASK 0x7C007C00
#define AT_2_BACK_MASK 0x03E003E0

#define AT_PACK(jFore, jBack, jAlpha)	((jFore << (AM_ALPHA_BITS + AM_COLOR_BITS)) \
										| (jBack << AM_ALPHA_BITS) | jAlpha)

typedef struct _PF_MASKS {
	DWORD	dwRMask;
	DWORD	dwGMask;
	DWORD	dwBMask;
} PF_MASKS;

class Blender;

extern Blender *gpoBlender;

class FrameBuffer {
public:
	FrameBuffer(int nWidth, int nHeight, int nBits, FB_TYPE eFBType);
	~FrameBuffer();

	void *pGetBuffer(void) {
		if (mpPixels) {
			return mpPixels;
		} else {
			vLockBuffer();
			return mpPixels;
		}
	} // get a writeable pointer to the frame buffer
	void vReleaseBuffer(void) {}; // done writing to the buffer

	void vFrameDone(void); // write the frame out to the screen

	void vSetMode(FB_TYPE eFBType);

	Blender *poBlender(void) {return gpoBlender;}

	WORD *pwGetPalette(void) {
		if (mpoPal16) {
			return mpoPal16->pwGetPalette();
		} else {
			return NULL;
		}
	}

	void vSetPalette(Palette16 *poPal);

	int		mnPitch; // the distance in pixels from scanline n to n+1
	int		mnWidth; 
	int		mnHeight;
	int		mnBits;
	FB_TYPE	meFBType;

protected:

	void vLockBuffer(void);

	void vCreateCompatibleDIB(void);
	void vDestroyWindowDIB(void);
	void vInitWindow(void);

	void vInitDD(void);
	void vCloseDD(void);
		
	void		*mpPixels;
	Palette16	*mpoPal16;

	// stuff relating to DIBSection
	HBITMAP		mhDIB;
	void		*mpDIBPixels; // the original origin of the dib pixels
	MY_BM_INFO	mBMInfo;
	HWND		mhDDWindow;  // the covering window that gets events

	// stuff relating to direct draw 5
	BOOL				mbFlip;
	LPDIRECTDRAW		mlpDD;
	LPDIRECTDRAWSURFACE mlpDDSPrimary;
	LPDIRECTDRAWSURFACE mlpDDSBack;
	LPDIRECTDRAWSURFACE mlpDDSMemory;

};


//
// Blender -- handles blending and transforms for 16-bit graphics
//
//
// everything is done internally in 5-bit resolution, which means
// the output has only 5/5/5 resolution

typedef COLOR16 (FASTCALL * P_ALPHA_CALC_PROC)(COLOR16, COLOR16, ALPHA5);
typedef DWORD (FASTCALL * P_2_ALPHA_CALC_PROC)(DWORD, DWORD, ALPHA5);
typedef COLOR16 (FASTCALL * P_ALPHA_BLACK_PROC)(COLOR16, ALPHA5);
typedef DWORD (FASTCALL * P_2_ALPHA_BLACK_PROC)(DWORD, ALPHA5);
typedef COLOR16 (FASTCALL * P_PACK_PROC)(BYTE, BYTE, BYTE);

class Blender {
public:
	Blender(PF_MASKS &Masks);
	~Blender();

	COLOR16 dwPack(RGBQUAD *pRGB) {
		return dwPack(pRGB->rgbRed, pRGB->rgbGreen, pRGB->rgbBlue);
	}
	
	COLOR16 dwPack(BYTE jRed, BYTE jGreen, BYTE jBlue) {
		return mpdwPack(jRed, jGreen, jBlue);
	}


	// calculate alpha using a 5-bit lookup table
	COLOR16 dwCalcAlpha(COLOR16 dwBack, COLOR16 dwFore, BYTE jAlpha) {
		return mpdwAlphaCalc(dwBack, dwFore, (ALPHA5)(jAlpha >> 3));
	}

	DWORD dwCalc2Alpha(DWORD dwBack, DWORD dwFore, BYTE jAlpha) {
		return mpdw2AlphaCalc(dwBack, dwFore, (ALPHA5)(jAlpha >> 3));
	}

	COLOR16 dwCalcAlpha5(COLOR16 dwBack, COLOR16 dwFore, ALPHA5 jAlpha) {
		return mpdwAlphaCalc(dwBack, dwFore, jAlpha);
	}

	DWORD dwCalc2Alpha5(DWORD dwBack, DWORD dwFore, ALPHA5 jAlpha) {
		return mpdw2AlphaCalc(dwBack, dwFore, jAlpha);
	}

	COLOR16 dwCalcAlphaBlack5(COLOR16 dwFore, ALPHA5 jAlpha) {
		return mpdwAlphaBlack(dwFore, jAlpha);
	}

	DWORD dwCalc2AlphaBlack5(DWORD dwFore, ALPHA5 jAlpha) {
		return mpdw2AlphaBlack(dwFore, jAlpha);
	}


	// these will calculate 2 at a time, if there are two pixels packed
	// into the 32-bit DWORD
	DWORD dwCalcGray50(DWORD dwColor) {
		return (dwColor >> 1) & mdwBlack50Mask;
	}

	DWORD dwCalcGray25(DWORD dwColor) {
		return (dwColor >> 2) & mdwBlack25Mask;
	}

	//
	// amount to shift a 5-bit value from the table into the appropriate
	// slot in a 16-bit value
	BYTE	mjRShift5to16;
	BYTE	mjGShift5to16;
	BYTE	mjBShift5to16;

	//
	// amount to shift an 8-bit value to put the bits into the lowest
	// part of the byte (i.e. 3 bits down for a 5-bit value)
	//
	BYTE	mjRBaseShift;
	BYTE	mjGBaseShift;
	BYTE	mjBBaseShift;

	//
	// amount to shift a value that has been shifted to the bottom
	// to move it into the correct slot in a 16-bit value
	//
	BYTE	mjRShift;
	BYTE	mjGShift;
	BYTE	mjBShift;

	// we'll store the number of bits to rotate a DWORD left to move
	// the color pixels into position for table lookup
	int		mnRRotToFore;
	int		mnRRotToBack;
	int		mnGRotToFore;
	int		mnGRotToBack;
	int		mnBRotToFore;
	int		mnBRotToBack;

protected:
	void vInitAlphaTable(void);

	COLOR5 jGetAlphaTable(COLOR5 jFore, COLOR5 jBack, ALPHA5 jAlpha) {
		//assert(jAlpha < AM_ALPHA_MAX);
		return majAlphaTable[AT_PACK(jFore, jBack, jAlpha)];
	}
	COLOR5 jGetAlphaTable(DWORD dwPacked) {
		//assert(dwPacked < AT_SIZE);
		return majAlphaTable[dwPacked];
	}
	void vSetAlphaTable(COLOR5 jFore, COLOR5 jBack, ALPHA5 jAlpha, COLOR5 jValue) {
		//assert(jAlpha < AM_ALPHA_MAX);
		majAlphaTable[AT_PACK(jFore, jBack, jAlpha)] = jValue;
	}

	P_ALPHA_CALC_PROC		mpdwAlphaCalc;
	P_2_ALPHA_CALC_PROC		mpdw2AlphaCalc;
	P_ALPHA_BLACK_PROC		mpdwAlphaBlack;
	P_2_ALPHA_BLACK_PROC	mpdw2AlphaBlack;
	P_PACK_PROC				mpdwPack;

	// some stuff for the fast shift to black
	DWORD	mdwBlack50Mask;
	DWORD	mdwBlack25Mask;

	COLOR5	majAlphaTable[AT_SIZE];
};


int nGetDisplayFormat(PF_MASKS *pMasks = NULL);

int nMaskToNumBits(DWORD dwMask, int *pnLowBit);
//
// OLD CODE
//

/*
	COLOR16 dwMoveTowardsWhite(COLOR16 dwColor, BYTE jAmount) {
		// cheap way to handle full and none
		if (jAmount == 0xFF) {
			return 0xFFFF;
		}
		if (jAmount == 0x00) {
			return dwColor;
		}
		jAmount >>= (8 - AM_ALPHA_BITS);

		COLOR5 jFull = AM_COLOR_MASK;
		COLOR16 dwRet = ((COLOR16)jAlphaCalc(jAmount, jFull, jGetRed(dwColor)) << mjShiftR);

		// do the same for green and blue
		dwRet |= ((COLOR16)jAlphaCalc(jAmount, jFull, jGetGreen(dwColor)) << mjShiftB);
		dwRet |= ((COLOR16)jAlphaCalc(jAmount, jFull, jGetBlue(dwColor)) << mjShiftG);

		return dwRet;
	}
	COLOR16 dwMoveTowardsBlack(COLOR16 dwColor, BYTE jAmount) {
		// cheap way to handle full and none
		if (jAmount == 0xFF) {
			return 0x0000;
		}
		if (jAmount == 0x00) {
			return dwColor;
		}
		jAmount >>= (8 - AM_ALPHA_BITS);

		COLOR5 jFull = 0;
		COLOR16 dwRet = ((COLOR16)jAlphaCalc(jAmount, jFull, jGetRed(dwColor)) << mjShiftR);

		// do the same for green and blue
		dwRet |= ((COLOR16)jAlphaCalc(jAmount, jFull, jGetGreen(dwColor)) << mjShiftB);
		dwRet |= ((COLOR16)jAlphaCalc(jAmount, jFull, jGetBlue(dwColor)) << mjShiftG);

		return dwRet;
	}

	DWORD dwCalc16AlphaFast(DWORD dwBack, DWORD dwFore, BYTE jAlpha) {
		// we can make this even faster with a math lookup table of say 5 by 5 bits.

		// calculate the basic red alpha
		DWORD dwBack = dwBack & mdwMaskR;
		DWORD dwRet = (((((dwFore & mdwMaskR) - dwBack) * jAlpha) >> 8) + dwBack) & mdwMaskR;

		// do the same for green and blue
		dwBack = dwBack & mdwMaskG;
		dwRet |= (((((dwFore & mdwMaskG) - dwBack) * jAlpha) >> 8) + dwBack) & mdwMaskG;
		dwBack = dwBack & mdwMaskB;
		dwRet |= (((((dwFore & mdwMaskB) - dwBack) * jAlpha) >> 8) + dwBack) & mdwMaskB;
		
		return dwRet;
	}

	// a slightly optimized version that calculates the most common format: 565 RGB
	DWORD dwCalc565AlphaFast(DWORD dwBack, DWORD dwFore, BYTE jAlpha) {
		// calculate the basic red alpha
		DWORD dwBack = dwBack & MASK_R_565;
		DWORD dwRet = (((((dwFore & MASK_R_565) - dwBack) * jAlpha) >> 8) + dwBack) & MASK_R_565;

		// do the same for green and blue
		dwBack = dwBack & MASK_G_565;
		dwRet |= (((((dwFore & MASK_G_565) - dwBack) * jAlpha) >> 8) + dwBack) & MASK_G_565;
		dwBack = dwBack & MASK_B_565;
		dwRet |= (((((dwFore & MASK_B_565) - dwBack) * jAlpha) >> 8) + dwBack) & MASK_B_565;
		
		return dwRet;
	}

*/

#endif