#include "os.h"
#include "version.h"
#include "game.h"
#include "app.h"
#include "inifile.h"
#include "rooms.h"
#include "room0.h"
#include "roomint.h"
#include "roomend.h"
#include "roomcrdt.h"
#include "objectdb.h"
#include "drawobj.h"
#include "player.h"
#include "net.h"
#include "mapplane.h"
#include "intplane.h"
#include "unit.h"
#include "randgen.h"
#include "procs.h" 
#include "MsgInternal.h"
#include <string.h>
#include "soundlist.h"
#include "NonPlayerClass.h"
#include "animationdb.h"
#include "spell.h"
#include "missiongoal.h"
#include "splash.h"
//extern BOOL win32s;
//extern BOOL startNetNewGame;

ASIGSSession *gpoNet;
extern Application *pApp;
extern Palette *spoPal;

#define DEBUG_QUICKSTART_NONETWORKGAME 1

#define TICKS_PER_GAME_CYCLE 3
#define MAX_TICKS_WAIT_FOR_NET 120
#define TICKS_PER_NET_CHECK 10
#define NUM_CYCLES_BEFORE_FORCE_DRAW 3
#define MAX_CYCLE_OVERAGE (TICKS_PER_GAME_CYCLE * 10)

#define GAME_TEMPLATE 1
#define CLANTEMPLATES_BASE 10
#define CLANTEMPLATES_COUNT 4

typedef struct _GAME_INIT_INFO {
	int		nRandSeed;
	char	strMap[128];
} GAME_INIT_INFO;

#define MAX_WAIT_FOR_INIT 30

int localFrameThrowAwayConstant;

Game::Game()
{
	mbClearGameMap = TRUE;
	mnClearGameMapCyclesInterval = 60;

	mpoMapPlane = NULL;
	mpoIntPlane = NULL;

	mbBadSynch = FALSE;
	gpoNet = NULL;
	mbNetGame = FALSE;
	mbGameStarted = FALSE;
	mpoLocalPlayer = NULL;
	mnPlayers = 0;

	mnGameCycles = 0;
	mnLastGameCycleTime = 0;
	mbGameCycling = TRUE;
	mnTicksPerGameCycle = TICKS_PER_GAME_CYCLE;
	mnCycleOverage = 0;
	mnDrawsSkipped = 0;
	mnLastDrawCycle = 0;

	mnCurrentNetTurn = 0;
	mnTurnPostingDelay = 3;
	mnPostingNetTurn = mnCurrentNetTurn + mnTurnPostingDelay;
	mnCyclesPerNetTurn = 2;
	mnLastNetTurnCycle = 0;
	mnLastNetStopTime = 0;
	mbNetPause = FALSE;

	nextRoomNum = ROOM_INTRO;

}

Game::~Game()
{
	if (gpoNet)
	{
		delete gpoNet;
		gpoNet = NULL;
	}
}


long Game::lGetChecksum(void)
{
	// checksum all member variables that are important to net synch

	long lCheck = 0;

	// do self
	lCheck += mnCurrentNetTurn + mnPlayers + mnGameCycles;
	
	// do players
	int i;
	for (i = 0; i < mnPlayers; ++i) {
		Player *poPlayer = moPlayers.GetItem(i);
		if (poPlayer) {
			// guard against transpositions by rotating before each add
			lCheck = (long)_lrotl(lCheck, 1);
			lCheck += poPlayer->lGetChecksum();
		}
	}

	// do map (which does all map objects)
	lCheck += gpoMap->lGetChecksum();

	// do random number generator
	lCheck += pApp->oGlobalRandom.lGetChecksum();

	return lCheck;

}

void Game::vDumpSyncText(FILE *fileDump)
{
	// do self
	fprintf(fileDump, "**Game Network Synchronization Dump\n");
	fprintf(fileDump, "This player number: %d\n", mpoLocalPlayer->mnPlayer);
	fprintf(fileDump, "Current net turn: %d\n", mnCurrentNetTurn);
	fprintf(fileDump, "Game cycles: %d\n", mnGameCycles);
	fprintf(fileDump, "Number of players: %d\n", mnPlayers);
	
	// do players
	int i;
	for (i = 0; i < mnPlayers; ++i) {
		Player *poPlayer = moPlayers.GetItem(i);
		if (poPlayer) {
			poPlayer->vDumpSyncText(fileDump);
		}
	}

	// do map (which does all map objects)
	gpoMap->vDumpSyncText(fileDump);

	// do random number generator
	pApp->oGlobalRandom.vDumpSyncText(fileDump);
}

WORD Game::wGetMiniMapColor(int nColorIndex)
{
	assert(nColorIndex < MAX_PLAYERS);
	assert(MAX_PLAYERS == 8);

	static ULONG maunColorTags[MAX_PLAYERS][3] =
	{
		{TAG_mp1R, TAG_mp1G, TAG_mp1B},
		{TAG_mp2R, TAG_mp2G, TAG_mp2B},
		{TAG_mp3R, TAG_mp3G, TAG_mp3B},
		{TAG_mp4R, TAG_mp4G, TAG_mp4B},
		{TAG_mp5R, TAG_mp5G, TAG_mp5B},
		{TAG_mp6R, TAG_mp6G, TAG_mp6B},
		{TAG_mp7R, TAG_mp7G, TAG_mp7B},
		{TAG_mp8R, TAG_mp8G, TAG_mp8B},
	};

	return (WORD)gpoBlender->dwPack(
		(BYTE)mpoTemplate->lGetValue(maunColorTags[nColorIndex][0], 0),
		(BYTE)mpoTemplate->lGetValue(maunColorTags[nColorIndex][1], 0),
		(BYTE)mpoTemplate->lGetValue(maunColorTags[nColorIndex][2], 0));
}

void Game::vClearPlayers(void)
{
	moPlayers.DisposeAll();
	mpoLocalPlayer = NULL;
	mnPlayers = 0;
	moGameSync.SetPlayerCount(mnPlayers);
}

void Game::vAddPlayer(Player *poPlayer)
{
	moPlayers.SetItem(poPlayer->mnPlayer, poPlayer);
	++mnPlayers;
	moGameSync.SetPlayerCount(mnPlayers);
	if (poPlayer->mbHuman && poPlayer->mbLocal) {
		assert(!mpoLocalPlayer);
		mpoLocalPlayer = poPlayer;
	}
}

void Game::vSetupGame(void)
{
	int nPlayers;
	int nMe;
	int i;
	UINT unPlayer;
	int nSize;

	mpoTemplate = (Template*)pApp->pResourceManager->LoadResource(RES_TEMPLATE, GAME_TEMPLATE);
	assert(mpoTemplate);

	// TEMP -- check multi-loop bitmaps
	//Bitmap *poTestBMP = (Bitmap *)pApp->pResourceManager->LoadResource(RES_BITMAP, 99999);
	//delete poTestBMP;
	// TEMP -- quick check of random number generator
	/*
	int k;
	RandomGenerator oGenerator(999);
	for (k = 0; k < 10; ++k) {
		int	nBuckets[16];
		for (i = 0; i < 16; ++ i) {
			nBuckets[i] = 0;
		}
		for (i = 0; i < 10000; ++i) {
			int j;
			unsigned int unRand = oGenerator.unGetRandom();
			for (j = 0; j < 8; ++ j) {
				int nBucket = unRand & 0xf;
				nBuckets[nBucket]++;
				unRand >>= 4;
				//vDebugOut("%08x ", oGenerator.unGetRandom());
			}
			//vDebugOut("\n");
		}
		for (i = 0; i < 16; ++ i) {
			vDebugOut("%4d ", nBuckets[i]);
		}
		vDebugOut("\n");
	}
	*/

#if DEBUG_QUICKSTART_NONETWORKGAME
	VALIDATE_DLL_EXIT nRet = DLL_EXIT_FAILED_TO_UPDATE;
#else
	//
	// attempt a multi-player game
	gpoNet = new ASIGSSession;
	assert(gpoNet);
	VALIDATE_DLL_EXIT nRet = gpoNet->Create(pApp->pWindow->hWnd, "c:\\dev\\scicpp\\", "c:\\dev\\scicpp\\debug\\ph3.exe", "vampire");
#endif
	
	if (nRet == DLL_EXIT_CONTINUE_TO_CONNECT) {
		// we've got a net game -- initialize info
		mbNetGame = TRUE;
		nPlayers = gpoNet->GetNumberOfPlayers();
		nMe = gpoNet->GetMyNumber();
	} else {
		mbNetGame = FALSE;
		nPlayers = 1;
		nMe = 0;
	}

	game->vClearPlayers();

	for (i = 0; i < nPlayers; ++i) {
		Player *poPlayer;
		if (i == nMe) {
			poPlayer = new Player(PL_HUMAN, PL_LOCAL, i);
		} else {
			poPlayer = new Player(PL_HUMAN, PL_REMOTE, i);
		}
		game->vAddPlayer(poPlayer);
	}

	// TBD - SINGLE PLAYER GAME ADD COMPUTER PLAYERS
	if (nPlayers == 1)
	{
		Player *poPlayer;
		i = 1;
		poPlayer = new Player(PL_COMPUTER, PL_LOCAL, i);
		game->vAddPlayer(poPlayer);
	}

	//
	// if I'm the game master, I should send out the initialization info
	//
	GAME_INIT_INFO game_info;
	game_info.nRandSeed = timeGetTime();
	//if (nMe == 0 && nPlayers > 1) {
	if (nMe == 0 && mbNetGame) 
	{
		strcpy(game_info.strMap, "default");
		gpoNet->SendMessageAll((char*)&game_info, sizeof(GAME_INIT_INFO));
	} 
	else 
	{
		//if (nPlayers > 1) 
		if (mbNetGame)
		{
			int nWait = 0;
			while (++nWait < MAX_WAIT_FOR_INIT) {
				nSize = gpoNet->ReceiveMessage((char*)&game_info, sizeof(GAME_INIT_INFO), &unPlayer);
				if (nSize == 0) {
					Sleep(200);
				} else {
					break;
				}
			}
			assert(nSize);
			assert(unPlayer == 0);
		}
	}

	pApp->oGlobalRandom.vSeed(game_info.nRandSeed);

	game->SetClearGameMapInterval((int)(20.0 * mpoTemplate->fGetValue(TAG_cgmi, 3)));
	game->SetNetworkGameSynchronizationInterval((int)(20.0 * mpoTemplate->fGetValue(TAG_ngsi, 0.25)));
	
	Unit::vInitStatics();	//initialize the member static variables for unit
	mclrHighlight = PaletteFindNearest(spoPal->colors, 
		(BYTE)mpoTemplate->lGetValue(TAG_Hred, 255), 
		(BYTE)mpoTemplate->lGetValue(TAG_Hgre, 255), 
		(BYTE)mpoTemplate->lGetValue(TAG_Hblu, 0));

	localFrameThrowAwayConstant = mpoTemplate->lGetValue(TAG_sped, 15);

	// non-player class base template
	int nVal = mpoTemplate->lGetValue(TAG_npcb, 40);
	UINT unNpcBase = 40;
	if (nVal >= 0)
	{
		UINT unNpcBase = (UINT)nVal;
	}
	// non-player class count
	nVal = mpoTemplate->lGetValue(TAG_npcc, 5);
	UINT unNpcCount = 5;
	if (nVal >= 0)
	{
		unNpcCount = (UINT)nVal;
	}
	
	// LOAD THE NPC TEMPLATES
	Template * poNpcTemplate;
	UINT unNpcTemplate;
	UINT unNpcTop = unNpcBase + unNpcCount;
	NonPlayerClass * poNpc = NULL;
	for(unNpcTemplate = unNpcBase; unNpcTemplate < unNpcTop; unNpcTemplate++)
	{
		poNpcTemplate = (Template *)pApp->pResourceManager->LoadResource(RES_TEMPLATE, unNpcTemplate);
		poNpc = new NonPlayerClass(*poNpcTemplate);
		moNpcList.AddToEnd(poNpc);
	}

	// LOAD THE CLAN TEMPLATES
	Template * poClanTemplate;
	Clan * poClan;
	int nClanTemplate;
	for(nClanTemplate = CLANTEMPLATES_BASE; nClanTemplate < CLANTEMPLATES_COUNT + CLANTEMPLATES_BASE; nClanTemplate++)
	{
		poClanTemplate = (Template *)pApp->pResourceManager->LoadResource(RES_TEMPLATE, nClanTemplate);
		poClan = new Clan(*poClanTemplate);
		moClans.AddToEnd(poClan);
	}

	// LOAD AVAILABLE SPELLS INTO GAME
	int nGameSpellTemplateBase;
	nGameSpellTemplateBase = mpoTemplate->lGetValue(TAG_gstb, 61);
	int nGameSpellTemplateCount;
	nGameSpellTemplateCount =  mpoTemplate->lGetValue(TAG_gstc, 21);

	Template * poSpellTemplate;
	int nSpellTemplate = 0;
	while(nSpellTemplate < nGameSpellTemplateCount)
	{
		Spell * poSpell;
		poSpellTemplate = (Template *)pApp->pResourceManager->LoadResource(RES_TEMPLATE, nSpellTemplate + nGameSpellTemplateBase);
		assert(poSpellTemplate);
		poSpell = new Spell(*poSpellTemplate);
		assert(poSpell);
		moSpells.AddToEnd(poSpell);
		nSpellTemplate++;
	}

	// TBD - SET THE CLAN FOR EACH PLAYER
	for (int nCount = 0;nCount<moPlayers.GetSize();nCount++)
	{
		//moPlayers.GetItem(nCount)->mpoClan = moClans.GetItem(1); TW - commented this line out
		moPlayers.GetItem(nCount)->mpoClan = moClans.GetItem(nCount);

		//setup player spells
		moPlayers.GetItem(nCount)->vInitSpellSpace(nGameSpellTemplateCount);

	}

	mpoMapPlane = new MapPlane();
	mpoIntPlane = new InterfacePlane();

	// CENTER THE VIEW ON THE VAMPIRE LORD
	for (i= 0; i < moPlayers.GetSize(); ++i)
	{
		Player *poPlayer = moPlayers.GetItem(i);
		assert (poPlayer);
		if (mpoLocalPlayer == poPlayer)
		{
			Unit * poVampireLord = NULL;
			poPlayer->GetVampireLord(&poVampireLord);
			if (poVampireLord)
			{
				gpoMap->vCenterNode(poVampireLord->poNode());
			}
			break;
		}

	}

	//MapNode *poNode = gpoMap->poGetNode(10, 10);
	//Unit *poUnit = new Unit(NULL, poMapPlane, poNode, 400);

	vStartGame();
}

void Game::vStartGame(void)
{
	mbGameStarted = TRUE;
	mbGameCycling = TRUE;
	mnLastGameCycleTime = time;
	mnLastNetCheckTime = time;
	vSetPause(FALSE);

	mnCycleOverage = 0;
	mnCurrentNetTurn = 0;
	mnPostingNetTurn = mnCurrentNetTurn + mnTurnPostingDelay;
	mnLastNetTurnCycle = 0;
	mnLastNetStopTime = 0;
	mbNetPause = FALSE;
	mnGameCycles = 0;

	// TBD - queueing background music
	if (pApp->pSoundManager)
	{
		pApp->pSoundManager->EmitAmbient(SOUND_MoodMusicLoop1, SOUNDPRIORITY_BACKGROUNDMUSIC, TRUE, 128);
	}
		
}

void Game::vKillGame(void)
{
	mbGameStarted = FALSE;

	// STOP BACKGROUND MUSIC
	
}

void Game::vCheckNet(void) {
	if (!mbNetGame) {
		return;
	}
	//if ((mnPauseTime + time - mnLastNetCheckTime) < TICKS_PER_NET_CHECK) {
	//	return;
	//}
	mnLastNetCheckTime = time;

	BYTE ajBuf[MAX_NET_MSG_SIZE];
	UINT unPlayer;

	int nSize = gpoNet->ReceiveMessage((char*)ajBuf, MAX_NET_MSG_SIZE, &unPlayer);
	while (nSize) {

		BYTE jKind = *(BYTE *) ajBuf;

		switch(jKind)
		{
			case NET_MSG_TURN:
				{
					assert(unPlayer < (UINT)mnPlayers);

					NET_TURN_HEAD *pHead = (NET_TURN_HEAD*)ajBuf;

					int nTurn = pHead->lTurn;
					
					Player *poPlayer = moPlayers.GetItem(unPlayer);
					assert(poPlayer);

					Command *poCommand = new Command(ajBuf + sizeof(NET_TURN_HEAD));
					poCommand->mlChecksum = pHead->lChecksum;

					poPlayer->vPostRemoteCommand(poCommand, nTurn);
				}
				break;
		
			case NET_MSG_INTERNAL:
				{	
					assert(unPlayer < (UINT)mnPlayers);

					NET_INTERNAL_HEAD *pHead = (NET_INTERNAL_HEAD *)ajBuf;
					
					MsgInternal *poMsgInternal = new MsgInternal(ajBuf + sizeof(NET_INTERNAL_HEAD));

					switch (poMsgInternal->getType())
					{
						case MSGINTERNAL_CHECKSUM:
							{	
								Player *poPlayer = moPlayers.GetItem(unPlayer);
								assert(poPlayer);  // invalid Player
								
								mbBadSynch = !(moGameSync.IsValid(unPlayer, pHead->lTurn, pHead->lChecksum));
							}
							break;
						case MSGINTERNAL_QUITGAME:
							{
								game->QuitGame(TRUE);
							}

						case MSGINTERNAL_PAUSE:
						case MSGINTERNAL_UNPAUSE:
						case MSGINTERNAL_NOTSET:
						default:
							assert(0); // this internal message not handled
							break;
					}
					delete poMsgInternal;
				}				

				break;
			default:
				assert(0);  // network message type not handled
		}
		nSize = gpoNet->ReceiveMessage((char*)ajBuf, MAX_NET_MSG_SIZE, &unPlayer);
	}
}

void Game::vGameTick(void) {

	// if we're paused for some other reason than waiting for the net,
	// bail
	if (bPaused() && !mbNetPause) {

		// we still should draw
		pApp->pWindow->PaintIfDirty();
		return;
	}

	// if we're not yet into the game proper, bail
	//
	if (!mbGameStarted) {

		// we still should draw
		pApp->pWindow->PaintIfDirty();
		return;
	}

	// we should definitely try to empty the net queue
	//
	vCheckNet();

	if (mbBadSynch)
	{
		// TBD - quit game to main menu when network synch fails

		// games are out of synch so 
		vMessage("Network synchronization failure.\n\nENTER to create c:\\errorlog.txt");
		
		// create error log
		FILE * file;
		file = fopen("c:\\errorlog.txt", "w+");
		vDumpSyncText(file);
		fclose(file);

		game->QuitGame(TRUE);
	}

	// if we're not cycling, perhaps we should check again for a net turn
	if (!mbGameCycling) {
		mbGameCycling = bHandleNetTurn();
		if (mbGameCycling) {
			vDebugOut("Turns arrived after %d ticks\n", time - mnLastNetStopTime);
			vSetPause(FALSE);
			mbNetPause = FALSE;
		}
	}


	// if we've been waiting for a net turn long enough, pause the game until
	// it arrives.
	if (!mbGameCycling && !mbNetPause && (time - mnLastNetStopTime) > MAX_TICKS_WAIT_FOR_NET) {
		vDebugOut("Going into pause for lack of turns\n");
		mbNetPause = TRUE;
		vSetPause(TRUE);
	}

	// TBD -- if we've been waiting for a turn long enough, kick off the person we've
	// been waiting for, and keep going. DJM

	//
	// get the next net turn
	//
	assert ((mnGameCycles - mnLastNetTurnCycle) <= mnCyclesPerNetTurn);

	// check for a new net turn and send it out
	if ((mnGameCycles - mnLastNetTurnCycle) == mnCyclesPerNetTurn) {
		mnLastNetTurnCycle = mnGameCycles;
		mbGameCycling = bHandleNetTurn();
		if (!mbGameCycling) {
			// we didn't have a turn from one or more players
			// start timing how long before we need to pause
			mnLastNetStopTime = time;
			vDebugOut("Waiting for net turn %d.\n", mnCurrentNetTurn);
		}
	}

	// DJM -- first pass game cycle loop
	mnCycleOverage = (time - mnLastGameCycleTime) - mnTicksPerGameCycle;

	// make sure we aren't going to go more than 3 cycles without drawing
	/*
	if (mnCycleOverage > mnTicksPerGameCycle * NUM_CYCLES_BEFORE_FORCE_DRAW) {
		bForceDraw = TRUE;
		//mnCycleOverage = mnTicksPerGameCycle * 3;
	} else {
		bForceDraw = FALSE;
	}
	*/

	// if the cycle overage gets way out of hand, we need
	// to shrink it
	if (mnCycleOverage > MAX_CYCLE_OVERAGE) {
		mnCycleOverage = MAX_CYCLE_OVERAGE;
	}

	if (mbGameCycling && mnCycleOverage >= 0) {
			
		// try to catch up if we're behind
		// by pretending the last game cycle was longer ago
		// than it really was
		mnLastGameCycleTime = time - mnCycleOverage;

		vGameCycle();

	}
}


void Game::SetClearGameMapInterval(int nVal)
{
	mnClearGameMapCyclesInterval = nVal;
}

void Game::SetNetworkGameSynchronizationInterval(int nVal)
{
	mnNetworkGameSynchronizationInterval = nVal;
}

void Game::ToggleClearGameMap()
{
	mbClearGameMap = mbClearGameMap == FALSE;
}

void Game::vGameCycle(void)
{
	++mnGameCycles;
	
	if (GoalCount())
	{
		if (GoalCountCompleted() == GoalCount())
		{
			/*
			SetNextRoom(ROOM_INTRO);
			LoadTheRoom(ROOM_INTRO);
			vKillGame();
			return;
			*/
			WinScreen *poWin = new WinScreen();
		///	game->vSetPause(TRUE);
		///	game->QuitGame(TRUE);
		}
	}

	// clear game map
	if (mbClearGameMap)
	{
		if (!mnClearGameMapCyclesInterval)
		{
			gpoMap->vClearVisibility();
		}
		else
		{
			if (mnGameCycles % mnClearGameMapCyclesInterval == 0)
			{
				gpoMap->vClearVisibility();
			}
		}
	}

	// TBD -- add new list of just stuff that needs game cycles
	//int nSize = goObjectDB.nTotalSlots();
	//int i;
	DrawObj **ppoBegin = goObjectDB.poRawDB()->GetBegin();
	DrawObj **ppoObj = ppoBegin;
	while (ppoObj != goObjectDB.poRawDB()->GetEnd()) {
		DrawObj *poObj = *ppoObj++;
		if (poObj) {
			poObj->vGameCycle();

			// handle the problem of the list growing while we walk it
			if (ppoBegin != goObjectDB.poRawDB()->GetBegin()) {
				ppoObj = goObjectDB.poRawDB()->GetBegin() + (ppoObj - ppoBegin);
				ppoBegin = goObjectDB.poRawDB()->GetBegin();
			}
		}
	}
	//dish out game cycles to individual objects that may need them
	pApp->pSoundManager->GameCycle();
	gpoInterface->vGameCycle();
	goAnimationDB.vAnimationCycle();

	// if we're way behind, don't draw
	if (mbGameCycling &&
		(mnGameCycles - mnLastDrawCycle) < NUM_CYCLES_BEFORE_FORCE_DRAW && 
		mnCycleOverage > localFrameThrowAwayConstant) 
	{
		// skip this paint in order to catch up
		++mnDrawsSkipped;
#ifdef _DEBUG
		if (mnDrawsSkipped % 100 == 0) {
			vDebugOut(
				"Cycles: %d, Draw skipped: %d, Percent skipped: %.1f, Cycle overage: %d\n", 
				mnGameCycles, 
				mnDrawsSkipped, 
				(double)mnDrawsSkipped/mnGameCycles * 100.0,
				mnCycleOverage);
		}
#endif
	} else {
		// everything's copacetic -- go ahead and paint
		mnLastDrawCycle = mnGameCycles;
		pApp->pWindow->PaintIfDirty();;
	}
}

NonPlayerClass * Game::GetNonPlayerClass(ENUM_NPC eNpc)
{
	return moNpcList.GetItem((int)eNpc);
}

Spell * Game::GetSpell(int nIndex)
{
	if (nIndex < 1 || nIndex > 21) nIndex = 1;
	nIndex--;
	return moSpells.GetItem(nIndex);
}

//uses true indexing
Spell *Game::GetTrueSpell(int nIndex)
{
	if (nIndex < 0 || nIndex >= mpoTemplate->lGetValue(TAG_gstc, 0))
	{
		assert(0);
		nIndex = 0;
	}
	return moSpells.GetItem(nIndex);
}

BOOL Game::bHandleNetTurn()
{
	int i;

	if (!mnPlayers) {
		return FALSE;
	}

	BOOL bBadNetSynch = FALSE;

	// check for all turns available
	for (i = 0; i < mnPlayers; ++i) {
		Player *poPlayer = moPlayers.GetItem(i);
		if (poPlayer) {
			if (!poPlayer->bTurnAvailable(mnCurrentNetTurn)) {
				///// TBD - COMPUTER PLAYER COMMANDS
				if (!poPlayer->mbHuman)
				{
					Command *poCommand = new Command;
					poCommand->meType = CM_NONE;
					poCommand->mnPlayer = i;
					poPlayer->vPostRemoteCommand(poCommand, mnCurrentNetTurn);
					i--;
					continue;
				}
				// The turns aren't there. Stop game processing.
				return FALSE;
			}
		}
	}


	for (i = 0; i < mnPlayers; ++i) {
		Player *poPlayer = moPlayers.GetItem(i);
		if (poPlayer) {
			poPlayer->vNetTurn(mnCurrentNetTurn);
		}
	}


	// TBD -- desynchronize this from the handling of net turns
	//
	// check for local player turn, post one if there isn't
	// one already
	if (!mpoLocalPlayer->bTurnAvailable(mnPostingNetTurn)) {
		Command *poCommand = new Command;
		poCommand->meType = CM_NONE;
		mpoLocalPlayer->vPostCommand(poCommand);
	}

	++mnCurrentNetTurn;
	++mnPostingNetTurn;

	// IF MULTIPLAYER GAME 
	//if (mnPlayers > 1)
	if (mbNetGame)
	{
		// IF SYNCHRONIZATION TURN
		if ((mnCurrentNetTurn % mnNetworkGameSynchronizationInterval)==0)
		{
			// SEND SYNCHRONIZATION INFO
			poLocalPlayer()->SendChecksumInfo();		
			// CHECK SYNCHRONIZATION INFO
			mbBadSynch = !(moGameSync.IsValid(poLocalPlayer()->mnPlayer, mnCurrentNetTurn, lGetChecksum()));
		}
	}

	return TRUE;
}

void Game::LoadTheRoom(int roomNum)
{
	sciGame::LoadTheRoom(roomNum);

	switch (roomNum) {
	case ROOM_INTRO:
		curRoom = new RoomSelect();
		break;
	case ROOM_0:
		curRoom = new Room0();
		break;
	case ROOM_END:
		curRoom = new RoomEnd();
		break;
	case ROOM_CREDITS:
		curRoom = new RoomCredits();
		break;
	}
	curRoom->Init();
}

void Game::AddGoal(MissionGoal * poMissionGoal)
{
	assert(poMissionGoal);
	moGoals.AddToEnd(poMissionGoal);
}

Bitmap * Game::GetHighlightBitmap()
{
	return mpoIntPlane->GetHighlightBitmap();
}

int Game::GoalCount()
{
	return moGoals.GetSize();
}

int Game::GoalCountCompleted()
{
	int nCount = 0;
	int n = moGoals.GetSize();
	while(n)
	{
		n--;
		MissionGoal * poGoal = moGoals.GetItem(n);
		if (poGoal->GetCompleted())
		{
			nCount++;
		}	
	}
	return nCount;
}