// GAMEMAP.CPP the map handler
//
// DJM
#include "os.h"
#include "scimain.h"
#include "mapnode.h"
#include "gamemap.h"
#include "mapplane.h"
#include "obstaclepoint.h"
#include "player.h"
#include "unit.h"
#include "intplane.h"
#include "colordef.h"
#include "building.h"
#include "procs.h"
#include "spawn.h"
#include "ztimer.h"

#define MAP_CHECKSUM_FREQ 5

#define DEBUG_LOCALMESSAGES 0

GameMap *gpoMap;

char cMagicNumber[] = "TIMW";	

/**********
 * GameMap
 **********/

GameMap::GameMap(MapPlane *poPlane): mbSelectingWithMouse(FALSE), mbLeftMouseIsDown(FALSE), mnSetAttributeID(ATTR_TENACITY)
{
	mnPasses = DP_NUM_LAYERS;
	mnPosX = 0;
	mnPosY = 0;
	mnWidth = 0;
	mnVersion = 0;
	mnHeight = 0;
	nPrevX = -1;
	nPrevY = -1;
	mpoPlane = poPlane;
	SetMode(UIMODE_SELECT);
	mnKeyScrollSpeed = game->mpoTemplate->lGetValue(TAG_Kscr,2);
	mpjVisibility = NULL;
	mpoSinglePlayerStart = NULL;
}

void GameMap::vVisibilitySetAll(BYTE jSetting)
{
	FillMemory(mpjVisibility, mnWidth * mnHeight, jSetting);
}

void GameMap::vClearVisibility(void)
{
	// TEMP TEST -- DJM
	//return;
#if 0
	ZeroMemory(mpjVisibility, mnWidth * mnHeight);
#else
	int nSize = mnWidth * mnHeight;
	BYTE *pjTemp = mpjVisibility;
	while(nSize--) {
		if (*pjTemp >= MAP_VISIBILITY_EXPOSED_MAP) {
			*pjTemp = MAP_VISIBILITY_EXPOSED_MAP;
		}
		pjTemp++;
	}
#endif
}

#pragma optimize( "agt", on )

void GameMap::vApplyVisibility(void)
{
	//TEMP TEST -- DJM
	//return;

	if (!mpoPlane) return;

	FrameBuffer *poFrameBuf = pApp->poFrameBuf();

	assert(poFrameBuf);
	assert(poFrameBuf->mnBits == 16);

	//WORD wBackColor = (WORD)gpoBlender->dwPack(0, 0, 0);

	WORD *pwPixels = (WORD*)(poFrameBuf->pGetBuffer());
	int nPitch = poFrameBuf->mnPitch / 2; // handle pitch in words
	int nPixelX = mpoPlane->GetX();
	int nPixelY = mpoPlane->GetY();

	int nPixelWidth = mpoPlane->nWidth();
	int nPixelHeight = mpoPlane->nHeight();

	int nFirstCol = mnSpacingX / 2;
	int nSizeX = (nPixelWidth - nFirstCol) / mnSpacingX + 1;
	int nLastCol =  nPixelWidth - ((nSizeX - 2) * mnSpacingX) - nFirstCol;
	int nLimitX = mnPosX + nSizeX;

	int nFirstRow = mnSpacingY / 2;
	int nSizeY = (nPixelHeight - nFirstRow) / mnSpacingY + 1;
	int nLastRow = nPixelHeight - ((nSizeY - 2) * mnSpacingY) - nFirstRow;
	int nLimitY = mnPosY + nSizeY;

	int i, j;

	assert(nLastRow >= 0);
	assert(nLastCol >= 0);

	// get the pixel pointer in the right position
	pwPixels += nPixelX + nPixelY * nPitch;

	int nRow = nFirstRow;

	for (j = mnPosY; j < nLimitY; ++j) {
		int nCol = nFirstCol;
		WORD *pwRow = pwPixels; // save the row beginning
		for (i = mnPosX; i < nLimitX; ++i) {
			ALPHA5 jVisibility = (ALPHA5)(jGetVisibility(i, j) >> 3); // take it to 5 bits
			if (!jVisibility) {

				// if jVisibility == 0, just plot zeros
				WORD *pwTemp = pwPixels;
				int k = nRow;
				while (k--) {
					memset(pwTemp, 0, nCol * 2);
					pwTemp += nPitch;
				}
			} else if (jVisibility == 16) {
				// if jVisibility == 128 (16 in 5-bit), use the fast reduction
				WORD *pwTemp = pwPixels;
				int k = nRow;
				//int jAlpha = 255 - jVisibility;
				while (k--) {
					int l = nCol;

					// do the odd word
					if (l & 1) {
						*pwTemp = (WORD)(gpoBlender->dwCalcGray50(*pwTemp));
						++pwTemp;
					}

					// now do the rest as DWORDs
					l >>= 1;
					while (l--) {
						*(DWORD*)pwTemp = gpoBlender->dwCalcGray50(*(DWORD*)pwTemp);
						pwTemp += 2;
					}

					pwTemp += nPitch - nCol;
				}
			/*
			} else if (jVisibility == 8) {
				// if jVisibility == 64 (8 in 5-bit), use the fast reduction
				WORD *pwTemp = pwPixels;
				int k = nRow;
				//int jAlpha = 255 - jVisibility;
				while (k--) {
					int l = nCol;
					while (l--) {
						*pwTemp = (WORD)(gpoBlender->dwCalcGray25(*pwTemp));
						++pwTemp;
					}
					pwTemp += nPitch - nCol;
				}
			*/
			} else if (jVisibility < 31) {

				// if jVisibility < 255 (31 in 5-bit), then reduce toward black
				WORD *pwTemp = pwPixels;
				int k = nRow;
				while (k--) {
					int l = nCol;

					// do the odd word
					if (l & 1) {
						*pwTemp = (WORD)(gpoBlender->dwCalcAlphaBlack5(*pwTemp, jVisibility));
						++pwTemp;
					}

					// do the rest as DWORDs
					l >>= 1;
					while (l--) {
						*(DWORD*)pwTemp = gpoBlender->dwCalc2AlphaBlack5(*(DWORD*)pwTemp, jVisibility);
						pwTemp += 2;
					}
					pwTemp += nPitch - nCol;
				}
			}
			// there's no need to handle jVisibility == 255, it just means normal

			// move to the next node
			pwPixels += nCol;
			if (i == nLimitX - 2) {
				nCol = nLastCol;
			} else {
				nCol = mnSpacingX;
			}
		}

		// ten lines down
		pwPixels = pwRow + (nRow * nPitch);
		if (j == nLimitY - 2) {
			nRow = nLastRow;
		} else {
			nRow = 10;
		}
	}

	poFrameBuf->vReleaseBuffer();
}

				
	
long GameMap::lGetChecksum(void)
{
	// checksum all member variables that are important to net synch

	long lCheck = 0;

	// do self
	lCheck += mnHeight + mnWidth;
	
	// do nodes
	int i,j;
	for (i = 0; i < mnHeight; ++i) {
		for (j = 0; j < mnWidth; ++j) {
			MapNode *poNode = poGetNode(j, i);
			if (poNode) {
				// guard against transpositions by rotating before each add
				lCheck = (long)_lrotl(lCheck, 1);
				lCheck += poNode->lGetChecksum();
			}
		}
	}

	return lCheck;
}

void GameMap::vDumpSyncText(FILE *fileDump)
{
	// do self
	fprintf(fileDump, "*GameMap\n");
	fprintf(fileDump, "Map height: %d\n", mnHeight);
	fprintf(fileDump, "Map width: %d\n", mnWidth);
	
	// do nodes
	int i,j;
	for (i = 0; i < mnHeight; ++i) {
		for (j = 0; j < mnWidth; ++j) {

			// every tenth column print marker
			fprintf(fileDump,"--Row, Column: %d, %d\n", i, j);
			MapNode *poNode = poGetNode(j, i);
			if (poNode) {
				poNode->vDumpSyncText(fileDump);
			}
		}
	}
}

#define OVERDRAW_LEFT	10
#define OVERDRAW_RIGHT	10
#define OVERDRAW_TOP	6
#define OVERDRAW_BOTTOM	12

void GameMap::vDraw(Boolean bUpdate)
{
	int i, j;
	int k;
	int nLimitX, nLimitY, nBeginX, nBeginY;
	int nScreenX, nScreenY;
	int nLayers;

#if 0
	RECT rMap;

	// clip to the map rect
	SetRect(&rMap, mnScreenOrgX, mnScreenOrgY, 
		mnScreenOrgX + (mnScreenWidth * mnSpacingX),
		mnScreenOrgY + (mnScreenHeight * mnSpacingY));
	mpoPlane->vLocalToGlobal(&rMap);
	IntersectRect(&rMap, &rMap, mpoPlane->prGetClip());
#endif

	nLimitX = min(mnPosX + mnScreenWidth + OVERDRAW_RIGHT, mnWidth);
	nLimitY = min(mnPosY + mnScreenHeight + OVERDRAW_BOTTOM, mnHeight);

	nBeginX = max(0, (int)mnPosX - OVERDRAW_LEFT);
	nBeginY = max(0, (int)mnPosY - OVERDRAW_TOP);

	if (gbToggleObstacleGrid) {
		nLayers = DP_NUM_LAYERS;
	} else {
		nLayers = DP_NUM_LAYERS - 1;
	}

	mpoPlane->vClearDrawnList();

	//TIMER_BEGIN();

	for (k = DP_GROUND; k < nLayers; ++k) {

		nScreenY = mnScreenOrgY - ((mnPosY - nBeginY) * mnSpacingY);

		for (j = nBeginY; j < nLimitY; ++j) {

			nScreenX = mnScreenOrgX - ((mnPosX - nBeginX) * mnSpacingX);
			for (i = nBeginX; i < nLimitX; ++i) {
				MapNode *poNode;

				poNode = poGetNode(i, j);
				if (poNode) {
					poNode->vDraw((DRAW_PASS)k, nScreenX, nScreenY);
				}
				nScreenX += mnSpacingX;
			}
			nScreenY += mnSpacingY;
		}
	}

	//TIMER_END();

	if (mbSelectingWithMouse)
	{
		// DONE - PB - MAKE IT SO THAT STRETCHWINDOWSELECTION WORKS WITH 
		//    SCROLLING TOO.

		int tmpTop    = min(mnStartSelectionY - (nOrgPixelY() - mnStartScreenOrgY), mnEndSelectionY);
		int tmpLeft   = min(mnStartSelectionX - (nOrgPixelX() - mnStartScreenOrgX), mnEndSelectionX);

		mpoPlane->vLocalToGlobal(&tmpLeft, &tmpTop);

		WORD wObjWidth	= max(1, abs(mnEndSelectionX + (nOrgPixelX() - mnStartScreenOrgX) - mnStartSelectionX));
		WORD wObjHeight = max(1, abs(mnEndSelectionY + (nOrgPixelY() - mnStartScreenOrgY) - mnStartSelectionY));

		int tmpRight = (tmpLeft + wObjWidth);
		int tmpBottom = (tmpTop + wObjHeight);

		tmpTop    = min(mnStartSelectionY - (nOrgPixelY() - mnStartScreenOrgY), mnEndSelectionY);
		tmpLeft   = min(mnStartSelectionX - (nOrgPixelX() - mnStartScreenOrgX), mnEndSelectionX);

		
		RECT rLocal = {tmpLeft, tmpTop, tmpLeft + wObjWidth, tmpTop + wObjHeight};

		// TBD - PB - DO THIS WITHOUT USING INTERMEDIATE BITMAP
		Bitmap *pBM = new Bitmap(wObjWidth, wObjHeight);
		pBM->DrawRect(game->ColorHighlight(), rLocal, mpoPlane, 1);
		delete pBM;
		 
	}

	vApplyVisibility();

#if DEBUG_LOCALMESSAGES
	DebugString("Finished drawing map\n");
#endif
	if (bUpdate) {
		pApp->pWindow->Update(*mpoPlane->prGetClip());
	}

}

/*
MapNode *GameMap::poGetNode(int nX, int nY)
{
	assert(nY >= 0);
	assert(nX >= 0);
	return (MapNode *)moNodes.GetItem(nX, nY);
}
*/

void GameMap::vSetNode(int nX, int nY, MapNode *poNode)
{
	moNodes.SetItem(nX, nY, poNode);
}

MapNode *GameMap::poNodeFromEvent(sciEvent &oEvent)
{
	// we assume the event is in local plane coordinates
	if (oEvent.x < 0 || oEvent.y < 0 || 
		oEvent.x > ((mnSpacingX * mnScreenWidth)  - (mnSpacingX / 2) - 1) ||
		oEvent.y > ((mnSpacingY * mnScreenHeight) - (mnSpacingY / 2) - 1) ) {
// commented these 2 lines out Don - TW
//		oEvent.x > mpoPlane->nWidth() ||
//		oEvent.y > mpoPlane->nHeight() ) {
		
		// off the plane -- no hit
		return NULL;
	}

	// round to the nearest grid space
	int nGridX = ((oEvent.x + (mnSpacingX / 2)) / mnSpacingX) + mnPosX;
	int nGridY = ((oEvent.y + (mnSpacingY / 2)) / mnSpacingY) + mnPosY;

	return poGetNode(nGridX, nGridY);
}

//Used mainly for getting node information when the mouse is just resting (no generated event)
MapNode *GameMap::poNodeFromMouse() {
	POINT pt;
	GetCursorPos(&pt);
	ScreenToClient(pApp->pWindow->hWnd, &pt);
	sciEvent oEvent;
	oEvent.x = pt.x;
	oEvent.y = pt.y;
	oEvent.vLocalize((sciPlane*)mpoPlane);
	return poNodeFromEvent(oEvent);
}

BOOL GameMap::bScrollUp(int nHowMuch)
{
	if (mnPosY == 0) {
		return FALSE;
	} else {
		mnPosY = max(0, mnPosY - nHowMuch);
		if (pApp->pSoundManager)
		{
			pApp->pSoundManager->SetViewWindowPos(mnPosX, mnPosY);
		}
		return TRUE;
	}
}
	
BOOL GameMap::bScrollDown(int nHowMuch)
{
	int nLimit = mnHeight - mnScreenHeight; 
	if (mnPosY == nLimit) {
		return FALSE;
	} else {
		mnPosY = min(nLimit, mnPosY + nHowMuch);
		if (pApp->pSoundManager)
		{
			pApp->pSoundManager->SetViewWindowPos(mnPosX, mnPosY);
		}
		return TRUE;
	}
}
	
BOOL GameMap::bScrollLeft(int nHowMuch)
{
	if (mnPosX == 0) {
		return FALSE;
	} else {
		mnPosX = max(0, mnPosX - nHowMuch);
		if (pApp->pSoundManager)
		{
			pApp->pSoundManager->SetViewWindowPos(mnPosX, mnPosY);
		}
		return TRUE;
	}
}

BOOL GameMap::bScrollRight(int nHowMuch)
{
	int nLimit = mnWidth - mnScreenWidth; 
	if (mnPosX == nLimit) {
		return FALSE;
	} else {
		mnPosX = min(nLimit, mnPosX + nHowMuch);
		if (pApp->pSoundManager)
		{
			pApp->pSoundManager->SetViewWindowPos(mnPosX, mnPosY);
		}
		return TRUE;
	}
}

void GameMap::vPosn(int nNewX, int nNewY)
{
	if (nNewX < 0)
		nNewX = 0;
	else if (nNewX > mnWidth - mnScreenWidth)
		nNewX = mnWidth - mnScreenWidth;

	if (nNewY < 0)
		nNewY = 0;
	else if (nNewY > mnHeight - mnScreenHeight) 
		nNewY = mnHeight - mnScreenHeight; 

	mnPosX = nNewX;
	mnPosY = nNewY;

	if (pApp->pSoundManager)
	{
		pApp->pSoundManager->SetViewWindowPos(mnPosX, mnPosY);
	}
	
	mpoPlane->vMarkUpdate();
}

void GameMap::vInitialize(void)
{
	int i, j;

	// this is temporary code, just to have a map
	//
	// we'll call the texture bitmaps 100-199
	//

	// create a random 100x100 map
	mnWidth = 100;
	mnHeight = 100;
	mnSpacingX = 17;
	mnSpacingY = 10;
	mnScreenWidth = 28;
	mnScreenHeight = 48;
	mnScreenOrgX = 0;//160;
	mnScreenOrgY = 0;

	moNodes.vSetWidth(mnWidth);

	//tw - map now has a node for every position
	for (i = 0; i < mnWidth; ++i) {
		for (j = 0; j < mnHeight; ++j) {

			MapNode *poNode = poGetNode(i, j);

			if (!poNode) {
				poNode = new MapNode(this, i, j);
				if (poNode) {
					vSetNode(i, j, poNode);
				} else {
					continue;
				}
			}

			if ((i % 4 == 0) && (j % 4 == 0)) {
				// add a ground texture
				DrawObj *poGroundObj = new DrawObj(mpoPlane, 100 + random(2));

				if (!poGroundObj) {
					continue;
				}

				poGroundObj->vSetSelectable(FALSE);
				poGroundObj->vSetLayer(DP_GROUND);

				poNode->vAddObj(poGroundObj);

				if (random(20) == 0) {
					DrawObj *poTree = new DrawObj(mpoPlane, 200 + random(2));

					poTree->vSetLayer(DP_OBSTACLES);
					poNode->vAddObj(poTree);
					poNode->meType = TT_OBSTACLE;
				}

				if (random(30) == 0) {
					DrawObj *poBuilding = new DrawObj(mpoPlane, 300 + random(2));

					poBuilding->vSetLayer(DP_OBSTACLES);
					poNode->vAddObj(poBuilding);
					poNode->meType = TT_OBSTACLE;
				}
			}
		}
	}

	// TBD -- add units
	vUpdateObstacles();
}

//If poObj is passed, update all DrawObj's obstacles EXCEPT poObj(this actually gets decided in MapNode's vUpdateObstacle)
void GameMap::vUpdateObstacles(DrawObj *poObj) {	
	int i,j;
	MapNode *poNode;

	//need to clear all nodes first
	for (i = 0; i < mnWidth; ++i) {
		for (j = 0; j < mnHeight; ++j) {
			poNode = poGetNode(i, j);
			assert(poNode);
			poNode->meType = TT_CLEAR;
			poNode->meMMType = MMTT_NONE;
			poNode->vClearCover();
		}
	}

	//go through all nodes and do updates
	for (i = 0; i < mnWidth; ++i) {
		for (j = 0; j < mnHeight; ++j) {
			poNode = poGetNode(i, j);
			poNode->vUpdateObstacles(poObj);
		}
	}

}
void GameMap::vAddSelection(DrawObj *poItem)
{
	assert(poItem);
	poItem->vSetSelected(TRUE);
	gpoInterface->vAddSelection(poItem);
	moSelection.AddToEnd(poItem);
}

void GameMap::vClearAllSelections(void)
{
	//gpoInterface->vHidePortraits();
	gpoInterface->vClearPortraitArray();	//clear out array table for new entries
	//int i;
	//int nSize = moSelection.GetSize();
	DrawObj **ppoObj = moSelection.GetBegin();
	while (ppoObj != moSelection.GetEnd()) {
		DrawObj* poItem = *ppoObj++;
		poItem->vSetSelected(FALSE);
	}
	moSelection.RemoveAll();
}

void GameMap::vRemoveSelection(DrawObj *poItem)
{
	assert(poItem);
	poItem->vSetSelected(FALSE);
	gpoInterface->vRemoveSelection(poItem);
	moSelection.Remove(poItem);
}

void GameMap::vCommitSelection(void)
{
	int nSize = moSelection.GetSize();
	assert(nSize <= MAX_SELECTION_SIZE);
	assert(game && game->poLocalPlayer());
	
	gpoInterface->vHidePortraits();
	gpoInterface->vShowPortraits(&moSelection);
	
	Command *poCommand = new Command;
	poCommand->meType = CM_SELECT;
	poCommand->mnSelectSize = nSize;

	int i;
	for (i = 0; i < nSize; ++i) {
		DrawObj *poObj = moSelection.GetItem(i);
		assert(poObj);
		poCommand->manSelection[i] = poObj->nGetID();
	}
	game->poLocalPlayer()->vPostCommand(poCommand, FALSE);
}

void GameMap::vAudioConfirmCommand(void)
{
	int nSize = moSelection.GetSize();
	int i;
	for (i = 0; i < nSize; ++i) {
		DrawObj *poObj = moSelection.GetItem(i);

		if (poObj->poOwner() == game->poLocalPlayer()) {
			poObj->vPlayRandomSound(TAG_Soks);
			break;
		}
	}
}


Boolean GameMap::HandleEvent(sciEvent & oEvent)
{
	if (!oEvent.claimed) {
		if (oEvent.type == KEY_PRESS) 
		{
			// TBD - PAB - FOR DEBUGGING PURPOSES
			if (GetMode() == UIMODE_SETATTR)
			{
				switch (oEvent.c) 
				{
				case VK_F2:
					mnSetAttributeID = (ATTR_CODE) (ATTR_NEWUPGRADE_00 + UPGRADE_Sight);
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case VK_F3:
					mnSetAttributeID = (ATTR_CODE) (ATTR_NEWUPGRADE_00 + UPGRADE_Attack);
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case VK_F4:
					mnSetAttributeID = (ATTR_CODE) (ATTR_NEWUPGRADE_00 + UPGRADE_Defend);
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case VK_F5:
					mnSetAttributeID = (ATTR_CODE) (ATTR_NEWUPGRADE_00 + UPGRADE_Stealth);
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case VK_F6:
					mnSetAttributeID = (ATTR_CODE) (ATTR_NEWUPGRADE_00 + UPGRADE_Detection);
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case VK_F7:
					mnSetAttributeID = (ATTR_CODE) (ATTR_NEWUPGRADE_00 + UPGRADE_Speed);
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case VK_F8:
					mnSetAttributeID = (ATTR_CODE) (ATTR_NEWUPGRADE_00 + UPGRADE_Spells);
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case VK_F9:
					mnSetAttributeID = (ATTR_CODE) (ATTR_NEWUPGRADE_00 + UPGRADE_Level);
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case 'O':
					mnSetAttributeID = ATTR_ANXIETY;
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case 'T':
					mnSetAttributeID = ATTR_TENACITY;
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case 'A':
					mnSetAttributeID = ATTR_AGGRESSION;
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case 'C':
					mnSetAttributeID = ATTR_CONVERSION;
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case '0':
				case '1':
				case '2':
				case '3':
				case '4':
				case '5':
				case '6':
					Unit::SetAttribute(NULL, mnSetAttributeID, oEvent.c - '0');
					oEvent.claimed = TRUE;
					return oEvent.claimed;
					break;
				case ' ':
					SetMode(UIMODE_SELECT);
					break;
				}
			}
			else if (GetMode() == UIMODE_CAST)
			{
				int nAdjust = 0;
				if (oEvent.modifiers & SHIFT_DOWN)
				{
					nAdjust = 10;
				}
				if(oEvent.c >= '0' && oEvent.c <= '9')
				{
					Unit::SetAttribute(NULL, ATTR_READYSPELL, oEvent.c - '0' + nAdjust);
				}
			}
			else if (GetMode() == UIMODE_GIVE)
			{
				switch (oEvent.c) 
				{
					case VK_F2:
						mnSetAttributeID = (ATTR_CODE) (0);
						oEvent.claimed = TRUE;
						return oEvent.claimed;
						break;
					case VK_F3:
						mnSetAttributeID = (ATTR_CODE) (1);
						oEvent.claimed = TRUE;
						return oEvent.claimed;
						break;
					case VK_F4:
						mnSetAttributeID = (ATTR_CODE) (2);
						oEvent.claimed = TRUE;
						return oEvent.claimed;
						break;				
					case ' ':
						SetMode(UIMODE_SELECT);
						break;
				}
			}
			switch (oEvent.c) 
			{
				case VK_ESCAPE:
					oEvent.claimed = TRUE;
					SetMode(UIMODE_SELECT);
					break;
				case VK_LEFT:
					oEvent.claimed = bScrollLeft(mnKeyScrollSpeed);
					break;
				case VK_RIGHT:
					oEvent.claimed = bScrollRight(mnKeyScrollSpeed);
					break;
				case VK_UP:
					oEvent.claimed = bScrollUp(mnKeyScrollSpeed);
					break;
				case VK_DOWN:
					oEvent.claimed = bScrollDown(mnKeyScrollSpeed);
					break;
				case 'E':			// ENTER COMMANDS FOR SELECTED UNITS
					{
						if (!game->bNetGame())
						{
							if (GetMode() != UIMODE_ENTERCOMMANDS)
							{
								// CLEAR SEQUENCER FOR SELECTED UNITS
								int i = moSelection.GetSize();
								while (i>0)
								{
									i--;
									Unit * poUnit = (Unit *) moSelection.GetItem(i);
									if (poUnit->bIsUnit())
									{
										poUnit->CommandSequenceClear();
									}
								}
								
								SetMode(UIMODE_ENTERCOMMANDS);
							}
							else
							{
								vClearAllSelections();
								vCommitSelection();
								SetMode(UIMODE_SELECT);
							}

							oEvent.claimed = TRUE;
						}
					}
					break;
				case 'W':			// WAIT COMMAND FOR SELECTED UNITS
					if (GetMode() == UIMODE_ENTERCOMMANDS)
					{
						Command * poCommand = new Command;
						poCommand->meType = CM_WAIT_500MS;
						game->poLocalPlayer()->vPostCommand(poCommand);
						oEvent.claimed = TRUE;
					}
					break;
//
// functionality moved to intbuttn.cpp
//
/*				case 'A':				// ATTACK TARGET
					SetMode(UIMODE_ATTACK);
					oEvent.claimed = TRUE;
					break;
				case 'G':				// GUARD TARGET
					SetMode(UIMODE_GUARD);
					oEvent.claimed = TRUE;
					break;
				case 'P':				// PATROL
					if (GetMode() == UIMODE_PATROL)
					{
						SetMode(UIMODE_SELECT);
					}
					else
					{
						SetMode(UIMODE_PATROL);
					}
					oEvent.claimed = TRUE;
					break;

				case 'F':				// FEED ON TARGET
					SetMode(UIMODE_FEED);
					oEvent.claimed = TRUE;
					break;

				case 'V':				// VAMPIRIZE
					SetMode(UIMODE_CONVERT);
					oEvent.claimed = TRUE;
					break;
				case 'C':				// CAST SPELL
					SetMode(UIMODE_CAST);
					oEvent.claimed = TRUE;
					break;
*/

//
// functionality moved to intbuttn.cpp
//
/*				case 'K':				// CLOA'K'
					if (moSelection.GetSize())
					{
						Command * poCommand = new Command;
						poCommand->meType = CM_CLOAK;
						game->poLocalPlayer()->vPostCommand(poCommand);
						oEvent.claimed = TRUE;
					}
					break;
				case 'S':				// SCOUT
*/
				case ' ':
					vClearAllSelections();
					vCommitSelection();
					SetMode(UIMODE_SELECT);
					oEvent.claimed = TRUE;
					break;
				case 'B':				// BOOK OF SPELLS
					SetMode(UIMODE_UNDEFINED);
					oEvent.claimed = TRUE;
					break;
//
// functionality moved to intbuttn.cpp
//
/*				case 'T':
					SetMode(UIMODE_TAKE);
					oEvent.claimed = TRUE;
					break;
				
				case 'I':
					mnSetAttributeID = (ATTR_CODE) (3);
					SetMode(UIMODE_GIVE);
					oEvent.claimed = TRUE;
					break;

				case 'M':
					SetMode(UIMODE_MOVE);
					oEvent.claimed = TRUE;
					break;
*/				case 'N':
					mnSetAttributeID = ATTR_TENACITY;
					SetMode(UIMODE_SETATTR);
					oEvent.claimed = TRUE;
					break;
				default:
					break;			
			}
		}
		
		else if (oEvent.type == MOUSE_DOWN) 
		{
			if (GetMode() == UIMODE_CAST)
			{
				MapNode * poNode = poNodeFromEvent(oEvent);
				if (poNode)
				{
					Command *poCommand = new Command;
					poCommand->meType = CM_CAST;
					poCommand->mbTargetUnit = FALSE;
					
					poCommand->mnX = poNode->mnXPos;
					poCommand->mnY = poNode->mnYPos;
					poCommand->mnUnit = 0;
					poCommand->mnNewValue = 0;	// SPELL NUMBER
					game->poLocalPlayer()->vPostCommand(poCommand, FALSE);
				}
				oEvent.claimed = TRUE;

				if (!(oEvent.modifiers & CTRL_DOWN))
				{
					SetMode(UIMODE_SELECT);
				}
			}
			else
			{
				SetMode(UIMODE_SELECT);
				if (oEvent.modifiers & LEFT_MOUSE)  // LEFT MOUSE
				{
					  
					// TBD - this should start selection
					mbLeftMouseIsDown = TRUE;

					mnStartSelectionX = oEvent.x;
					mnEndSelectionX = oEvent.x;
					mnStartSelectionY = oEvent.y;
					mnEndSelectionY = oEvent.y;
					mnStartScreenOrgX = nOrgPixelX();
					mnStartScreenOrgY = nOrgPixelY();
	/*
	 *	 
	 *	COMMENTED TO MIMIC WARCRAFT NO DESELECT CONTROL BEHAVIOR
	 *
	 *				vClearAllSelections();
	 *				vCommitSelection();
	 */				
					//window->Update(*prGetClip());
				}
			}
		}
		else if (oEvent.type == MOUSE_MOVE)
		{
			if (mbLeftMouseIsDown)
			{
				mbSelectingWithMouse = TRUE;
			}

			if (mbSelectingWithMouse)
			{
				mnEndSelectionX = oEvent.x;
				mnEndSelectionY = oEvent.y;
				SetCursor(mhCursor);
			}
			else
			{
				// TBD - HANDLE CURSORS BETTER.  THIS IS JUST FOR TEMPORARY PURPOSES
				if (oEvent.y < 320 && oEvent.x > 3 && oEvent.x < 567)
				{
					SetCursor(mhCursor);
				}
				else
				{
					SetCursor(LoadCursor(NULL, IDC_ARROW));
				}
			}
		}
		else if (oEvent.type == MOUSE_UP)
		{
			if (oEvent.modifiers & LEFT_MOUSE)
			{
				mbLeftMouseIsDown = FALSE;
			}
			if (mbSelectingWithMouse) 
			{
				if (oEvent.modifiers & LEFT_MOUSE) // LEFT MOUSE
				{
					mbSelectingWithMouse = FALSE;

					int nEndX = (mnEndSelectionX + nOrgPixelX()) / nGetSpacingX();
					int nEndY = (mnEndSelectionY + nOrgPixelY()) / nGetSpacingY();
					int nStartX = (mnStartSelectionX + mnStartScreenOrgX) / nGetSpacingX();
					int nStartY = (mnStartSelectionY + mnStartScreenOrgY) / nGetSpacingY();

					int nBottomX = max(0, min(nStartX, nEndX));
					int nTopX = min(mnWidth - 1, max(nStartX, nEndX));
					int nBottomY = max(0, min(nStartY, nEndY));
					int nTopY = min(mnHeight - 1, max(nStartY, nEndY));

					// SELECT EVERYTHING IN THE RECTANGLE
					BOOL bAnythingSelected = FALSE;
					int nTemporaryMaxSelection = 10;  // TBD - NO MIN?
					while(nBottomY <= nTopY)
					{
						for(int i=nBottomX;i<=nTopX;i++)
						{
							MapNode * poNode = poGetNode(i, nBottomY);
							
							if (poNode->mpoUnit)
							{
								if (poNode->mpoUnit->poOwner() == game->poLocalPlayer())
								{
									if (!bAnythingSelected)
									{
										vClearAllSelections();
										bAnythingSelected = TRUE;
									}
									nTemporaryMaxSelection--;
									if (nTemporaryMaxSelection > 0)
									{
										vAddSelection(poNode->mpoUnit);
									}
									else
									{
										break;
									}
								}
							}
						}
						nBottomY++;
					}
					if (bAnythingSelected)
					{
						vCommitSelection();
					}
				}
				SetMode(UIMODE_SELECT);				
			}
		}

		if (oEvent.claimed) {
			DebugString("Scroll key hit\n");
			mpoPlane->window->Update(*(mpoPlane->prGetClip()));
			//vDraw();
			//pApp->pWindow->ForceUpdate();
		}
	}
	return oEvent.claimed;
}

// Given 2 points nX,Ny on the GameMap, it will return where those 2 points are in in
// terms of the map position 0,0 to mnWidth,mnHeight. Returns -1 in members if out of range.
POINT GameMap::oCalcPosition (int nX, int nY) {
	POINT pt;
	pt.x = -1;
	pt.y = -1;

	if ((nX >= mnScreenOrgX) && (nY >= mnScreenOrgY)) {
//		MonoOut ("posX: %d, posY: %d", mnPosX, mnPosY);
//		MonoOut ("Nx:%d Ny:%d", ((nX / mnSpacingX) + mnPosX), ((nY / mnSpacingY) + mnPosY));
		pt.x = (nX / mnSpacingX) + mnPosX;
		pt.y = (nY / mnSpacingY) + mnPosY;
	}
	
	return pt;
}

//Given a map node(nPosX, nPosY), this will return the screen coords where this map node occurs 
//globally within the game window. 
//
//Used for mouse postioning during mapediting.
//
//If node values are out of range, defaults to position of cursor.
POINT GameMap::oCalcCoords (int nPosX, int nPosY) {
	POINT pt;
	
	GetCursorPos(&pt);
	ScreenToClient(pApp->pWindow->hWnd, &pt);

	//make sure we're in legal range
	if ((nPosX >= mnPosX) && (nPosY >= mnPosY) && 
		(nPosX <= (mnWidth - 1)) && (nPosY <= (mnHeight -1)) ) {
		pt.x = ( (nPosX - mnPosX) * mnSpacingX) + mpoPlane->GetX();
		pt.y = ( (nPosY - mnPosY) * mnSpacingY) + mpoPlane->GetY();
		//ScreenToClient(pApp->pWindow->hWnd, &pt);
		//MonoOut ("mX=%d mY=%d", pt.x, pt.y);
	}
	
	return pt;
}


void GameMap::vReset(void) {
	int i,j;
	moSelection.RemoveAll();
	mpoPlane->vClearDrawnList();
	
	if ((mnWidth) && (mnHeight)) {

		for (i = 0; i < mnWidth; ++i) {
			for (j = 0; j < mnHeight; ++j) {
				MapNode *poNode = poGetNode(i, j);
				assert(poNode);
				poNode->vReset();
			}
		}
		moNodes.DisposeAll();
		moNodes.RemoveAll();
		mnWidth = 0;
		mnHeight = 0;
	}
}

Building * GameMap::GetAnotherRandomBuilding(Building * pOldBuilding)
{
// - SCAN THE BUILDINGS
// - PICK ONE RANDOMLY
	int nRange = moBuildings.GetSize();
	int nIndex;
	int nTries;
	nTries = nRange * 2;
	while (nTries > 0)
	{
		nTries--;
		nIndex = GlobalRand(nRange);
		Building * pBuilding = moBuildings.GetItem(nIndex);
		if (pOldBuilding != pBuilding)
		{
			return pBuilding;
		}
	}
	return pOldBuilding;
}

void GameMap::AddBuilding(Building * poBuilding)
{
	assert(poBuilding);
	moBuildings.AddToEnd(poBuilding);
}

void GameMap::RemoveBuilding(Building * poBuilding)
{
	assert(poBuilding);
	moBuildings.Remove(poBuilding);
}

//
//  HRESULT GameMap::Load(const char *poMapName)
//
//		returns:	S_OK, 
//					STG_E_FILENOTFOUND, 
//					STG_E_READFAULT
//
HRESULT GameMap::Load(const char *poMapName) {
	DebugString ("Loading Map...\n");

	if (gpoInterface)
	{
		//hide portraits and such
		vClearAllSelections();
		vCommitSelection();
	}
	
	FILE *file;
	char cFileName[64];
	//sprintf(cFileName, "%s\\%s.%s", ".", _strlwr(poMapName), "map");
	sprintf(cFileName, "%s\\%s.%s", ".", poMapName, "map");

	if ( (file = fopen(cFileName, "rb")) == NULL ) {
		vDebugOut( "The map file was not opened\n" );
		return STG_E_FILENOTFOUND;
	}
   
	long lMagicNum;
	fread(&lMagicNum, sizeof(long), 1, file);
	
	//check to make sure we're reading OUR .MAP file data
	if (lMagicNum != *(long *)cMagicNumber) {
		DebugString ("WRONG '.MAP' FILE TYPE\n");
		fclose(file);
		return STG_E_READFAULT;
	}
	
	//clear out current map nodes and data
	//vReset();	
	vDeleteMap();
	
	// Map Version
	WORD wVersion;
	fread (&wVersion, sizeof(WORD), 1, file);
	mnVersion = wVersion;
	
	switch (mnVersion)
	{
	case 1:
		// Map properties
		fread (&mnWidth, sizeof(int), 1, file);
		fread (&mnHeight, sizeof(int), 1, file);
		fread (&mnSpacingX, sizeof(int), 1, file);	//TBD - get rid of this line - TW
		fread (&mnSpacingY, sizeof(int), 1, file);	//TBD - get rid of this line - TW
		fread (&mnScreenWidth, sizeof(int), 1, file);
		fread (&mnScreenHeight, sizeof(int), 1, file);
		break;
	}

	mpjVisibility = new BYTE[mnWidth * mnHeight];
	ZeroMemory(mpjVisibility, mnWidth * mnHeight);

	mnScreenWidth = mpoPlane->nWidth() / mnSpacingX;
	mnScreenHeight = mpoPlane->nHeight() / mnSpacingY;

	moNodes.vSetWidth(mnWidth);

	int i,j;
	WORD wNodeColID, wNodeRowID;

	for (i = 0; i < mnWidth; ++i) 
	{
		for (j = 0; j < mnHeight; ++j) 
		{
			MapNode *poNode = new MapNode(this, i, j);
			assert(poNode);
			vSetNode(i, j, poNode);
		}
	}

	for (i = 0; i < mnWidth; ++i) 
	{
		for (j = 0; j < mnHeight; ++j) 
		{
			MapNode *poNode = poGetNode(i, j);
			assert(poNode);
			poNode->vLoadMap(file, wVersion, mpoPlane);
		}
		//read in the checksum data 
		if ((i % MAP_CHECKSUM_FREQ) == 0) 
		{
			fread (&wNodeColID, sizeof(WORD), 1, file);	//read 'x' 1st
			fread (&wNodeRowID, sizeof(WORD), 1, file);  //then 'y'
		
			if ((i != wNodeColID) || (j != wNodeRowID)) 
			{
				// we have a major problem here
				vDebugOut ("Map data out of sync\n");
				fclose(file);
				return STG_E_READFAULT;
			}
		}

	}

	fclose(file);

	vUpdateObstacles();

	return S_OK;
}

void GameMap::vSaveMap(char *poMapName, WORD wVersion) {
	FILE *file;

	char *pcName = _strlwr( _strdup( poMapName ) );
	char cFileName[64];
	sprintf(cFileName, "%s\\%s.%s", ".", pcName, "map");
	
	long nMagicNum = *(int *)cMagicNumber;

	moSelection.RemoveAll();
	file = fopen(cFileName, "wb");
	
	// Magic Number
	fwrite (&nMagicNum, sizeof(long), 1, file);
	// Map Version
	fwrite (&wVersion, sizeof(WORD), 1, file);

	switch (wVersion)
	{
	case 1:
		// Map information
		fwrite (&mnWidth, sizeof(int), 1, file);
		fwrite (&mnHeight, sizeof(int), 1, file);
		fwrite (&mnSpacingX, sizeof(int), 1, file);
		fwrite (&mnSpacingY, sizeof(int), 1, file);
		fwrite (&mnScreenWidth, sizeof(int), 1, file);
		fwrite (&mnScreenHeight, sizeof(int), 1, file);
		break;
	}
	
	int i,j;
	WORD wNodeColID, wNodeRowID;
	for (i = 0; i < mnWidth; ++i) {
		for (j = 0; j < mnHeight; ++j) {
			MapNode *poNode = poGetNode(i, j);
			poNode->vSaveMap(file, wVersion);
			//write out the checksum data 
		}
		if ((i % MAP_CHECKSUM_FREQ) == 0) {
			wNodeColID = i;	//width
			wNodeRowID = j;	//height
			fwrite (&wNodeColID, sizeof(WORD), 1, file);	//write 'x' 1st
			fwrite (&wNodeRowID, sizeof(WORD), 1, file); //then 'y'
		}
	}

	
	
	fclose(file);

}

void GameMap::SetMode(enumGameMode eGameMode)
{ 
	switch(eGameMode)
	{
	case UIMODE_UNDEFINED:
	case UIMODE_BOOK:
	case UIMODE_ENTERCOMMANDS:
	case UIMODE_SELECT:
		mhCursor = LoadCursor(NULL, IDC_ARROW);
		break;
	case UIMODE_ATTACK:
	case UIMODE_GUARD:
	case UIMODE_FOLLOW:
	case UIMODE_CASTSPELL:
	case UIMODE_PATROL:
	case UIMODE_SCOUT:
	case UIMODE_FEED:
	case UIMODE_CONVERT:
	case UIMODE_MOVE:
		mhCursor = LoadCursor(NULL, IDC_CROSS);
		break;
	}
	SetCursor(mhCursor);
	meGameMode = eGameMode; 
}

void GameMap::vCenterNode(MapNode *poNode) {
	//setup max/min  limits
	int nLeft = 0;
	int nTop = 0;
	int nRight = mnWidth - mnScreenWidth;
	int nBot = mnHeight - mnScreenHeight;

	int nPosX = poNode->mnXPos - mnScreenWidth  / 2;
	int nPosY = poNode->mnYPos - mnScreenHeight / 2;

	mnPosX = nPosX;
	if (nPosX < nLeft) {
		mnPosX = nLeft;
	} else {
		if (nPosX > nRight) {
			mnPosX = nRight;
		}
	}

	mnPosY = nPosY;
	if (nPosY < nTop) {
		mnPosY = nTop;
	} else {
		if (nPosY > nBot) {
			mnPosY = nBot;
		}
	}
	pApp->pSoundManager->SetViewWindowPos(mnPosX, mnPosY);
	mpoPlane->vMarkUpdate();

}


//For dragging an inventory item onto the map. Returns the map node it can go into
//unless it is an obstacle, then NULL is returned
MapNode* GameMap::poDraggedItemNode(sciEvent &oEvent)
{
	MapNode *poNode = poNodeFromEvent(oEvent);
	if (poNode)
	{
		if (poNode->meType != TT_OBSTACLE)
		{
			return poNode;
		}
	}
	return NULL;
}


//disposes of every node and clears all lists
void GameMap::vDeleteMap()
{
	mnPosX = 0;
	mnPosY = 0;

	int i,j;

//	((MapPlane*)mpoPlane)->vClearDrawnList();
	mpoPlane->vClearDrawnList();

	for (i = 0; i < mnWidth; ++i)
	{
		for (j = 0; j < mnHeight; ++j) 
		{
			MapNode *poNode = poGetNode(i, j);
			if (poNode)
			{
				delete poNode;
			}
		}
	}

	moNodes.RemoveAll();
	moSelection.RemoveAll();
	moBuildings.RemoveAll();
	if (mpjVisibility) {
		delete [] mpjVisibility;
		mpjVisibility = NULL;
	}

}
	
//Makes a new blank map with a simple ground texture
void GameMap::vNewMap(int nNewWidth, int nNewHeight)
{
	//first dispose of current map
	vDeleteMap();

	////
	mnSpacingX = 17;
	mnSpacingY = 10;
	mnScreenWidth = 28;
	mnScreenHeight = 48;
	mnScreenOrgX = 0;//160;
	mnScreenOrgY = 0;
	////

	//now make a new 'blank' map based on new width and height
	
	mnWidth = nNewWidth;
	mnHeight = nNewHeight;

	mnScreenWidth = mpoPlane->nWidth() / mnSpacingX;
	mnScreenHeight = mpoPlane->nHeight() / mnSpacingY;
	
	moNodes.vSetWidth(mnWidth);
	mnVersion = MAP_VERSION;

	int i,j;
	for (i = 0; i < mnWidth; ++i) {
		for (j = 0; j < mnHeight; ++j) 
		{
			MapNode *poNode = new MapNode(this, i, j);
			vSetNode(i, j, poNode);

			if ((i % 4 == 0) && (j % 4 == 0)) {
				// add a ground texture
				DrawObj *poGroundObj = new DrawObj(mpoPlane, 10101 + random(5));

				if (!poGroundObj) {
					continue;
				}

				poGroundObj->vSetSelectable(FALSE);
				poGroundObj->vSetLayer(DP_GROUND);

				poNode->vAddObj(poGroundObj);

			}
		}
	}
	
	//mpjVisibility disposed of in vDeleteMap above
	mpjVisibility = new BYTE[mnWidth * mnHeight];
	vVisibilitySetAll(255);

	if (gpoInterface)
	{
		gpoInterface->vReAdjustScroller();
	}

}

//Returns a node closest to the points. 
//If points fall off map, the node will be closest node on the map's edge
MapNode *GameMap::poClosestNode(int nX, int nY)
{
	int nNewX, nNewY;

	if (nX < 0)
		nNewX = 0;
	else if (nX >= mnWidth)
		nNewX = mnWidth - 1;
	else
		nNewX = nX;

	if (nY < 0)
		nNewY = 0;
	else if (nY >= mnHeight)
		nNewY = mnHeight - 1;
	else
		nNewY = nY;
	vDebugOut ("x: %d y:%d\n",nNewX, nNewY);
	return (MapNode *)moNodes.GetItem(nNewX, nNewY);
}

//a child was modified therefore modify all children of other parents of same ID
void GameMap::vResetChildren(DrawObj *poParent)
{
	int i,j;
	for (i = 0; i < mnWidth; ++i) {
		for (j = 0; j < mnHeight; ++j) 
		{
			MapNode *poNode = (MapNode *)moNodes.GetItem(i,j);
			assert(poNode);
			poNode->vResetChildren(poParent);
		}
	}
}
