// GAMEMAP.H -- the map handler
//
// DJM

#ifndef _GAMEMAP_
#define _GAMEMAP_
#include "scimain.h"

#ifndef _COMMAND_
#include "command.h"
#endif 


#define MAP_VERSION		1		
////////////////////////////
// Version 1 - no changes //




///////////////////////////////////////////////////////////
//														 //
//	The visibility 'number' for deteriming what parts of //
//  the map are exposed for visibilty purposes			 //	
//														 //
///////////////////////////////////////////////////////////
#define MAP_VISIBILITY_EXPOSED_MAP	128


class DebugHandler;
class MapNode;
class MapPlane;
class DrawObj;
class Building;
class Spawn;

typedef enum
{
	UIMODE_UNDEFINED,
	UIMODE_SELECT,
	UIMODE_ATTACK,
	UIMODE_GUARD,
	UIMODE_FOLLOW,
	UIMODE_CASTSPELL,
	UIMODE_PATROL,
	UIMODE_SCOUT,
	UIMODE_BOOK,
	UIMODE_ENTERCOMMANDS,
	UIMODE_FEED,
	UIMODE_CONVERT,
	UIMODE_MOVE,
	UIMODE_SETATTR,
	UIMODE_TAKE,
	UIMODE_GIVE,
	UIMODE_DROP,
	UIMODE_CAST
} enumGameMode;

class GameMap : public Object {
	friend class DebugHandler;
	friend class MapEditPlane;
protected:

	int mnWidth;
	int mnHeight;

	int mnScreenWidth;
	int mnScreenHeight;
	
	int mnPosX;
	int mnPosY;

	int mnSpacingX; // in pixels
	int mnSpacingY;

	int mnScreenOrgX; // in pixels/
	int mnScreenOrgY;

	int mnPasses;
	int mnKeyScrollSpeed;

	List2D<MapNode> moNodes; // list of MapNodes

	BYTE	*mpjVisibility;   // the visibility map

	RefList<DrawObj> moSelection; // visible selection list

	HCURSOR mhCursor;
	
	RefList<Building> moBuildings;
	
public:
	void AddBuilding(Building * poBuilding);
	void RemoveBuilding(Building * poBuilding);
	Building * GetAnotherRandomBuilding(Building * pOldBuilding);
	ATTR_CODE mnSetAttributeID;

protected:
	BOOL mbSelectingWithMouse;
	BOOL mbLeftMouseIsDown;
	int mnStartSelectionX;
	int mnStartSelectionY;
	int mnStartScreenOrgX;
	int mnStartScreenOrgY;
	int mnEndSelectionX;
	int mnEndSelectionY;

public:
	GameMap(MapPlane *poPlane);
	BOOL IsSelecting() { return mbSelectingWithMouse; }
	void vCenterNode(MapNode *poNode);	//center poNode in the middle of the game screen

	// debugging stuff -- checksum and text dumping
	long lGetChecksum(void);
	void vDumpSyncText(FILE *fileDump);
	
	BOOL bScrollUp(int nHowMuch);
	BOOL bScrollDown(int nHowMuch);
	BOOL bScrollLeft(int nHowMuch);
	BOOL bScrollRight(int nHowMuch);

	BOOL bKeyScrollUp()		{return bScrollUp(mnKeyScrollSpeed);}
	BOOL bKeyScrollDown()	{return bScrollDown(mnKeyScrollSpeed);}
	BOOL bKeyScrollLeft()	{return bScrollLeft(mnKeyScrollSpeed);}
	BOOL bKeyScrollRight()	{return bScrollRight(mnKeyScrollSpeed);}

	void vResetChildren(DrawObj *poParent);	//a child was modified therefore modify all children of other parents of same ID

	void vPosn(int nNewX, int nNewY);

	void vDraw(Boolean bUpdate = TRUE);

	void vInitialize(void);
	void vReset(void);

	void vAddSelection(DrawObj *poItem);
	void vClearAllSelections(void);
	void vRemoveSelection(DrawObj *poItem);
	void vCommitSelection(void); // must be called to make a command happen
	void vAudioConfirmCommand(void);

	Boolean HandleEvent(sciEvent & oEvent);

	MapNode *poGetNode(int nX, int nY) {
		assert(nY >= 0);
		assert(nX >= 0);
		return (MapNode *)moNodes.GetItem(nX, nY);
	}

	// range checked version
	MapNode *poGetNodeRC(int nX, int nY) {
		if (nX < 0 || nX >= mnWidth || nY < 0 || nY >= mnHeight) {
			return NULL;
		} else {
			return (MapNode *)moNodes.GetItem(nX, nY);
		}
	}

	void vSetNode(int wX, int wY, MapNode *poNode);
	MapNode *poNodeFromEvent(sciEvent &oEvent);
	MapNode *poNodeFromMouse(void); //returns node based on mouse position
	MapNode *poClosestNode(int nX, int nY);	//returns a node closest to the points. If points fall off map, the node will be closest node on the map's edge

	void vObstructVisibility(int nX, int nY, BYTE jVisibility) 
	{
		int nWhich = nY * mnWidth + nX;
		mpjVisibility[nWhich] = jVisibility;
	}

	void vSetVisibility(int nX, int nY, BYTE jVisibility) {
		int nWhich = nY * mnWidth + nX;
		if (jVisibility > mpjVisibility[nWhich]) {
			mpjVisibility[nWhich] = jVisibility;
		}
	}
	//void vSetVisibility(MapNode *poNode, BYTE jVisibility) {
	//	vSetVisibility(poNode->mnXPos, poNode->mnYPos, jVisibility);
	//}
	BYTE jGetVisibility(int nX, int nY) {
		return mpjVisibility[nY * mnWidth + nX];
	}
	void vClearVisibility(void);
	void vApplyVisibility(void);
	void vVisibilitySetAll(BYTE jSetting);

	MapPlane *mpoPlane;	// our plane

	int nGetPosX () {return mnPosX;};
	int nGetPosY () {return mnPosY;};
	int nGetSpacingX () {return mnSpacingX;};
	int nGetSpacingY () {return mnSpacingY;};
	int nGetWidth() {return mnWidth;};
	int nGetHeight() {return mnHeight;};
	int nGetScreenHeight() { return mnScreenHeight; }
	int nGetScreenWidth() { return mnScreenWidth; }

	int nOrgPixelX() {return mnPosX * mnSpacingX;}
	int nOrgPixelY() {return mnPosY * mnSpacingY;}

	POINT oCalcPosition (int nX, int nY);
	POINT oCalcCoords (int nPosX, int nPosY);

	HRESULT Load(const char *poMapName);
	void vSaveMap(char *poMapName, WORD wVersion);
	void vUpdateObstacles(DrawObj *poObj = NULL);	//if poObj is passed update all DrawObj's obstacles EXCEPT poObj
	int nSelectionSize(void) {return moSelection.GetSize();}

	//previous position - for use when dragging objects around map
	int nPrevX;
	int nPrevY;

	//For dragging an inventory item onto the map. Returns the map node it can go into
	//unless it is an obstacle, then NULL is returned
	MapNode *poDraggedItemNode(sciEvent &oEvent);

	void vDeleteMap();	//disposes of every node and clears all lists
	void vNewMap(int nNewWidth, int nNewHeight);//makes a new blank map with a simple ground texture
	int nVersion() {return mnVersion;}
	
	Spawn			*mpoSinglePlayerStart;		//the starting point for single player games only
	List<Spawn>		moMultiPlayerStarts;		//starting points for all players
	List<Spawn>		moVampireHunterStarts;		//starting/spawing points for hunters
	List<Spawn>		moPeasantStarts;			//starting/spawing points for peasant types

public:
	void SetMode(enumGameMode eGameMode);
	enumGameMode GetMode() { return meGameMode; }
protected:
	enumGameMode meGameMode;
	int			 mnVersion;

};


extern GameMap *gpoMap;

#endif