#include "scimain.h"
#include "mapnode.h"
#include "mapplane.h"
#include "unit.h"
#include "building.h"
#include "command.h"
#include "objectdb.h"
#include "path.h"
#include "template.h"
#include "mapedit.h"
#include "guard.h"
#include "movement.h"
#include "procs.h"
#include "math.h"
#include "intplane.h"
#include "vampire.h"
#include "item.h"


void HasInventory::Drop(MapNode * poWhere, Item * poWhat)
{
	// FROM INVENTORY
	// TO MAPNODE
	assert(poWhat && poWhere);
	
	int i = moItems.Find(poWhat);
	assert(i != ID_NONE);  // SHOULD BE THERE

	moItems.RemoveIndex(i);

	poWhat->vSetLayer(DP_ITEMS);
	poWhere->vAddObj(poWhat);
	poWhat->vClearUnitOwner();

	Unit *poUnit = (Unit*)this;
	if (poUnit->bIsSelected())
		gpoInterface->vRefreshInventory();

}

void HasInventory::Take(MapNode * poWhere, Item * poWhat)
{
	// FROM MAPNODE
	// TO INVENTORY
	assert(poWhat && poWhere);

	int i = moItems.Find(poWhat);
	assert(i == ID_NONE);			// SHOULD NOT BE THERE
	assert(poWhere->bContains(poWhat));  // SHOULD BE THERE

	if (moItems.GetSize() < 3) 
	{
		poWhere->vRemoveObj(poWhat);
		moItems.AddToEnd(poWhat);
		poWhat->vSetOwner((Unit *)this);

		Unit *poUnit = (Unit*)this;
		if (poUnit->bIsSelected())
			gpoInterface->vRefreshInventory();
	}

}

void HasInventory::Give(HasInventory * poWhom, Item * poWhat)
{
	assert(poWhat && poWhom);

	// FROM INVENTORY
	// TO OTHER INVENTORY
	int i = moItems.Find(poWhat);
	assert(i != ID_NONE);
	if (poWhom->moItems.GetSize() < 3)
	{
		moItems.Remove(poWhat);
		poWhom->moItems.AddToEnd(poWhat);
		poWhat->vSetOwner((Unit *)poWhom);

		Unit *poUnit1 = (Unit*)this;
		Unit *poUnit2 = (Unit*)poWhom;
		if (poUnit1->bIsSelected() || poUnit2->bIsSelected())
			gpoInterface->vRefreshInventory();
	}

}

//Drop all inventory items (if any are present) in the given map node 'poWhere'
void HasInventory::DropAll(MapNode *poWhere)
{
	assert(poWhere);
	int nSize = moItems.GetSize();
	if (nSize)
	{
		int i;
		for (i = 0; i < nSize; ++ i)
		{
			Item *poItem = moItems.GetItem(i);
			assert(poItem);
			poWhere->vAddObj(poItem);
		}
		moItems.RemoveAll();
	}
}