// INVPLANE.CPP -- the inventory plane for single or dual portraits
//
// TW

#include "scimain.h"
#include "invplane.h"
#include "intplane.h"
#include "unit.h"
#include "inventory.h"
#include "list.h"
#include "item.h"
#include "statusln.h"
#include "itemdragplane.h"

InventoryPlane::InventoryPlane() : sciPlane(pApp->pWindow, FALSE, 1000),
	moItem1(), moItem2(), moItem3()
{
//	vSetBackgroundColor(5); //just temporary
	mpoUnit = NULL;
	mnNumItems = 0;
	
	moItem1.vSetPlane(this);
	moItem1.SetPri(mnPriority * 10);
	moItem1.Init();
	
	moItem2.vSetPlane(this);
	moItem2.SetPri(mnPriority + 10);
	moItem2.Init();

	moItem3.vSetPlane(this);
	moItem3.SetPri(mnPriority + 10);
	moItem3.Init();

}

InventoryPlane::~InventoryPlane()
{
}

void InventoryPlane::Doit() {
	sciPlane::Doit();
}

void InventoryPlane::Render (RECT &rUpdateRect) {
	sciPlane::Render(rUpdateRect);
}

Boolean InventoryPlane::HandleEvent(sciEvent & oEvent) {
	return sciPlane::HandleEvent(oEvent);
}

void InventoryPlane::Show (Unit *poUnit) {
	mpoUnit = poUnit;
	sciPlane::Show();
	vShowItems();
}

void InventoryPlane::vShowItems()
{
	assert(mpoUnit);
	RefList<Item> *poItemList = mpoUnit->poItemList();
	mnNumItems = poItemList->GetSize();

	switch (mnNumItems)
	{
	case 0:
		moItem1.Hide();
		moItem2.Hide();
		moItem3.Hide();
		break;
	case 1:
		moItem1.Show(poItemList->GetItem(0));
		moItem2.Hide();
		moItem3.Hide();
		break;
	case 2:
		moItem1.Show(poItemList->GetItem(0));
		moItem2.Show(poItemList->GetItem(1));
		moItem3.Hide();
		break;
	case 3:
		moItem1.Show(poItemList->GetItem(0));
		moItem2.Show(poItemList->GetItem(1));
		moItem3.Show(poItemList->GetItem(2));
		break;
	default:
		//Don assured me that 3 items would be the maximum limit
		assert(0);
		break;
	}
	vMarkUpdate();
	vDebugOut ("size: %d\n", poItemList->GetSize());
}

void InventoryPlane::Hide()
{
	mnNumItems = 0;
	sciPlane::Hide();
	moItem1.Hide();
	moItem2.Hide();
	moItem3.Hide();
}

void InventoryPlane::Posn(int nX, int nY, BOOL bUpdateWindow)
{
	sciPlane::Posn(nX, nY, bUpdateWindow);
	moItem1.Posn(moItem1.nLeft(), moItem1.nTop());
	moItem2.Posn(moItem2.nLeft(), moItem2.nTop());
	moItem3.Posn(moItem3.nLeft(), moItem3.nTop());
}

void InventoryPlane::vSetStatusLine(StatusLine *poStatusLine)
{
	moItem1.vSetStatusLine(poStatusLine);
//	moItem1.Init();
	moItem2.vSetStatusLine(poStatusLine);
	moItem3.vSetStatusLine(poStatusLine);
}



InventoryPlaneItem::InventoryPlaneItem() 
{
	mpoStatusText = NULL;
	mpoStatusLine = NULL;
	mpoItem = NULL;
}

InventoryPlaneItem::~InventoryPlaneItem() 
{
	mpoItem = NULL;

	if (mpoStatusText)
	{
		delete[] mpoStatusText;
		mpoStatusText = NULL;
	}
}

void InventoryPlaneItem::vSetStatusText(const char *pstrNewText)
{
	int nStrLen;

	if (mpoStatusText && pstrNewText && !strcmp(mpoStatusText, pstrNewText)) {
		return;
	}

	if (mpoStatusText) {
		delete[] mpoStatusText;
	}

	if (pstrNewText) {
		nStrLen = strlen(pstrNewText);
		mpoStatusText = new char[nStrLen + 1];
		strcpy(mpoStatusText, pstrNewText);
	}
}

void InventoryPlaneItem::Posn(int nX, int nY)
{
	sciProp::Posn(nX, nY);
	//sciProp::Posn(moRect.left, moRect.top);

/*	int nWidth, nHeight;

	nWidth = moRect.right - moRect.left;
	nHeight = moRect.bottom - moRect.top;
	
	//this positioning is a little different. Uses points in moRect as the starting point
	int nStartX = moRect.left + nX;
	int nStartY = moRect.top + nY;
	SetRect(&moRect, nStartX, nStartY, nStartX + nWidth, nStartY + nHeight);
*/
}

//Returns TRUE if points fall in moRect's area

Boolean InventoryPlaneItem::bOnMe(int nX, int nY)
{
	if ((nX >= moRect.left) && (nX <= moRect.right) &&
		(nY >= moRect.top) && (nY <= moRect.bottom)) 
	{
		return TRUE;

	}
	
	return FALSE;
}

void InventoryPlaneItem::Hide()
{
	sciProp::Hide();
	mpoItem = NULL;
	vSetStatusText("Open Slot");
}

void InventoryPlaneItem::Show(Item *poItem)
{
	SetView(poItem->GetInventoryBitmapId());
	vSetStatusText(poItem->GetItemName());
	mpoItem = poItem;
/*	if (bitmap)
	{
		SetRect(&rcWindow, left, top, left + bitmap->Width(), top + bitmap->Height());
	}
	top = rcWindow.top;
	left = rcWindow.left;
	bottom = rcWindow.bottom;
	right = rcWindow.right;
	Posn(left,top);
*/	
	sciProp::Show();

//	plane->window->Update(rcWindow);

}

void InventoryPlaneItem::Render (RECT &rUpdateRect)
{
	if (!hidden)
		sciProp::Render(rUpdateRect);
}

Boolean InventoryPlaneItem::HandleEvent(sciEvent &oEvent)
{

	switch (oEvent.type) {

	case MOUSE_MOVE:
		if (bOnMe(oEvent.x, oEvent.y)) {
			//vDebugOut("On me\n");
			//vDebugOut("x: %d, y:%d\n", plane->GetCursorX(), plane->GetCursorY());
			assert(mpoStatusLine);
			if (mpoStatusText) {
				mpoStatusLine->vAddToBar((InventoryPlaneItem*)this, mpoStatusText);			
				gpoInterface->vUpdateVampireBtnState( (Vampire*)(((InventoryPlane*)plane)->poUnit()) );
			}
			oEvent.claimed = TRUE;
		} else {
			assert(mpoStatusLine);
			if (mpoStatusLine->bOnBar((InventoryPlaneItem*)this)) {
				mpoStatusLine->vClearFromBar((InventoryPlaneItem*)this);
			}
		}
		break;
	}


	if (hidden)
	{
		return oEvent.claimed;
	} 
	else
	{

		switch (oEvent.type) {

		case MOUSE_DOWN:
			if (bOnMe(oEvent.x, oEvent.y)) 
			{
				switch (oEvent.modifiers)
				{
				case LEFT_MOUSE:
					{
						vDebugOut("Left mouse\n");
						Unit *poUnit = ((InventoryPlane*)plane)->poUnit();
						assert(poUnit);
						assert(mpoItem);
						int nIndex = poUnit->nIndex(mpoItem);
						assert (nIndex != ID_NONE);
						ItemDragPlane *poDragPlane = new ItemDragPlane(plane->window->nGetHighPriority() + 100, view, poUnit, nIndex);
						Hide();
						break;
					}
				case RIGHT_MOUSE:
					{
						vDebugOut("Right mouse\n");
						break;
					}
				}
				oEvent.claimed = TRUE;
			}
			break;
		}
		
		return oEvent.claimed;
	}
}