#include "scimain.h"
#include "drawobj.h"
#include "mapplane.h"
#include "gamemap.h"
#include "objectdb.h"
#include "procs.h"
#include "colordef.h"
#include "unit.h"
#include "player.h"
#include "debug.h"
#include "mapedit.h"
//#include "obstaclepoint.h"
#include "item.h"
#include "vampire.h"

Item::Item(MapPlane *poPlane, int nTemplateID = 0):
	DrawObj(poPlane, nTemplateID),mpoOwner(0),mpoUnit(0)
{
	meClass = OC_ITEM;
	mfPowerReservoir = 2e5;

	meType = (ITEM_TYPE)mpoTemplate->lGetValue(TAG_type, 0);
	mnChanceToStealIn10000 = (int)(10000.0 * mpoTemplate->fGetValue(TAG_Pcts, 0.0));
}

Item::~Item()
{

}

void Item::vGameCycle()
{
	switch(meType)
	{
	case ITEM_RADIANT:
		{
			Unit * mpoUnitOwner = NULL;

			if (mpoUnit)
			{
				mpoUnitOwner = mpoUnit;
			}
			else if ((poNode()->mpoUnit)&&(poNode()->mpoUnit->bIsVampire()))
			{
				mpoUnitOwner = poNode()->mpoUnit;
				vSetOwner(mpoUnitOwner->poOwner());
			}
			// ALL % VALUES ARE PER SECOND

			// CHECK FOR VAMPIRE ON TOP / OR OWNING
			if (mpoUnitOwner)
			{
				// INCREASE VAMPIRE POWER 20%*
				mpoUnitOwner->GainPowerFromOwnedStone(this);
			}
			else
			{
				if (mpoOwner)
				{
					// SCAN FOR VAMPIRES ON OWNING CLAN IN RANGE 20*
						// INCREASE VAMPIRE POWER 10%* to 0%* WITHIN RANGE*
					vScan();
				}
			}
		}
		break;
	case ITEM_NONRADIANT:
		break;
	}



}

void Item::vSetOwner(Player * poNewOwner)
{
	if (mpoOwner)
	{
		mpoOwner->ClearFlags(mpoTemplate->lGetValue(TAG_Ispf, -1));
	}
	assert(poNewOwner);
	mpoOwner = poNewOwner;
	mpoOwner->SetFlags(mpoTemplate->lGetValue(TAG_Ispf, -1));
}

void Item::vSetOwner(Unit * poNewOwner)
{
	assert(poNewOwner);
	vClearOwner();
	mpoUnit = poNewOwner;
	mpoOwner = poNewOwner->poOwner();
	mpoOwner->SetFlags(mpoTemplate->lGetValue(TAG_Ispf, -1));
	mpoUnit->SetFlag(mpoTemplate->lGetValue(TAG_Isuf, -1));
}

void Item::vClearOwner()
{
	if (mpoOwner)
	{
		mpoOwner->ClearFlags(mpoTemplate->lGetValue(TAG_Ispf, -1));
	}
	if (mpoUnit)
	{
		mpoUnit->ClearFlag(mpoTemplate->lGetValue(TAG_Isuf, -1));
	}
	mpoOwner = NULL;
	mpoUnit = NULL;
}

void Item::vClearUnitOwner()
{
	if (mpoUnit)
	{
		mpoUnit->ClearFlag(mpoTemplate->lGetValue(TAG_Isuf, -1));
	}
	mpoUnit = NULL;
}

BOOL Item::bCanSteal()
{
	return (GlobalRand(10000) < mnChanceToStealIn10000);
}

void Item::CheckNodeForAllies(MapNode * poNode, int nDistance)
{
	assert(poNode);
	if (poNode->mpoUnit)
	{
		// VAMPIRE
		if (poNode->mpoUnit->bIsVampire())			
		{
			// ON SAME TEAM AS mpoOwner
			if (((Vampire*)poNode->mpoUnit)->IsOnSameTeamAs(mpoOwner))
			{
				FLOAT fPercentPowerAmount = ((FLOAT)mpoTemplate->lGetValue(TAG_rang, 20) - (FLOAT)nDistance) / (FLOAT)mpoTemplate->lGetValue(TAG_rang, 20);
				((Vampire*)poNode->mpoUnit)->GainPowerFromStone(this, fPercentPowerAmount);
			}
		}
	}
}

void Item::vScan()
{
	assert(mpoOwner);	// SHOULD HAVE PLAYER OWNER
	assert(!mpoUnit);	// SHOULD NOT HAVE UNIT OWNER
	assert(mpoCurNode);
		
	// CREATE LIST OF OBJECTS WITHIN VISIBLE RANGE
	int nCenterX = poNode()->mnXPos;
	int nCenterY = poNode()->mnYPos;

	int nVisibleRange = mpoTemplate->lGetValue(TAG_rang, 20);

	int nLeft = max(0, nCenterX - nVisibleRange);
	int nRight = min(gpoMap->nGetWidth() - 1, nCenterX + nVisibleRange);
	int nTop = max(0, nCenterY - nVisibleRange);
	int nBottom = min(gpoMap->nGetHeight() - 1, nCenterY + nVisibleRange);

	int nPlayer;
	if (mpoOwner)
	{
		nPlayer = mpoOwner->mnPlayer;
	}

	// 
	//  TBD - BETTER SCAN ALGORITHM.  ALSO, INCLUDE VISIBILITY,
	//			LINE OF SIGHT,
	//
	int nRange = 1;
	while (nRange < nVisibleRange)
	{
		MapNode * poNode;

		int y = nTop;
		int l = max(nLeft, nCenterX - nRange);
		int r = min(nRight, nCenterX + nRange);
		while(y <= nBottom)
		{	
			poNode = gpoMap->poGetNode(l, y);
			CheckNodeForAllies(poNode, nRange);
			poNode = gpoMap->poGetNode(r, y);
			CheckNodeForAllies(poNode, nRange);
			y++;
		}
		int x = nLeft + 1;
		int b = min(nBottom, nCenterY + nRange);
		int t = max(nTop, nCenterY - nRange);
		while (x < nRight)
		{
			poNode = gpoMap->poGetNode(x, b);
			CheckNodeForAllies(poNode, nRange);
			poNode = gpoMap->poGetNode(x, t);
			CheckNodeForAllies(poNode, nRange);
			x++;
		}
		nRange++;
	}
}

Boolean Item::HandleEvent(sciEvent & oEvent)
{
	if (gbMapEditing) 
	{
		DrawObj::HandleEvent(oEvent);
		return oEvent.claimed;
	}
	switch(oEvent.type)
	{
	case MOUSE_DOWN:
		if (bOnMe(oEvent))
		{
			if (((gpoMap->GetMode() == UIMODE_TAKE)&&(oEvent.modifiers & LEFT_MOUSE))||(oEvent.modifiers & RIGHT_MOUSE))
			{
				// ISSUE THE TAKE COMMAND
				Command *poCommand = new Command;
				poCommand->meType = CM_TAKE;
				poCommand->mbTargetUnit = TRUE;
				poCommand->mnUnit = mnObjID;
				game->poLocalPlayer()->vPostCommand(poCommand, FALSE);

				gpoMap->SetMode(UIMODE_SELECT);
				oEvent.claimed = TRUE;
			}
		}
		break;
	}
	return oEvent.claimed;
}

int Item::GetInventoryBitmapId()
{
	return mpoTemplate->lGetValue(TAG_Bpor, 0);
}

const char * Item::GetItemName()
{
	return mpoTemplate->strGetString(TAG_NAME, "Unknown");
}


void Item::vInsertSelf(sciEvent &oEvent) {
	MapNode *poNode = mpoPlane->mpoMap->poNodeFromEvent(oEvent);
	if (poNode) {

		//most likely vDrag will have already inserted us into the node(and into moDrawn)
		//but if it didn't we need to catch it here.
//		vSetLayer(DP_ITEMS);
		poNode->vAddObj(this); //this lines also removes self from previous node
		//gpoMapEdit->mpoMapNode->vReset(); //clear out clipboard - no need see above line

	} else {
		// we clicked somewhere not on the game map so get rid of self
		delete this;
	}
}


// For adding to the map.
// The draw object being dragged around makes a copy of itself and inserts in into the
// node nearest the event
void Item::vCopySelf(sciEvent &oEvent) 
{
	MapNode *poNode = mpoPlane->mpoMap->poNodeFromEvent(oEvent);
	if (poNode) {
		Item *poCopy = new Item(mpoPlane, nTemplateID());	
		poCopy->vSetLayer(mnLayer);
		poNode->vAddObj(poCopy);
	}
}

FLOAT Item::DrainPower(FLOAT fPower)
{
	// RETURNS AMOUNT OF POWER DRAINED FROM STONE
	FLOAT mfPowerDifference = mfPowerReservoir;
	mfPowerReservoir = max(0, mfPowerReservoir - fPower);
	return (mfPowerDifference - mfPowerReservoir);
}

void Item::vTakeActionOnNewOwner()
{
	//if (mpoTemplate->lGetValue(TAG_spel, 0))
	{
		//mpoUnit->AddSpell(mpoTemplate->lGetValue(TAG_spel, 0));
	}
}
