#ifndef _LIST_H_
#define _LIST_H_

#include "object.h"

#define DEFAULT_DELTA 5

#define ID_NONE	-1

//typedef Object *POBJECT;
class ListBase : public Object {
public:
	ListBase();
	virtual ~ListBase();
	void AddToEnd(Object *po);
	void AddToFront(Object *po);
	void AddAfter(Object *pSeekObject, Object *po);
	void Insert(Object *po, int iIndex);
	void Remove(Object *po);
	void RemoveIndex(int iIndex);
	void RemoveAll();
	void SetItem(int iIndex, Object *po);
	int  Find(Object *poThis);
	Object *GetFirst();
	//Object *GetNext();
	Object *GetLast();
	//Object *GetPrevious();
	Object *GetItem(int iIndex) {
		assert(iIndex < iListCount);
		return ppoList[iIndex];
	}
	Object *GetItemSafe(int iIndex) {
		if (iIndex < iListCount) {
			return ppoList[iIndex];
		} else {
			return NULL;
		}
	}

	Object *PopLast();
	void DisposeAll();
	void MoveToFront(Object *po);
	void Shrink(void);
	int  GetSize(void) {return iListCount;};
	void Preallocate(int nItems); // allocate space for n items

	// iterator-type syntax
	Object **GetBegin(void) {return ppoList;}
	Object **GetEnd(void) {return ppoList + iListCount;} // one past end

protected:
	void Grow(int nExtraNeeded = 0);		// Allocate more space for Objects.
	virtual void DisposeItem(Object *po);

	int	iListCount;		// The number of items in the list
	Object **ppoList;	// Pointers to the objects in the list
	int iSpaceAllocated;	// How much space allocated for the list
	//int iNextIndex;		// Where GetNext looks
	//int iPreviousIndex;		// Where GetPrevious looks
	//Object **apReturnObject;
	//int iListDelta;
};

template <class T> 
class List : public ListBase {
public:
	void AddToEnd(T *po) {ListBase::AddToEnd((Object*)po);};
	void AddToFront(T *po) {ListBase::AddToFront((Object*)po);};
	void AddAfter(T *poSeek, T *po) {ListBase::AddAfter((Object*)poSeek, (Object*)po);};
	void Insert(T *po, int iIndex) {ListBase::Insert((Object*)po, iIndex);};
	void Remove(T *po) {ListBase::Remove((Object*)po);};
	void SetItem(int iIndex, T *po) {ListBase::SetItem(iIndex, (Object*)po);};
	int  Find(T *poThis) {return ListBase::Find((Object *)poThis);};
	T *GetFirst() {return (T*)ListBase::GetFirst();};
	//T *GetNext() {return (T*)ListBase::GetNext();};
	T *GetLast() {return (T*)ListBase::GetLast();};
	T *PopLast() {return (T*)ListBase::PopLast();};
	//T *GetPrevious() {return (T*)ListBase::GetPrevious();};
	T *GetItem(int iIndex) {return (T*)ListBase::GetItem(iIndex);};
	void MoveToFront(T *po) {ListBase::MoveToFront((Object*)po);};
	T **GetBegin(void) {return (T**)ListBase::GetBegin();}
	T **GetEnd(void) {return (T**)ListBase::GetEnd();} // one past end

	// sciList compatibility
	void AddToBack(T *po) {ListBase::AddToEnd((Object*)po);};
	void Set(int iIndex, T *po) {ListBase::SetItem(iIndex, (Object*)po);};
	void Delete(T *po) {ListBase::Remove((Object*)po);};
	int Size() {return ListBase::GetSize();}
	Boolean IsEmpty() {return ListBase::GetSize() == 0;}
	Boolean Contains(T *poThis) {return ListBase::Find((Object *)poThis) != ID_NONE;};
	T *At(int iIndex) {return (T*)ListBase::GetItemSafe(iIndex);};
	//void Release();
};

template <class T> 
class RefList : public List<T> {
public:
	~RefList() {iListCount = 0;}

protected:
	// this is the change -- doesn't dispose pointers
	void DisposeItem(Object *po) {return;};
};

template <class T> 
class List2D : public List<T> {
public:
	List2D(int nWidth = 1) {mnWidth = nWidth;};

	void vSetWidth(int nWidth) {mnWidth = nWidth;};

	T *GetItem(int nX, int nY) {
		return List<T>::GetItem(nX + nY * mnWidth);
	};
	void SetItem(int nX, int nY, T *poItem) {
		List<T>::SetItem(nX + nY * mnWidth, poItem);
	};

protected:
	int mnWidth;
};

template <class T> 
class RefList2D : public List2D<T> {
public:
	~RefList2D() {iListCount = 0;}

protected:
	// this is the change -- doesn't dispose pointers
	void DisposeItem(Object *po) {return};
};


// Here's a kludge to store items of some form of long.
// You could do LongList<long> or LongList<DWORD> or something.
//
// It would work for WORD items, but using 2X the amount of space
//
// This is assuming that (Object*) is a 32-bit value. Very bad form.
//
// What can I say? I'm rushed.
template <class T> 
class LongList : public ListBase {
public:
	void AddToEnd(T lVal) {ListBase::AddToEnd((Object*)lVal);};
	void AddToFront(T lVal) {ListBase::AddToFront((Object*)lVal);};
	void AddAfter(T lSeek, T lVal) {ListBase::AddAfter((Object*)lSeek, (Object*)lVal);};
	void Insert(T lVal, int iIndex) {ListBase::Insert((Object*)lVal, iIndex);};
	void Remove(T lVal) {ListBase::Remove((Object*)lVal);};
	void SetItem(int iIndex, T lVal) {ListBase::SetItem(iIndex, (Object*)lVal);};
	int  Find(T lVal) {return ListBase::Find((Object *)lVal);};
	T GetFirst() {return (T)ListBase::GetFirst();};
	//T GetNext() {return (T)ListBase::GetNext();};
	T GetLast() {return (T)ListBase::GetLast();};
	T PopLast() {return (T)ListBase::PopLast();};
	//T GetPrevious() {return (T)ListBase::GetPrevious();};
	T GetItem(int iIndex) {return (T)ListBase::GetItem(iIndex);};
	void MoveToFront(T lVal) {ListBase::MoveToFront((Object*)lVal);};

	~LongList() {iListCount = 0;}
protected:
	void DisposeItem(Object *po) {return;};
};

#endif
