// MAPEDIT.CPP -- the map editor
//
// TW

#include "mapedit.h"
#include "gamemap.h"
#include "mapplane.h"
#include "intplane.h"
#include "colordef.h"
#include "drawobj.h"
#include "unit.h"
#include "procs.h"
#include "building.h"
#include "viewbmp.h"
#include "debug.h"
#include "obstaclepoint.h"
#include "item.h"
#include "soundlist.h"
#include "addplane.h"

#define CROSS_HAIR 999

#define MAP_MIN_WIDTH	40	//minimum allowable map width
#define MAP_MIN_HEIGHT	40	//minimum allowable map height

MapEditPlane *gpoMapEdit = NULL;

MapEditPlane::MapEditPlane() 
: sciPlane(pApp->pWindow, FALSE, 100) {

	vSetPri(gpoInterface->nGetPri() + 5);
	//vSetRect(0,0, 206, 480);
	vSetRect(158,gpoMap->mpoPlane->nHeight(), 640, 480);
	vSetBackgroundColor(C_BLACK);
	meMode = EM_NONE;
	meLayer = DP_GROUND;
	mpoMapNode = new MapNode(gpoMap, 0, 0); //create a clipboard node
	meObstacleMode = V_EXTERIOR;
	mbListChanged = FALSE;
	mbDragUnits = FALSE;
	
	mnStartOriginX = 0;
	mnStartOriginY = 0;	
	mnCurrentX = 0;
	mnCurrentY = 0;
	mpoCrossHair = new ViewBitmap(CROSS_HAIR);
	mnLastCel = 0;
	mnLastLoop = 0;

	mpoText1 = new sciText;
	mpoText1->vSetPlane(this);
//	mpoText1->SetText("XXXXXXXXXXXXX");	//place holder
//	mpoText1->vSetSize();
	mpoText1->SetBackgroundColor(C_BLACK);
//	mpoText1->SetWidth(206);
//	mpoText1->SetHeight(300);

	mpoText1->Posn(0,0);
	mpoText1->Init();

	mpoText2 = new sciText;
	mpoText2->vSetPlane(this);
	mpoText2->SetBackgroundColor(C_BLACK);
	mpoText2->Posn(175,0);
	mpoText2->Init();



	mpoMousePos = new sciText;
	mpoMousePos->vSetPlane(this);
//	mpoMousePos->SetText("XXXXXXXXXXXXX\nXXXXXXXXXXXXX");	//place holder
//	mpoMousePos->vSetSize();
	mpoMousePos->SetBackgroundColor(C_BLACK);
	mpoMousePos->Posn(350,0);
	mpoMousePos->Init();
	mpoMousePos->SetPri(50);
	
	mpoAddPlane = new AddPlane(mnPriority + 10);

	vSetObject(NULL);
}

MapEditPlane::~MapEditPlane() {
	meMode = EM_NONE;
	mpoObject = NULL;

	if (mpoMapNode->mpoUnit) {
		delete mpoMapNode->mpoUnit;
		mpoMapNode->mpoUnit = NULL;
	}
	
	if (mpoMapNode) {
		delete mpoMapNode;
		mpoMapNode = NULL;
	}
	
	if (mpoCrossHair) {
		delete mpoCrossHair;
		mpoCrossHair = NULL;
	}
	if (mpoMousePos) {
		delete mpoMousePos;
		mpoMousePos = NULL;
	}

	//no need for delete here as the window disposes of all planes in its list
	mpoAddPlane = NULL;

	moOffsets.DisposeAll();
	moOffsets.RemoveAll();
//	moInteriorUnits.DisposeAll();
	moInteriorUnits.RemoveAll();

	moExtPointList.DisposeAll();
	moExtPointList.RemoveAll();
	moIntPointList.DisposeAll();
	moIntPointList.RemoveAll();

	pApp->pWindow->Update(mrClip);

}

void MapEditPlane::Render (RECT &updateRect) {
	sciPlane::Render(mrClip);

	
/*	if (bOnClipBoard()) {
//		POINT pt;
//		GetCursorPos(&pt);
//		ScreenToClient(pApp->pWindow->hWnd, &pt);
//		sciEvent oEvent;
//		oEvent.x = pt.x;
//		oEvent.y = pt.y;
//		oEvent.vLocalize((sciPlane*)gpoMap->mpoPlane);
		MapNode *poNode = gpoMap->poNodeFromMouse();
		if (poNode) {
			mpoMapNode->mnXPos = poNode->mnXPos;
			mpoMapNode->mnYPos = poNode->mnYPos;
			if (mpoMapNode->mpoUnit) {
				mpoMapNode->mpoUnit->vSetPixelPos(poNode->nPixelX(), poNode->nPixelY());
			}
			int k;
			for (k = DP_GROUND; k < DP_NUM_LAYERS; ++k) {
				mpoMapNode->vDraw((DRAW_PASS)k, 0,0);
			}
		}
	}
*/
	if (meMode == EM_OBSTACLE) {
		mpoObject->vDrawObstacles();
	}

	if (meMode == EM_ORIGIN) {
		assert(mpoObject);
		vSetText();	// need to refresh the coordinates
		mpoCrossHair->vDraw(mpoObject->nPixelX() - gpoMap->mpoPlane->nMapOrgPixelX(), mpoObject->nPixelY() - gpoMap->mpoPlane->nMapOrgPixelY(), (sciPlane*)gpoMap->mpoPlane);
	}

	mpoText1->vDraw();
	mpoText2->vDraw();
}

void MapEditPlane::Doit() {
	sciPlane::Doit();


	MapNode *poNode = gpoMap->poNodeFromMouse();
	if (poNode)
		//sprintf (mstr, "x: %d, y: %d\nfps %3.1f\nnX: %d, nY: %d", pApp->nMouseX, pApp->nMouseY, pApp->mfFPS, poNode->mnXPos, poNode->mnYPos);
		sprintf (mstr, "x: %d, y: %d\nnX: %d, nY: %d", pApp->nMouseX, pApp->nMouseY, poNode->mnXPos, poNode->mnYPos);
	else
		//sprintf (mstr, "x: %d y: %d\nfps %3.1f", pApp->nMouseX, pApp->nMouseY,pApp->mfFPS);
		sprintf (mstr, "x: %d y: %d", pApp->nMouseX, pApp->nMouseY);
	assert (strlen(mstr) < 256); //mstr is too small
	mpoMousePos->SetText(mstr);
	mpoMousePos->vSetSize();


//	if (mpoObject && !mpoObject->bDoneCycling())
//	{
//		mpoObject->vAnimationCycle();
//	} else {
		gpoInterface->vUpdateScroller();
//	}
	
	//if an animation is taking place we need to update the text data
	if (mpoObject && ((mpoObject->nGetCel() != mnLastCel) || (mpoObject->nGetLoop() != mnLastLoop)) )
	{
		mnLastCel = mpoObject->nGetCel();
		mnLastLoop = mpoObject->nGetLoop();
		vSetText();
	}
}

Boolean MapEditPlane::HandleEvent(sciEvent &oEvent) {

	switch (oEvent.type) {

	case KEY_PRESS:
		switch (oEvent.c) {
		//add an item
		case 'A':
			vDebugOut("adding...\n");
			oEvent.claimed = TRUE;
			meMode = EM_ADD;
			mbDragUnits = FALSE;
			int i;
			bGetNumber("Enter Template number", i, 5);
			if (bValidTemplate (i)) {
				vDebugOut("%d valid\n",i);
				//meMode = EM_NONE;
				mpoAddPlane->vAdd(i);

/*				long lClass = lCheckTemplate(i, TAG_clas, OC_DRAW);
				MapNode *poNode = gpoMap->poNodeFromEvent(oEvent);
				switch (lClass) {
				case OC_OBSTACLE: 
					{ 
						vDebugOut ("Obstacle Object\n");
						DrawObj *poNewObstObj = new DrawObj(gpoMap->mpoPlane, i);
						poNewObstObj->vSetLayer(DP_OBSTACLES);
						vSetObject(poNewObstObj);
						//poNode->vAddObj(poNewDrawObj);
						mpoMapNode->vAddObj(poNewObstObj); //put on clipboard
						pApp->pWindow->vAddEventHog(poNewObstObj);
						vMarkUpdate();
					}
					break;

				case OC_FLOOR: 
					{ 
						vDebugOut ("Floor Object\n");
						DrawObj *poNewFloorObj = new DrawObj(gpoMap->mpoPlane, i);
						poNewFloorObj->vSetLayer(DP_FLOOR);
						vSetObject(poNewFloorObj);
						//poNode->vAddObj(poNewFloorObj);
						mpoMapNode->vAddObj(poNewFloorObj); //put on clipboard
						pApp->pWindow->vAddEventHog(poNewFloorObj);
						vMarkUpdate();
					}
					break;

				case OC_UNIT:
					{
						vDebugOut ("Unit Object\n");
						// we may have a unit object that is still hanging around that we need
						// to clear from memory
						if (mpoMapNode->mpoUnit) {
							delete mpoMapNode->mpoUnit;
							mpoMapNode->mpoUnit = NULL;
						}
						long lUnit = lCheckTemplate(i, TAG_unit, UT_VAMPIRE);
						if (lUnit == UT_VAMPIRE)
						{
							Vampire *poNewVampObj = new Vampire(game->poLocalPlayer(), gpoMap->mpoPlane, mpoMapNode, i);	//put on clipboard
							if (i >=20210 && i<= 20243)
								poNewVampObj->mjPlayerLink = 1;
							else
								poNewVampObj->mjPlayerLink = 2;
							vSetObject(poNewVampObj);
							pApp->pWindow->vAddEventHog(poNewVampObj);
						}
						else
						{
							//TBD - add other UT_types
							//Unit *poNewUnitObj = new Unit(NULL, gpoMap->mpoPlane, mpoMapNode, i);	//put on clipboard
							Unit * poNewUnitObj = NULL;
							Unit::Create(&poNewUnitObj, NULL, gpoMap->mpoPlane, mpoMapNode, i);
							poNewUnitObj->mjPlayerLink = 0;
							vSetObject(poNewUnitObj);
							pApp->pWindow->vAddEventHog(poNewUnitObj);
						}

						//we're adding a unit to the map, so get the player # it will be for
						//int nPlayerLink;

						// TBD - do this nicely - have to get this working now
						//if (bGetNumber("Enter Player Number:\n\nENTER to continue\nESC to cancel", nPlayerLink, 0)) 
						//{
						//	if (!( (nPlayerLink >=  0) && (nPlayerLink <= 255) )) 
						//	{
						//		vMessage ("Player Number Too Large::set to 0");
						//		nPlayerLink = 0;
						//	}
						//}
						vMarkUpdate();
					}
					break;

				case OC_BUILDING:
					{
						vDebugOut ("Building Object\n");
						Building *poNewBuilding = new Building(gpoMap->mpoPlane, i);
						
						vClearUnitIntLists();
						//read in obstacle lists
						vClearObstLists();
						poNewBuilding->vReadInLists();

						poNewBuilding->vSetLayer(DP_OBSTACLES);
						vSetObject(poNewBuilding);
						//poNode->vAddObj(poNewBuilding);
						mpoMapNode->vAddObj(poNewBuilding); //put on clipboard
						pApp->pWindow->vAddEventHog(poNewBuilding);
						vMarkUpdate();
					}
					break;

				case OC_ITEM:
					{
						vDebugOut ("Item Object\n");

						Item * poNewItem = new Item(gpoMap->mpoPlane, i);

						poNewItem->vSetLayer(DP_ITEMS);
						vSetObject(poNewItem);
						mpoMapNode->vAddObj(poNewItem);
						pApp->pWindow->vAddEventHog(poNewItem);
						vMarkUpdate();
					}
					break;

				default:
					meMode = EM_NONE;
					break;
				}
*/			}else {
				vDebugOut("%d not valid\n",i);
				meMode = EM_NONE;
			}
			break; // 'A'

		//delete selected objects
		case 'D':
			{
				if (meMode == EM_NONE)
				{
					DrawObj *poItem = (DrawObj *)gpoMap->moSelection.GetLast();
					if (poItem) {
						game->sound2->Play(SOUND_HomerScream);
					}
					while (poItem) {
						gpoMap->moSelection.Remove(poItem);
						gpoMap->mpoPlane->vRemoveFromDrawnList(poItem);
						delete poItem;
						poItem = (DrawObj *)gpoMap->moSelection.GetLast();
					}

					mpoObject = NULL;
					gpoMap->vUpdateObstacles();
					//gopMap->vDraw();
					vSetText();
					oEvent.claimed = TRUE;
				}
			}
			break;

		//Toggling the obstacle grid on/off
		case 'O': 
			gbToggleObstacleGrid = abs(gbToggleObstacleGrid - 1);
			gpoMap->vDraw();
			vDebugOut ("%d", gbToggleObstacleGrid);
			oEvent.claimed = TRUE;
			break; // 'O'
		
		//Start new map
		case 'N':
			vStartNewMap();
			oEvent.claimed = TRUE;
			break;
			
		//Toggling the interiors on/off
		case 'V': 
			gbToggleInteriors = abs(gbToggleInteriors - 1);
			gpoMap->vDraw();
			oEvent.claimed = TRUE;				
			break; // 'V'

		case 'Z': 
			mpoAddPlane->vAdd(20210);
			//mpoAddPlane->vAdd(1101);
			oEvent.claimed = TRUE;				
			break; // 'V'

		} // oEvent.c
		break; //KEY_PRESS
	
	} //oEvent.type

	return oEvent.claimed;

}

void MapEditPlane::vSetObject(DrawObj *poObj) {
	mpoObject = poObj;
	mnLastCel = 0;
	mnLastLoop = 0;
	vSetText();
	if (mpoObject)
		mpoText2->Show();
	else
		mpoText2->Hide();
	vMarkUpdate();
}

void MapEditPlane::vSetText() {
	char str[1024];
	char str2[1024];
	int j = sprintf (str, "Map Editing\n\n");
	int k;
	
	j += sprintf (str + j, "Map Version: %d\n", gpoMap->nVersion());
	j += sprintf (str + j, "Edit Mode: %s\n\n", astrEditModes[meMode]);
	if (mpoObject && (!mpoObject->bIsUnit()) )
	{
		j += sprintf (str + j, "Layer: %d\n\n", mpoObject->nLayer());
	}

	if (mpoObject) {
		j += sprintf (str + j, "ObjectID: %d\n", mpoObject->nGetID());
		j += sprintf (str + j, "BitmapID: %d\n", mpoObject->nBitmapID());
		j += sprintf (str + j, "TemplateID: %d\n", mpoObject->nTemplateID());
		j += sprintf (str + j, "Class: %d\n", mpoObject->eClass());
		k = sprintf (str2, "Current Cel: %d\n", mpoObject->nGetCel());
		k += sprintf (str2 + k, "Total Num Cels: %d\n\n", mpoObject->nGetNumCels());
		k += sprintf (str2 + k, "Current Loop: %d\n", mpoObject->nGetLoop());
		k += sprintf (str2 + k, "Total Num Loops: %d\n", mpoObject->nGetNumLoops());
//		k += sprintf (str2 + k, "Current Loop: %d\n", mpoObject->mpoBitmap->nGetLoop());
//		k += sprintf (str2 + k, "Total Num Loops: %d\n", mpoObject->mpoBitmap->GetNumLoops());

		switch (meMode) {
		case EM_OBSTACLE:
			//if we're a building and we have an inside
			if (mpoObject->bIsBuilding() && mpoObject->nInsideBitmapID()) {
					j += sprintf (str + j, "\nObstacle Editing");
					j += sprintf (str + j, "\nExterior/Interior");
			} else {
				j += sprintf (str + j, "\nObstacle Editing");
			}
			break;
		case EM_ORIGIN:
			j += sprintf (str + j, "\nOrigin Point:");
			j += sprintf (str + j, "\n x: %d, y: %d", mnCurrentX, mnCurrentY);
			break;
		}
	}
	mpoText1->SetText(str);
	mpoText1->vSetSize();

	if (mpoObject)
	{
		mpoText2->SetText(str2);
		mpoText2->vSetSize();
	}


}
void MapEditPlane::vSetLists(Building *poBldg) {
	long plListSize;
	int nWidth = gpoMap->nGetWidth();		//map width
	int nHeight = gpoMap->nGetHeight();		//map height

	// clear out the lists
	vClearUnitIntLists();
//	moOffsets.DisposeAll();
//	moOffsets.RemoveAll();
//	moInteriorUnits.DisposeAll();
//	moInteriorUnits.RemoveAll();

	long *plIntList = poBldg->mpoTemplate->plGetList(TAG_Libs, &plListSize);
	if (plListSize) {
		// we have data in Libs tag
		if (plListSize % 2 == 0) {	//make sure we have even multiple of coords
			int nXPos = poBldg->mpoCurNode->mnXPos;
			int nYPos = poBldg->mpoCurNode->mnYPos;
			int nNewNodeX, nNewNodeY;
			int i;
			for (i=0; i < plListSize; i+=2) {
				nNewNodeX = nXPos + plIntList[i];
				nNewNodeY = nYPos + plIntList[i+1];
				if ( (nNewNodeX >= 0) && (nNewNodeX < nWidth) &&
					  (nNewNodeY >= 0) && (nNewNodeY < nHeight) ) {
					MapNode *poNode = gpoMap->poGetNode(nNewNodeX, nNewNodeY);
					assert (poNode);
					
					if (poNode->mpoUnit) {
						//add to lists
						ObstaclePoint *poPnt = new ObstaclePoint(plIntList[i], plIntList[i+1]);
						moOffsets.AddToEnd(poPnt);
						moInteriorUnits.AddToEnd(poNode->mpoUnit);
					}

//					poNode->meType = TT_CLEAR;
//					poNode->vClearCover();	//keeping this line in automatically pops the guys out. 
											//this could be an editing option
				}
			}
		}
	}

}

// Returns TRUE if building can move to new node nearest oEvent. Returns FALSE if interior units
// would result in moving onto other non-interior units or off the map or if 2 interior nodes
// will end up overlapping
Boolean MapEditPlane::bLegalMove(Building *poObj, sciEvent &oEvent) {
	MapNode *poNode = gpoMap->poNodeFromEvent(oEvent);

	int i;
	int nWidth = gpoMap->nGetWidth();
	int nHeight = gpoMap->nGetHeight();

	// dont't allow an interior from this building to be dragged onto another interior from 
	// another building
	int nSize = moIntPointList.GetSize();
	if (nSize && poNode) {
		if (poObj->mpoCurNode != poNode) {
			// the mouse is in a different node than the current object
			for (i = 0; i < nSize; ++i) {
				ObstaclePoint *poPnt= moIntPointList.GetItem(i);
//				Unit *poUnit = moInteriorUnits.GetItem(i);
				int nNewNodeX = poNode->mnXPos + poPnt->mnXPos;
				int nNewNodeY = poNode->mnYPos + poPnt->mnYPos;
				
//				vDebugOut ("%d %d %d %d\n", nNewNodeX, nNewNodeY, nWidth, nHeight);
				if ((nNewNodeX < 0) || (nNewNodeX > nWidth  - 1) ||
				    (nNewNodeY < 0) || (nNewNodeY > nHeight - 1) ) {
					return FALSE;
				}

				MapNode *poOffsetNode = gpoMap->poGetNode(nNewNodeX, nNewNodeY);
				assert(poOffsetNode);
				if (poOffsetNode->poGetCover() && poOffsetNode->poGetCover() != poObj) {
					return FALSE;
				}

			}
		}
	}
	
	
	// now check to see if units in a building would get dragged onto a unit outside
	nSize = moOffsets.GetSize();
	if (nSize && poNode) {
		if (poObj->mpoCurNode != poNode) {
			// the mouse is in a different node than the current object
			for (i = 0; i < nSize; ++i) {
				ObstaclePoint *poPnt= moOffsets.GetItem(i);
//				Unit *poUnit = moInteriorUnits.GetItem(i);
				int nNewNodeX = poNode->mnXPos + poPnt->mnXPos;
				int nNewNodeY = poNode->mnYPos + poPnt->mnYPos;
				
//				vDebugOut ("%d %d %d %d\n", nNewNodeX, nNewNodeY, nWidth, nHeight);
				if ((nNewNodeX < 0) || (nNewNodeX > nWidth  - 1) ||
				    (nNewNodeY < 0) || (nNewNodeY > nHeight - 1) ) {
					return FALSE;
				}
				
				MapNode *poOffsetNode = gpoMap->poGetNode(nNewNodeX, nNewNodeY);
				assert(poOffsetNode);
				if (poOffsetNode->mpoUnit) {
					return FALSE;
				}
			}
		}
	}
	
	return TRUE;
}

void MapEditPlane::vUpdateUnitPositions(Building *poObj) {
	int nSize = moOffsets.GetSize();
	int i;
	int nNodeX = poObj->mpoCurNode->mnXPos;
	int nNodeY = poObj->mpoCurNode->mnYPos;
	
	if (nSize) {
		for (i = 0; i < nSize; ++i) {
			ObstaclePoint *poPnt= moOffsets.GetItem(i);
			Unit *poUnit = moInteriorUnits.GetItem(i);
			MapNode *poNode = gpoMap->poGetNode(nNodeX + poPnt->mnXPos, nNodeY + poPnt->mnYPos);
			assert(poPnt);
			assert(poNode);
			assert(poUnit);

			poUnit->vMapShift(poNode);	//move unit to new node and update appropiate properties
		}
	
	}
}


// Clear obstacle toggling lists
void MapEditPlane::vClearObstLists() {
	mbListChanged = FALSE;
	moExtPointList.DisposeAll();
	moExtPointList.RemoveAll();
	moIntPointList.DisposeAll();
	moIntPointList.RemoveAll();
}

// Clear unit interior lists(when dragging) 
void MapEditPlane::vClearUnitIntLists() {
	moOffsets.DisposeAll();
	moOffsets.RemoveAll();
	//moInteriorUnits.DisposeAll();
	moInteriorUnits.RemoveAll();
}

void MapEditPlane::vToggleObstacle(sciEvent oEvent) {
	MapNode *poNode = gpoMap->poNodeFromEvent(oEvent);

	if (poNode) {
		mbListChanged = TRUE;

		//calculate the offset of the node nearest the event to the object's node
		int nXCalc = poNode->mnXPos - mpoObject->poNode()->mnXPos;
		int nYCalc = poNode->mnYPos - mpoObject->poNode()->mnYPos;

		ObstaclePoint *poPnt, *poOtherListPnt;
		//find out if point is or isn't in the list and toggle it
		switch (meObstacleMode) {
		case V_EXTERIOR:
			poPnt = poFindPoint(nXCalc, nYCalc,&moExtPointList);
			poOtherListPnt = poFindPoint(nXCalc, nYCalc,&moIntPointList); //see if point is in interior list
			if (poOtherListPnt)								//if point is in interior list
				moIntPointList.Remove(poOtherListPnt);	//then delete it since we're toggling it in the exterior list
				delete poOtherListPnt;
			break;
		case V_INTERIOR:
			poPnt = poFindPoint(nXCalc, nYCalc,&moIntPointList);
			poOtherListPnt = poFindPoint(nXCalc, nYCalc,&moExtPointList);	//see if point is in exterior list
			if (poOtherListPnt)								//if point is in exterior list
				moExtPointList.Remove(poOtherListPnt);	//then delete it since we're toggling it in the interior list
				delete poOtherListPnt;
			break;
		}

		if (poPnt) {
			//we we're in the list so remove and dispose
			switch (meObstacleMode) {
			case V_EXTERIOR:
				moExtPointList.Remove(poPnt);
				break;
			case V_INTERIOR:
				moIntPointList.Remove(poPnt);
				break;
			}
			delete poPnt;
		}
		else {
			// we weren't in the list so make a point and insert
			poPnt = new ObstaclePoint(nXCalc, nYCalc);
			switch (meObstacleMode) {
			case V_EXTERIOR:
				moExtPointList.AddToEnd(poPnt);
				break;
			case V_INTERIOR:
				moIntPointList.AddToEnd(poPnt);
				break;
			}
		}
		
		vDebugOut ("x:%d, y:%d Sz:%d\n", nXCalc, nYCalc, moExtPointList.GetSize()); 
	
	} // if(poNode)

}





ObstaclePoint *MapEditPlane::poFindPoint (int nX, int nY, List<ObstaclePoint> *poPointList) {
	int nSize, i;

	nSize = poPointList->GetSize();
	for (i = 0; i < nSize; ++i) {
		ObstaclePoint *poPoint = poPointList->GetItem(i);

		if (poPoint && (poPoint->mnXPos == nX) && (poPoint->mnYPos == nY) )  {
			return poPoint;
		}
	}

	return NULL;
}

// returns TRUE if something is on the clipboard
Boolean MapEditPlane::bOnClipBoard() {
	assert (mpoMapNode);
	if ( (mpoMapNode->mpoUnit != NULL) || (mpoMapNode->nDrawnSize() != 0) ) {
		return TRUE;
	} else {
		return FALSE;
	}
}

Boolean MapEditPlane::bOnClipBoard(Building *poBldg) {
	assert (mpoMapNode);
	return mpoMapNode->bContains((DrawObj*)poBldg);
}

void MapEditPlane::vSetOriginStart(short nX, short nY) {
	mnStartOriginX = nX;		//the original starting X origin point
	mnStartOriginY = nY;		//the original starting Y origin point

	//Set as currents as well
	mnCurrentX = nX;			//the current location for X
	mnCurrentY = nY;			//the current location for Y

}

void MapEditPlane::vSetCurrent(short nX, short nY) {
	mnCurrentX = nX;			//the current location for X
	mnCurrentY = nY;			//the current location for Y
}

void MapEditPlane::vXListScroll(int nHowMuch) {
	int nSize, i;
	mbListChanged = TRUE;
	
	nSize = moExtPointList.GetSize();
	for (i = 0; i < nSize; ++i) {
		ObstaclePoint *poPoint = moExtPointList.GetItem(i);
		if (poPoint)  {
			poPoint->mnXPos += nHowMuch;
		}
	}

	nSize = moIntPointList.GetSize();
	for (i = 0; i < nSize; ++i) {
		ObstaclePoint *poPoint = moIntPointList.GetItem(i);
		if (poPoint)  {
			poPoint->mnXPos += nHowMuch;
		}
	}

}

void MapEditPlane::vYListScroll(int nHowMuch) {
	int nSize, i;
	mbListChanged = TRUE;

	nSize = moExtPointList.GetSize();
	for (i = 0; i < nSize; ++i) {
		ObstaclePoint *poPoint = moExtPointList.GetItem(i);
		if (poPoint)  {
			poPoint->mnYPos += nHowMuch;
		}
	}

	nSize = moIntPointList.GetSize();
	for (i = 0; i < nSize; ++i) {
		ObstaclePoint *poPoint = moIntPointList.GetItem(i);
		if (poPoint)  {
			poPoint->mnYPos += nHowMuch;
		}
	}

}
void MapEditPlane::vStartNewMap()
{
	int nWidth = 0, nHeight = 0;
	bGetNumber("New Map\nEnter Width:", nWidth, 3);
	bGetNumber("New Map\nEnter Height:", nHeight, 3);
	
	if (nWidth >= MAP_MIN_WIDTH && nHeight >= MAP_MIN_HEIGHT)
	{
		vDebugOut ("New W:%d, new H:%d\n", nWidth,nHeight);
		gpoMap->vNewMap(nWidth, nHeight);
	}
	else
	{
		vDebugOut ("Map size too small\n");
	}
	
}

void MapEditPlane::vSetMode(EDIT_MODE eMode)
{	
	meMode = eMode;
	gpoMap->vUpdateObstacles();
}

//Setup object up for a possible move
void MapEditPlane::vMove(DrawObj *poObj) 
{
	mpoAddPlane->vMove(poObj);
} 
