// VWDB.CPP -- the main vampire wars database code
//
// DJM

#include "scimain.h"
#include "mapnode.h"
#include "drawobj.h"
#include "building.h"
#include "mapplane.h"
#include "gamemap.h"
#include "objectdb.h"
#include "unit.h"
#include "debug.h"
#include "colordef.h"
#include "vampire.h"
#include "item.h"
#include "procs.h"
#include "missiongoal.h"
#include "missile.h"

extern BYTE gajCommandTag[11];

/**********
 * MapNode
 **********/
MapNode::MapNode(GameMap *poMap, int nX, int nY)
{
	mpoMap = poMap;
	mnXPos = nX;
	mnYPos = nY;
	mpoUnit = NULL;
	mpoParent = NULL;
	mpoTouchedChain = NULL;
	mpoCovering = NULL;
	mnHDist = 0; // heuristic distance from goal
	mnGDist = 0; // generated distance from goal
	mnFDist = 0; // above two added -- purely for convenience
	mbOpen = FALSE;	 // is this one of the nodes we're working on?
	meMMType = MMTT_NONE;
}

MapNode::~MapNode()
{
	if (mpoUnit)
	{
		delete mpoUnit;
		mpoUnit = NULL;
	}
}

long MapNode::lGetChecksum(void)
{
	// checksum all member variables that are important to net synch

	long lCheck = 0;
	BOOL bUnit = mpoUnit != 0;
	int nSize = moDrawn.GetSize();

	// do self
	lCheck += (long)meType + mnXPos + mnYPos + nSize + (int)bUnit;
	
	// do objects
	int i;

	for (i = 0; i < nSize; ++i) {
		DrawObj *poObj = moDrawn.GetItem(i);
		if (poObj) {
			lCheck = (long)_lrotl(lCheck, 1);
			lCheck += poObj->lGetChecksum();
		}
	}

	// do unit
	if (mpoUnit) {
		lCheck += mpoUnit->lGetChecksum();
	}

	return lCheck;
}

void MapNode::vDumpSyncText(FILE *fileDump)
{
	BOOL bUnit = mpoUnit != 0;
	int nSize = moDrawn.GetSize();

	// do self
	fprintf(fileDump, "*MapNode\n");
	fprintf(fileDump, " X, Y: %d, %d\n", mnXPos, mnYPos);
	fprintf(fileDump, " Type code: %d\n", (int)meType);
	fprintf(fileDump, " Unit?: %d\n", (int)bUnit);
	fprintf(fileDump, " Number of draw objects: %d\n", nSize);
	
	// do objects
	int i;

	for (i = 0; i < nSize; ++i) {
		DrawObj *poObj = moDrawn.GetItem(i);
		if (poObj) {
			poObj->vDumpSyncText(fileDump);
		}
	}

	// do unit
	if (mpoUnit) {
		mpoUnit->vDumpSyncText(fileDump);
	}
}

BOOL MapNode::bBlocked(BOOL *pbTemp)
{
	BOOL bRet;

	if (meType == TT_OBSTACLE || mpoUnit) {
		bRet = TRUE;

		// are we only temporarily blocked?
		if (pbTemp) {
			if (meType != TT_OBSTACLE && mpoUnit && mpoUnit->bMoving()) {
				*pbTemp = TRUE;
			} else {
				*pbTemp = FALSE;
			}
		}
	} else {
		bRet = FALSE;
	}

	return bRet;
}



//
// THESE ARE THE NEW ACCESS FUNCTIONS -- USE THEM
//
void MapNode::vRemoveObj(DrawObj *poObj)
{
	moDrawn.Remove(poObj);
}

void MapNode::vAddObjToFront(DrawObj * poObj)
{
	poObj->vSetNode(this);
	//don't allow multiple copies of same object into list
	if (moDrawn.Find(poObj) == ID_NONE)
	{
		moDrawn.AddToFront(poObj);
	}
}


void MapNode::vAddObj(DrawObj *poObj)
{
	poObj->vSetNode(this);
	//don't allow multiple copies of same object into list
	if (moDrawn.Find(poObj) == ID_NONE)
	{
		moDrawn.AddToEnd(poObj);
	}
}


void MapNode::vReallyDraw(DRAW_PASS ePass, int nX, int nY)
{
	int nSize, i;

	nSize = moDrawn.GetSize();
	for (i = 0; i < nSize; ++i) {
		DrawObj *poObj = moDrawn.GetItem(i);
		if ( (ePass == DP_FLOOR) && poObj->bIsBuilding() ) {
			((Building*)poObj)->vCheckState();
		}
		else if (poObj->eClass() == OC_MISSILE && jGetVisibility() <= MAP_VISIBILITY_EXPOSED_MAP)
		{
			if (!((Missile *)poObj)->bDrawInFogOfWar())
			{
				continue;
			}
		}
		if (poObj && poObj->nLayer() == (int)ePass) {
			poObj->vDraw();
		}
	}

	// here's where we will draw the unit on this square
	if (mpoUnit && (ePass == DP_OBSTACLES) && jGetVisibility() > MAP_VISIBILITY_EXPOSED_MAP) {
		mpoUnit->vCheckVisibility();	//check and set visibility if in or out of buildings
		mpoUnit->vDraw();
	}

	if ( (ePass == DP_DEBUG) && gbToggleObstacleGrid && ((meType == TT_OBSTACLE) || (meType == TT_INTERIOR) || mpoUnit) ) {
	//if ( (ePass == DP_DEBUG) && gbToggleObstacleGrid && (meMMType == MMTT_ROADS) ) {
		vHighlight();
	}

}


//Update obstacles of all DrawObjects in moDrawn list.
//The lone exception is if poObj is passed as a non NULL parameter and it's in 
//the moDrawn list. Then and only then will the obstacles for that DrawObj NOT be updated
//
void MapNode::vUpdateObstacles(DrawObj *poObj) {
	int nSize, i;

	nSize = moDrawn.GetSize();
	for (i = 0; i < nSize; ++i) {
		DrawObj *poDrawObj = moDrawn.GetItem(i);
		if (poDrawObj != poObj)
			poDrawObj->vUpdateObstacles();
	}
}

// Reset(dispose) node for map loading purposes but doesn't dispose of self
void MapNode::vReset(void) {
	moDrawn.DisposeAll();
	delete mpoUnit;
	mpoUnit = NULL;
}

void MapNode::vSaveMap(FILE *file, WORD wVersion) {
	
	//I highly doubt a node will ever have more than 256 things in it at any time
	BYTE jSize = (BYTE)moDrawn.GetSize();	
	BYTE jAdjSize = jSize;				//adjusted size of items in moDrawn minus any 'children'
	int i;
	int nNoUnit = 0;		// used if node doesn't have a unit in it
	BYTE jNoPlayer = 0;	// used if node doesn't have a unit in it
		
	switch (wVersion) {
	case 1:
		for (i = 0; i < jSize; ++i) {
			DrawObj *poObj = moDrawn.GetItem(i);
			//if children are in moDrawn list, don't count them
			if (poObj->poParent() != NULL)
				--jAdjSize;
		}

		//write the number of non-unit objects in this node (ground,trees,items,houses,etc) minus children
		fwrite (&jAdjSize, sizeof(BYTE), 1, file);

		if (mpoUnit) {
			mpoUnit->vSaveMap(file, wVersion);
		}
		else 
		{
			//no unit in node so write out null data
			// RESOURCE ID
			fwrite (&nNoUnit, sizeof(int), 1, file);		
			// LEVEL
			fwrite (&nNoUnit, sizeof(int), 1, file);		
			// PLAYER
			fwrite (&jNoPlayer, sizeof(BYTE), 1, file);
		}


		for (i = 0; i < jSize; ++i) {
			DrawObj *poObj = moDrawn.GetItem(i);
			//Save objects that aren't children
			if (poObj->poParent() == NULL)
				poObj->vSaveMap(file, wVersion);
		}
		break;
	}
}

void MapNode::vLoadMap(FILE *file, WORD wVersion, MapPlane *poPlane) {
	BYTE jSize;
	int i, nResID, nLayer, nLevel;
	BYTE jPlayer;	
	List<Player> *poPlayerList = game->poPlayers();
	Player *poPlayer;

	switch (wVersion) {
	case 1: 
		{
			//read in the number of non-unit objects in this node (ground,trees,items,houses,etc)
			fread (&jSize, sizeof(BYTE), 1, file);

			//TBD - let the individual unit load itself
			//load the unit for the node(if there is one)
			fread (&nResID, sizeof(int), 1, file);		
			fread (&nLevel, sizeof(int), 1, file);		
			fread (&jPlayer, sizeof(BYTE), 1, file);

			long lPosition = ftell(file);
			BYTE ajCommandTag[sizeof(gajCommandTag)];
			BOOL bCommandSet = FALSE;

			int nSizeRead = fread(ajCommandTag, sizeof(gajCommandTag), 1, file);
			if (nSizeRead == 1)
			{
				if (strcmp((char *)gajCommandTag, (char *)ajCommandTag) == 0)
				{
					bCommandSet = TRUE;					
				}
			}

			if (!bCommandSet)
			{
				// restore the file pointer 
				fseek(file, lPosition, SEEK_SET);
			}

			if ( nResID == 0 && nLevel)
			{
				nLevel = nLevel & 0x0000FFFF;
				if (jPlayer <= poPlayerList->GetSize() && jPlayer > 0)
				{
					// PULL FROM LIST OF UNIT TEMPLATES FOR CLAN OF PLAYER N
					nResID = poPlayerList->GetItem(jPlayer - 1)->mpoClan->GetTemplateIdForSexAndLevel(SEX_MALE, nLevel);
				}
			}
			
			if ( nResID && jPlayer && (jPlayer <= poPlayerList->GetSize()) ) 
			{
				poPlayer = poPlayerList->GetItem(jPlayer - 1);
				assert(poPlayer);
				Unit *poUnit = new Vampire(poPlayer, poPlane, this, nResID);
				poUnit->mjPlayerLink = jPlayer; 
				assert(!bCommandSet);
			}
			else 
			{
				if ( nResID && (jPlayer == 0) )
				{
					// Unit *poUnit = new Unit(NULL, poPlane, this, nResID);
					Unit *poUnit = NULL;
					Unit::Create(&poUnit, NULL, poPlane, this, nResID);
					if (bCommandSet)
					{
						poUnit->vLoadMap(file, 1);
					}
				}
			}
			//else we just ignore the unit and never do anything with it(not enough players for unit)

			for (i = 0; i < jSize; ++i) {
				fread (&nResID, sizeof(int), 1, file);
				fread (&nLayer, sizeof(int), 1, file);

				long lClass = lCheckTemplate(nResID, TAG_clas, OC_DRAW);
				switch (lClass) {
				case OC_DRAW: 
					{ 
						//draw objects
						DrawObj *poDrawObj = new DrawObj(poPlane, nResID);
						poDrawObj->vSetSelectable(FALSE);
						poDrawObj->vSetLayer(nLayer);
						vAddObj(poDrawObj);
						break;
					}

				case OC_FLOOR: 
					{ 
						//'floor' objects
						DrawObj *poDrawObj = new DrawObj(poPlane, nResID);
//						poDrawObj->vSetSelectable(FALSE);
						poDrawObj->vSetLayer(nLayer);
						vAddObj(poDrawObj);
						break;
					}

				case OC_OBSTACLE:
					{
						//obstacles
						DrawObj *poObst = new DrawObj(poPlane, nResID);
						//poObst->vSetLayer(nLayer);
						poObst->vSetLayer(DP_OBSTACLES); //TBD - fix this
						vAddObj(poObst);
						meType = TT_OBSTACLE;
						break;
					}
				case OC_BUILDING:
					{
						//buildings
						Building *poBuilding = new Building(poPlane, nResID);
						poBuilding->vSetLayer(DP_OBSTACLES); //TBD - fix this
						//poBuilding->vSetLayer(nLayer);
						vAddObj(poBuilding);
						meType = TT_OBSTACLE;
						break;
					}
				case OC_ITEM:
					{
						//items
						Item * poItem = new Item(poPlane, nResID);
						poItem->vSetLayer(nLayer);
						vAddObj(poItem);
						break;
					}
				case OC_GOAL:
					{
						MissionGoal * poNewGoal = new MissionGoal(poPlane, nResID);
						poNewGoal->vSetLayer(DP_DEBUG);
						poNewGoal->vLoadMap(file, wVersion);
						vAddObj(poNewGoal);
					}
					break;
				}

			}
			break;
		}
	}
}


void MapNode::vHighlight() {

	int nWidth  =  mpoMap->nGetSpacingX();
	int nHeight =  mpoMap->nGetSpacingY();
	//center the rectangle 'around' the node point
	int nX = ( (nPixelX() - mpoMap->nOrgPixelX()) - (nWidth / 2) );
	int nY = ( (nPixelY() - mpoMap->nOrgPixelY()) - (nHeight /2) );
	
	RECT rLocal = {nX, nY, nX+nWidth, nY+nHeight};
	
	Bitmap *pBM = new Bitmap(nWidth, nHeight);
	int nColor = C_OBSTACLE;
	switch (meType) {
	case TT_OBSTACLE:
		nColor = C_OBSTACLE;
		break;
	case TT_INTERIOR:
		nColor = C_INTERIOR;
		break;
	}
	pBM->DrawRect(nColor, rLocal, gpoMap->mpoPlane, 1); //temp - color 159 is a yellowish color, just to stand out
	delete pBM;
}


//a child was modified(moved) therefore modify(move) all children of other parents of same ID
void MapNode::vResetChildren(DrawObj *poParent)
{
	int nSize, i;
	int nParentID = poParent->nTemplateID();

	nSize = moDrawn.GetSize();
	for (i = 0; i < nSize; ++i) {
		DrawObj *poDrawObj = moDrawn.GetItem(i);
		if (poDrawObj != poParent) //no need to bother to update the poParent since it was done way back in AddPlane
		{
			if (poDrawObj->nTemplateID() == nParentID)
			{
				poDrawObj->vSetChildNodeOff(this);
			}
		}
	}
}


vClan::vClan(Template & oClanTemplate):mpoTemplate(&oClanTemplate)
{
	mfMinDamage = (float)mpoTemplate->lGetValue(TAG_cmdm, 3);
	mfMaxDamageAttack = (float)mpoTemplate->lGetValue(TAG_cMda, 8);
	mfMaxDamageDefend = (float)mpoTemplate->lGetValue(TAG_cMdd, 3);
	mnDistanceIncludeFriendlies = mpoTemplate->lGetValue(TAG_Difu, 5);
}

Template * vClan::GetTemplate() { return mpoTemplate; }

int vClan::GetBaseSight() 
{ 
	return mpoTemplate->lGetValue(TAG_Bsig, 6);
}

int vClan::GetAdrenalineDuration()
{
	return (int)(20 * mpoTemplate->fGetValue(TAG_AdrD, 1.0));
}

int vClan::GetAdrenalineResetTime()
{
	return (int)(20 * mpoTemplate->fGetValue(TAG_AdrR, 4.0));
}

double vClan::GetAdrenalineSpeedBoost()
{
	return mpoTemplate->fGetValue(TAG_AdrB, 1.3);
}

double vClan::GetPowerCoefficient()
{
	return (double) (mpoTemplate->lGetValue(TAG_PwrC, 75) / 100.0);
} 

float vClan::GetSpellCostCoefficient1()
{
	return (float)((100 - mpoTemplate->lGetValue(TAG_SCC1, 10)) / 100.0);
}

float vClan::GetSpellCostCoefficient2()
{
	return (float)((100 - mpoTemplate->lGetValue(TAG_SCC2, 20)) / 100.0);
}

double vClan::GetHealthCoefficient()
{
	return (double) (mpoTemplate->lGetValue(TAG_HltC, 125) / 100.0);
}

int Clan::GetBaseSight() 
{ 
	return mpoTemplate->lGetValue(TAG_Bsig, 8);
}

int Clan::GetAlertCycleLimit()
{
	return mnAlertCycleLimit;
}

Clan::Clan(Template & oClanTemplate):vClan(oClanTemplate)
{
	// TIME BETWEEN ALERTS
	mnAlertCycleLimit = (int)(20 * mpoTemplate->fGetValue(TAG_Mtba, 5.0));

	mnDistanceToLookForAnother = mpoTemplate->lGetValue(TAG_Dlfa, 3);

	mnIdleTimeLimit = mpoTemplate->lGetValue(TAG_nitl, 8) * 20;

	mnNewConvertHealth = mpoTemplate->lGetValue(TAG_nhnc, 50);
	mnNewConvertPower = mpoTemplate->lGetValue(TAG_npnc, 50);
	mnRangeToDecloak = mpoTemplate->lGetValue(TAG_nrde,5);

	mnCountFeedingOnOnePeasant = mpoTemplate->lGetValue(TAG_nMfs, 1);
	mfPercentMinimumFeedingHealth = (float)mpoTemplate->lGetValue(TAG_pmfh, 10) / (float)100.0;
	mfPercentHealthConversionFloor = (float)mpoTemplate->lGetValue(TAG_pmch, 5) / (float)100.0;
	mfMaxPercentPowerToHealth = (float)mpoTemplate->lGetValue(TAG_mpph, 10) / ((float)20.0 * (float)100.0);
	mfHealthPerPower = (float)mpoTemplate->lGetValue(TAG_hppr, 2);
	mnConvertSpeed = (int)mpoTemplate->lGetValue(TAG_vspd, 3) * 20;
	mfPowerRegenPercent = (float)mpoTemplate->lGetValue(TAG_pwrp, 5) / ((float)100.0 * (float)20.0);
	mnCostToConvert = (int)mpoTemplate->lGetValue(TAG_pctc, 50);
		 
	mnBaseAttackSpeed = mpoTemplate->lGetValue(TAG_aspd, 20);

	mnTimeUpgradeLevel = mpoTemplate->lGetValue(TAG_TuLn, 25);
	mnTimeUpgradeSkillFrom[0] = mpoTemplate->lGetValue(TAG_TuS1, 8);
	mnTimeUpgradeSkillFrom[1] = mpoTemplate->lGetValue(TAG_TuS2, 15);

	mfPowerUpgradeLevel = (float)(mpoTemplate->lGetValue(TAG_PpLn, 75)) / (float)100.0;
	mfPowerUpgradeSkillFrom[0] = (float)(mpoTemplate->lGetValue(TAG_PpS1, 20)) / (float)100.0;
	mfPowerUpgradeSkillFrom[1] = (float)(mpoTemplate->lGetValue(TAG_PpS2, 30)) / (float)100.0;
	mfPowerSaveForEmergency = (float)(mpoTemplate->lGetValue(TAG_Ppse, 10)) / (float)100.0;

	mnInterfaceTemplateNumber = mpoTemplate->lGetValue(TAG_IntT, 0);

	// TOP LEVEL TO BOTTOM LEVEL
	mnCountUnitsPerLevel[0] = mpoTemplate->lGetValue(TAG_nUL5, 1);
	mnCountUnitsPerLevel[1] = mpoTemplate->lGetValue(TAG_nUL4, 3);
	mnCountUnitsPerLevel[2] = mpoTemplate->lGetValue(TAG_nUL3, 9);
	mnCountUnitsPerLevel[3] = mpoTemplate->lGetValue(TAG_nUL2, 27);
	mnCountUnitsPerLevel[4] = mpoTemplate->lGetValue(TAG_nUL1, 81);
	mnCountUnitsPerLevel[5] = mpoTemplate->lGetValue(TAG_nUL0, 100);

 	const char * pszName = mpoTemplate->strGetString(TAG_NAME, NULL);
	if (pszName)
	{
		strcpy(mstrName, pszName);
	}

	// get the palettes
	long *plPalNums;
	long nPals;
	int i;
	
	plPalNums = mpoTemplate->plGetList(TAG_Lpal, &nPals);

	assert(nPals == MAX_PLAYERS);

	for (i = 0; i < MAX_PLAYERS; ++i) {
		int nPalNum = plPalNums[i];

		Palette *poPal = (Palette*)pApp->pResourceManager->LoadResource(RES_PALETTE, nPalNum);
		if (poPal && poPal->GetData()) {
			maoPalettes[i].vConvert(poPal);
			pApp->pResourceManager->FreeResource(poPal);
		}
	}
}

int Clan::GetUnitsPerLevel(int nLevel, int nTopLevel)
{
	assert(nTopLevel>0 && nTopLevel < 8);
	assert(nLevel>0 && nLevel<=nTopLevel);

	int nIndex = nTopLevel - nLevel;
	if (nLevel < 0)
	{
		return 0;
	}
	else
	{
		assert(nIndex <= 5);
		return mnCountUnitsPerLevel[nIndex];
	}
}

FLOAT Clan::GetPercentMinimumFeedingHealth() { return mfPercentMinimumFeedingHealth; }
FLOAT Clan::GetPercentHealthConversionFloor() { return mfPercentHealthConversionFloor; }
FLOAT Clan::GetPowerRegenPercent() { return mfPowerRegenPercent; }
int Clan::GetPowerCostToConvert() { return mnCostToConvert; }
int Clan::GetNewConvertHealth() { return mnNewConvertHealth; }
int Clan::GetNewConvertPower() { return mnNewConvertPower; }

//WORD *Clan::pwGetClanPalette(int nColorIndex)
int Clan::GetTemplateIdForSexAndLevel(ENUM_SEX eSex, int nLevel)
{
	long lListSize = 0;
	
	long *plIntList;
	if (eSex == SEX_MALE)
	{
		plIntList = mpoTemplate->plGetList(TAG_Lmut, &lListSize);
	}
	else 
	{
		plIntList = mpoTemplate->plGetList(TAG_Lfut, &lListSize);
	}
	assert(lListSize);
	return plIntList[nLevel - 1];
}

int Clan::GetAnxietyBlastForFeeding()
{
	return mpoTemplate->lGetValue(TAG_ABFF, 10);
}

int Clan::GetRangeCallForHelp()
{
	return mpoTemplate->lGetValue(TAG_RCFH, 20);
}

int Clan::GetAnxietyBlastForConverting()
{
	return mpoTemplate->lGetValue(TAG_ABFC, 40);
}

int Clan::GetAnxietyBlastForAttacking()
{
	return mpoTemplate->lGetValue(TAG_ABFA, 3);
}

int Clan::GetAnxietyBlastForKilling()
{
	return mpoTemplate->lGetValue(TAG_ABFK, 40);
}

int Clan::GetAnxietyCausedByFinding()
{
	return mpoTemplate->lGetValue(TAG_ACFU, 100);
}

float Clan::GetAnxietyCausedByUncloaking()
{
	// THIS IS PER GAME CYCLE OF VIEWING UNCLOAKED VAMPIRE
	return (float)(mpoTemplate->fGetValue(TAG_ACUW, 0.5));
}

