// MAPPLANE.CPP -- the the map plane
//
// DJM

#include "scimain.h"
#include "mapnode.h"
#include "gamemap.h"
#include "mapplane.h"
#include "drawobj.h"
#include "game.h"
#include "player.h"
#include "unit.h"
#include "vampire.h"
#include "procs.h"

#define DEBUG_LOCALMESSAGES 0

MapPlane::MapPlane()
: sciPlane(pApp->pWindow, FALSE, 100)
{
	mpoMap = new GameMap(this);

	Player *poPlayer =  game->poLocalPlayer();
	assert(poPlayer);
	//load map width and height from appropiate interface template
	int nWidth = lCheckTemplate (poPlayer->mpoClan->nInterfaceTemplateNumber(), TAG_mapW, 0);
	int nHeight = lCheckTemplate (poPlayer->mpoClan->nInterfaceTemplateNumber(), TAG_mapH, 0);
	assert(nWidth);	//if we assert, we're missing the tags from the appropiate interface template
	assert(nHeight);

	//vSetRect(160, 0, 640, 480);
	//vSetRect(0,0,640,323);	//tw - newer coords to coincide with interface art
	vSetRect(0, 0, nWidth, nHeight);
	assert(mpoMap);
	
	char strFileName[255];
	wsprintf(strFileName, "%s", game->mpoTemplate->strGetString(TAG_SMAP, "TEST"));
	HRESULT hr = mpoMap->Load(strFileName);
	if (FAILED(hr))
	{
		mpoMap->vNewMap(100,100);
		char strMsg[255];
		switch(hr)
		{
		case STG_E_FILENOTFOUND:
			wsprintf(strMsg, "Map \"%s\" not found.\n\nENTER to continue", strFileName);
			vMessage(strMsg);
			break;
		case STG_E_READFAULT:
			wsprintf(strMsg, "Problem with Map \"%s\".\n\nENTER to continue", strFileName);
			vMessage(strMsg);
			break;
		}
	}

	mnMouseScrollSpeed = game->mpoTemplate->lGetValue(TAG_Mscr,2);
	//mpoMap->vDraw();

	gpoMap = mpoMap;
	if (pApp->pSoundManager)
	{
		pApp->pSoundManager->SetWorldCoordinates(gpoMap->nGetWidth() - gpoMap->nGetScreenWidth(), gpoMap->nGetHeight() - gpoMap->nGetHeight());
	}
}

MapPlane::~MapPlane()
{
	if (mpoMap)
	{
		delete mpoMap;
		mpoMap = NULL;
	}
}

void MapPlane::Render(RECT & updateRect)
{
	mpoMap->vDraw();
}

#define SCROLL_BORDER 3
#define SCROLL_BORDERLEFT 3
#define SCROLL_BORDERBOTTOMWITHDRAG 160

void MapPlane::Doit()
{
	BOOL bXScrolled = FALSE;
	BOOL bYScrolled = FALSE;
	BOOL bWithinWindow = TRUE;

	//save a little time by getting these values once
	int nMouseX = pApp->nMouseX;
	int nMouseY = pApp->nMouseY;
	int nWidth  = pApp->pWindow->Width();
	int nHeight = pApp->pWindow->Height();

	// call the super
	sciPlane::Doit();

//	vDebugOut ("x:%d y:%d\n", pApp->nMouseX, pApp->nMouseY);
	
	//see if mouse is outside of window
	if (nMouseX < 0 || nMouseY < 0 || nMouseX > nWidth || nMouseY > nHeight) {
		bWithinWindow = FALSE;
	} 
	

	if (bWithinWindow) {

		// Calculate X
		if (bWithinWindow && nMouseX <= SCROLL_BORDERLEFT && nMouseX >= 0) {
			bXScrolled = mpoMap->bScrollLeft(mnMouseScrollSpeed);
		} else if (bWithinWindow && nMouseX >= (nWidth - SCROLL_BORDER) && nMouseX <= nWidth) {
			bXScrolled = mpoMap->bScrollRight(mnMouseScrollSpeed);
		} 
		
		// Calculate Y
		if (bWithinWindow && nMouseY <= SCROLL_BORDER && nMouseY >= 0) 
		{
			bYScrolled = mpoMap->bScrollUp(mnMouseScrollSpeed);
		} 
		else if (mpoMap->IsSelecting())
		{
			if (bWithinWindow && nMouseY >= (nHeight - SCROLL_BORDERBOTTOMWITHDRAG) && nMouseY <= nHeight) 
			{
				bYScrolled = mpoMap->bScrollDown(mnMouseScrollSpeed);
			}
		}
		else if (bWithinWindow && nMouseY >= (nHeight - SCROLL_BORDER) && nMouseY <= nHeight) 
		{
			bYScrolled = mpoMap->bScrollDown(mnMouseScrollSpeed);
		} 


		if (bXScrolled || bYScrolled) {
#if DEBUG_LOCALMESSAGES
			DebugString("Mouse scroll\n");
#endif
			window->Update(mrClip);
		}
	
	}	// bWithinWindow
	
}


Boolean MapPlane::HandleEvent(sciEvent & oEvent)
{
	//call the super first
	sciPlane::HandleEvent(oEvent);
	if (oEvent.claimed)
		return oEvent.claimed;


	// hand off to the drawn objects list

	// iterate in reverse
	DrawObj **ppoObj = moDrawnList.GetEnd();
	DrawObj *poObj;
	while (ppoObj != moDrawnList.GetBegin()) {
		poObj = *(--ppoObj);

		if (poObj->HandleEvent(oEvent)) {
			break;
		}
		//poObj = moDrawnList.GetPrevious();
	}

	if (!oEvent.claimed)
	{
		switch(oEvent.type)
		{
			case MOUSE_DOWN:
			{
				if (oEvent.modifiers & RIGHT_MOUSE)  // RIGHT MOUSE 
				{
					MapNode *poNode = mpoMap->poNodeFromEvent(oEvent);
					if (poNode)
					{
						vSetMovement(poNode);
					}
					oEvent.claimed = TRUE;
				}
				else
				{
					if (mpoMap->GetMode() == UIMODE_MOVE || mpoMap->GetMode() == UIMODE_SELECT)
					{
						if ( mpoMap->nSelectionSize() > 0 )
						{
							MapNode *poNode = mpoMap->poNodeFromEvent(oEvent);
							if (poNode)
							{
								vSetMovement(poNode);
							}
							mpoMap->SetMode(UIMODE_SELECT);
							oEvent.claimed = TRUE;
						}
					}
				}
// obsolete
				/*
					if (mpoMap->GetMode() == UIMODE_MOVE)
					{
						if ( (mpoMap->nSelectionSize() > 0) )
						{
							MapNode *poNode = mpoMap->poNodeFromEvent(oEvent);
							if (poNode) 
							{
								Command *poCommand = new Command;
								poCommand->mnX = poNode->mnXPos;
								poCommand->mnY = poNode->mnYPos;

								if (mpoMap->GetMode() == UIMODE_PATROL)
								{
									poCommand->meType = CM_PATROL_WAYPOINT;
								}
								else
								{
									poCommand->meType = CM_MOVE;
								}
								game->poLocalPlayer()->vPostCommand(poCommand);
								//gpoMap->vAudioConfirmCommand();
							}
						}
						oEvent.claimed = TRUE;
						mpoMap->SetMode(UIMODE_SELECT);
					}
					else if (mpoMap->GetMode() == UIMODE_GIVE)
					{
						if ( (mpoMap->nSelectionSize() > 0) )
						{
							MapNode *poNode = mpoMap->poNodeFromEvent(oEvent);
							if (poNode) 
							{
								Command *poCommand = new Command;
								poCommand->mnX = poNode->mnXPos;
								poCommand->mnY = poNode->mnYPos;
								poCommand->mnNewValue = (int)gpoMap->mnSetAttributeID;
								poCommand->meType = CM_DROP;

								game->poLocalPlayer()->vPostCommand(poCommand);
							}
						}
						oEvent.claimed = TRUE;
						mpoMap->SetMode(UIMODE_SELECT);
					}
				}
*/			}
			case MOUSE_MOVE:
			{

			}
		}
	}
			
	// just for now, hand it off to the map
	// TBD make the map a key handler and kill this? DJM
	if (!oEvent.claimed) 
	{
		mpoMap->HandleEvent(oEvent);
	}
	return oEvent.claimed;
}

void MapPlane::vSetMovement (MapNode *poNode)
{
	// TBD - do not route these to npc's
	if ( (mpoMap->nSelectionSize() > 0) )
	{
		if (poNode) 
		{
			Command *poCommand = new Command;
			poCommand->mnX = poNode->mnXPos;
			poCommand->mnY = poNode->mnYPos;

			if (mpoMap->GetMode() == UIMODE_PATROL)
			{
				poCommand->meType = CM_PATROL_WAYPOINT;
				game->poLocalPlayer()->vPostCommand(poCommand, FALSE);
			}
			else
			{
				poCommand->meType = CM_MOVE;
				game->poLocalPlayer()->vPostCommand(poCommand);
			}
			//gpoMap->vAudioConfirmCommand();
		}
	} 
}


//The check for dragging an item from inventory onto a unit (or its status indicator)
//Returns NULL if nX,nY not on any drawn Unit else
//returns a pointer to the unit that nX,nY is on. 
Unit* MapPlane::poOnUnit(sciEvent &oEvent)
{

	DrawObj **ppoObj = moDrawnList.GetEnd();
	DrawObj *poObj;
	while (ppoObj != moDrawnList.GetBegin()) 
	{
		poObj = *(--ppoObj);
		if ( poObj->bIsUnit() && poObj->bIsSelectable() )
		{
			if  ( poObj->bOnMe(oEvent) )
			{
				vDebugOut("on unit\n");
				return (Unit*)poObj;
			}
			else 
			{ 
				Unit *poUnit = (Unit*)poObj;
				if (poUnit->bIsVampire())
				{
					if ( ((Vampire*)poUnit)->moStatus.OnMe(oEvent) )
					{
						vDebugOut("on unit's status indicator\n");
						return poUnit;
					}
				}
			}
		}
		//poObj = moDrawnList.GetPrevious();
	}

	return NULL;

}
