#ifndef _MISSILE_H_
#define _MISSILE_H_

typedef enum TAG_MISSILE_TYPE
{
	MISSILE_DEATHFROMABOVE,
	MISSILE_SMOKE,
	MISSILE_DRIFTINGSMOKE,
	MISSILE_LINGERINGSMOKE,
	MISSILE_TRACER,
	MISSILE_GROUNDHOMING,
	MISSILE_GROUNDSPIRAL,
	MISSILE_HIGHARC,
	MISSILE_FOG,
	MISSILE_TIMER,
	MISSILE_DROPBOMB,
	MISSILE_INSTANT,
	MISSILE_TIMERFADEDRAWOBJ,
	MISSILE_TRACERANIMATED,
	MISSILE_ANIMATEANDFADEDRAWOBJ,
	MISSILE_ANIMATESTOPANDFADEAWAY,
} MISSILE_TYPE;

typedef enum TAG_SPELL_TYPE
{
	SPELL_CALL = 1,
	SPELL_DEATHFROMABOVE,
	SPELL_FORCELEECH,
	SPELL_LIGHTNINGBOLT,
	SPELL_DOOMSPIRAL,
	SPELL_DEATHRAIN,
	SPELL_INVISIBILITY,
	SPELL_CALLFOG,
	SPELL_SWAP,
	SPELL_PHANTOMS,
	SPELL_EYEOFTHEDRAGON,
	SPELL_MINDCLOUD,
	SPELL_TALECSNARE,
	SPELL_BRAINWORM,
	SPELL_VEILOFMIST,
	SPELL_NECROMANCE,
	SPELL_GHOSTTOUCH,
	SPELL_CALLTHESUN,
	SPELL_PROTECTPEASANT,
	SPELL_DISPERSEMOB,
	
} SPELL_TYPE;

typedef enum
{
	SMOKETRAIL_NONE,
	SMOKETRAIL_FIREBALL,
	SMOKETRAIL_RESIDUAL,
	SMOKETRAIL_FADECYCLE,
} TYPE_SMOKETRAIL;

typedef enum
{
	DAMAGE_NONE,
	DAMAGE_AREASQUARE,
} TYPE_DAMAGE;

typedef enum
{
	IMPACT_NONE,
	IMPACT_DEATHFROMABOVE,
	IMPACT_DRAINPOWERANDRETURN,
	IMPACT_INCREASEPOWER,
	IMPACT_UNITREAPPEARS,
	IMPACT_SPYOFF,
	IMPACT_MELT,
	IMPACT_CLEARINVISIBLE,
	IMPACT_DEATHRAIN,
	IMPACT_BOMBLET,
} TYPE_IMPACT;

typedef enum 
{
	DRIFT_NONE,
	DRIFT_STRAIGHTUP,
	DRIFT_WIND,
} TYPE_DRIFT;

class Missile : public DrawObj
{
public:
	Missile(MISSILE_TYPE eType, MapPlane *poPlane, Unit * poFromUnit,  
				 MapNode * poSourceNode, MapNode * poTargetNode, int nTemplateID = 0, int nWait = 0, BOOL bSolid = FALSE, float fPercent = (float)0.0,Unit * poTargetUnit = NULL, double X = 10.0, double Y = 10.0, int nFallOff = 2, int nDirection = 1);

	~Missile();

// OVERRIDES
	void Destroy();
	void SetDamage(TYPE_DAMAGE eDamage);
	void vGameCycle();				
	Boolean HandleEvent(sciEvent & oEvent);
	void vCopySelf(sciEvent &oEvent);
	void vInsertSelf(sciEvent &oEvent);
	void vDraw();
	void GetPlayPosition(unsigned int * unX, unsigned int *unY);

	void SubCycle(int nSubCycle);
	void GetPixelPosition(int * pnX, int * pnY);
	void UpdateNode();

	void SetAlpha(BYTE jAlpha);
	void vSetPixelPos(int nX, int nY);
	void SetCycleCountdown(BOOL bCycleCountdown);
	void SetCyclesRemaining(UINT nCyclesRemaining);
	void SetFalloffCycle(int nFalloffCycle);
	void SetImpact(TYPE_IMPACT eImpact);
	void EmbedPower(float fPower);
	void EmbedUnit(Unit * poUnit);
	void SetUnitWasSelected(BOOL bValue);
	void SetSpeed(int nSpeed) {mnSpeed = nSpeed;}
	int GetSpeed() {return mnSpeed;}
	void SetFlashEmbeddedUnitVisibility(BOOL bValue);
	void SetDirection(int nDegrees);
	void SetRotation(int nDegrees);
	void SetVelocity(double dVelocity);
	void SetHeight(double dZ);
	void SetObstructView(BOOL bValue);
	void SetSpell(Spell * poSpell);
	void SetDrift(TYPE_DRIFT eDrift);
	void SetDrawObj(int nUnit);
	BOOL bDrawInFogOfWar() { return mbDrawInFogOfWar; }
protected:
	BOOL mbDrawInFogOfWar;
	Unit * mpoFoggedUnit;
	void PlaySoundCreate();
	void PlaySoundDestroy();
	Spell * mpoSpell;
	BOOL mbObstructView;
	int mnFalloffCycle;
	int mnDirectionDegrees;
	int mnRotationDegrees;
	double mdVelocity;

	float mfPower;
	MISSILE_TYPE meType;
	BOOL mbFlashEmbeddedUnitVisibility;
	Bitmap * mpoShadowBitmap;
	Unit * mpoSourceUnit;
	Unit * mpoTargetUnit;
	Unit * mpoUnit;
	BOOL mbUnitWasSelected;

	MapNode		*mpoLastNode;	// the one we left
	RefList<MapNode>	moCurrentRoute; // reverse order (end-to-begin)

	MapNode * GetNodeForCurrentPosition();
	MapNode * poGetNextMoveNode();
	MapNode * poPopOneNode();
	void vCalcRoute();
	void vMoveOneCycle();
	void vMoveToNextNode(void);
	void vMoveToNode(MapNode *poNode);
	MapNode * poGetTargetNode(void);

	int mnPartialMove;
	int mnLastTargetX, mnLastTargetY, mnStoppedCycles,mnTempBlockedCycles;
	int mnSpeed;
	BOOL mbMoving;
	BOOL mbBetweenPoints;

	double mdX;
	double mdY;
	double mdZ;
	double mddX;
	double mddY;
	double mddZ;
	double mdddZ;
	double mdTargetX;
	double mdTargetY;

	//int mnCel;
	int mnCelCount;
	int mnFallOff;
	int mnDirection;
	int mnShadeCount;
	int mnShade;
	int mnState;
	int mnWait;
	UINT mnCyclesRemaining;

	BOOL mbSolid;
	BOOL mbCastShadow;
	BOOL mbCheck;
	BOOL mbCycleCountdown;
	BOOL mbImpactObject;
	BOOL mbSoundEmitter;

	TYPE_SMOKETRAIL meSmokeTrail;
	TYPE_DAMAGE meDamageType;
	TYPE_IMPACT meImpact;

	float PowerFromUnit(float fMaxPower, Unit * poFromUnit);

	void Damage();
	void Damage_AreaSquare();
	void Impact();
	void Impact_Bomblet();
	void Impact_ClearInvisible();
	void Impact_DeathFromAbove();
	void Impact_DeathRain();
	void Impact_DrainPowerAndReturn();
	void Impact_IncreasePower();
	void Impact_Melt();
	void Impact_SpyOff();
	void Impact_UnitReappears();
};

#define PI (3.14159265359)

#endif