#include "scimain.h"
#include "drawobj.h"
#include "mapplane.h"
#include "gamemap.h"
#include "objectdb.h"
#include "procs.h"
#include "colordef.h"
#include "unit.h"
#include "player.h"
#include "debug.h"
#include "mapedit.h"
#include "obstaclepoint.h"
#include "cycler.h"
#include "missiongoal.h"

MissionGoal::MissionGoal(MapPlane * poPlane, int nTemplateID):DrawObj(poPlane, nTemplateID)
{
	strText = NULL;
	LPTSTR strDefault = "BLAH";
	TextObject = new sciText;
	SetText(mpoTemplate->strGetString(TAG_NAME, strDefault));

	meType = (TYPE_MISSIONGOAL)mpoTemplate->lGetValue(TAG_type, 0);
	mbSecret = mpoTemplate->lGetValue(TAG_scrt, 0);
	TextObject->vSetPlane(poPlane);
	TextObject->SetWidth(200);
	TextObject->SetHeight(100);
	TextObject->Init();
	TextObject->Posn(0, 0);
	mbRequired = TRUE;
	mbCompleted = FALSE;

	mbTargetObject = FALSE;
	mnTargetObjectID = -1;
	mnTargetX = -1;
	mnTargetY = -1;

	game->AddGoal(this);
}

MissionGoal::~MissionGoal()
{
	mpoCurNode->vRemoveObj(this);
	mpoCurNode = NULL;
	mpoPlane->vRemoveFromDrawnList(this);

	if (strText) 
	{
		free(strText);
		strText = NULL;
	}
}

void MissionGoal::vSetNode(MapNode *poNode)
{
	DrawObj::vSetNode(poNode);

	switch (meType)
	{
	case MISSIONGOAL_DESTROYOBJECT:
		if (mpoCurNode->mpoUnit)
		{
			mbTargetObject = TRUE;
			mnTargetObjectID = mpoCurNode->mpoUnit->nGetID();
		}
		break;
	case MISSIONGOAL_MOVETOAREA:
		mnTargetX = mpoCurNode->mnXPos;
		mnTargetY = mpoCurNode->mnYPos;
		break;
	case MISSIONGOAL_NUNITSOFMLEVEL:
		break;
	case MISSIONGOAL_FINDITEM:
		break;
	}

}

void MissionGoal::vGameCycle()
{
	if (!mbCompleted)
	{
		switch (meType)
		{
		case MISSIONGOAL_DESTROYOBJECT:
			if (mbTargetObject)
			{
				DrawObj * poObj = goObjectDB.poGetObject(mnTargetObjectID);

				if (poObj)
				{
					if (poObj->GetHealth() <= 0)
					{
						mbCompleted = TRUE;
					}
				}
			}
			else
			{
				mbCompleted = TRUE;
			}
			break;
		case MISSIONGOAL_MOVETOAREA:
			MapNode * poNode;
			poNode = gpoMap->poGetNodeRC(mnTargetX, mnTargetY);
			if (poNode)
			{
				if (poNode->mpoUnit)
				{
					// TBD - JUST CHECKING TO SEE IF ANY PLAYER 
					if (poNode->mpoUnit->poOwner())
					{
						mbCompleted = TRUE;
					}
				}
			}
			else
			{
				mbCompleted = TRUE;
			}
			break;
		case MISSIONGOAL_NUNITSOFMLEVEL:
			break;
		case MISSIONGOAL_FINDITEM:
			break;
		}
	}
}

void MissionGoal::SetRequired(BOOL bVal)
{
	mbRequired = bVal != FALSE;
}

BOOL MissionGoal::GetRequired()
{
	return mbRequired;
}

void MissionGoal::SetText(LPCTSTR strNew)
{
	int nLength = lstrlen(strNew) + 1;
	if (nLength > 1)
	{
		strText = (LPTSTR)realloc(strText, nLength * sizeof(TCHAR));
		lstrcpy(strText, strNew);
		TextObject->SetText(strText);
	}
	else
	{
		if (strText)
		{
			free(strText);
			strText = NULL;
		}
	}
}

LPTSTR MissionGoal::GetText()
{
	return strText;
}
	
void MissionGoal::vDraw()
{
	if (gbMapEditing)
	{
		DrawObj::vDraw();
		TextObject->Posn(nPixelX() - mpoPlane->nMapOrgPixelX() + 20, 
						   nPixelY() - mpoPlane->nMapOrgPixelY() - 15 );
		TextObject->vDraw();
		if (mbTargetObject)
		{
			DrawObj * poObj = goObjectDB.poGetObject(mnTargetObjectID);

			if (poObj)
			{
				poObj->vHighlight();
				poObj->vDraw();
			}
		}

	}
}

void MissionGoal::vSaveMap(FILE *file, WORD wVersion) 
{
	int nResID = mpoTemplate->resourceID;
	int nLayer = mnLayer;

	switch (wVersion) {
	case 1:
		fwrite(&nResID, sizeof(int), 1, file);	//write the resourceID of the template
		fwrite(&nLayer, sizeof(int), 1, file); //write the draw layer the object is in
		break;
	}
	int nLen = lstrlen(strText) + 1;
	fwrite(&nLen, sizeof(int), 1, file);
	fwrite(strText, nLen, 1, file);
	fwrite(&mbRequired, sizeof(BOOL), 1, file);
	fwrite(&mbSecret, sizeof(BOOL), 1, file);
}

void MissionGoal::vLoadMap(FILE *file, WORD wVersion) 
{
	// LOAD SETTINGS IN ADDITION TO NORMAL DRAWOBJ
	int nLen = 0;
	fread (&nLen, sizeof(int), 1, file);
	TCHAR tstr[255];
	fread (tstr, nLen, 1, file);
	SetText(tstr);
	fread(&mbRequired, sizeof(BOOL), 1, file);
	fread(&mbSecret, sizeof(BOOL), 1, file);
}

void MissionGoal::vMapEditHandleEvent(sciEvent &oEvent) 
{
	DrawObj::vMapEditHandleEvent(oEvent);

	if (oEvent.claimed == TRUE) {
		return;
	}

	switch (gpoMapEdit->eMode()) 
	{
	case EM_NONE:
		if (oEvent.type == KEY_PRESS)
		{
			if (mbSelected && mpoPlane->mpoMap->nSelectionSize() == 1) 
			{
				TCHAR strGoal[128];
				ZeroMemory(strGoal, 0);
				switch (oEvent.c) {
				case VK_F2:
					if (bGetString("Enter description of goal:\n\nENTER to continue\nESC to cancel", strGoal, 128)) 
					{
						SetText(strGoal);
					}
					break;
				}
			}
		}
	}
}