#ifndef _MOB_H_
#define _MOB_H_

#ifndef _SOUNDLIST_H_
#include "soundlist.h"
#endif

extern int gnMobCount;

class MobCoordinator: public SoundEmitter
{
public:
	MobCoordinator(Unit& moUnitLeader)
	{
		gnMobCount++;
		mpoLeader = &moUnitLeader;	
		mState = 0;
		mnLastPlayCycle = 1;
		mnSound = 0;
	}
	~MobCoordinator();
	void GetPlayPosition(unsigned int * unX, unsigned int * unY);
protected:

	// NOTE THAT IT IS POSSIBLE TO NOT HAVE A LEADER UNIT
	Unit * mpoLeader;
	RefList<Unit> moFollowers;
	int mnLastPlayCycle;
	int mnSound;
public:
	void vPlayRandomSound(long lTag);
	int mState;
	int GetSize() 
	{ 
		int nSize = 0; 
		if (mpoLeader) nSize++;
		nSize += moFollowers.GetSize();
		return nSize;
	}
	void GameCycle();
	void Disperse();
	BOOL bHasMember(Unit * poUnit)
	{
		if (moFollowers.Find(poUnit) != ID_NONE) return TRUE;
		if (mpoLeader == poUnit) return TRUE;
		return FALSE;
	}
	void AlertLeaderOfVampireLocation(Unit * poUnit);
	void AttackUnit(Unit * poUnit)
	{
		Command oCommand;
		oCommand.meType = CM_ATTACK;
		oCommand.mbTargetUnit = TRUE;
		oCommand.mnUnit = poUnit->nGetID();
		mpoLeader->vCommand(oCommand, TRUE);
		if (mState < 2)
		{
			// ATTACK
			pApp->pSoundManager->EmitFromObject(mpoLeader, SOUND_ItsAVampire, SOUNDPRIORITY_AMBIENTSOUNDS);
			mState = 2;
		}

		AlertFollowersToAttackUnit(poUnit);
	}
	Unit * GetLeader() { return mpoLeader; }
	Unit * GetUnitToFollow();
	void Regroup();

	void AlertFollowersToAttackUnit(Unit * poUnit)
	{
		int nCount = moFollowers.GetSize();
		Unit * poFollower;
		while (nCount)
		{
			nCount--;
			poFollower = moFollowers.GetItem(nCount);
			Command oCommand;
			oCommand.meType = CM_ATTACK;
			oCommand.mbTargetUnit = TRUE;
			oCommand.mnUnit = poUnit->nGetID();
			poFollower->vCommand(oCommand, TRUE);
		}
	}

	void AddUnitAsFollower(Unit * poUnit, Unit * poFollowThisUnit = NULL);

	void RemoveUnit(BOOL * bOutOk, Unit * poUnit)
	{
		*bOutOk = TRUE;

		if (poUnit == mpoLeader)
		{
			mpoLeader = NULL;
			if (moFollowers.GetSize())
			{
				ChooseNewLeader();
			}
		}
		else
		{
			int nIndex = moFollowers.Find(poUnit);
			if (nIndex == ID_NONE)
			{
				*bOutOk = FALSE;
			}
			else
			{
				moFollowers.RemoveIndex(nIndex);
			}
		}
	}


	void ChooseNewLeader(); 
};


#endif