#include "template.h"
#include "mapnode.h"
#include "Vampire.h"
#include "NonPlayerClass.h"

NonPlayerClass::NonPlayerClass(Template & oNpcTemplate):vClan(oNpcTemplate)
{
	mnSpell = 0;

	// TBD - GET CYCLES PER SECOND INSTEAD OF 20
	mfAnxietyFalloffRate = (float)mpoTemplate->lGetValue(TAG_AnFR, 2) / (float)20;
}

// Default Upgrade Settings
int NonPlayerClass::GetUpgradeStealth()
{
	return mpoTemplate->lGetValue(TAG_uSTE, 0);
}

int NonPlayerClass::GetUpgradeDetection()
{
	return mpoTemplate->lGetValue(TAG_uDET, 0);
}

int NonPlayerClass::GetUpgradeAttack()
{
	return mpoTemplate->lGetValue(TAG_uATT, 0);
}

int NonPlayerClass::GetUpgradeDefend()
{
	return mpoTemplate->lGetValue(TAG_uDEF, 0);
}

int NonPlayerClass::GetUpgradeCasting()
{
	return mpoTemplate->lGetValue(TAG_uCAS, 0);
}

int NonPlayerClass::GetUpgradeSpeed()
{
	return mpoTemplate->lGetValue(TAG_uSPD, 0);
}

int NonPlayerClass::GetUpgradeSight()
{
	return mpoTemplate->lGetValue(TAG_uSIG, 0);
}

int NonPlayerClass::GetUpgradeLevel()
{
	return mpoTemplate->lGetValue(TAG_uLEV, 0);
}

BOOL NonPlayerClass::bCastsSpells()
{
	return mpoTemplate->lGetValue(TAG_bCas, 0);
}

BOOL NonPlayerClass::bConvertibleByBiting()
{
	return mpoTemplate->lGetValue(TAG_bCnv, 0);
}


ENUM_THRALLACTION NonPlayerClass::eWhenThralled()
{
	int n = mpoTemplate->lGetValue(TAG_eThr, 0);
	if (n<0 || n > 2) n = 0;
	return (ENUM_THRALLACTION)n;
}


// Default Converted Vampire Settings
int NonPlayerClass::GetVampireStealth()
{
	return mpoTemplate->lGetValue(TAG_vSTE, 0);
}

int NonPlayerClass::GetVampireDetection()
{
	return mpoTemplate->lGetValue(TAG_vDET, 0);
}

int NonPlayerClass::GetVampireAttack()
{
	return mpoTemplate->lGetValue(TAG_vATT, 0);
}

int NonPlayerClass::GetVampireDefend()
{
	return mpoTemplate->lGetValue(TAG_vDEF, 0);
}

int NonPlayerClass::GetVampireCasting()
{
	return mpoTemplate->lGetValue(TAG_vCAS, 0);
}

int NonPlayerClass::GetVampireSpeed()
{
	return mpoTemplate->lGetValue(TAG_vSPD, 0);
}

int NonPlayerClass::GetVampireSight()
{
	return mpoTemplate->lGetValue(TAG_vSIG, 0);
}

int NonPlayerClass::GetVampireLevel()
{
	return mpoTemplate->lGetValue(TAG_vLEV, 0);
}


HRESULT NonPlayerClass::bGetFirstSpell(int * pnSpellId)
{
	mnSpell = 0;
	return bGetNextSpell(pnSpellId);
}

HRESULT NonPlayerClass::bGetNextSpell(int * pnSpellId)
{
	*pnSpellId = 0;
	mnSpell++;
	return S_OK;
}

int NonPlayerClass::nGetMaxAnxiety()
{
	return mpoTemplate->lGetValue(TAG_MAnx, 300);
}

int NonPlayerClass::nGetAnxietyEnterMob()
{
	return mpoTemplate->lGetValue(TAG_AnEM, 200);
}

int NonPlayerClass::nGetAnxietyLeaveMob()
{
	return mpoTemplate->lGetValue(TAG_AnLM, 125);
}

float NonPlayerClass::fGetAnxietyFalloffRate()
{
	return mfAnxietyFalloffRate;
}

int NonPlayerClass::GetAnxietyIfInvitedToJoinMob()
{
	return mpoTemplate->lGetValue(TAG_AnII, 175);
}

HRESULT NonPlayerClass::vCreateConvert(Vampire ** ppoVampire, Vampire * poParentVampire, Unit * poOriginalUnit, MapNode * poNode, MapPlane * poPlane)
{
	// CREATE A VAMPIRE FROM THE PARENT VAMPIRE'S CLAN/PLAYER/OTHER INFO
	Clan * poClan = (Clan *) poParentVampire->vGetClan();

	// NEW VAMPIRE'S LEVEL ADJUSTMENT FROM CONVERTED CLAN
	int nLevel = GetVampireLevel();

	// TBD - RETRIEVE SEX FROM poUnit
	if (nLevel<1 || nLevel>6) nLevel = 1;

	int nTemplateId = poClan->GetTemplateIdForSexAndLevel(poOriginalUnit->eGetSex(), nLevel);
	*ppoVampire = new Vampire(poParentVampire->poOwner(), poPlane, poNode, nTemplateId);
	

	// RETURN THE NEW VAMPIRE
	if (ppoVampire)
	{
		Vampire & rVampire = **ppoVampire;
		rVampire.manDesired[UPGRADE_Speed] = GetVampireSpeed();
		rVampire.manDesired[UPGRADE_Sight] = GetVampireSight();
		rVampire.manDesired[UPGRADE_Stealth] = GetVampireStealth();
		rVampire.manDesired[UPGRADE_Spells] = GetVampireCasting();
		rVampire.manDesired[UPGRADE_Detection] = GetVampireDetection();
		rVampire.manDesired[UPGRADE_Defend] = GetVampireDefend();
		rVampire.manDesired[UPGRADE_Attack] = GetVampireAttack();

		rVampire.manUpgrade[UPGRADE_Speed] = GetVampireSpeed();
		rVampire.manUpgrade[UPGRADE_Sight] = GetVampireSight();
		rVampire.manUpgrade[UPGRADE_Stealth] = GetVampireStealth();
		rVampire.manUpgrade[UPGRADE_Spells] = GetVampireCasting();
		rVampire.manUpgrade[UPGRADE_Detection] = GetVampireDetection();
		rVampire.manUpgrade[UPGRADE_Defend] = GetVampireDefend();
		rVampire.manUpgrade[UPGRADE_Attack] = GetVampireAttack();

		// SET CLOAKED BITMAP FOR NEW VAMPIRE
		rVampire.SetBitmapCloaked(poOriginalUnit->lBitmapNormal());
		return S_OK;
	}
	else
		return E_OUTOFMEMORY;	
}