// PLAYER.CPP -- the info about one of the players
//
// DJM

#include "player.h"
#include "objectdb.h"
#include "drawobj.h"
#include "net.h"
#include "MsgInternal.h"
#include "unit.h"
#include "vampire.h"
#include "soundlist.h"
#include <math.h>
#include "spell.h"

Player::Player(BOOL bHuman, BOOL bLocal, int nPlayer)
{
	mdwFlags = 0;
	mbHuman = bHuman;
	mbLocal = bLocal;

	mpoClan = NULL;
	mnTopLevel = 0;
	mpoPlane = NULL;
	mpoMap = NULL;
	mnPlayer = nPlayer;
	mnSelectSize = 0;
	mnColorIndex = nPlayer; // djm for now, no selection of color
	mwMiniMapColor = game->wGetMiniMapColor(mnColorIndex);

	// fill up the turn stack with null turns to start
	int i;
	int nTurns = game->nPostingDelay();
	for (i = 0; i < nTurns; ++i) {
		Command *poCommand = new Command;
		poCommand->meType = CM_NONE;
		poCommand->mnPlayer = mnPlayer;
		moTurns.vPushTurn(i, poCommand);
	}

	mpjSpells = NULL;

	//TBD - what will the starting upgrades be??? right now, max out everything
	//assigning 0's will mean that upgrade isn't known yet
	manUpgradeLevel[0] = MAX_LEVEL_UPGRADE;
	for (i = 1; i < UPGRADE_COUNT; ++ i)
		manUpgradeLevel[i] = MAX_UPGRADES_UPGRADE;

/*
	//use these as tests for interface for starting upgrades
	manUpgradeLevel[UPGRADE_Attack] = MAX_UPGRADES_UPGRADE;
	manUpgradeLevel[UPGRADE_Defend] = MAX_UPGRADES_UPGRADE;
	manUpgradeLevel[UPGRADE_Detection] = MAX_UPGRADES_UPGRADE;
	manUpgradeLevel[UPGRADE_Sight] = MAX_UPGRADES_UPGRADE;
	manUpgradeLevel[UPGRADE_Speed] = MAX_UPGRADES_UPGRADE;
	manUpgradeLevel[UPGRADE_Spells] = MAX_UPGRADES_UPGRADE;
	manUpgradeLevel[UPGRADE_Stealth] = MAX_UPGRADES_UPGRADE;
*/

}

Player::~Player()
{
	if (mpjSpells) {
		delete mpjSpells;
		mpjSpells = NULL;
	}
	// do nothing for now
}

void Player::vAddUnit(Unit *poUnit)
{
	moUnits.AddToEnd(poUnit);

	// paulb - just track top level during add's - once loaded, the leader does not change
	if (poUnit->nLevel() > mnTopLevel)
	{
		SetTopLevel(poUnit->nLevel());
	}
}

void Player::SetTopLevel(int nLevel)
{
	if (mnTopLevel < nLevel)
	{
		mnTopLevel = nLevel;
	}
}

int Player::GetTopLevel()
{
	return mnTopLevel;
}

void Player::SetFlags(int nFlag)
{
	if (nFlag != -1)
	{
		DWORD dwFlag = (DWORD)pow((int)2, nFlag);
		mdwFlags = mdwFlags | dwFlag;
	}
}

void Player::ClearFlags(int nFlag)
{
	if (nFlag != -1)
	{
		DWORD dwFlag = (DWORD)pow((int)2, nFlag);
		mdwFlags = mdwFlags & (~dwFlag);
	}
}

void Player::GetVampireLord(Unit ** poUnit)
{
	int i = moUnits.GetSize();
	Unit * poReturn;

	*poUnit = NULL;
	while(i)
	{
		i--;
		poReturn = moUnits.GetItem(i);
		if (poReturn->nLevel() == mnTopLevel)
		{
			*poUnit = poReturn;
			return;
		}
	}
	*poUnit = NULL;
	return;
}

long Player::lGetChecksum(void)
{
	// checksum all member variables that are important to net synch

	long lCheck = 0;

	// do self
	lCheck += (long)mbHuman + mnPlayer + mnSelectSize + moUnits.GetSize();

	int i;
	for (i = 0; i < mnSelectSize; ++i) {
		lCheck += manSelection[i];
	}
	
	return lCheck;
}

void Player::vDumpSyncText(FILE *fileDump)
{
	// do self
	fprintf(fileDump, "*Player\n");
	fprintf(fileDump, " Player number: %d\n", mnPlayer);
	fprintf(fileDump, " Human? %d\n", (int)mbHuman);
	fprintf(fileDump, " Number of units: %d\n", moUnits.GetSize());
	fprintf(fileDump, " Selection size: %d\n", mnSelectSize);

	int i;
	for (i = 0; i < mnSelectSize; ++i) {
		fprintf(fileDump, "  Selection %d: %d\n", i, manSelection[i]);
	}

}

void Player::vPostRemoteCommand(Command *poCommand, int nTurn)
{
//	assert(!mbLocal);
	assert(poCommand);
	assert(poCommand->mnPlayer == mnPlayer);

	// besides the assertion,
	// we'll ignore any command that is trying to control the wrong player's
	// units.
	if (poCommand->mnPlayer != mnPlayer) {
		//
		// TBD -- think of an appropriate response -- probable hacking going on
		poCommand->meType = CM_NONE;
	}

	//assert(!moTurns.bTurnAvailable(nTurn));

	moTurns.vPushTurn(nTurn, poCommand);
}

void Player::vPostCommand(Command *poCommand, BOOL bAudioConfirm)
{
	assert(mbLocal);
	assert(poCommand);

	poCommand->mnPlayer = mnPlayer;
	poCommand->mlChecksum = 0;//game->lGetChecksum();

	if (gpoMap->GetMode() == UIMODE_ENTERCOMMANDS)
	{
		if (poCommand->meType == CM_SELECT)
		{
			gpoMap->SetMode(UIMODE_SELECT);
		}
		else if (poCommand->meType != CM_NONE)
		{
			for (int i = 0; i < mnSelectSize; ++i) 
			{
				DrawObj *poObj = goObjectDB.poGetObject(manSelection[i]);
				if (poObj) 
				{
					if (poObj->bIsUnit())
					{
						Unit * poUnit = (Unit *)poObj;
						if (!poUnit->poOwner())
						{
							poUnit->vCommand(*poCommand, FALSE);
						}
					}
				}
			}
			poCommand->meType = CM_NONE;
		}
	}
	else if ((poCommand->meType != CM_NONE)&&(poCommand->meType != CM_ATTR)&&(poCommand->meType != CM_SELECT))
	{
		// CHECK FOR ABILITY TO PERFORM COMMAND DURING POST
		// IF ONLY ONE UNIT IS SELECTED
		BOOL bCancel = FALSE;
		Vampire * poObj = NULL;
		for (int i=0; i < mnSelectSize; i++)
		{
			poObj = (Vampire *)goObjectDB.poGetObject(manSelection[i]);
			if (poObj->bIsVampire()) 
			{
				// IF ANY VAMPIRES SELECTED THEN ONE SHOULD BE ABLE TO COMPLETE
				// COMMAND
				if (poObj->bAbleToPerformCommand(*poCommand))
				{
					// AT LEAST ONE SELECTED VAMPIRE CAN PERFORM COMMAND
					bCancel = FALSE;
					break;
				}
				bCancel = TRUE;
			}
		}
		if (bCancel)
		{
			poObj->bAbleToPerformCommand(*poCommand, TRUE);
			delete poCommand;
			return;
		}
		if (bAudioConfirm)
		{
			gpoMap->vAudioConfirmCommand();
		}
	}

	//
	// post to my own list
	//
	int nPostTurn = game->nPostingTurn();
	while (moTurns.bTurnAvailable(nPostTurn)) {
		++nPostTurn;
		if (nPostTurn - game->nNetTurn() >= MAX_TURN_QUEUE) {
			DebugString("Turn queue exhausted!!");
			return;
		}
	}
	moTurns.vPushTurn(nPostTurn, poCommand);

	// post to other players' lists
	if (game->bNetGame()) {
		BYTE ajBuf[MAX_NET_MSG_SIZE];
		int nSize;
		NET_TURN_HEAD *pHead = (NET_TURN_HEAD *)ajBuf;
		pHead->jKind = NET_MSG_TURN;
		pHead->lTurn = nPostTurn;
		pHead->lChecksum = poCommand->mlChecksum;

		nSize = poCommand->nCreateBinaryCommand(ajBuf + sizeof(NET_TURN_HEAD));

		nSize += sizeof(NET_TURN_HEAD);

		gpoNet->SendMessageAll((char*)ajBuf, nSize);
	}
}

void Player::SendChecksumInfo()
{
	if (!(game->bNetGame())) return;

	MsgInternal * poMsgInternal = NULL;

	BYTE ajBuf[MAX_NET_MSG_SIZE];

	int nSize;
	NET_INTERNAL_HEAD *pHead = (NET_INTERNAL_HEAD *)ajBuf;

	pHead->jKind = NET_MSG_INTERNAL;
	pHead->lTurn = game->nNetTurn();
	pHead->lChecksum = game->lGetChecksum();
	poMsgInternal = new MsgInternal(mnPlayer, MSGINTERNAL_CHECKSUM);
	if (poMsgInternal)
	{
		nSize = poMsgInternal->nCreateBinary(ajBuf + sizeof(NET_INTERNAL_HEAD), MAX_NET_MSG_SIZE);
		nSize += sizeof(NET_INTERNAL_HEAD);
		gpoNet->SendMessageAll((char *) ajBuf, nSize);
		delete poMsgInternal;
	}
}

BOOL Player::bTurnAvailable(int nTurn)
{
	return moTurns.bTurnAvailable(nTurn);
}

long Player::lTurnChecksum(int nTurn)
{
	// pab - turn must exist - checked in CommandStack::lTurnChecksum
#ifdef _DEBUG
	Command * poCommand = moTurns.poPeekTurn(nTurn);
#endif

	return moTurns.lTurnChecksum(nTurn);
}

void Player::vNetTurn(int nTurn)
{
	int i;

	assert(moTurns.bTurnAvailable(nTurn));

	Command *poCommand = moTurns.poPeekTurn(nTurn);

	assert(poCommand);

	if (poCommand->mnPlayer != mnPlayer) 
	{
		vDebugOut("Command by player %d in player %d's stack\n", poCommand->mnPlayer, mnPlayer);
		poCommand->meType = CM_NONE;
	}

	switch (poCommand->meType) {
	case CM_NONE:
		// do nothing
		break;
	case CM_CLOAK:
		{
			int nCount = 0;
			int nCountCloaked = 0;
			for (i = 0; i < mnSelectSize; ++i) 
			{
				Vampire * poObj = (Vampire *)goObjectDB.poGetObject(manSelection[i]);
				if (poObj && poObj->bIsUnit()) 
				{
					nCount++;
					if (poObj->bIsVampire() && poObj->bCloaked())
					{
						nCountCloaked++;
					}
				}
			}
			if (nCountCloaked > (nCount / 2))
			{
				poCommand->mnNewValue = FALSE;
				// TBD - SOUND - UNCLOAKING
			}
			else
			{
				poCommand->mnNewValue = TRUE;
				// TBD - SOUND - CLOAKING
			}
			for (i = 0; i < mnSelectSize; ++i) 
			{
				Unit * poObj = (Unit *)goObjectDB.poGetObject(manSelection[i]);
				if (poObj && poObj->bIsUnit()) 
				{
					poObj->vCommand(*poCommand);
				}
			}
		}
		break;
	case CM_SELECT:
		//
		// this is a meta-turn that changes the current selection
		//
		mnSelectSize = poCommand->mnSelectSize;
		for (i = 0; i < mnSelectSize; ++i) {
			manSelection[i] = poCommand->manSelection[i];
		}
		break;
	case CM_ATTR:
		if (poCommand->mbTargetUnit)
		{
			DrawObj *poObj = goObjectDB.poGetObject(poCommand->mnUnit);
			{
				assert(poObj);
				if (poObj)
				{
					poObj->vCommand(*poCommand);
				}
			}
			break;
		}
	default:
		for (i = 0; i < mnSelectSize; ++i) {
			DrawObj *poObj = goObjectDB.poGetObject(manSelection[i]);
			if (poObj) 
			{
				poObj->vCommand(*poCommand);
			}
		}
		break;
	}

	//
	// at this point, everyone should be done with the command, so
	// dispose of it.
	//
	moTurns.vPopTurn(nTurn);
}

int Player::GetUnitCountOfLevel(int nLevel)
{
	int nReturnCount = 0;
	int nCount = moUnits.GetSize();
	while(nCount>0)
	{
		nCount--;
		Vampire * poUnit = (Vampire *)moUnits.GetItem(nCount);
		assert(poUnit->bIsVampire());
		if (poUnit->nLevel() == nLevel)
		{
			nReturnCount++;
		}
		else if (poUnit->bIsUpgradeInProgress())
		{
			UPGRADE_NAME upgrade = UPGRADE_NONE;
			int nProgress = 0;
			poUnit->GetUpgradeInProgress(&upgrade, &nProgress);
			if (upgrade == UPGRADE_Level)
			{
				if (poUnit->nLevel() == (nLevel - 1))
				{
					nReturnCount++;
				}
			}
		}
	}
	return nReturnCount;
}

Vampire * Player::FindFriendlyUnitToRetreatTowards(Vampire * poThisUnit)
{
	Vampire * poVampireReturn = NULL;
	int nSize = moUnits.GetSize();
	int nBestScore = 200000000;  // ARBITRARY
	int nScore;
	int i = nSize;
	const int nThisX = poThisUnit->nGetX();
	const int nThisY = poThisUnit->nGetY();

	// FIRST JUST TRY TO FIND NEAREST VAMPIRE OF A HIGHER LEVEL
	while (i)
	{
		i--;
		Vampire * poVampire = (Vampire *)moUnits.GetItem(i);
		if (poVampire == poThisUnit) continue;
		if (poVampire->nLevel() > poThisUnit->nLevel())
		{
			nScore = poVampire->lDistToSpace(nThisX, nThisY);
			if (nScore < nBestScore)
			{
				nBestScore = nScore;
				poVampireReturn = poVampire;
			}
		}
	}
	// IF NONE FOUND
	if (!poVampireReturn)
	{
		// NOW JUST TRY TO FIND NEAREST VAMPIRE THAT CAN HELP
		i = nSize;
		while(i)
		{
			i--;
			Vampire * poVampire = (Vampire *)moUnits.GetItem(i);
			if (poVampire == poThisUnit) continue;

			nScore = poVampire->lDistToSpace(nThisX, nThisY);
			if (nScore < nBestScore)
			{
				nBestScore = nScore;
				poVampireReturn = poVampire;
			}
		}
	}

	return poVampireReturn;
}

int Player::GetLiveFriendlyUnitCount()
{
	// TBD -- ADD SUPPORT TO COUNT ALLIED UNITS
	return moUnits.GetSize();
}

//The number of known upgrades (Level excluded)
int Player::nKnownUpgrades()
{
	int nCount = 0;
	for (int i = 1; i < UPGRADE_COUNT; ++ i)
	{
		if (manUpgradeLevel[i] != 0)
			++nCount;
	}
	return nCount;
}

//Returns the 'position' of the known upgrade. 
//Ex - a return value of 2 says that nUpgrade is the 2nd in the list of known upgrades. The enum list is defined in UNIT.H
// A return of 0 means the upgrade isn't known
int Player::nUpgradePosition(int nUpgrade) 
{
	int nPos = 0;
	for (int i = 1; i< UPGRADE_COUNT; ++i)
	{
		if (manUpgradeLevel[i] != 0)
			++nPos;

		if (i == nUpgrade)
			return nPos;

	}

	return 0;	//none found
}

int Player::CallForHelpFromFriendlies(Vampire * poVampire, int nRange)
{
	// RETURNS THE NUMBER OF FRIENDLIES WHO ARE RESPONDING WITH HELP
	assert(nRange>0 && nRange < 40);
	assert(poVampire);

	int nX = poVampire->nGetX();
	int nY = poVampire->nGetY();
	int i = moUnits.GetSize();
	int count = 0;
	while(i)
	{
		i--;
		Vampire * poVampireFriendly = (Vampire *)moUnits.GetItem(i);
		if (poVampireFriendly == poVampire) continue;
		if (poVampireFriendly->lDistToSpace(nX, nY) <= nRange)
		{
			count += poVampireFriendly->RequestHelp(poVampire);
		}
	}
	return count;
}

//allocate space for mpjSpells based on the number of spells in the game
void Player::vInitSpellSpace(int nNumSpells)
{
	mpjSpells = new BYTE[nNumSpells];

	for (int i = 0; i < nNumSpells; ++i)
	{
		mpjSpells[i] = 0;
	}

	//TBD - for now add all the know spells but setup startup spells someplace
	for (int j = 0; j < nNumSpells; ++j)
	{
		Spell *poSpell = game->GetTrueSpell(j);
		vAddSpell(j, poSpell->GetRequiredLevel());
		//tbd - eventually we'll add spells like this:
		//vAddSpell(poSpell->nTemplateID() - game->mpoTemplate->lGetValue(TAG_gstb,0), poSpell->GetRequiredLevel());
	}

}		


//returns the value of indexed item in mpjSpells 
BYTE Player::jSpellKnown(int nNdx)
{
	if (nNdx >= game->mpoTemplate->lGetValue(TAG_gstc, 0)) {
		assert(0);
		return 0;
	}

	return mpjSpells[nNdx];
}

void Player::vAddSpell(int nNdx, int nLevel)
{
	if (nNdx >= game->mpoTemplate->lGetValue(TAG_gstc, 0)) {
		assert(0);
		return;
	}

	mpjSpells[nNdx] = (BYTE)nLevel;
	
}