// PORTRAIT.CPP -- A  portrait of a selected unit
//
// TW


#include "portrait.h"
#include "unit.h"
#include "sciplane.h"
#include "bargraph.h"
#include "statusln.h"
#include "vampire.h"
#include "intplane.h"
#include "invplane.h"

//Level indicator defines
#define LEVEL_X_OFFSET	12	//X coordinate offset from portrait for 'Level'
#define LEVEL_Y_OFFSET	4	//Y coordinate offset from portrait for 'Level'
#define LEVEL_FONT		3	//font number for level

//Cloak indicator
#define CLOAK_X_OFFSET	24	//X coordinate offset from portrait for 'Cloak'
#define CLOAK_Y_OFFSET	4	//Y coordinate offset from portrait for 'Cloak'


//Dual Portrait Inventory plane size defines (offset from upper left corner of portrait)
#define DP_PLANE_OFF_LEFT	11
#define DP_PLANE_OFF_TOP	32
#define DP_PLANE_OFF_RIGHT	39
#define DP_PLANE_OFF_BOTTOM	98

#define DP_ITEM_WIDTH		28	//width of single inventory item slot
#define DP_ITEM_HEIGHT		24	//width of single inventory item slot


Unit *Portrait::mpoUnitToSummarize = NULL;


Portrait::Portrait(sciPlane *poPlane, Template *poTemplate) {
	assert (poPlane);
	plane = poPlane;
	mpoUnit = NULL;
	mpoStatusLine = NULL;
	mnCurUnitLevel = -1;
	mbDrawUpgrade = FALSE;
	mpoLevel = NULL;
	mpoCloak = NULL;
	mpoPower = NULL;
	mpoHealth = NULL;
	mpoUpgrade = NULL;
	mpoStatusText = NULL;
	mpoTemplate = poTemplate;
	assert(poTemplate);
}

Portrait::~Portrait() {
	mpoTemplate = NULL;
	if (mpoLevel) {
		delete mpoLevel;
		mpoLevel = NULL;
	}
	if (mpoCloak) {
		delete mpoCloak;
		mpoCloak = NULL;
	}
	if (mpoPower) {
		delete mpoPower;
		mpoPower = NULL;
	}
	if (mpoHealth) {
		delete mpoHealth;
		mpoHealth = NULL;
	}
	if (mpoUpgrade) {
		delete mpoUpgrade;
		mpoUpgrade = NULL;
	}
	if (mpoStatusText)
	{
		delete[] mpoStatusText;
		mpoStatusText = NULL;
	}
}

void Portrait::vSetUp() {
	BYTE jRed, jGreen, jBlue;
	int nWidth, nHeight, nOffX, nOffY;

	nWidth = mpoTemplate->lGetValue(TAG_IppW, 0);
	nHeight = mpoTemplate->lGetValue(TAG_IppH, 0);
	nOffX = mpoTemplate->lGetValue(TAG_IppX, 0);
	nOffY = mpoTemplate->lGetValue(TAG_IppY, 0);
	mpoPower = new BarGraph(nWidth, nHeight, 100, BG_VERTICAL);
	mpoPower->vSetXY(nOffX, nOffY);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGr, 0);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGg, 155); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGb, 0);
	mpoPower->vSetFillColor(jRed,jGreen,jBlue);

	nWidth = mpoTemplate->lGetValue(TAG_IphW, 0);
	nHeight = mpoTemplate->lGetValue(TAG_IphH, 0);
	nOffX = mpoTemplate->lGetValue(TAG_IphX, 0);
	nOffY = mpoTemplate->lGetValue(TAG_IphY, 0);
	mpoHealth = new BarGraph(nWidth, nHeight, 100, BG_VERTICAL);
	mpoHealth->vSetXY(nOffX, nOffY);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGr, 236);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGg, 0); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGb, 0);
	mpoHealth->vSetFillColor(jRed,jGreen,jBlue);

	nWidth = mpoTemplate->lGetValue(TAG_IpuW, 0);
	nHeight = mpoTemplate->lGetValue(TAG_IpuH, 0);
	nOffX = mpoTemplate->lGetValue(TAG_IpuX, 0);
	nOffY = mpoTemplate->lGetValue(TAG_IpuY, 0);
	mpoUpgrade = new BarGraph(nWidth, nHeight, 100, BG_HORIZONTAL);
	mpoUpgrade->vSetXY(nOffX, nOffY);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGr, 0);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGg, 0); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGb, 214);
	mpoUpgrade->vSetFillColor(jRed,jGreen,jBlue);

	mpoLevel = new sciText();
	mpoLevel->vSetPlane(plane);
	mpoLevel->SetFont(LEVEL_FONT);
	mpoLevel->Init();
	mpoLevel->SetText("00");
	mpoLevel->vSetSize();
	mpoLevel->Posn(left,top);

	mpoCloak = new sciText();
	mpoCloak->vSetPlane(plane);
	mpoCloak->SetFont(LEVEL_FONT);
	mpoCloak->Init();
	mpoCloak->SetText("U");
	mpoCloak->vSetSize();
	mpoCloak->Posn(left,top);

	mnLocalPlayer = game->poLocalPlayer()->mnPlayer;
}

void Portrait::Posn(int posnX, int posnY) {
	sciProp::Posn(posnX, posnY);
	mpoLevel->Posn(posnX + LEVEL_X_OFFSET, posnY + LEVEL_Y_OFFSET);
	mpoCloak->Posn(posnX + CLOAK_X_OFFSET, posnY + CLOAK_Y_OFFSET);
}

void Portrait::SetPri(int newPri) {
	sciProp::SetPri(newPri);
	mpoLevel->SetPri(newPri + 5);
	mpoCloak->SetPri(newPri + 5);
}

void Portrait::Show(Unit *poUnit) {
	mpoUnit = poUnit;		//set up backpointer
	mnCurUnitLevel = -1;	//clear level
	mbDrawUpgrade = FALSE;
//	vUpdate(poUnit);	//update and set current levels/settings
	vSetStatusText(poUnit->GetName());
	if (poUnit->bIsVampire())
	{
		if (poUnit->bIsLocalVampire())
		{
			meUnitType = PT_LOCAL_VAMPIRE;
		} else {
			meUnitType = PT_ENEMY_VAMPIRE;
		}
	} else {
		meUnitType = PT_PEASANT;
	}
	
	sciProp::Show();

	//local vampires or marked enemy vampire portraits can have a level display
	switch (meUnitType)
	{
	case PT_LOCAL_VAMPIRE:
		vSetLevel();
		mpoLevel->Show();
		break;
	case PT_ENEMY_VAMPIRE:
		if (((Vampire*)mpoUnit)->bMarked(mnLocalPlayer))
		{
			vSetLevel();
			mpoLevel->Show();
		}
		break;
	}

	//local vampire portraits are the only ones that need the cloak display
	mbCloaked = mpoUnit->bCloaked();
	if (poUnit->bIsLocalVampire() && !(poUnit->bCloaked()) )
		mpoCloak->Show();

	Doit();				//update and set current levels/settings
	
}

void Portrait::Hide() {
	sciProp::Hide();
	mpoUnit = NULL;
	mpoLevel->Hide();
	mpoCloak->Hide();
}

void Portrait::Render(RECT &updateRect) {
	if (!hidden) {
		sciProp::Render(updateRect);
		//mpoLevel.vDraw();
		switch (meUnitType)
		{
		case PT_LOCAL_VAMPIRE:
			mpoPower->vDraw(left + mpoPower->nGetX(), top + mpoPower->nGetY(), plane);
			mpoHealth->vDraw(left + mpoHealth->nGetX(), top + mpoHealth->nGetY(), plane);
			if (mbDrawUpgrade) {
				mpoUpgrade->vDraw(left + mpoUpgrade->nGetX(), top + mpoUpgrade->nGetY(), plane);
				mbDrawUpgrade = FALSE;
			}
			break;

		case PT_ENEMY_VAMPIRE:
		case PT_PEASANT:
			mpoHealth->vDraw(left + mpoHealth->nGetX(), top + mpoHealth->nGetY(), plane);
			break;
		
		}
	}
}

Boolean Portrait::HandleEvent(sciEvent &oEvent) {
	if (hidden) {
		return FALSE;
	}

	switch (oEvent.type) {

	case MOUSE_MOVE:
		if (OnMe(oEvent.x, oEvent.y)) {
			//vDebugOut("On me\n");
			//vDebugOut("x: %d, y:%d\n", plane->GetCursorX(), plane->GetCursorY());
			mpoUnitToSummarize = mpoUnit;
			
			if (!mpoStatusLine->bOnBar((Portrait*)this)) 
			{
				//char str[256];
				//sprintf(str, "test message AG:%d TE:%d", mpoUnit->GetAggression(), mpoUnit->GetTenacity());
				//sprintf(str, "test message");
				mpoStatusLine->vAddToBar((Portrait*)this, mpoStatusText);
				gpoInterface->vUpdateVampireBtnState((Vampire*)mpoUnit);
			}
			
			oEvent.claimed = TRUE;
		} else 	{
			//mouse isn't on portrait, but check to see if we JUST were
			assert(mpoStatusLine);
//			if (mpoUnitToSummarize == mpoUnit) {
//				mpoUnitToSummarize = NULL;
//			}
			if (mpoStatusLine->bOnBar((Portrait*)this)) {
				//we were just on portrait so remove from status line
				mpoUnitToSummarize = NULL;
				mpoStatusLine->vClearFromBar((Portrait*)this); //the plane->vMarkUpdate occurs here
				gpoInterface->vUpdateSelectedVampires();
			}

		}
		break;

//	case MOUSE_DOWN:
//		oEvent.claimed = TRUE;
//		break;

	}
	
	return (bHeadHandleEvent(oEvent));
	//return oEvent.claimed;

}

//A left click on the 'head' portion will select just that unit, or a right click on the 'head'
//will center that unit on the map
Boolean Portrait::bHeadHandleEvent(sciEvent &oEvent) {
	switch (oEvent.type) {
	
	case MOUSE_DOWN:
		{
			if (OnMe(oEvent.x, oEvent.y)) {
				oEvent.claimed = TRUE;
				switch (oEvent.modifiers) {
				case LEFT_MOUSE:
					gpoMap->vClearAllSelections();
					gpoMap->vAddSelection(mpoUnit);
					gpoMap->vCommitSelection();
					//vDebugOut("left\n");
					break;
				case RIGHT_MOUSE:
					gpoMap->vCenterNode(mpoUnit->poNode());
					vDebugOut("right\n");
					break;
				}
			}

			break;
		}

	}
	
	return oEvent.claimed;

}

void Portrait::vSetStatusText(const char *pstrNewText)
{
	int nStrLen;

	if (mpoStatusText && pstrNewText && !strcmp(mpoStatusText, pstrNewText)) {
		return;
	}

	if (mpoStatusText) {
		delete[] mpoStatusText;
	}

	if (pstrNewText) {
		nStrLen = strlen(pstrNewText);
		mpoStatusText = new char[nStrLen + 1];
		strcpy(mpoStatusText, pstrNewText);
	}
}

//Assumes mpoUnit is a vampire of some sort
void Portrait::vSetLevel() {
	// update/set level text indicator
	assert(mpoUnit);
	
	if (mpoUnit->bIsLocalVampire())
	{
		//set level for local vampire	
		mnCurUnitLevel = mpoUnit->nLevel();
	}
	else
	{
		//set perceived level for enemy vampire
		mnCurUnitLevel = ((Vampire*)mpoUnit)->moStatus.nDetectionLevel();
	}

	char str[2];
	sprintf(str, "%d", mnCurUnitLevel);
	mpoLevel->SetText(str);
}

void Portrait::Doit() {
	if (!hidden) {
		Boolean bUpdate = FALSE;
		assert(mpoUnit);
		
		switch (meUnitType)
		{
		case PT_LOCAL_VAMPIRE:
			{
				//check for level change
				//
				if (mpoUnit->nLevel() != mnCurUnitLevel) {
					vSetLevel();
					//plane->window->Update(rcWindow);
					bUpdate = TRUE;
				}

				
				//check for cloak change
				//
				if (mpoUnit->bCloaked() != mbCloaked) {
					mbCloaked = mpoUnit->bCloaked();
					if (mbCloaked)
						mpoCloak->Hide();
					else
						mpoCloak->Show();
					//plane->window->Update(rcWindow);
					bUpdate = TRUE;
				}


				//update health bar graph
				//
				if (mpoUnit->GetMaxHealth() == mpoHealth->nMaxLimit()) {

					//update graph if there is any change
					if (mpoUnit->GetHealth() != mpoHealth->nCurLevel()) {
						mpoHealth->vFill(mpoUnit->GetHealth()); 
						//plane->window->Update(rcWindow);
						bUpdate = TRUE;
					}

				} else {
					//unit changed level since the max health is different
					mpoHealth->vNewUpperLimit(mpoUnit->GetMaxHealth());
					mpoHealth->vFill(mpoUnit->GetHealth()); 
					//plane->window->Update(rcWindow);
					bUpdate = TRUE;
				}


				//update power bar graph
				//
				if (mpoUnit->GetMaxPower() == mpoPower->nMaxLimit()) {

					//update graph if there is any change
					if (mpoUnit->GetPower() != mpoPower->nCurLevel()) {
						mpoPower->vFill(mpoUnit->GetPower()); 
						//plane->window->Update(rcWindow);
						bUpdate = TRUE;
					}

				} else {
					//unit changed level since the max power is different
					mpoPower->vNewUpperLimit(mpoUnit->GetMaxPower());
					mpoPower->vFill(mpoUnit->GetPower()); 
					//plane->window->Update(rcWindow);
					bUpdate = TRUE;
				}

				//update any upgrades in progress
				//
				if (mpoUnit->bIsVampire()) {
					if (mpoUnit->bIsUpgradeInProgress()) {
						UPGRADE_NAME eName;
						int	nPercent;
						((Vampire*)mpoUnit)->GetUpgradeInProgress(&eName, &nPercent);
						if (eName == UPGRADE_NONE)
							nPercent = 0;
						mpoUpgrade->vFill(nPercent);
						mbDrawUpgrade = TRUE;
						//plane->window->Update(rcWindow);
						bUpdate = TRUE;
					}
				}
			
				break; //PT_LOCAL_VAMPIRE
			}

		case PT_ENEMY_VAMPIRE:
			{
				//check for level change
				//
				if (((Vampire*)mpoUnit)->bMarked(mnLocalPlayer))
				{
					if (((Vampire*)mpoUnit)->moStatus.nDetectionLevel() != mnCurUnitLevel) 
					{
						if (mpoLevel->IsHidden())
						{
							mpoLevel->Show();
						}
						vSetLevel();
						//plane->window->Update(rcWindow);
						bUpdate = TRUE;
					}
				}

				//update health bar graph
				//
				if (mpoUnit->GetMaxHealth() == mpoHealth->nMaxLimit()) {

					//update graph if there is any change
					if (mpoUnit->GetHealth() != mpoHealth->nCurLevel()) {
						mpoHealth->vFill(mpoUnit->GetHealth()); 
						//plane->window->Update(rcWindow);
						bUpdate = TRUE;
					}

				} else {
					//unit changed level since the max health is different
					mpoHealth->vNewUpperLimit(mpoUnit->GetMaxHealth());
					mpoHealth->vFill(mpoUnit->GetHealth()); 
					//plane->window->Update(rcWindow);
					bUpdate = TRUE;
				}


				break; //PT_ENEMY_VAMPIRE
			}

		case PT_PEASANT:
			{
				//update health bar graph
				//
				if (mpoUnit->GetMaxHealth() == mpoHealth->nMaxLimit()) {

					//update graph if there is any change
					if (mpoUnit->GetHealth() != mpoHealth->nCurLevel()) {
						mpoHealth->vFill(mpoUnit->GetHealth()); 
						//plane->window->Update(rcWindow);
						bUpdate = TRUE;
					}

				} else {
					//unit changed level since the max health is different
					mpoHealth->vNewUpperLimit(mpoUnit->GetMaxHealth());
					mpoHealth->vFill(mpoUnit->GetHealth()); 
					//plane->window->Update(rcWindow);
					bUpdate = TRUE;
				}

				break; //PT_PEASANT
			}
		
		} //switch(meUnitType)

		
		
		if (bUpdate)		
		{
			plane->vMarkUpdate();
		}

	}
}



///////////////////////////////////////////////////////////////////////////////////
//									//
// DEFINES FOR SINGLE PORTRAIT		//
//									//	
///////////////////////////////////////////////////////////////////////////////////

//Level indicator defines
#define SP_LEVEL_X_OFFSET	30//49	//X coordinate offset from portrait for 'Level'
#define SP_LEVEL_Y_OFFSET	4//24	//Y coordinate offset from portrait for 'Level'
#define SP_LEVEL_FONT		3	//font number for level

//Cloak indicator
#define SP_CLOAK_X_OFFSET	36//54	//X coordinate offset from portrait for 'Cloak'
#define SP_CLOAK_Y_OFFSET	4//24	//Y coordinate offset from portrait for 'Cloak'


SinglePortrait::SinglePortrait(sciPlane *poPlane, Template *poTemplate) :	Portrait(poPlane, poTemplate) {
	mpoHead = NULL;
	mpoInvPlane = NULL;
}

SinglePortrait::~SinglePortrait() {
	if (mpoHead) {
		delete mpoHead;
		mpoHead = NULL;
	}

	//mpoInvPlane is deleted when vWindow is disposed(member list moPlanes is deleted)
	mpoInvPlane = NULL;

}

Boolean SinglePortrait::bHeadHandleEvent(sciEvent &oEvent) {
	switch (oEvent.type) {
	
	case MOUSE_DOWN:
		{
			if (mpoHead->OnMe(oEvent.x, oEvent.y)) {
				oEvent.claimed = TRUE;
				switch (oEvent.modifiers) {
				case LEFT_MOUSE:
					//no reason to select this unit since it already is
					//vDebugOut("left\n");
					break;
				case RIGHT_MOUSE:
					gpoMap->vCenterNode(mpoUnit->poNode());
					vDebugOut("right\n");
					break;
				}
			}

			break;
		}

	}
	
	return oEvent.claimed;

}

void SinglePortrait::vSetUp() {
	BYTE jRed, jGreen, jBlue;
	int nWidth, nHeight, nOffX, nOffY;

	nWidth = mpoTemplate->lGetValue(TAG_IspW, 0);
	nHeight = mpoTemplate->lGetValue(TAG_IspH, 0);
	nOffX = mpoTemplate->lGetValue(TAG_IspX, 0);
	nOffY = mpoTemplate->lGetValue(TAG_IspY, 0);
	mpoPower = new BarGraph(nWidth, nHeight, 100, BG_VERTICAL);
	mpoPower->vSetXY(nOffX, nOffY);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGr, 0);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGg, 155); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGb, 0);
	mpoPower->vSetFillColor(jRed,jGreen,jBlue);

	nWidth = mpoTemplate->lGetValue(TAG_IshW, 0);
	nHeight = mpoTemplate->lGetValue(TAG_IshH, 0);
	nOffX = mpoTemplate->lGetValue(TAG_IshX, 0);
	nOffY = mpoTemplate->lGetValue(TAG_IshY, 0);
	mpoHealth = new BarGraph(nWidth, nHeight, 100, BG_VERTICAL);
	mpoHealth->vSetXY(nOffX, nOffY);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGr, 236);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGg, 0); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGb, 0);
	mpoHealth->vSetFillColor(jRed,jGreen,jBlue);

	nWidth = mpoTemplate->lGetValue(TAG_IsuW, 0);
	nHeight = mpoTemplate->lGetValue(TAG_IsuH, 0);
	nOffX = mpoTemplate->lGetValue(TAG_IsuX, 0);
	nOffY = mpoTemplate->lGetValue(TAG_IsuY, 0);
	mpoUpgrade = new BarGraph(nWidth, nHeight, 100,BG_HORIZONTAL);
	mpoUpgrade->vSetXY(nOffX, nOffY);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGr, 0);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGg, 0); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGb, 214);
	mpoUpgrade->vSetFillColor(jRed,jGreen,jBlue);

	mpoHead = new sciView();
	mpoHead->vSetPlane(plane);
	mpoHead->SetView(4512);
	mpoHead->Init();
	mpoHead->Hide();

	mpoLevel = new sciText();
	mpoLevel->vSetPlane(plane);
	mpoLevel->SetFont(SP_LEVEL_FONT);
	mpoLevel->Init();
	mpoLevel->SetText("00");
	mpoLevel->vSetSize();
	mpoLevel->Posn(left,top);

	mpoCloak = new sciText();
	mpoCloak->vSetPlane(plane);
	mpoCloak->SetFont(SP_LEVEL_FONT);
	mpoCloak->Init();
	mpoCloak->SetText("U");
	mpoCloak->vSetSize();
	mpoCloak->Posn(left,top);

	int nOffL, nOffT, nOffR, nOffB;
	nOffL = mpoTemplate->lGetValue(TAG_IipL, 0);
	nOffT = mpoTemplate->lGetValue(TAG_IipT, 0);
	nOffR = mpoTemplate->lGetValue(TAG_IipR, 0);
	nOffB = mpoTemplate->lGetValue(TAG_IipB, 0);
	nWidth = mpoTemplate->lGetValue(TAG_IiiW, 0);
	nHeight = mpoTemplate->lGetValue(TAG_IiiH, 0);

	mpoInvPlane = new InventoryPlane();
	mpoInvPlane->vSetRect(nOffL, nOffT, nOffR, nOffB);
	mpoInvPlane->vSetRect1(0,0, nWidth - 1, nHeight - 1);
	mpoInvPlane->vSetRect2(nWidth, 0, (nWidth * 2) - 1, nHeight - 1);
	mpoInvPlane->vSetRect3(nWidth * 2, 0, (nWidth * 3) - 1, nHeight - 1);
	mpoInvPlane->Hide();
}

void SinglePortrait::SetPri(int newPri) {
	Portrait::SetPri(newPri);
	assert(mpoHead);
	//mpoHead->SetPri(newPri + 5);
	mpoHead->SetPri(newPri - 1);
	//mpoLevel->SetPri(newPri + 10);
	mpoCloak->SetPri(newPri + 10);
}

void SinglePortrait::Posn(int posnX, int posnY) {
	sciProp::Posn(posnX, posnY);
	mpoLevel->Posn(posnX + SP_LEVEL_X_OFFSET, posnY + SP_LEVEL_Y_OFFSET);
	mpoCloak->Posn(posnX + SP_CLOAK_X_OFFSET, posnY + SP_CLOAK_Y_OFFSET);

	int nOffX = mpoTemplate->lGetValue(TAG_IsHX, 0);
	int nOffY = mpoTemplate->lGetValue(TAG_IsHY, 0);
	mpoHead->Posn(posnX + nOffX, posnY + nOffY);

	int nOffL, nOffT;
	nOffL = mpoTemplate->lGetValue(TAG_IipL, 0);
	nOffT = mpoTemplate->lGetValue(TAG_IipT, 0);
	mpoInvPlane->Posn(posnX + nOffL + plane->GetX(), posnY + nOffT + plane->GetY());
}

void SinglePortrait::Render(RECT &updateRect)
{
//	Portrait::Render(updateRect);
	if (!hidden) {
		//mpoLevel.vDraw();
		switch (meUnitType)
		{
		case PT_LOCAL_VAMPIRE:
			mpoPower->vDraw(left + mpoPower->nGetX(), top + mpoPower->nGetY(), plane);
			mpoHealth->vDraw(left + mpoHealth->nGetX(), top + mpoHealth->nGetY(), plane);
//tbd - remove these comments once a hole has been cut out for upgrade
//			if (mbDrawUpgrade) {
//				mpoUpgrade->vDraw(left + mpoUpgrade->nGetX(), top + mpoUpgrade->nGetY(), plane);
//				mbDrawUpgrade = FALSE;
//			}
			break;

		case PT_ENEMY_VAMPIRE:
		case PT_PEASANT:
			mpoHealth->vDraw(left + mpoHealth->nGetX(), top + mpoHealth->nGetY(), plane);
			break;
		
		}
		sciProp::Render(updateRect);

	//tbd - remove this entire if statement once a hole has been cut out for upgrade
	if (!hidden) {
		//mpoLevel.vDraw();
		switch (meUnitType)
		{
		case PT_LOCAL_VAMPIRE:
			if (mbDrawUpgrade) {
				mpoUpgrade->vDraw(left + mpoUpgrade->nGetX(), top + mpoUpgrade->nGetY(), plane);
				mbDrawUpgrade = FALSE;
			}
			break;
		}
	}


	}

	mpoInvPlane->vMarkUpdate();
}

void SinglePortrait::Show(Unit *poUnit) {
//	mpoHead->SetView(4512); //TBD - for now
	mpoHead->SetView(poUnit->lBitmapSoloPortrait());
	Portrait::Show(poUnit);
	mpoHead->Show();
	mpoInvPlane->Show(poUnit);
}

void SinglePortrait::Hide() {
	Portrait::Hide();
	mpoHead->Hide();
	mpoInvPlane->Hide();
}

void SinglePortrait::vSetStatusLine(StatusLine *poStatusLine)
{
	Portrait::vSetStatusLine(poStatusLine);
	mpoInvPlane->vSetStatusLine(poStatusLine);
}

void SinglePortrait::vRefreshInventory()
{
	mpoInvPlane->vShowItems();
}

///////////////////////////////////////////////////////////////////////////////////
//									//
// DEFINES FOR DUAL PORTRAITS		//
//									//	
///////////////////////////////////////////////////////////////////////////////////

//Level indicator defines
#define DP_LEVEL_X_OFFSET	17	//X coordinate offset from portrait for 'Level'
#define DP_LEVEL_Y_OFFSET	109	//Y coordinate offset from portrait for 'Level'
#define DP_LEVEL_FONT		3	//font number for level

//Cloak indicator
#define DP_CLOAK_X_OFFSET	27	//X coordinate offset from portrait for 'Cloak'
#define DP_CLOAK_Y_OFFSET	109	//Y coordinate offset from portrait for 'Cloak'

//Power bar graph defines
#define DP_POWER_WIDTH		3	//width of bar
#define DP_POWER_HEIGHT		56	//height of bar
#define DP_POWER_OFF_X		5	//x offset from upper left of portrait
#define DP_POWER_OFF_Y		62	//y offset from upper left of portrait

//Health bar graph defines
#define DP_HEALTH_WIDTH		3	//width of bar
#define DP_HEALTH_HEIGHT	56	//height of bar
#define DP_HEALTH_OFF_X		41	//x offset from upper left of portrait
#define DP_HEALTH_OFF_Y		62	//y offset from upper left of portrait

//Upgrade bar graph defines
#define DP_UPGRADE_WIDTH	30	//width of bar
#define DP_UPGRADE_HEIGHT	2	//height of bar
#define DP_UPGRADE_OFF_X	10	//x offset from upper left of portrait
#define DP_UPGRADE_OFF_Y	101	//y offset from upper left of portrait

//Head defines
#define DP_HEAD_OFF_X		8	//x offset from upper left of background view
#define DP_HEAD_OFF_Y		105	//x offset from upper left of background view

DualPortrait::DualPortrait(sciPlane *poPlane, Template *poTemplate) :	Portrait(poPlane, poTemplate) {
	mpoHead = NULL;
	mpoInvPlane = NULL;
}

DualPortrait::~DualPortrait() {
	if (mpoHead) {
		delete mpoHead;
		mpoHead = NULL;
	}

	//mpoInvPlane is deleted when vWindow is disposed(member list moPlanes is deleted)
	mpoInvPlane = NULL;
}

Boolean DualPortrait::bHeadHandleEvent(sciEvent &oEvent) {
	switch (oEvent.type) {
	
	case MOUSE_DOWN:
		{
			if (mpoHead->OnMe(oEvent.x, oEvent.y)) {
				oEvent.claimed = TRUE;
				switch (oEvent.modifiers) {
				case LEFT_MOUSE:
					gpoMap->vClearAllSelections();
					gpoMap->vAddSelection(mpoUnit);
					gpoMap->vCommitSelection();
					//vDebugOut("left\n");
					break;
				case RIGHT_MOUSE:
					gpoMap->vCenterNode(mpoUnit->poNode());
					vDebugOut("right\n");
					break;
				}
			}

			break;
		}

	}
	
	return oEvent.claimed;

}


void DualPortrait::vSetUp() {
	BYTE jRed, jGreen, jBlue;

	mpoPower = new BarGraph(DP_POWER_WIDTH, DP_POWER_HEIGHT, 100, BG_VERTICAL);
	mpoPower->vSetXY(DP_POWER_OFF_X, DP_POWER_OFF_Y);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGr, 0);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGg, 155); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_PBGb, 0);
	mpoPower->vSetFillColor(jRed,jGreen,jBlue);

	mpoHealth = new BarGraph(DP_HEALTH_WIDTH, DP_HEALTH_HEIGHT, 100, BG_VERTICAL);
	mpoHealth->vSetXY(DP_HEALTH_OFF_X, DP_HEALTH_OFF_Y);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGr, 236);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGg, 0); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_HBGb, 0);
	mpoHealth->vSetFillColor(jRed,jGreen,jBlue);

	mpoUpgrade = new BarGraph(DP_UPGRADE_WIDTH,DP_UPGRADE_HEIGHT,100,BG_HORIZONTAL);
	mpoUpgrade->vSetXY(DP_UPGRADE_OFF_X, DP_UPGRADE_OFF_Y);
	jRed   = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGr, 0);
	jGreen = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGg, 0); 
	jBlue  = (BYTE)game->mpoTemplate->lGetValue(TAG_UBGb, 214);
	mpoUpgrade->vSetFillColor(jRed,jGreen,jBlue);

	mpoHead = new sciView();
	mpoHead->vSetPlane(plane);
	mpoHead->SetView(4520);
	mpoHead->Init();
	mpoHead->Hide();

	mpoLevel = new sciText();
	mpoLevel->vSetPlane(plane);
	mpoLevel->SetFont(DP_LEVEL_FONT);
	mpoLevel->Init();
	mpoLevel->SetText("00");
	mpoLevel->vSetSize();
	mpoLevel->Posn(left,top);

	mpoCloak = new sciText();
	mpoCloak->vSetPlane(plane);
	mpoCloak->SetFont(DP_LEVEL_FONT);
	mpoCloak->Init();
	mpoCloak->SetText("U");
	mpoCloak->vSetSize();
	mpoCloak->Posn(left,top);

	mpoInvPlane = new InventoryPlane();
	mpoInvPlane->vSetRect(DP_PLANE_OFF_LEFT, DP_PLANE_OFF_TOP, DP_PLANE_OFF_RIGHT, DP_PLANE_OFF_BOTTOM);
	mpoInvPlane->vSetRect1(0, 0, DP_ITEM_WIDTH - 1, DP_ITEM_HEIGHT - 1);
	mpoInvPlane->vSetRect2(0, DP_ITEM_HEIGHT, DP_ITEM_WIDTH - 1, DP_ITEM_HEIGHT * 2 - 1);
	mpoInvPlane->vSetRect3(0, DP_ITEM_HEIGHT * 2, DP_ITEM_WIDTH - 1 , DP_ITEM_HEIGHT * 3 - 1);
	mpoInvPlane->Hide();

}

void DualPortrait::SetPri(int newPri) {
	Portrait::SetPri(newPri);
	assert(mpoHead);
	mpoHead->SetPri(newPri + 5);
	mpoLevel->SetPri(newPri + 10);
	mpoCloak->SetPri(newPri + 10);
}
		
void DualPortrait::Posn(int posnX, int posnY) {
	sciProp::Posn(posnX, posnY);
	mpoLevel->Posn(posnX + DP_LEVEL_X_OFFSET, posnY + DP_LEVEL_Y_OFFSET);
	mpoCloak->Posn(posnX + DP_CLOAK_X_OFFSET, posnY + DP_CLOAK_Y_OFFSET);
	mpoHead->Posn(posnX + DP_HEAD_OFF_X, posnY + DP_HEAD_OFF_Y);
	mpoInvPlane->Posn(posnX + DP_PLANE_OFF_LEFT + plane->GetX(), posnY + DP_PLANE_OFF_TOP + plane->GetY());
}

void DualPortrait::Render(RECT &updateRect)
{
	Portrait::Render(updateRect);
	mpoInvPlane->vMarkUpdate();
}

void DualPortrait::Show(Unit *poUnit) {
	mpoHead->SetView(4520); //TBD - for now
//	mpoHead->SetView(poUnit->lBitmapPortrait());
	Portrait::Show(poUnit);
	mpoHead->Show();
	mpoInvPlane->Show(poUnit);
}

void DualPortrait::Hide() {
	Portrait::Hide();
	mpoHead->Hide();
	mpoInvPlane->Hide();
}

void DualPortrait::vSetStatusLine(StatusLine *poStatusLine)
{
	Portrait::vSetStatusLine(poStatusLine);
	mpoInvPlane->vSetStatusLine(poStatusLine);
}

void DualPortrait::vRefreshInventory()
{
	mpoInvPlane->vShowItems();
}

int DualPortrait::nInventorySize() 
{
	return mpoInvPlane->nNumItems();
}
