#include "os.h"

#include "resman.h"
#include "list.h"
#include "prf.h"
#include "app.h"
#include "util.h"
#include "sound.h"
#include "viewbmp.h"
#include "template.h"

#include <string.h>
#include <stdlib.h>

extern Application *pApp;
extern CHAR4 c4ResourceTypes[];
extern char *szResourceFilename;
extern char cdDriveLetter;
extern int numBytesAllocated;
extern char *resourcePaths[];

ResourceManager::ResourceManager(char *prfName)
{
	PRF *pPRF;
	HANDLE h;
	WIN32_FIND_DATA findFileData;
	Boolean found;
	char str[64];
	int i;
	char path[64];
	mbStop = FALSE;

	if (prfName == NULL) {
		name = NULL;
		i = 0;
		while (resourcePaths[i][0]) {
			strcpy(path, resourcePaths[i]);
			strcat(path, "\\");
			strcat(path, "*.prf");
			h = FindFirstFile(path, &findFileData);
			if (h != INVALID_HANDLE_VALUE) {
				found = TRUE;
				while (found) {
					pPRF = new PRF();
					pPRF->Open(findFileData.cFileName, "rb");	// long filename
					// pPRF->Open(findFileData.cAlternateFileName, "rb");
					// // 8.3 filename
					listOfPRFs.AddToFront(pPRF);
					found = FindNextFile(h, &findFileData);
				}
				FindClose(h);
			}
			++i;
		}
		// Try to find the .PRFs on a CD if a CD was located.
		if (cdDriveLetter) {
			str[0] = cdDriveLetter;
			str[1] = 0;
			strcat(str, ":\\*.prf");
			h = FindFirstFile(str, &findFileData);
			if (h != INVALID_HANDLE_VALUE) {
				found = TRUE;
				while (found) {
					pPRF = new PRF();
					str[0] = cdDriveLetter;
					str[1] = 0;
					strcat(str, ":\\");
					strcat(str, findFileData.cFileName);
					pPRF->Open(str, "rb");	// long filename
					// pPRF->Open(findFileData.cAlternateFileName, "rb");
					// // 8.3 filename
					listOfPRFs.AddToFront(pPRF);
					found = FindNextFile(h, &findFileData);
				}
				FindClose(h);
			}
		}
	} else {
		name = new char[strlen(prfName) + 1];
		strcpy(name, prfName);
		i = 0;
		pPRF = NULL;
		while (resourcePaths[i][0] && !pPRF) {
			strcpy(path, resourcePaths[i]);
			strcat(path, "\\");
			strcat(path, prfName);
			strcat(path, ".prf");
			h = FindFirstFile(path, &findFileData);
			if (h != INVALID_HANDLE_VALUE) {
				pPRF = new PRF();
				pPRF->Open(path, "rb");	// long filename
				FindClose(h);
			}
			++i;
		}
		if (pPRF)
			listOfPRFs.AddToFront(pPRF);
		if (cdDriveLetter) {
			i = 0;
			pPRF = NULL;
			while (resourcePaths[i][0] && !pPRF) {
				path[0] = cdDriveLetter;
				path[1] = 0;
				strcat(path, ":\\");
				strcat(path, resourcePaths[i]);
				strcat(path, "\\");
				strcat(path, prfName);
				strcat(path, ".prf");
				h = FindFirstFile(path, &findFileData);
				if (h != INVALID_HANDLE_VALUE) {
					pPRF = new PRF();
					pPRF->Open(path, "rb");	// long filename
					FindClose(h);
				}
				++i;
			}
			if (pPRF)
				listOfPRFs.AddToFront(pPRF);
		}
	}

	pApp->listOfResourceManagers->AddToEnd(this);
}

ResourceManager::~ResourceManager()
{
	ResourceNode *pNode;
	PRF *pPRF;

	if (name)
		delete[] name;
	pApp->listOfResourceManagers->Remove(this);
	pNode = (ResourceNode *) (listOfResources.GetFirst());
	while (pNode) {
		listOfResources.Remove(pNode);
		delete pNode->resource;
		delete pNode;
		pNode = (ResourceNode *) (listOfResources.GetFirst());
	}
	pPRF = (PRF *) (listOfPRFs.GetFirst());
	while (pPRF) {
		listOfPRFs.Remove(pPRF);
		delete pPRF;
		pPRF = (PRF *) (listOfPRFs.GetFirst());
	}
}

Resource *ResourceManager::LoadResource(RESOURCE_TYPES newType, int newResourceID, Boolean bStop)
{
	mbStop = bStop;
	ResourceNode *pNode = NULL;
	char prompt[64];

	// Try to locate previously loaded resource.
	pNode = FindNode(newType, newResourceID);

	if (pNode == NULL) {
		// Load a new resource.
		pNode = new ResourceNode;
		switch (newType) {
		default:
			pNode->resource = new Resource(newType, newResourceID, this);
			break;
		case RES_BITMAP:
			vDebugOut("Loading Bitmap %d.\n", newResourceID);
			pNode->resource = new ViewBitmap(newResourceID, this);
			if (!pNode->resource->GetData()) {
				delete pNode->resource;
				pNode->resource = new Bitmap(newResourceID, this);
			}
			break;

		case RES_SOUND:
			wsprintf(prompt, "Loading Sound %d.\n", newResourceID);
			DebugString(prompt);
			pNode->resource = new Sound(newResourceID, this);
			if (pNode->resource->GetData() == NULL)
			{
				delete pNode->resource;
				pNode->resource = NULL;
			}
			break;
		case RES_PALETTE:
			pNode->resource = new Palette(newResourceID);
			break;
		case RES_TEMPLATE:
			vDebugOut("Loading Template %d.\n", newResourceID);
			pNode->resource = new Template(newResourceID);
			/*
			if (!pNode->resource->GetSize())
			{
				delete pNode->resource;
				pNode->resource = NULL;
			}*/
			break;
		}
		if (pNode->resource) {
			pNode->usageCount = 1;
			listOfResources.AddToEnd(pNode);
			return pNode->resource;
		} else {
			// wsprintf(prompt, "Cannot find %d of resource type %d.",
			// newResourceID, newType);
			// MessageBox(NULL, prompt, "Error", MB_OK);
			//if (pNode->resource)
			//	delete pNode->resource;
			delete pNode;
			return NULL;
		}
	} else {
		// Increment usage count.
		++pNode->usageCount;
		return pNode->resource;
	}
}

void ResourceManager::FreeResource(Resource * resource)
{
	ResourceNode *pNode;

	// Try to locate previously loaded resource.
	pNode = FindNode(resource);
	if (pNode) {
		if (pNode->usageCount > 0)
			--pNode->usageCount;
	} else {
		DebugString("Freeing an invalid resource!\n");
	}
}

void ResourceManager::PurgeResource(Resource * resource)
{
	ResourceNode *pNode;

	pNode = FindNode(resource);
	if (pNode) {
		listOfResources.Remove(pNode);
		delete pNode->resource;
		delete pNode;
	}
}

void ResourceManager::PurgeResources(int bytesCanUse)
{
	ResourceNode *pNode;
	ResourceNode **ppoNode;

	ppoNode = listOfResources.GetBegin();
	while ((numBytesAllocated > bytesCanUse) && ppoNode != listOfResources.GetEnd()) {
		pNode = *ppoNode;
		if (pNode->usageCount == 0) {
			// Don't put out a message when purging *all* resources - too
			// many messages.
			if (bytesCanUse)
				DebugString("Purging a resource.\n");
			listOfResources.Remove(pNode);
			delete pNode->resource;
			delete pNode;
		} else {
			ppoNode++;
		}
	}
}

ResourceNode *ResourceManager::FindNode(RESOURCE_TYPES newType, int newResourceID)
{
	ResourceNode *pNode;
	ResourceNode **ppoNode;

	ppoNode = listOfResources.GetBegin();
	while (ppoNode != listOfResources.GetEnd()) {
		pNode = *ppoNode++;
		if (pNode->resource->resourceID == newResourceID && pNode->resource->type == newType) {
			return pNode;
		}
	}
	return NULL;
}

ResourceNode *ResourceManager::FindNode(Resource * resource)
{
	ResourceNode *pNode = NULL;
	ResourceNode **ppoNode;

	ppoNode = listOfResources.GetBegin();
	while (ppoNode != listOfResources.GetEnd()) {
		pNode = *ppoNode++;
		if (pNode->resource == resource) {
			return pNode;
		}
	}
	return NULL;
}

void ResourceManager::PreloadResource(RESOURCE_TYPES newType, int newResourceID)
{
	FreeResource(LoadResource(newType, newResourceID));
}

char *ResourceManager::GetName()
{
	return name;
}
