#include "os.h"

#include <stdlib.h>
#include <string.h>

#include <memory.h>
#include <time.h>
#include <io.h>

#include "resource.h"
#include "resman.h"
#include "util.h"
#include "app.h"
#include "unihand.h"
#include "prf.h"
#include "window.h"
#include "dsstream.h"

extern Application *pApp;
extern char *resourcePaths[];
extern char cdDriveLetter;
extern char *gsSaveDir;
extern LPDIRECTSOUNDBUFFER	lpDSBStreamBuffer;
extern LPDIRECTSOUNDNOTIFY lpDirectSoundNotify;

#define DWDSTREAMFILESIZETHRESHOLD 1024*1024

// BOTH OF THESE LISTS nEED TO BE PARALLEL TO THE ENUM IN RESOURCE.H
CHAR4 c4ResourceTypes[] =
{
	MAKEC4ID('\0', '\0', '\0', '\0'),
	MAKEC4ID('B', 'T', 'M', 'P'),
	MAKEC4ID('W', 'A', 'V', 'E'),
	MAKEC4ID('C', 'W', ' ', ' '),
	MAKEC4ID('P', 'A', 'L', ' '),
	MAKEC4ID('M', 'P', ' ', ' '),
	MAKEC4ID('S', 'Y', 'N', 'C'),
	MAKEC4ID('A', 'N', 'M', 'A'),
	MAKEC4ID('F', 'O', 'N', 'T'),
	MAKEC4ID('B', 'T', 'B', 'L'),
	MAKEC4ID('P', 'L', 'Y', 'R'),
	MAKEC4ID('P', 'O', 'P', 'L'),
	MAKEC4ID('B', 'J', 'P', 'L'),
	MAKEC4ID('M', 'S', 'K', '1'),
	MAKEC4ID('P', 'N', 'G', ' '),
	MAKEC4ID('V', 'I', 'E', 'W'),
	MAKEC4ID('M', 'A', 'P', 'O'),
	MAKEC4ID('M', 'A', 'P', ' '),
	MAKEC4ID('T', 'M', 'P', 'L'),
};

char *resourceExtensions[] =
{
	0,
	"BMP",
	"WAV",
	"CW",
	"PAL",
	"MP",
	"RLS",
	"ANM",
	"FNT",
	"BTL",
	"PLY",
	"PLY",
	"PLY",
	"MSK",
	"PNG",
	"TEX",
	"MPO",
	"MAP",
	"TLB",
	"DWD"
};

Resource::Resource(RESOURCE_TYPES newType, int newResourceID, ResourceManager * newResourceManager)
{
	type = newType;
	resourceID = newResourceID;
	numOfBytes = 0;
	data = NULL;
	if (newResourceManager)
		resourceManager = newResourceManager;
	else
		resourceManager = pApp->pResourceManager;

	if (resourceID == 0)		// 0 is an invalid resource number.
		return;

//#ifdef _DEBUG
	// Check for a file resource if not using a custom resource manager.
	if (resourceManager == pApp->pResourceManager)

		ReadFile();
	if (data != NULL)
		return;
//#endif // _DEBUG
	ReadPRF();

	if (data == NULL && resourceManager->bStop())
	{
		char sFilename[512];
		sprintf(sFilename, "Resource %d.%s not found.", resourceID, resourceExtensions[type]);
		MessageBox(NULL, sFilename, "ERROR!", MB_OK | MB_ICONERROR);
		exit(3);
	}
}

Resource::~Resource()
{
	if (data) {
		delete[] data;
	}
}

void Resource::ReadPRF()
{
	PRF *pPRF;
	PRF **ppoPRF;

	numOfBytes = 0;
	data = NULL;
	ppoPRF = resourceManager->listOfPRFs.GetBegin();
	while (ppoPRF != resourceManager->listOfPRFs.GetEnd()) {
		pPRF = *ppoPRF++;
		pPRF->Load(c4ResourceTypes[type], (WORD) resourceID, &data, &numOfBytes);
		if (data) {
			// Resource found. Terminate while() loop.
			break;
		}
	}
}

DWORD fsize(FILE * file)
{
	long oldPosition;
	long newPosition;

	oldPosition = ftell(file);
	fseek(file, 0, SEEK_END);
	newPosition = ftell(file);
	fseek(file, oldPosition, SEEK_SET);
	return newPosition;
}

void Resource::vWriteFile()
{
	FILE *file;
	char sFilename[64];

	//if (!data) {
	//	return;
	//}

	if (resourceManager && resourceManager->GetName()) {
		sprintf(sFilename, "%s\\%s\\%d.%s", gsSaveDir, resourceManager->GetName(),
				resourceID, resourceExtensions[type]);
	} else {
		sprintf(sFilename, "%s\\%d.%s", gsSaveDir,
				resourceID, resourceExtensions[type]);
	}
	file = fopen(sFilename, "wb");

	if (file) {
		vWriteData(file);
		fclose(file);
	}
}

void Resource::vWriteData(FILE *pFile)
{
	// this should be subclassed by apps that want to create
	// their own files. Don't call the superclass; write a new
	// function that writes out the appropriate file format
	if (data) {
		fwrite(data, numOfBytes, 1, pFile);
	}
}


void Resource::ReadFile()
{
	FILE *file;
	char filename[64];
	int i;


	i = 0;
	file = NULL;
	while (resourcePaths[i][0] && !file) {
		if (resourceManager && resourceManager->GetName())
			sprintf(filename, "%s\\%s\\%d.%s", resourcePaths[i], resourceManager->GetName(),
					resourceID, resourceExtensions[type]);
		else
			sprintf(filename, "%s\\%d.%s", resourcePaths[i],
					resourceID, resourceExtensions[type]);
		file = fopen(filename, "rb");
		++i;
	}
	// Don't search the CD for single files.
	/* i = 0; if (cdDriveLetter) { while (!file && resourcePaths[i][0]) {
	   sprintf(filename, "%c:\\%s\\%d.%s", cdDriveLetter,
	   resourcePaths[i], resourceID, resourceExtensions[type]); file =
	   fopen(filename, "rb"); ++i; } } 
	*/
	if (file) {
		numOfBytes = fsize(file);

		if (type == RES_SOUND)
		{
			// IF SIZE > STREAM SIZE THRESHOLD
			if (numOfBytes > DWDSTREAMFILESIZETHRESHOLD)
			{
				fclose(file);

				// THIS IS A STREAMER

				// PREPARE FOR STREAMING
				numOfBytes = 9 + strlen(filename) + 1;
				data = new BYTE[numOfBytes];
				// STORE THE STREAMER TAG
				memcpy(data, "STREAMER\0", 9);
				// STORE THE FILENAME
				memcpy(data + 9, filename, strlen(filename));
				memset(data + numOfBytes - 1, 0, 1);
				
				// "STREAMER\0xxxFILENAMExxx\0"
				return;
			}
			// OTHERWISE FALLTHROUGH
		}

		data = new BYTE[numOfBytes];
		fread(data, numOfBytes, 1, file);
		fclose(file);
	}
}



int Resource::GetSize()
{
	return numOfBytes;
}

BYTE *Resource::GetData()
{
	// Don't allow null pointers.
	if (this == NULL) {
#ifdef _DEBUG
		DebugString("Locking a NULL resource!\n");
#endif // _DEBUG
		return NULL;
	}
	return data;
}

