#include "os.h"
#include "scibtngp.h"

sciRadioButton::sciRadioButton(int newID, Boolean newSelected)
 : sciCheckBox(newSelected)
{
	buttonID = newID;
	buttonGroupList = NULL;
	mpoButtonGroup = NULL;
	mbCanUnselect = FALSE;
}

sciRadioButton::~sciRadioButton()
{
}

Boolean sciRadioButton::HandleEvent(sciEvent &event)
{
	Boolean bWasSelected = IsSelected();
	
	sciCheckBox::HandleEvent(event);

	//if selected button is clicked on we don't want to unselect it unless mbCanUnselect is TRUE
	if(bWasSelected && !IsSelected())
	{
		if (mbCanUnselect) {
			Unselect();
		} else {
			Select();
		}
		UpdateCel();
	}

	//if another button was selected we need to unselect all others
	sciRadioButton *radioButton;
	if(!bWasSelected && IsSelected())
	{
		for(int i = 0; i < buttonGroupList->Size(); i++)
		{
			radioButton = ((sciRadioButton *)(buttonGroupList->At(i)));
			if(radioButton != this && radioButton->IsSelected())
			{
				radioButton->Unselect();
				radioButton->UpdateCel();
			}
		}
	}
	return(event.claimed);
}

int sciRadioButton::GetID()
{
	return(buttonID);
}

void sciRadioButton::SetID(int newID)
{
	buttonID = newID;
}

void sciRadioButton::SetButtonGroupList(RefList<sciRadioButton> *newButtonList)
{
	buttonGroupList = newButtonList;
}
	



sciButtonGroup::sciButtonGroup()
{
	buttonList = NULL;
}
sciButtonGroup::~sciButtonGroup()
{
	if (buttonList) {
		//buttonList->Release();
		delete buttonList;
		buttonList = NULL;
	}
}
Boolean sciButtonGroup::AddToGroup(sciRadioButton *newButton)
{
	if(!buttonList)
		buttonList = new RefList<sciRadioButton>();

	if(!buttonList->Contains(newButton))
	{
		buttonList->AddToBack(newButton);
		newButton->SetButtonGroupList(buttonList);
		newButton->vSetButtonGroup(this);
		newButton->plane = plane;
	}
	return FALSE;
}

sciRadioButton *sciButtonGroup::GetSelectedButton(void)
{
	for(int i = 0; i < buttonList->Size(); i++)
	{
		if(((sciRadioButton *)(buttonList->At(i)))->IsSelected())
			return((sciRadioButton *)(buttonList->At(i)));
	}
	return NULL;
}

int sciButtonGroup::GetSelectedButtonID(void)
{
	for(int i = 0; i < buttonList->Size(); i++)
	{
		if(((sciRadioButton *)(buttonList->At(i)))->IsSelected())
			return(((sciRadioButton *)buttonList->At(i))->GetID());
	}
	return NULL;
}

void sciButtonGroup::SelectButtonByID(int newID)
{
	// Select specified button and unselect others.  If ID is invalid,
	// everything gets unselected.

	for(int i = 0; i < buttonList->Size(); i++)
	{
		if(((sciRadioButton *)buttonList->At(i))->GetID() == newID) {
			((sciRadioButton *)buttonList->At(i))->Select();
			((sciRadioButton *)buttonList->At(i))->DoLeftClick();
		} else {
			((sciRadioButton *)buttonList->At(i))->Unselect();
		}
	}
}

void sciButtonGroup::EnableAll()
{
	for(int i = 0; i < buttonList->Size(); i++)
		((sciRadioButton *)buttonList->At(i))->Enable();
}
void sciButtonGroup::DisableAll()
{
	for(int i = 0; i < buttonList->Size(); i++)
		((sciRadioButton *)buttonList->At(i))->Disable();
}
void sciButtonGroup::UnselectAll()
{
	for(int i = 0; i < buttonList->Size(); i++)
		((sciRadioButton *)buttonList->At(i))->Unselect();
}

void sciButtonGroup::Show()
{
	sciView::Show();
	for(int i = 0; i < buttonList->Size(); i++)
		((sciRadioButton *)(buttonList->At(i)))->Show();
}

void sciButtonGroup::Hide()
{
	for(int i = 0; i < buttonList->Size(); i++)
		((sciRadioButton *)(buttonList->At(i)))->Hide();
	sciView::Hide();
}





sciTab::sciTab(int newID, Boolean newSelected)
 : sciRadioButton(newID, newSelected)
{
	justification = JUST_HORIZONTAL | JUST_VERTICAL;
	featureList = NULL;
}

sciTab::~sciTab()
{
	if (featureList) {
		//featureList->Release();
		delete featureList;
		featureList = NULL;
	}
}

void sciTab::AddToTab(sciFeature *newFeature)
{
	if(!featureList)
		featureList = new RefList<sciFeature>();

	if(!featureList->Contains(newFeature))
	{
		featureList->AddToBack(newFeature);
		newFeature->plane = plane;
	}
}

void sciTab::Select()
{
	SetPri(5);
	sciRadioButton::Select();
	SetFont(2, 3);
	Show();
	if (featureList) {
		for (int i = 0; i < featureList->Size(); ++i) {
			((sciFeature *)featureList->At(i))->Show();
		}
	}
}

void sciTab::Unselect()
{
	SetPri(4 - buttonID);
	sciRadioButton::Unselect();
	SetFont(5, 6);
	Show();
	if (featureList) {
		for (int i = 0; i < featureList->Size(); ++i) {
			((sciFeature *)featureList->At(i))->Hide();
		}
	}
}
