#include "os.h"

#include "app.h"
#include "rc.h"
#include "scichkbx.h"
#include "util.h"
#include "sigs.h"
#include "game.h"
#include "sciroom.h"
#include "sciplane.h"
#include "scicursr.h"
#include "sciscrip.h"

extern Application *pApp;
extern Game *game;
extern SIGS *sigs;

sciCheckBox::sciCheckBox(Boolean newSelected)
{
	if (newSelected) {
		Select();
	} else {
		Unselect();
	}
	//justification = JUST_HORIZONTAL | JUST_VERTICAL; //tw- I dont like this default
	justification = JUST_LEFT | JUST_TOP;
	textOffset = 0;
	SetCheckBoxDefaultCels();
	UpdateCel();
}

sciCheckBox::~sciCheckBox()
{
}

Boolean sciCheckBox::HandleEvent(sciEvent &event)
{
	if (!enabled || hidden || event.claimed)
		return sciButton::HandleEvent(event);

	switch (event.type)
		{
		case MOUSE_UP:
			if (OnMe(event.x, event.y) || mouseDown) {
				if ((event.modifiers & RIGHT_MOUSE) && mouseDown && OnMe(event.x, event.y)) {
					if (selected) {
						Unselect();
					} else {
						Select();
					}
				}
				if ((event.modifiers & LEFT_MOUSE) && mouseDown && OnMe(event.x, event.y)) {
					if (selected) {
						Unselect();
					} else {
						Select();
					}
				}
			}
			break;

		case KEY_PRESS:
			if ((tolower(event.c) == hotkey) || (tolower(event.c) == hotkey2))
			{
				SetState(DEFAULT_BUTTON_DEPRESSED);
				plane->window->ForceUpdate();
				Sleep(100);
				SetState(DEFAULT_BUTTON_NORMAL);
				if (selected) {
					Unselect();
				} else {
					Select();
				}
				event.claimed = TRUE;
				return TRUE;
			}
			else if (tolower(event.c) == rightHotkey)
			{
				SetState(DEFAULT_BUTTON_DEPRESSED);
				plane->window->ForceUpdate();
				Sleep(100);
				SetState(DEFAULT_BUTTON_NORMAL);
				if (selected) {
					Unselect();
				} else {
					Select();
				}
				event.claimed = TRUE;
				return TRUE;
			}
			break;

		}
	
	return(sciButton::HandleEvent(event));
}

void sciCheckBox::Select()
{
	selected = TRUE;
	UpdateCel();
}

void sciCheckBox::Unselect()
{
	selected = FALSE;
	UpdateCel();
}

Boolean sciCheckBox::IsSelected()
{
	return(selected);
}

void sciCheckBox::UpdateCel(void)
{
	if (selected) {
		normal = normalSelected;
		normalFont = normalSelectedFont;

		depressed = depressedSelected;
		depressedFont = depressedSelectedFont;

		disabled = disabledSelected;
		disabledFont = disabledSelectedFont;

		hilighted = hilightedSelected;
		hilightedFont = hilightedSelectedFont;
	} else {
		normal = normalNonSelected;
		normalFont = normalNonSelectedFont;

		depressed = depressedNonSelected;
		depressedFont = depressedNonSelectedFont;

		disabled = disabledNonSelected;
		disabledFont = disabledNonSelectedFont;

		hilighted = hilightedNonSelected;
		hilightedFont = hilightedNonSelectedFont;
	}
	sciButton::UpdateCel();
}

void sciCheckBox::SetCheckBoxCels(int newNormal, int newDepressed, int newDisabled,
 int newHilighted, int newNormalSelected, int newDepressedSelected,
 int newDisabledSelected, int newHilightedSelected)
{
	normalNonSelected = newNormal;
	depressedNonSelected = newDepressed;
	disabledNonSelected = newDisabled;
	hilightedNonSelected = newHilighted;
	normalSelected = newNormalSelected;
	depressedSelected = newDepressedSelected;
	disabledSelected = newDisabledSelected;
	hilightedSelected = newHilightedSelected;

	UpdateCel();
}

// 1st four paramters are when the check box ISN'T selected, the next 4 are for when it is
void sciCheckBox::SetFontCheckBox(int newNormalF, int newDepressedF, int newDisabledF,
	int newHilightedF, int newNormalSelectedF, int newDepressedSelectedF,
	int newDisabledSelectedF, int newHilightedSelectedF) {

	normalNonSelectedFont = newNormalF;
	depressedNonSelectedFont = newDepressedF;
	disabledNonSelectedFont = newDisabledF;
	hilightedNonSelectedFont = newHilightedF;

	normalSelectedFont = newNormalSelectedF;
	depressedSelectedFont = newDepressedSelectedF;
	disabledSelectedFont = newDisabledSelectedF;
	hilightedSelectedFont = newHilightedSelectedF;

	UpdateCel();
}



void sciCheckBox::SetCheckBoxDefaultCels()
{
	normalNonSelected = DEFAULT_BUTTON_NORMAL;
	depressedNonSelected = DEFAULT_BUTTON_DEPRESSED;
	disabledNonSelected = DEFAULT_BUTTON_DISABLED;
	hilightedNonSelected = DEFAULT_BUTTON_HILIGHTED;
	normalSelected = DEFAULT_BUTTON_NORMAL_SELECTED;
	depressedSelected = DEFAULT_BUTTON_DEPRESSED_SELECTED;
	disabledSelected = DEFAULT_BUTTON_DISABLED_SELECTED;
	hilightedSelected = DEFAULT_BUTTON_HILIGHTED_SELECTED;

	UpdateCel();
}

void sciCheckBox::SetTextOffset(int newOffset)
{
	textOffset = newOffset;
	//sciButton::UpdateText();
}
void sciCheckBox::Posn(int x, int y)
{
	sciButton::Posn(x,y);
	if(text)
	{
		text->Posn(x + textOffset, y);			  //TODO: Fix this from happening
	}
}
