#include "os.h"
#include "scilist.h"

sciListNode::sciListNode()
	{
	nextNode = NULL;
	prevNode = NULL;
	object = NULL;
	}

sciListNode::~sciListNode()
	{
	if (object)
		delete object;
	object = NULL;
	nextNode = NULL;
	prevNode = NULL;
	}

sciList::sciList()
	{
	frontNode = NULL;
	backNode = NULL;
	nextNode = NULL;
	size = 0;
	}

sciList::~sciList()
	{
	while (Size())
		{
		Delete(At(0));
		}
	}

void sciList::AddToFront(sciObject *obj)
	{
	sciListNode *newNode;

	newNode = new sciListNode();
	newNode->object = obj;
	newNode->nextNode = frontNode;
	if (frontNode)
		{
		frontNode->prevNode = newNode;
		}
	else
		{
		backNode = newNode;
		}
	frontNode = newNode;
	++size;
	}

void sciList::AddToBack(sciObject *obj)
	{
	sciListNode *newNode;

	newNode = new sciListNode();
	newNode->object = obj;
	newNode->prevNode = backNode;
	if (backNode)
		{
		backNode->nextNode = newNode;
		}
	else
		{
		frontNode = newNode;
		}
	backNode = newNode;
	++size;
	}

void sciList::AddAfter(sciObject *seekObject, sciObject *anObject)
	{
	sciListNode *newNode;
	sciListNode *seekNode;

	seekNode = FindNode(seekObject);
	if (seekNode)
		{
		newNode = new sciListNode();
		newNode->object = anObject;
		newNode->nextNode = seekNode->nextNode;
		seekNode->nextNode = newNode;
		if (newNode->nextNode)
			{
			newNode->nextNode->prevNode = newNode;
			}
		else
			{
			backNode = newNode;
			}
		newNode->prevNode = seekNode;
		++size;
		}
	else
		{
		// Error!!! seekObject not found.
		}
	}

void sciList::Delete(sciObject *delObject)
	{
	sciListNode *seekNode;

	seekNode = FindNode(delObject);
	if (seekNode)
		{
		if (seekNode == frontNode)
			{
			frontNode = seekNode->nextNode;
			}
		if (seekNode == backNode)
			{
			backNode = seekNode->prevNode;
			}
		if (seekNode->prevNode)
			{
			seekNode->prevNode->nextNode = seekNode->nextNode;
			}
		if (seekNode->nextNode)
			{
			seekNode->nextNode->prevNode = seekNode->prevNode;
			}
		seekNode->object = NULL;
		delete seekNode;
		--size;
		}
	else
		{
		// Error!!! delObject not found.
		}
	}

int sciList::Size()
	{
	return size;
	}

Boolean sciList::IsEmpty()
	{
	if (Size() == 0)
		return TRUE;
	else
		return FALSE;
	}

Boolean sciList::Contains(sciObject *obj)
	{
	sciListNode *seekNode;

	seekNode = FindNode(obj);
	if (seekNode)
		return TRUE;
	else
		return FALSE;
	}

void sciList::Release()
	{
	sciListNode *tempNode;

	while (Size())
		{
		// Move through the list deleting nodes but not deleting the objects
		// that the node points to.
		// Leave the next/prev points a mess - they'll soon be all gone so
		// no need to keep them correct.
		tempNode = frontNode->nextNode;
		frontNode->object = NULL;
		delete frontNode;
		frontNode = tempNode;
		--size;
		}
	backNode = NULL;
	}

/*
sciObject *sciList::At(int i)
	{
	int currentPosition;
	sciListNode *currentNode;

	currentPosition = 0;
	currentNode = frontNode;
	while (currentPosition != i)
		{
		if (currentNode)
			{
			currentNode = currentNode->nextNode;
			}
		++currentPosition;
		}
	if (currentNode)
		return currentNode->object;
	else
		return NULL;
	}

void sciList::Set(int i, sciObject *obj)
	{
	int currentPosition;
	sciListNode *currentNode;

	// If the list is too small, make it bigger.
	while (i >= Size())
		{
		AddToBack(NULL);
		}

	currentPosition = 0;
	currentNode = frontNode;
	while (currentPosition != i)
		{
		currentNode = currentNode->nextNode;
		++currentPosition;
		}
	currentNode->object = obj;
	}
*/

sciListNode *sciList::FindNode(sciObject *obj)
	{
	sciListNode *currentNode;

	currentNode = frontNode;
	while (currentNode && (currentNode->object != obj))
		{
		currentNode = currentNode->nextNode;
		}
	if (currentNode)
		return currentNode;
	else
		return NULL;
	}
