#include "os.h"
#include "app.h"
#include "palette.h"
#include "sciplane.h"
#include "owindow.h"
//#include "scilist.h"
#include "scicursr.h"
#include "sciscrip.h"
#include "scigame.h"
#include "scilist.h"
#include "scicycle.h"
#include "sciroom.h"
#include "bitmap.h"
#include "resman.h"
#include "viewbmp.h"
#include "game.h"


sciPlane::sciPlane(Window * parentWindow, Boolean bModal, int nPriority)
{
	pBackgroundBitmap = NULL;
	pBackgroundColor = NULL;

	mbModal = bModal;
	if (parentWindow == NULL) {
		window = new OtherWindow;
		window->MakeWindow();
		isNotHidden = FALSE;
		game->curRoom->AddPlane(this);
		itsMyWindow = TRUE;
	} else {
		window = parentWindow;
		isNotHidden = TRUE;
		itsMyWindow = FALSE;
	}
	picNumber = -1;
	mnPriority = nPriority;

	window->vAddPlane(this);
	// window->plane = this;
	top = window->rcInner.top;
	left = window->rcInner.left;
	bottom = window->rcInner.bottom;
	right = window->rcInner.right;

	vSetRect(window->rcInner);

	cursor = new sciCursor();
	cursor->SetHotspot(1, 0);
	cursor->SetView(1);
	cursor->plane = this;
	cursor->Init();
	cursor->SetCycle(new sciForward);
	if ((parentWindow == NULL) && bModal) {
		cursor->Hide();
		// cursor->Show();
	} else {
		cursor->Hide();
	}
	cursor->Hide(TRUE);
	DeleteFromCast(cursor);
}

sciPlane::~sciPlane()
{
	if (IsModal() && (game->curRoom->GetModalPlane() == this))
		game->curRoom->SetModalPlane(NULL);
	game->curRoom->RemovePlane(this);
	if (window) {
		window->vDeletePlane(this);
	}
	delete cursor;
	while (cast.Size()) {
		delete cast.At(0);
	}
	while (props.Size()) {
		delete props.At(0);
	}
	if (itsMyWindow) {
		delete window;
	}
	if (pBackgroundBitmap) {
		delete pBackgroundBitmap;
		pBackgroundBitmap = NULL;
	}
	if (pBackgroundColor) {
		delete pBackgroundColor;
		pBackgroundColor = NULL;
	}

}

void sciPlane::AddToCast(sciFeature * feature)
{
	if (!cast.Contains(feature)) {
		cast.AddToFront(feature);
		feature->cast = &cast;
	}
}

void sciPlane::DeleteFromCast(sciFeature * feature)
{
	if (cast.Contains(feature)) {
		cast.Delete(feature);
		feature->cast = NULL;
	}
}

void sciPlane::AddToProps(sciProp * prop)
{
	if (!props.Contains(prop)) {
		props.AddToFront(prop);
	}
}

void sciPlane::DeleteFromProps(sciProp * prop)
{
	if (props.Contains(prop)) {
		props.Delete(prop);
	}
}

Boolean sciPlane::HandleEvent(sciEvent & event)
{
	sciFeature *tmpFeature;
	int i;

	if (!isNotHidden)
		return FALSE;

	if (event.type == MOUSE_MOVE) {
		if (cursor)
			cursor->HandleEvent(event);
	}
	if (event.claimed) {
		return (TRUE);
	} else {
		for (i = 0; i < cast.Size(); ++i) {
			tmpFeature = (sciFeature *) (cast.At(i));
			if (tmpFeature && tmpFeature->HandleEvent(event)) {
				return TRUE;
			}
		}
		return FALSE;
	}
}

void sciPlane::Doit()
{
	sciProp *tmpProp;
	int i;
	
	if (isNotHidden)
	{
		for (i = 0; i < props.Size(); ++i) {
			tmpProp = (sciProp *) (props.At(i));
			if (tmpProp)
				tmpProp->Doit();
		}
	}
}

void sciPlane::Render(RECT & updateRect)
{
	sciFeature *tmpFeature;
	int i;

	if (isNotHidden)
	{

		// clip to the plane, and if plane
		// is not in update area, don't draw
		RECT rClipped = updateRect;
		if (!IntersectRect(&rClipped, &rClipped, &mrClip)) {
			return;
		}

		// bg bitmap/color drawing
		if (pBackgroundBitmap != NULL) {
/*			RECT my;
			RECT clippedUpdateRect;

			clippedUpdateRect = mrClip;
			vLocalToGlobal(&clippedUpdateRect);

			IntersectRect(&clippedUpdateRect, &clippedUpdateRect, &updateRect);
			my.left = clippedUpdateRect.left - mrClip.left;
			my.top = clippedUpdateRect.top - mrClip.top;
			my.right = my.left + (clippedUpdateRect.right - clippedUpdateRect.left);
			my.bottom = my.top + (clippedUpdateRect.bottom - clippedUpdateRect.top);
			pBackgroundBitmap->CopyPixels(window->pWorkBitmap, clippedUpdateRect, my, TRUE);
*/			
			pBackgroundBitmap->vDraw(0,0,this);

		} 
		else {
			if (pBackgroundColor != NULL) {
				pBackgroundColor->vDraw(0,0,this);
			}
		}

		for (i = cast.Size() - 1; i >= 0; --i) {
			tmpFeature = (sciFeature *) (cast.At(i));
			if (tmpFeature) {
				tmpFeature->Render(updateRect);
			}
		}
		cursor->Render(updateRect);

	} //isNotHidden
}

void sciPlane::Show()
{
//	window->Show();	TW - if you need the window to be shown(why would you) then make a separate call
	isNotHidden = TRUE;
	if (IsModal())
		game->curRoom->SetModalPlane(this);
}

void sciPlane::Hide()
{
//	window->Hide();//	window->Show();	TW - if you need the window to be hidden(why would you) then make a separate call
	isNotHidden = FALSE;
	if (IsModal() && (game->curRoom->GetModalPlane() == this))
		game->curRoom->SetModalPlane(NULL);
}

void sciPlane::SetPic(int newPicNumber)
{
	if (picNumber != newPicNumber) {
		picNumber = newPicNumber;
		LoadPic();
	}
}

void sciPlane::LoadPic()
{
	//RECT rect;
	RECT rUpdate;
/*	BOOL activatePalette = FASLE;
	HDC hDC;
	Palette *newPalette;

	if (!itsMyWindow) {
		newPalette = (Palette *) pApp->pResourceManager->LoadResource(RES_PALETTE, picNumber);
		if (newPalette->GetData()) {
			// New palette found and loaded.
			if (pApp->pCurrentPalette)
				pApp->pResourceManager->FreeResource(pApp->pCurrentPalette);
			pApp->pCurrentPalette = newPalette;
			pApp->pCurrentPalette->ActivatePalette();
			activatePalette = TRUE;
		} else {
			// Palette doesn't exist, keep old one.
			pApp->pResourceManager->FreeResource(newPalette);
			activatePalette = TRUE;
		}
	} else {
		hDC = GetDC(window->hWnd);
		if (pApp->pCurrentPalette)
			pApp->pCurrentPalette->ActivatePalette(hDC);
		ReleaseDC(window->hWnd, hDC);
		activatePalette = FALSE;
	}
*/
	//if (window->pBackgroundBitmap)
	//	delete window->pBackgroundBitmap;
	if (pBackgroundBitmap) {
		delete pBackgroundBitmap;
	}
	pBackgroundBitmap = new ViewBitmap(picNumber, NULL);//, activatePalette); TW - commented out

	if (itsMyWindow) {
		MoveWindow(window->hWnd, window->x, window->y,
				   pBackgroundBitmap->Width(),
				   pBackgroundBitmap->Height(),
				   TRUE);
		left = window->x;
		top = window->y;
		right = left + pBackgroundBitmap->Width();
		bottom = top + pBackgroundBitmap->Height();
		window->rcInner.left = 0;
		window->rcInner.top = 0;
		window->rcInner.right = pBackgroundBitmap->Width();
		window->rcInner.bottom = pBackgroundBitmap->Height();
		if (window->pWorkBitmap)
			delete window->pWorkBitmap;
		window->pWorkBitmap = new Bitmap((WORD) (window->rcInner.right - window->rcInner.left), (WORD) (window->rcInner.bottom - window->rcInner.top));
	}
	//rect.top = 0;
	//rect.left = 0;
	//rect.right = pBackgroundBitmap->Width();
	//rect.bottom = pBackgroundBitmap->Height();
	//window->Update(rect);
	// mark the old position for updating
	SetRect(&rUpdate, left, top, right, bottom);
	window->Update(rUpdate);
}

void sciPlane::vSetBackgroundColor(BYTE jColor) {
	if (pBackgroundColor) {
		delete pBackgroundColor;
	}
	
	pBackgroundColor = new Bitmap((WORD)(mrClip.right - mrClip.left), (WORD)(mrClip.bottom - mrClip.top));
	RECT rNew;
	SetRect (&rNew, 0,0, mrClip.right - mrClip.left, mrClip.bottom - mrClip.top);
	pBackgroundColor->vErase8(rNew, jColor);
}

void sciPlane::vSetBackgroundColor(BYTE jRed, BYTE jGreen, BYTE jBlue)
{
	if (pBackgroundColor) {
		delete pBackgroundColor;
	}
	
	pBackgroundColor = new Bitmap((WORD)(mrClip.right - mrClip.left), (WORD)(mrClip.bottom - mrClip.top), 16);
	RECT rNew;
	SetRect (&rNew, 0,0, mrClip.right - mrClip.left, mrClip.bottom - mrClip.top);
	pBackgroundColor->vErase16(rNew, jRed, jGreen, jBlue);
}

void sciPlane::Posn(int posnX, int posnY, BOOL updateWindow)
{
	int width,
	 height;

	width = right - left;
	height = bottom - top;

	vSetRect(posnX, posnY, posnX + width, posnY + height);

	//window->x = left;
	//window->y = top;
	//if (updateWindow) {
		//MoveWindow(window->hWnd, left, top, width, height, TRUE);
		//if (window != pApp->pWindow)
			//UpdateWindow(pApp->pWindow->hWnd);
		//if (itsMyWindow && isNotHidden)
			//pApp->pWindow->ForceUpdate();
	//}
}

int sciPlane::GetX()
{
	return left;
}

int sciPlane::GetY()
{
	return top;
}

int sciPlane::GetCursorX() {
	POINT pt;
	if (GetCursorPos(&pt)) {
		ScreenToClient(pApp->pWindow->hWnd, &pt);
		return (pt.x - left);	//localize it
	} else {
		//an error occured in GetCursorPos
		return 0;
	}
}

int sciPlane::GetCursorY() {
	POINT pt;
	if (GetCursorPos(&pt)) {
		ScreenToClient(pApp->pWindow->hWnd, &pt);
		return (pt.y - top);	//localize it
	} else {
		//an error occured in GetCursorPos
		return 0;
	}
}


Boolean sciPlane::IsModal()
{
	return mbModal;
}

void sciPlane::vSetPri(int nPriority)
{
	if (window && nPriority != mnPriority) {
		RECT rUpdate;

		window->vDeletePlane(this);
		mnPriority = nPriority;
		window->vAddPlane(this);

		// mark the new area for updating
		SetRect(&rUpdate, left, top, right, bottom);
		window->Update(rUpdate);
	}
}

void sciPlane::vSetRect(int nLeft, int nTop, int nRight, int nBottom)
{
	//RECT rUpdate;

	// mark the old area for updating
	//SetRect(&rUpdate, left, top, right, bottom);
	window->Update(mrClip);

	top = nTop;
	bottom = nBottom;
	left = nLeft;
	right = nRight;

	mnWidth = right - left;
	mnHeight = bottom - top;

	// mark the new area for updating
	//SetRect(&rUpdate, left, top, right, bottom);
	vCalcClip();
	window->Update(mrClip);
};

void sciPlane::vCalcClip()
{
	RECT rSelf, rWindow;

	SetRect(&rSelf, left, top, right, bottom);
	SetRect(&rWindow, 0, 0, window->Width(), window->Height());
	IntersectRect(&mrClip, &rSelf, &rWindow);
}

Boolean sciPlane::OnMeLocalized(int onMeX, int onMeY)
{
	onMeX -= left;
	onMeY -= top;
	return OnMe(onMeX, onMeY);
}

Boolean sciPlane::OnMeGlobalized(int onMeX, int onMeY)
{
	onMeX += left;
	onMeY += top;
	return OnMe(onMeX, onMeY);
}

Boolean sciPlane::OnMe(int onMeX, int onMeY)
{
	if (isNotHidden &&
		((onMeX >= left) && (onMeX <= right)) &&
		((onMeY >= top) && (onMeY <= bottom))
		)
		return (TRUE);
	else
		return (FALSE);
}
