#include "os.h"

#include "scirail.h"
#include "app.h"
#include "resman.h"
#include "util.h"
#include "scigame.h"
#include "sciprop.h"
#include "scisound.h"
#include "sounder.h"
#include "game.h"

extern Application *pApp;

#include "string.h"

sciRailCycle::sciRailCycle(sciScript *newCaller)
	: sciCycle(newCaller)
	{
	itsStartTime = 0;
	itsNumQueuedSyncs = 0;
	itsRestImageNumber = 0;
	itsCurrentSyncNumber = 0;
	iCurrentFrame = 0;
	itsHead = 0;
	itsTail = 0;
	itsPreviousSyncTime = 0;
	pCurrentSyncData = NULL;
	bStartSyncImmediately = FALSE;
	bIStartedASound = FALSE;
	sound = NULL;
	}

sciRailCycle::~sciRailCycle()
	{
	//StopSyncing();
	while (itsNumQueuedSyncs)
		{
		--itsNumQueuedSyncs;
		delete[] itsDirectSyncData[itsHead];
		itsDirectSyncData[itsHead] = NULL;
		itsHead = (itsHead+1) % (int)kSyncQueueLength;
		}
	}

void sciRailCycle::AppendSync(int iResourceID)
	{
	Resource *pResource;

	pResource = client->resourceManager->LoadResource(RES_SYNC, iResourceID);
	AppendSync((RAIL_SYNC *)(pResource->GetData()));
	client->resourceManager->FreeResource(pResource);
	}

void sciRailCycle::AppendSync(int iSyncID, int iSubSyncNumber)
	{
	Resource *pResource;

	pResource = client->resourceManager->LoadResource(RES_SYNC, iSyncID);
	AppendSync(GetSync((WORD *)(pResource->GetData()), iSubSyncNumber));
	client->resourceManager->FreeResource(pResource);
	}

void sciRailCycle::AppendRandomSync(int iSyncID)
	{
	Resource *pResource;
	WORD *pRailSyncGroup;

	pResource = client->resourceManager->LoadResource(RES_SYNC, iSyncID);
	if (pResource)
		{
		pRailSyncGroup = (WORD *)(pResource->GetData());
		if (pRailSyncGroup)
			AppendSync(GetSync(pRailSyncGroup, random(*pRailSyncGroup)));
		//else
		//	DebugString("NULL sync.\n");
		}
	//else
	//	DebugString("Sync not found.\n");
	client->resourceManager->FreeResource(pResource);
	}

RAIL_SYNC *sciRailCycle::GetSync(WORD *pData, int iIndex)
	{
	WORD	far *theRailSyncPtr;
	WORD	theNumSyncFrames;

	if (pData == NULL || iIndex >= *pData || iIndex < 0)
		return NULL;

	#define kSyncFrameSize 2		// 2 WORD
	#define kRailSyncHeader 2		// 2 WORD

	theRailSyncPtr = ((WORD far *)pData)+1;

	while ((iIndex--) > 0) {
		theRailSyncPtr += kRailSyncHeader;
		theNumSyncFrames = *(theRailSyncPtr++);
		theRailSyncPtr += (theNumSyncFrames*kSyncFrameSize);
		}
	return ((RAIL_SYNC *)theRailSyncPtr);
	}

void sciRailCycle::AppendSync(RAIL_SYNC *pTheSyncData)
	{
	int sLength;
	Boolean bStartSync;

	if (!pTheSyncData)
		{
		//DebugString("Appending a NULL sync.\n");
		return;
		}
	
	// If no syncs are currently syncing, start this one after its been appended.
	bStartSync = !IsSyncing();

	itsNumQueuedSyncs++;
	if (pTheSyncData->wNumSyncFrames)
		{
		sLength = (int)(sizeof(RAIL_SYNC) + (sizeof(RAIL_SYNC_FRAME) * (pTheSyncData->wNumSyncFrames-1)));
		itsDirectSyncData[itsTail] = (RAIL_SYNC *)new BYTE[sLength];
		memcpy(itsDirectSyncData[itsTail], pTheSyncData, sLength);
		}
	else
		{
		// Special case empty syncs.
		sLength = (int)(sizeof(RAIL_SYNC) - sizeof(RAIL_SYNC_FRAME));
		itsDirectSyncData[itsTail] = (RAIL_SYNC *)new BYTE[sizeof(RAIL_SYNC)];
		memcpy(itsDirectSyncData[itsTail], pTheSyncData, sLength);
		itsDirectSyncData[itsTail]->rsFrames[0].wFrameNumber = 0;
		itsDirectSyncData[itsTail]->rsFrames[0].wDuration = 0;
		}

	itsTail = (itsTail+1) % (int)kSyncQueueLength;

	if (bStartSync && bStartSyncImmediately)
		BeginSync();
	}

void sciRailCycle::BeginSync()
	{
	pCurrentSyncData = itsDirectSyncData[itsHead];
	if (pCurrentSyncData->wNewRestImage != kRestImageNumber)
		SetRestImage(pCurrentSyncData->wNewRestImage);

	if ((pCurrentSyncData->wSoundID > 1) && (pCurrentSyncData->wSoundID < 65535))
		{
		if (sound)
			sound->Play(pCurrentSyncData->wSoundID);
		else
			game->sound1->Play(pCurrentSyncData->wSoundID);
		bIStartedASound = TRUE;
		}
	else
		{
		//DebugString("Trying to sync to sound 0, 1, or -1.\n");
		}
	itsStartTime = game->GetTrueTime();
	itsPreviousSyncTime = itsStartTime - 1;
	itsCurrentSyncNumber = 0;
	iCurrentFrame = pCurrentSyncData->rsFrames[itsCurrentSyncNumber].wFrameNumber;
	if (iCurrentFrame == -1)
		iCurrentFrame = itsRestImageNumber;
	client->SetCel(iCurrentFrame);
	}

void sciRailCycle::EndSync()
	{
	if (!IsSyncing())
		return;
//	if (pCurrentSyncData && pCurrentSyncData->wSoundID)
//		pApp->pSounder->EndSound();
	iCurrentFrame = itsRestImageNumber;
	client->SetCel(iCurrentFrame);
	itsNumQueuedSyncs--;
	delete[] itsDirectSyncData[itsHead];
	itsDirectSyncData[itsHead] = NULL;
	itsHead = (itsHead+1) % (int)kSyncQueueLength;
	itsStartTime = 0;
	pCurrentSyncData = NULL;
	bIStartedASound = FALSE;

	if (itsNumQueuedSyncs == 0)
		{
		}
	}

void sciRailCycle::Doit()
	{
	Boolean bFinished;
	Boolean bNeedsUpdating;               
	int dwTickCount;
	int sCurrentDuration;

	bFinished = FALSE;
	bNeedsUpdating = FALSE;
	if ((itsStartTime == 0) && IsSyncing())
		BeginSync();
	if (itsStartTime > 0)
		{
		// Advance sync number until it is current.
		dwTickCount = game->GetTrueTime();
		if (pCurrentSyncData->wNumSyncFrames)
			sCurrentDuration = pCurrentSyncData->rsFrames[itsCurrentSyncNumber].wDuration;
		else
			sCurrentDuration = 1;
		while (!bFinished &&
					dwTickCount > itsPreviousSyncTime + sCurrentDuration)
			{
			bNeedsUpdating = TRUE;
			++itsCurrentSyncNumber;
			if (itsCurrentSyncNumber >= pCurrentSyncData->wNumSyncFrames)
				{
				if (!bIStartedASound || (bIStartedASound && !pApp->pSounder->IsSoundPlaying()))
					{
					bFinished = TRUE;
					EndSync();
					if (IsSyncing())
						BeginSync();
					}
				else
					{
					itsCurrentSyncNumber = pCurrentSyncData->wNumSyncFrames - 1;
					if (itsCurrentSyncNumber < 0)
						itsCurrentSyncNumber = 0;
					itsPreviousSyncTime += sCurrentDuration;
					sCurrentDuration = 1;
					}
				}
			else
				{
				itsPreviousSyncTime += sCurrentDuration;
//				itsPreviousSyncTime = dwTickCount;
				if (pCurrentSyncData->wNumSyncFrames)
					sCurrentDuration = pCurrentSyncData->rsFrames[itsCurrentSyncNumber].wDuration;
				else
					sCurrentDuration = 1;
				}
			}
		if (bNeedsUpdating)
			{
			if (pCurrentSyncData)
				{
				iCurrentFrame = pCurrentSyncData->rsFrames[itsCurrentSyncNumber].wFrameNumber;
				if (iCurrentFrame == -1)
					iCurrentFrame = itsRestImageNumber;
				client->SetCel(iCurrentFrame);	// ???added during port - needed???
				}
			//Refresh();
			}
		}
	if (!IsSyncing())
		CycleDone();
	}

Boolean sciRailCycle::IsSyncing()
	{
 	return (itsNumQueuedSyncs > 0) || (bIStartedASound && pApp->pSounder->IsSoundPlaying());
	}

Boolean sciRailCycle::IsSyncQueueFull()
	{
 	return (itsNumQueuedSyncs == kSyncQueueLength);
	}

void sciRailCycle::SetRestImage(int theRestImage)
	{
 	itsRestImageNumber = theRestImage;
	}

void sciRailCycle::StopSyncing()
	{
	if (IsSyncing())
		{
		if (bIStartedASound)
			{
			if (sound)
				sound->Stop();
			else
				game->sound1->Stop();
			}
		bIStartedASound = FALSE;
		while (IsSyncing())
			EndSync();
		CycleDone();
		}
	}

void sciRailCycle::SetSound(sciSound *newSound)
	{
	sound = newSound;
	}
