#include "os.h"
#include "app.h"
#include "palette.h"
#include "bitmap.h"
#include "sciroom.h"
#include "sciplane.h"
#include "sciscrip.h"
#include "scicursr.h"
#include "util.h"

extern Application *pApp;
extern int maxMemoryAllocated;

sciRoom::sciRoom()
{
	plane = NULL;
	script = NULL;
	SetModalPlane(NULL);
	maxMemoryAllocated = 0;
}

sciRoom::~sciRoom()
{
	char str[65];

	wsprintf(str, "Max Memory used = %d.\n", maxMemoryAllocated);
	DebugString(str);

	if (plane) {
		delete plane;
	}
	while (otherPlanes.Size()) {
		plane = (sciPlane *) otherPlanes.At(0);
		delete plane;
	}
	plane = NULL;
	SetScript(NULL);
}

void sciRoom::Init()
{
	plane = new sciPlane(pApp->pWindow);
	SetPic(picNumber);

#if 0
	RECT destRect,
	 srcRect;
	Bitmap *tempBMP;

	destRect.top = 97;
	destRect.left = 224;
	destRect.right = 427;
	destRect.bottom = 321;

	srcRect.top = 0;
	srcRect.left = 0;
	srcRect.right = 203;
	srcRect.bottom = 224;

	tempBMP = new Bitmap(8060);
	tempBMP->CopyPixels(pApp->pWindow, destRect, srcRect);
	delete tempBMP;
#endif
}

Boolean sciRoom::HandleEvent(sciEvent & event)
{
#if 0
	int i;

	if (!event.claimed && event.plane) {
		if ((event.type == KEY_PRESS) && (!modalPlane || (modalPlane != event.plane))) {
			// Pass key presses to all planes.
			// Pass it to the main plane first, then to the otherPlanes.
			if (plane)
				plane->HandleEvent(event);
			i = 0;
			while (i < otherPlanes.Size()) {
				((sciPlane *) otherPlanes.At(i))->HandleEvent(event);
				++i;
			}
		} else {
			// Pass key presses to the plane, or the modal, if there is
			// one.
			if (modalPlane) {
				event.plane = modalPlane;
				modalPlane->HandleEvent(event);
			} else
				event.plane->HandleEvent(event);
		}
	}
#endif
	return event.claimed;

}

void sciRoom::SetScript(sciScript * newScript)
{
	if (script)
		delete script;
	script = newScript;
	if (script)
		script->Init(NULL);
}

void sciRoom::Doit()
{
	//int i;

	if (script) {
		script->Doit();
	}
#if 0
	if (modalPlane) {
		modalPlane->Doit();
		return;
	}

	if (plane)
		plane->Doit();

	i = 0;
	while (i < otherPlanes.Size()) {
		((sciPlane *) otherPlanes.At(i))->Doit();
		++i;
	}
#endif
}

void sciRoom::SetPic(int newPicNumber)
{
	picNumber = newPicNumber;
	if (plane)
		plane->SetPic(newPicNumber);
}

void sciRoom::AddPlane(sciPlane * newPlane)
{
	otherPlanes.AddToBack(newPlane);
	if (newPlane->IsModal()) {
		SetModalPlane(newPlane);
	}
}

void sciRoom::RemovePlane(sciPlane * poPlane)
{
	if (plane == poPlane) {
		plane = NULL;
	}
	otherPlanes.Delete(poPlane);
	if (modalPlane == poPlane) {
		SetModalPlane(NULL);
	}
}

void sciRoom::SetModalPlane(sciPlane * newPlane)
{
	modalPlane = newPlane;
	if (modalPlane)
		plane->cursor->Hide();
	else if (plane && plane->cursor)
		plane->cursor->Show();
}

sciPlane *sciRoom::GetModalPlane()
{
	return modalPlane;
}
