// SIGSwrap.h : SIGS session API wrapper object
// Written by: Brick Baldwin

#ifndef __SIGSWRAP_H__
#define __SIGSWRAP_H__

#include "SierraNWAPI.h"
#include "SNTypes.h"
#include "SNWValidateSDK.h"

#define MAX_SIGS_MSG_SIZE 512
#define MAX_GAME_OP_SIZE 100

/////////////////////////////////////////////////////////////////////////////
// ASIGSSession wrapper object
class ASIGSSession
{
public:
    // Construction/creation/destruction
    ASIGSSession();
    VALIDATE_DLL_EXIT       Create                  (HWND hwndGameApp     = NULL,
                                                     LPSTR pszInstallPath = NULL,
                                                     LPSTR pszRestartCmd  = NULL,
                                                     LPSTR pszServerName  = NULL);
    BOOL                    NewGame                 ();
    ~ASIGSSession();

    // Startup
    BOOL                    IsConnected             ();
    BOOL                    IsGameReady             ();
    BOOL                    IsCaptain               ();

    // Shutdown
    BOOL                    Close                   ();
    void                    ExitGame                ();

    // Status
    int                     GetNumberOfPlayers      ();
    int                     GetMyNumber             ();
    BOOL                    GetMyName               (LPSTR pszName,
                                                     UINT  nSize);
    BOOL                    GetGameOption           (char *cOption,
                                                     char *cVal,
                                                     UINT  nSize);

    // Yer basic I/O
    // ReceiveMessage : Returns 0 if no message available, else returns
    //                  the size of the message received.  cMsg points
    //                  to the buffer where the message is to be placed,
    //                  nMaxSize is the buffer size, and pnID is filled
    //                  with the player ID of the message sender.
    int                     ReceiveMessage          (char *cMsg,
                                                     UINT  nMaxSize,
                                                     UINT *pnID);
    void                    TellMeMsg               (MESS_RECEIVED_PROC my_function);
    BOOL                    SendMessageAll          (char *cMsg,
                                                     int   iSize);
    BOOL                    SendPlayerMessage       (int   iPlayer,
                                                     char *cMsg,
                                                     int   iSize);
    void                    FreeMessageBuffer       ();
    // Get the ping time to the game server to determine latency
    int                     PingGameServer          ();

    // Set a callback to receive special chat notifications
    void                    HookSpecialChat         (SPECIAL_CHAT_PROC pfn);

    // Set a callback to receive messages when players leave
    void                    HookPlayerLeave         (PLAYER_STATUS_PROC);

    // Flags
    BOOL                    m_bGameInProgress;
    BOOL                    m_bSIGSConnected;
    BOOL                    m_bShuttingDown;
    BOOL                    m_bHost;

// Private functions
private:
    VALIDATE_DLL_EXIT       Open                    ();
    BOOL                    InitFunctionPointers    ();
    BOOL                    EndThread               (HANDLE *hdThread);

    // The SIGS monitoring thread
    static HRESULT          SIGSThread              ();

    // Callback
    static void             MsgRcv                  (char *Msg, sMessInfo MsgInfo);

    // Mild hack to allow the thread and callback access to "this"
    static ASIGSSession    *m_pThis;

// Private data
private:
    HANDLE                  m_hSIGSThrd;
    HWND                    m_hwndGameApp;
    LPSTR                   m_pszServerName;
    LPSTR                   m_pszInstallPath;
    LPSTR                   m_pszRestartCmd;
    HINSTANCE               m_hndDLL; // Used to load unload the SIGS dll's

    // SIGS function pointers (DLL ProcAddresses)
    lpfnSNWValidateFunc               lpfnSNWValidate;
    lpfnInitializeDLLFunc             lpfnInitializeDLL;
    lpfnCloseDLLFunc                  lpfnCloseDll;
    lpfnCheckForAnyTCPMessageFunc     lpfnCheckForAnyTCPMessage;
    lpfnPeekForTCPMessageFunc         lpfnPeekForTCPMessage;
    lpfnRecvTCPMessageFunc            lpfnRecvTCPMessage;
    lpfnCheckGameConnectStatusFunc    lpfnCheckGameConnectStatus;
    lpfnGameCloseFunc                 lpfnGameClose;
    lpfnGetNumberOfPlayersFunc        lpfnGetNumberOfPlayers;
    lpfnSendTCPMessageFunc            lpfnSendTCPMessage;
    lpfnSendTCPPointMessageFunc       lpfnSendTCPPointMessage;
    lpfnGetPlayerXNameFunc            lpfnGetPlayerXName;
    lpfnGetMyPlayerNumberFunc         lpfnGetMyPlayerNumber;
    lpfnGetGameOptionFunc             lpfnGetGameOption;
    lpfnTellMeGameMessReceivedFunc    lpfnTellMeGameMessReceived;
    lpfnFreeGameMessageBufferFunc     lpfnFreeGameMessageBuffer;
    lpfnPingTimeFunc                  lpfnPingTime;
    lpfnTellMeSpecialChatReceivedFunc lpfnTellMeSpecialChatReceived;
    lpfnTellMePlayerLeftGameFunc      lpfnTellMePlayerLeftGame;
};

#endif
