#include "os.h"

#include "sound.h"
#include "SoundDwd.h"
#include "app.h"
#include "resman.h"
#include "unihand.h"
#include "util.h"
#include "dwdheader.h"

extern Application *pApp;
extern BYTE btSoundBitSupport;
extern int globalAvailVoices;
extern CRITICAL_SECTION cs1;

SoundDwd::SoundDwd(int nResourceID, ResourceManager *resourceManager)
	: Resource(RES_SOUNDDWD, nResourceID, resourceManager)
{
	mpnSoundNum = NULL;
	mnPriority = SOUNDPRIORITY_UNITSOUNDS;
	loop = FALSE;
	mhFile = NULL;
	mhDwd = NULL;
	mpDwdStream = NULL;
	mhEventAlert = NULL;
	mhEventStopped = NULL;
	mdwLengthOfDataSection = 0;
	pSamples = NULL;
	if (!data)
	{
		mbStreaming = FALSE;

		Sound * soundWAV = (Sound *)new Resource(RES_SOUND, nResourceID, resourceManager);

		if (soundWAV->GetData()) 
		{
			// convert to DWD
			ULONG ulSize = (ULONG)soundWAV->GetSize();
			numOfBytes = (int)ulSize;

			int err = dws_WAV2DWD(soundWAV->GetData(), &ulSize, NULL);

			//data = new BYTE[nLengthOfDwd];
			pSamples = new UniHandle(ulSize);
			BYTE * pjDwdData = (BYTE *) pSamples->Lock();
			err = dws_WAV2DWD(soundWAV->GetData(), &ulSize, pjDwdData);
			wSamplesPerSec = *(WORD*)(pjDwdData + 32);
			pSamples->Unlock();
		}
		else
		{
			numOfBytes = 0;
		}
		delete soundWAV;
	}
	else
	{
		// the first buffer from the DWD has been read into data

		// maintain a pointer into the file
		// play the sound and preload the back buffer ahead of the playing
		// section.
		if ((*(DWORD*)data) == DWDSTREAMTAG)
		{
			mhFile = *(HANDLE*)(data+sizeof(DWORD));

			mbStreaming = TRUE;
			
			mhDwd = CreateFileMapping(mhFile, NULL, PAGE_READWRITE, 0L, 0L, NULL);
			
			mpDwdStream = (BYTE *)MapViewOfFile(mhDwd, FILE_MAP_ALL_ACCESS, 0L, 0L, 0L);
			
			dws_DWDHEADER * pdHeader = (dws_DWDHEADER *) mpDwdStream;

			wSamplesPerSec = *(WORD *)pdHeader->wSamplingRate;

			mhEventAlert = CreateEvent(NULL, TRUE, FALSE, NULL);
/*
			mhEventStopped = CreateEvent(NULL, TRUE, FALSE, NULL);

			BYTE * pReadData = data + sizeof(DWORD) + sizeof(FILE *);
			dws_DWDHEADER * pdHeader = (dws_DWDHEADER *) pReadData;
			

			mdwLengthOfDataSection = *(DWORD *)pdHeader->dwLengthOfDataSection;
			wSamplesPerSec = *(WORD *)pdHeader->wSamplingRate;
			munRemainingSampleSize = *(DWORD *)(pReadData + sizeof(dws_DWDHEADER));

			mulBufferSize = (ULONG)GetSize() - (sizeof(DWORD) + sizeof(FILE *));

			pSamples = new UniHandle(2 * mulBufferSize);

			// get the main buffer and load the first sample
			//  from data (loaded in Resource())
			mpjBuffer = (BYTE *) pSamples->Lock();
			memcpy(mpjBuffer, pReadData, mulBufferSize);

			// get the back buffer
			mpjBackBuffer = mpjBuffer + mulBufferSize;

			// init the back buffer
			memcpy(mpjBackBuffer, mpjBuffer, sizeof(dws_DWDHEADER));

			// load the back buffer
			// LoadBackBuffer();
*/			
		}
		else
		{
			mbStreaming = FALSE;
			// copy data
			ULONG ulSize = (ULONG)GetSize();
			pSamples = new UniHandle(ulSize);
			BYTE * pjDwdData = (BYTE *) pSamples->Lock();
			memcpy(pjDwdData, data, ulSize);
			dws_DWDHEADER * pdHeader = (dws_DWDHEADER *) data;
			wSamplesPerSec = *(WORD *)pdHeader->wSamplingRate;
			pSamples->Unlock();
		}

		delete[] data;
		data = NULL;
	}
}

SoundDwd::~SoundDwd() {
		if (pSamples)
		{
			delete pSamples;
			pSamples = NULL;
		}
		if (mhEventAlert)
		{
			CloseHandle(mhEventAlert);
		}
	}


void SoundDwd::CleanUp() 
{
	if (mbStreaming)
	{
		if (mpDwdStream) 
		{
			UnmapViewOfFile(mpDwdStream);
			mpDwdStream = NULL;
		}
		if (mhDwd) 
		{
			CloseHandle(mhDwd);
			mhDwd = NULL;
		}
		if (mhFile)
		{
			CloseHandle(mhFile);
			mhFile = NULL;
		}
		mbStreaming = FALSE;
		vDebugOut("Clean up called for streamed sound.\n");
	}
}

void SoundDwd::Free()
	{
    CleanUp();
	resourceManager->FreeResource(this);
	}

BYTE *SoundDwd::GetData()
	{
	return (BYTE*)data;
	}

void SoundDwd::ThreadAlert(WORD soundnum, SoundManagerDiamondWare * soundman)
{
	mnSoundNum = soundnum;
	//smgr = soundman;
	SetEvent(mhEventAlert);
}

void SoundDwd::MainBufferStarted(WORD soundnum, SoundManagerDiamondWare * soundman)
{
	vDebugOut("MainBufferStarted %d\n", soundnum);
	mnSoundNum = soundnum;
	// now load back buffer
	if (LoadBackBuffer())
	{
		soundman->Continue(soundnum, this);

		ToggleBackBuffer();
	}
}

void SoundDwd::ToggleBackBuffer()
{
//	vDebugOut("ToggleBackBuffer\n");
	// toggle mpjBackBuffer
	if (mpjBackBuffer != mpjBuffer)
	{
		mpjBackBuffer = mpjBuffer;
	}
	else
	{
		mpjBackBuffer = mpjBuffer + mulBufferSize;
	}
}


BOOL SoundDwd::LoadBackBuffer()
{
	// loads sound into back buffer

	// DETERMINE LOCATION IN THE STREAM
//	fpos_t pos;
//	fgetpos(mpFile, &pos);

//	int nRemainingSampleSize = mulTotalSampleSize - (int)pos;

	int nBytesToRead;
	
	if (munRemainingSampleSize == 0)
	{
		//fseek(mpFile, 0, SEEK_END);
		//munRemainingSampleSize  = ftell(mpFile);
		//fseek(mpFile, 56, SEEK_SET);
	}

	if (munRemainingSampleSize > 0)
	{
		// DETERMINE LENGTH TO READ
		nBytesToRead = min(DWDSOUNDBUFFERSIZE, munRemainingSampleSize);
		
		// READ THE SAMPLES FROM THE FILE INTO BACKBUFFER
		dws_DPLAY dp1;

		do
		{
			dp1.flags = dws_dplay_SND;
			dws_DGetInfo(&dp1);
		} while (dp1.snd == mpjBackBuffer);
		
		//fread(mpjBackBuffer + sizeof(dws_DWDHEADER), nBytesToRead, 1, mpFile);
//		int lVal = ftell(mpFile);
		//vDebugOut("streamed more at %d\n", lVal);
		munRemainingSampleSize = munRemainingSampleSize - nBytesToRead;
	}
	else
	{
		// TBD - SHUT DOWN STREAMING OR LOOP -- NO MORE SAMPLES
		mbStreaming = FALSE;
		return FALSE;
	}

	// FIXUP THE HEADER FOR THE BACKBUFFER
	dws_DWDHEADER * pHeader = (dws_DWDHEADER * ) mpjBackBuffer;
	*(DWORD *)pHeader->dwLengthOfDataSection = nBytesToRead;
	*(DWORD *)pHeader->dwSamples = (nBytesToRead / (pHeader->jBitsPerSample >> 3)) / pHeader->jChannels;
	return TRUE;
}

/*void SoundDwd::Stream()
{
	// swaps the backbuffer with the playing sound buffer
	BYTE * pTemp = mpBackBufferSound;
	BYTE * pTemp = mpPlayingBuffer;

	pjDwdData = (BYTE *) pSamples->Lock();	 

	// load the backbuffer
	LoadBackBuffer();
}
*/
ULONG __stdcall ThreadProcPlayStreamedSound( LPVOID pStreamedSound )
{
	// RECEIVES SOUNDDWD POINTER
	SoundDwd * soundDwd = (SoundDwd *) pStreamedSound;

	// BEGIN PLAYING SOUND ******************************
	dws_DPLAY dplay;

	BYTE * pjSoundBuffer = (BYTE *) soundDwd->pSamples->Lock();	

	dplay.snd = pjSoundBuffer;
	dplay.flags = dws_dplay_SND | dws_dplay_PRIORITY;
	dplay.priority = 32000;
	if (soundDwd->wSamplesPerSec != DWS_PREFERRED_OUTPUTRATE)
	{
		dplay.pitch = (WORD)((double)dws_IDENTITY / ((double)soundDwd->wSamplesPerSec / (double)DWS_PREFERRED_OUTPUTRATE));
		dplay.flags |= dws_dplay_PITCH;
	}

	// begin playing the sound

	
	dws_DPlay(&dplay);  // TBD handle error
	//pApp->pSoundManager->AddToPlayList(soundDwd, dplay.soundnum);

	if (soundDwd->IsStreaming())
	{
		while (soundDwd->LoadBackBuffer())
		{
	//		vDebugOut("Streaming \n");
			dplay.snd = soundDwd->GetBackBuffer();
			dplay.presnd = dplay.soundnum;
			dplay.flags = dws_dplay_SND | dws_dplay_PRESND;
			
			vDebugOut("Played more\n");
			dws_DPlay(&dplay);

			soundDwd->ToggleBackBuffer();
			//vDebugOut("Waiting \n");
			//WaitForSingleObject(soundDwd->mhEventAlert, INFINITE);
			//ResetEvent(soundDwd->mhEventAlert);
			WaitForSingleObject(soundDwd->mhEventStopped, INFINITE);
			ResetEvent(soundDwd->mhEventStopped);

		}
		//vDebugOut("FINISHED Streaming \n");
	}

	return 0;
}


ULONG  __stdcall HackPlay( LPVOID pStreamedSound )
{
	
	EnterCriticalSection(&cs1);
	if (globalAvailVoices > 0)
	{
		// DECREMENT AVAILABLE VOICE COUNT
		globalAvailVoices--;
		LeaveCriticalSection(&cs1);
		
		// START PLAYING SOUND
		SoundDwd * soundDwd = (SoundDwd *) pStreamedSound;

		// BEGIN PLAYING SOUND ******************************
		dws_DPLAY dplay;

		BYTE * pjSoundBuffer = (BYTE *) soundDwd->pSamples->Lock();

		dplay.snd = pjSoundBuffer;

		dplay.flags = dws_dplay_SND| dws_dplay_PRIORITY;
		dplay.priority = soundDwd->mnPriority;
		if (soundDwd->loop == FALSE)
		{
			dplay.count = 1;
		}
		else
		{
			dplay.count = 0;
		}
		dplay.flags |= dws_dplay_COUNT | dws_dplay_LVOL | dws_dplay_RVOL;
		dplay.lvol |= dws_IDENTITY;
		dplay.rvol |= dws_IDENTITY;

		//dplay.synchronous = TRUE;
		if (soundDwd->wSamplesPerSec != DWS_PREFERRED_OUTPUTRATE)
		{
			dplay.pitch = (WORD)((double)dws_IDENTITY / ((double)soundDwd->wSamplesPerSec / (double)DWS_PREFERRED_OUTPUTRATE));
			dplay.flags |= dws_dplay_PITCH;
		}

		// begin playing the sound
		dws_DPlay(&dplay);  // TBD handle error
		int sndnum = dplay.soundnum;
		if (soundDwd->mpnSoundNum)
		{
			*(soundDwd->mpnSoundNum) = dplay.soundnum;
		}
		dplay.flags = dws_dplay_SOUNDNUM;
		BOOL bContinue = TRUE;
		while (bContinue)
		{
			EnterCriticalSection(&cs1);
			if (globalAvailVoices < -900)
			{
				LeaveCriticalSection(&cs1);
				break;
			}
			LeaveCriticalSection(&cs1);
			if (!dws_DGetInfo(&dplay))
			{
				
				bContinue = FALSE;
			}
			else
			{
				
				if (dplay.soundnum == 0)
				{
					bContinue = FALSE;
				}
				Sleep(120);
			}
		}
		//dws_DDiscard(sndnum);
		
		soundDwd->pSamples->Unlock();
		Sleep(350);
		// LOOP UNTIL SOUND IS FINISHED
		// 
		// INCREMENT AVAILABLE VOICE COUNT
		EnterCriticalSection(&cs1);
		globalAvailVoices++;
		LeaveCriticalSection(&cs1);
	}
	else
		LeaveCriticalSection(&cs1);


	return 0;
}

void funcPlayCallBack(DWORD msg, DWORD soundnum, DWORD event)
{
	switch (event)
	{
		case dws_event_SOUNDSTARTED:
			break;
		case dws_event_SOUNDCOMPLETE:
		case dws_event_SOUNDABORTED:
			EnterCriticalSection(&cs1);
			globalAvailVoices++;
			LeaveCriticalSection(&cs1);
			break;
	}		
}

ULONG __stdcall ThreadProcPlayStreamedSound2( LPVOID pStreamedSound )
{
	// RECEIVES SOUNDDWD POINTER
	SoundDwd * soundDwd = (SoundDwd *) pStreamedSound;

	// BEGIN PLAYING SOUND ******************************
	dws_DPLAY dplay;

	BYTE * pjSoundBuffer = (BYTE *) soundDwd->mpDwdStream;	

	dplay.snd = pjSoundBuffer;

	dplay.flags = dws_dplay_SND| dws_dplay_PRIORITY;
	dplay.priority = SOUNDPRIORITY_BACKGROUNDMUSIC;
	dplay.count = 0;
	dplay.flags |= dws_dplay_COUNT | dws_dplay_LVOL | dws_dplay_RVOL;
	dplay.lvol |= dws_IDENTITY;
	dplay.rvol |= dws_IDENTITY;

	//dplay.synchronous = TRUE;
	if (soundDwd->wSamplesPerSec != DWS_PREFERRED_OUTPUTRATE)
	{
		dplay.pitch = (WORD)((double)dws_IDENTITY / ((double)soundDwd->wSamplesPerSec / (double)DWS_PREFERRED_OUTPUTRATE));
		dplay.flags |= dws_dplay_PITCH;
	}

	// begin playing the sound

	dws_DPlay(&dplay);  // TBD handle error

	//pApp->pSoundManager->AddToPlayList(soundDwd, dplay.soundnum);
	SetEvent(soundDwd->mhEventAlert);
	return 0;
}
