// STATUS.CPP -- the status indicator for friendly or marked enemy vampires
//
// TW

#define LEVEL_FONT 2	//the font number for the level display

#include "status.h"
#include "unit.h"
#include "bitmap.h"
#include "scitext.h"
#include "texture.h"
#include "bargraph.h"

extern Palette *spoPal;

#define T_PAD			1	// top padding
#define B_PAD			3	// bottom padding
#define L_PAD			2	// left padding
#define R_PAD			3	// right padding

#define PAD				2	// pixel padding between level and bargraph
#define BG_W			8	// bargraph width
#define BG_H			2	// bargraph height

void StatusIndicator::vSetup(Unit *poUnit, sciPlane *poPlane) {
	assert (poUnit);
	assert(poPlane);
	plane = poPlane;
	mpoUnit = poUnit;
	mbHidden = TRUE;
	mnDisplayLevel = poUnit->nLevel();
	mnDisplayHealth = 0;

	poUnit->GetStatusIndicatorOffset(&mnXOffset, &mnYOffset);

	mpoLevel = new sciText();
	mpoLevel->Posn(0,0);
	mpoLevel->vSetPlane(poPlane);	
	char str[2];
	sprintf(str, "%d", poUnit->nLevel());
	mpoLevel->SetText(str);
	mpoLevel->vSetSize();
	mpoLevel->Init();

	int nCharWidth  = mpoLevel->nCharWidth('5');
	int nCharHeight = mpoLevel->nCharHeight('5');

	mpoLevelI = NULL;
	if (game->poLocalPlayer() == poUnit->poOwner())
	{
		int nBitID = game->mpoTemplate->lGetValue(TAG_Bsll, 0);
		assert(nBitID);
		if (nBitID) {
			mpoLevelI = (Bitmap*)pApp->pResourceManager->LoadResource(RES_BITMAP, nBitID);
		}
	} else { //enemy unit so use different bitmap
		int nBitID = game->mpoTemplate->lGetValue(TAG_Bsle, 0);
		assert(nBitID);
		if (nBitID) {
			mpoLevelI = (Bitmap*)pApp->pResourceManager->LoadResource(RES_BITMAP, nBitID);
		}
	}

	mpoGraph = new BarGraph(BG_W, BG_H,100);

	mpoHealth = new StatusIndicatorBG (nCharWidth + L_PAD + R_PAD, nCharHeight + T_PAD + B_PAD);
	vFillHealth(100);

	Init();
//	SetWidth(nCharWidth + L_PAD + R_PAD);
//	SetHeight(nCharHeight + T_PAD + B_PAD);
	if (mpoLevelI)
	{
		SetWidth(BG_W);
		SetHeight(mpoLevelI->Height() + PAD + BG_H);

	} else {
		SetWidth(BG_W);
		SetHeight(BG_H);
	}
}

StatusIndicator::~StatusIndicator() {
	mpoUnit = NULL;

	delete mpoHealth;
	mpoHealth = NULL;

	if (mpoGraph) {
		delete mpoGraph;
		mpoGraph = NULL;
	}
	
	if (mpoLevelI) {
		pApp->pResourceManager->FreeResource(mpoLevelI);
		mpoLevelI= NULL;
	}

}

// Parameters nX/nY are the coords of the node the unit currently is in
void StatusIndicator::vDraw(int nX, int nY) {
	if (!mbHidden) {
		//draw at uppper right of bitmap minus half of bitmap mpoHealth
		int nXPos = nX + mnXOffset;
		int nYPos = nY + mnYOffset;
		Posn(nXPos, nYPos);
//		mpoHealth->vDraw(nXPos, nYPos, plane);
//		mpoLevel->Posn(nXPos + L_PAD, nYPos + T_PAD);
//		mpoLevel->vDraw();
		
		if (mpoLevelI) 
		{
			mpoLevelI->SetCel(mpoUnit->nLevel() - 1);
			mpoLevelI->vDraw(nXPos, nYPos, plane);
			mpoGraph->vDraw(nXPos, nYPos + mpoLevelI->Height() + PAD, plane);
		} else {
			mpoGraph->vDraw(nXPos, nYPos, plane);
		}

			
//		RECT rUpdate;
//		rUpdate.left = nXPos;
//		rUpdate.top = nYPos;
//		rUpdate.right = nXPos + mpoHealth->Width();
//		rUpdate.bottom = nYPos + mpoHealth->Height();
//		pApp->pWindow->Update(rUpdate);
		//vDebugOut ("x: %d, y:%d\n", nX, nY);
	}
}

void StatusIndicator::vFillHealth(short nHealth) {
	RECT rDestRect;
	rDestRect.left = 0;
	rDestRect.top = 0;
	rDestRect.right = mpoHealth->Width();
	rDestRect.bottom = mpoHealth->Height();

/*	BYTE jRed = jColor(0, 255, nHealth);
	BYTE jGreen = jColor(255, 0, nHealth);
	BYTE jBlue = jColor(0,0,nHealth);
	vDebugOut ("Red:%d, Green:%d, Blue:%d, Health:%d\n", jRed, jGreen, jBlue, nHealth);
*/

	BYTE jRed = jColor(mpoUnit->mjFullRed, mpoUnit->mjEmptyRed, nHealth);
	BYTE jGreen = jColor(mpoUnit->mjFullGreen, mpoUnit->mjEmptyGreen, nHealth);
	BYTE jBlue = jColor(mpoUnit->mjFullBlue, mpoUnit->mjEmptyBlue, nHealth);

//	vDebugOut ("Red:%d, Green:%d, Blue:%d, Health:%d\n", jRed, jGreen, jBlue, nHealth);
	
	if (mpoHealth->nBits() == 8)
	{
		BYTE jClr = PaletteFindNearest(spoPal->colors, jRed, jGreen, jBlue);
		mpoHealth->vErase8(rDestRect, jClr);
	} else {
		mpoHealth->vErase16(rDestRect, jRed, jGreen, jBlue);
	}
	mpoHealth->vRoundCorners();
	mpoGraph->vSetFillColor(jRed, jGreen, jBlue);
	mpoGraph->vFill(nHealth);
}

//Calculate the correct status color depending on current health and the full/empty values
BYTE StatusIndicator::jColor(BYTE jFull, BYTE jEmpty, short nHealth) {
/*	if (jFull == jEmpty) {
		return jFull;
	} else if (jFull > jEmpty) {
		return (BYTE)( (jFull - jEmpty) * ((float)nHealth / 100) );
	} else { //jFull is less then jEmpty
		return (BYTE)( ((jEmpty - jFull) * ((100 - (float)nHealth) / 100)) + jFull );
	}
*/
	if (nHealth == 100)
		return jFull;

	if (nHealth > 50)
	{
		//if (jFull > jEmpty) {
		//	return (BYTE)( (jFull - jEmpty) * (((float)nHealth * 2 - 100) / 100) );
		//} else if (jEmpty > jFull) {
		if (jEmpty > jFull) {
			return (BYTE)( jEmpty - ((jEmpty - jFull) * (((float)nHealth * 2 - 100) / 100)) );
		} else {
			return jFull;
		}
	}
	else
	{
		if (jFull > jEmpty) {
			return (BYTE)( (jFull - jEmpty) * (((float)nHealth * 2) / 100) );
		 //if (jEmpty > jFull) {
		//	return (BYTE)( (jEmpty - jFull) * (((float)nHealth * 2) / 100) );
		} else {
			return jEmpty;
		}

	}
}


void StatusIndicator::vHide() {
	sciFeature::Hide();
	mbHidden = TRUE;
//	assert(mpoLevel);
//	mpoLevel->Hide();
}

void StatusIndicator::vShow() {
	sciFeature::Show();
	mbHidden = FALSE;
//	assert(mpoLevel);
//	mpoLevel->Show();
}

void StatusIndicator::vUpdateHealth(short nHealth) {
	if (nHealth >= 0 && nHealth <= 100) {
		if (nHealth != mnDisplayHealth) {
			mnDisplayHealth = nHealth;
			vFillHealth(mnDisplayHealth);
			mpoUnit->vMarkUpdate();
		}
	}
}

/*
void StatusIndicator::vUpdateLevel(short nLevel) {
	if (nLevel >= 0 && nLevel <= 100) {
		if (nLevel != mnDisplayLevel) {
//			vDebugOut("prev lev:%d new:%d\n",mnDisplayLevel, nLevel);
			mnDisplayLevel = nLevel;

			char str[2];
			sprintf(str, "%d", nLevel);
			
			//need to resize everything
			mpoLevel->SetText(str);
			mpoLevel->vSetSize();
//			delete mpoHealth;
//			mpoHealth = new StatusIndicatorBG (mpoLevel->nWidth() + L_PAD + R_PAD, mpoLevel->nHeight() + T_PAD + B_PAD);
			vFillHealth(mnDisplayHealth);
			mpoUnit->vMarkUpdate();
		}
	}
}
*/

Boolean StatusIndicator::HandleEvent(sciEvent &oEvent) 
{
	if (hidden)
		return FALSE;

	if ( (oEvent.type == MOUSE_DOWN) && OnMe(oEvent.x, oEvent.y) )
	{

		if (oEvent.modifiers & SHIFT_DOWN) 
		{
			if (mpoUnit->bIsSelected()) {
				gpoMap->vRemoveSelection(mpoUnit);
			} 
			else 
			{
				gpoMap->vAddSelection(mpoUnit);
			}
			gpoMap->vCommitSelection();

			// invalidate my area
			mpoUnit->vMarkUpdate();
			oEvent.claimed = TRUE;

		} 
		else if (oEvent.modifiers & LEFT_MOUSE) 
		{
			// just make a single selection
			gpoMap->vClearAllSelections();
			gpoMap->vAddSelection(mpoUnit);
			gpoMap->vCommitSelection();

			if (mpoUnit->poOwner() == game->poLocalPlayer()) 
			{
				mpoUnit->vPlayRandomSound(TAG_Syes);
			}
	
			// invalidate my area
			mpoUnit->vMarkUpdate();
			oEvent.claimed = TRUE;
		}
	}

	return oEvent.claimed;
}


StatusIndicatorBG::StatusIndicatorBG(WORD wNewWidth, WORD wNewHeight)
 : Bitmap(wNewWidth, wNewHeight,16)
{
}

void StatusIndicatorBG::vRoundCorners()
{
	if (mpPixels == 0)
		return;
	
	WORD wTransparent = 0;
/*	BYTE *pixels = mpPixels;

	*pixels = wTransparent;								//upper left
	*(pixels + Width() - 1) = wTransparent;				//upper right

	int nBotRow = (Height() - 1) * wWidthInBytes;
	*(pixels + nBotRow) = wTransparent;					//lower left
	*(pixels + nBotRow + Width() - 1) = wTransparent;	//lower right
*/
	POINT pt;
	
	pt.y = 0;
	pt.x = 0;
	vSetPixel(pt, wTransparent);					//upper left
	
	pt.x = Width() - 1;
	vSetPixel(pt, wTransparent);					//upper right

	pt.y = Height() - 1;
	pt.x = 0;
	vSetPixel(pt, wTransparent);					//lower left
	
	pt.x = Width() - 1;
	vSetPixel(pt, wTransparent);					//lower right

}