// TEMPLATE.CPP -- Templates for game objects
//
// DJM

#include "template.h"
#include "util.h"
#include <stdio.h>
#include <string.h>

#define MAX_TEMPLATE_LINE 1023

typedef enum _READ_MODE {
	M_NONE,
	M_VALUES,
	M_LISTS,
	M_STRINGS,
} READ_MODE;

// multiplier for 20.12 fixed point
#define FIXED_MULTIPLIER 4096

Template::Template(int nID)
:Resource(RES_TEMPLATE, nID, NULL)
{
	mpValueTags = NULL;
	mpListTags = NULL;
	mpStringTags = NULL;
	mplLists = NULL;
	mpchStrings = NULL;

	mnValueTags = 0;
	mnListTags = 0;
	mnStringTags = 0;

	if (!data) {
		vScanTextFile();
		if (!data) {
			vDebugOut("Failed to load template %d.\n", resourceID);
		}
	}

	if (data) {
		vScanData();
	}
}

long Template::lGetValue(long lTag, long lDefault)
{
	int i;
	for (i = 0; i < mnValueTags; ++i) {
		if (mpValueTags[i].ulTag == (ULONG)lTag) {
			return mpValueTags[i].lData / FIXED_MULTIPLIER;
		}
	}
	return lDefault;
}

double Template::fGetValue(long lTag, double fDefault)
{
	int i;
	for (i = 0; i < mnValueTags; ++i) {
		if (mpValueTags[i].ulTag == (ULONG)lTag) {
			return ((double)mpValueTags[i].lData) / FIXED_MULTIPLIER;
		}
	}
	return fDefault;
}

long *Template::plGetList(long lTag, long *plSize)
{
	int i;
	for (i = 0; i < mnListTags; ++i) {
		if (mpListTags[i].ulTag == (ULONG)lTag) {
			long *plList = mplLists + mpListTags[i].lData;

			*plSize = *plList;
			plList++;
			return plList;
		}
	}

	*plSize = 0;
	return NULL;
}

const char *Template::strGetString(long lTag, char *strDefault)
{
	int i;
	for (i = 0; i < mnStringTags; ++i) {
		if (mpStringTags[i].ulTag == (ULONG)lTag) {
			char *strTemp = mpchStrings + mpStringTags[i].lData;

			return strTemp;
		}
	}

	return strDefault;
}

void Template::vReload() {
	//for now just reload the text file
	vScanTextFile();
}


//
// Template::vScanTextFile
//
// This is a huge, unwieldy function that parses a file in something
// resembling INI file format and packs it into our binary format.
// It's not pretty, but string parsing code seldom is. There you go.
// DJM
//
// There should be 3 sections in this order: values, lists, and strings.
// The format for [values]:
//		[4 char tag]=[integer]
//		examples:
//		hlth=100
//		powr=75
//
// The format for [lists]:
//		[4 char tag]=[list of integers separated by commas]
//		examples:
//		Sdth=1000,1001,1002,1003
//
// The format for [strings]:
//		[4 char tag]=[string, optionally in quotes]
//		example:
//		name="Baital"
//
void Template::vScanTextFile(void)
{
	FILE *file;
	char str[MAX_TEMPLATE_LINE + 1];
	unsigned long ulTag;
	long lListPos = 0;
	long lStringPos = 0;
	long lCurTag = 0;
	long lListBeginPos = 0;
	long lValueTagsBegin = 0;
	long lListTagsBegin = 0;
	long lStringTagsBegin = 0;
	long lListSize = 0;
	long lData;
	double fData;
	long lFileSize;
	TAG_PAIR *pTags;
	DWORD *pulLists;
	char *pchStrings;
	char *strString;
	BYTE *pjBuffer;
	long	lTotalSize;

	char strFile[256];

	READ_MODE eMode = M_NONE;

	// open the file for reading
	// 
	sprintf(strFile, "%d.tlt", resourceID);
	file = fopen(strFile, "rt");
	if (!file) {
		goto err1;
	}

	// get the size for a quick estimate of 
	// number of entries
	fseek(file, 0, SEEK_END);
	lFileSize = ftell(file);
	fseek(file, 0, SEEK_SET);

	// initialize the data to WAY more than we'll need
	pTags = new TAG_PAIR[lFileSize / 6];
	if (!pTags) {
		goto err2;
	}
	pulLists = new DWORD[lFileSize / 2];
	if (!pulLists) {
		goto err3;
	}
	pchStrings = new char[lFileSize];
	if (!pchStrings) {
		goto err4;
	}

	// walk through the file a line at a time
	while (fgets(str, MAX_TEMPLATE_LINE, file)) {
		if (str[0] == ';')  {
			// comment
			continue;
		}

		// check for change of section
		if (strnicmp(str, "[values]", 8) == 0) {
			if (eMode != M_NONE) {
				// error
				vDebugOut("Found [values] twice or out of order in template %d.\n", resourceID);
				goto fixup;
			}
			eMode = M_VALUES;
			continue;

		} else if (strnicmp(str, "[lists]", 7) == 0) {
			if (eMode == M_STRINGS || eMode == M_LISTS) {
				// error
				vDebugOut("Found [lists] twice or out of order in template %d.\n", resourceID);
				goto fixup;
			}
			eMode = M_LISTS;
			lListTagsBegin = lCurTag;
			continue;

		} else if (strnicmp(str, "[strings]", 9) == 0) {
			if (eMode == M_STRINGS) {
				// error
				vDebugOut("Found [strings] twice or out of order in template %d.\n", resourceID);
				goto fixup;
			}
			eMode = M_STRINGS;
			lStringTagsBegin = lCurTag;
			continue;

		}

		// if there's no equal sign, it's not an entry and we skip it.
		strString = strchr(str, '=');
		if (!strString) {
			continue;
		}

		// split it at the equal sign
		strString[0] = '\0';
		strString++;

		if (eMode == M_NONE) {
			vDebugOut("tag %s found before any section headers in template %d.\n", str, resourceID);
			continue;
		}
	
		// now str points at the tag, and strString, the rest.
		{
			/*
			int i;
			ulTag = 0;
			BOOL bOK = TRUE;
			for (i = 0; i < 4; ++i) {
				unsigned char ch = str[i];
				if (!isalnum(ch)) {
					bOK = FALSE;
					break;
				}
				ulTag = ulTag << 8 | ch;
			}

			if (!bOK) {
				continue;
			}
			*/
			char *strTemp = str;
			while (strTemp && isspace(strTemp[0])) {
				++strTemp;
			}

			if (strlen(strTemp) < 4) {
				vDebugOut("Tag %s too short in template %d.\n", strTemp, resourceID);
				continue;
			}
			ulTag = MAKEC4IDSTR(strTemp);
		}

		pTags[lCurTag].ulTag = ulTag;

		switch (eMode) {
		case M_VALUES:
			//lData = 0;

			// values are now stored as fixed point.
			fData = 0.0;
			sscanf(strString, " %lf", &fData);
			lData = (long)(fData * FIXED_MULTIPLIER);
			pTags[lCurTag].lData = lData;
			++lCurTag;
			break;

		case M_LISTS:
			{
				lListSize = 0;
				lData = 0;

				// save where we're putting the list
				lListBeginPos = lListPos;
				++lListPos;

				int nFields = sscanf(strString, " %d", &lData);
				while (nFields && strString && strString[0]) {
					++lListSize;
					pulLists[lListPos] = lData;
					++lListPos;
					strString = strchr(strString, ',');
					if (strString) {
						++strString;
						nFields = sscanf(strString, " %d", &lData);
					}
				}
				pulLists[lListBeginPos] = lListSize;
				pTags[lCurTag].lData = lListBeginPos;

				++lCurTag;
				break;
			}

		case M_STRINGS:

			{
				long lStrLen = strlen(strString);

				// deal with space/returns at end
				while (lStrLen && isspace(strString[lStrLen - 1])) {
					--lStrLen;
					strString[lStrLen] = '\0';
				}
				// and at beginning
				while (isspace(strString[0])) {
					--lStrLen;
					++strString;
				}

				// if there are quotes, strip them off
				if (strString[0] == '\"') {
					--lStrLen;
					++strString;
				}
				if (strString[lStrLen - 1] == '\"') {
					--lStrLen;
					strString[lStrLen] = '\0';
				}

				strcpy(&pchStrings[lStringPos], strString);
				pTags[lCurTag].lData = lStringPos;
				lStringPos += lStrLen + 1;

				++lCurTag;
				break;
			}
		} // switch
	}

fixup:

	// fill in the fields properly

	// calculate how much space we need
	//
	// which is:	the header, with offsets to the sections 
	//					(in bytes from end of header)
	//				size of the tag pairs
	//				total size of list storage
	//				total size of string storage
	lTotalSize = sizeof(TEMPLATE_HEAD) + lCurTag * sizeof(TAG_PAIR) +
		lListPos * sizeof(long) + lStringPos;
	//numOfBytes = lTotalSize;

	pjBuffer = new BYTE[lTotalSize];
	if (!pjBuffer) {
		goto err5;
	}

	{
		if (data) {
			delete [] data;
		}
		data = pjBuffer;

		TEMPLATE_HEAD *pHead = (TEMPLATE_HEAD*)pjBuffer;
		TAG_PAIR *pTagPtr = (TAG_PAIR*)(pjBuffer + sizeof(TEMPLATE_HEAD));
		long *pListPtr = (long*)((char*)pTagPtr + lCurTag * sizeof(TAG_PAIR));
		char *pStringPtr = ((char*)pListPtr + lListPos * sizeof(long));

		// fix any positions that are unset (because there were no list or strings)
		
		
		if (lStringTagsBegin == 0) {
			lStringTagsBegin = lCurTag;
		}
		if (lListTagsBegin == 0) {
			lListTagsBegin = lStringTagsBegin;
		}

		pHead->lMagic = TEMPLATE_MAGIC;
		pHead->lVersion = TEMPLATE_VERSION;
		pHead->lListTagsBegin = lListTagsBegin * sizeof(TAG_PAIR);
		pHead->lStringTagsBegin = lStringTagsBegin * sizeof(TAG_PAIR);
		pHead->lListsBegin = lCurTag * sizeof(TAG_PAIR);
		pHead->lStringsBegin = pHead->lListsBegin + lListPos * sizeof(long);

		memcpy((void*)pTagPtr, (void*)pTags, lCurTag * sizeof(TAG_PAIR));
		memcpy((void*)pListPtr, (void*)pulLists, lListPos * sizeof(long));
		memcpy((void*)pStringPtr, (void*)pchStrings, lStringPos * sizeof(char)); // support for wide chars?

		// set up properly the main fields
		vScanData();
	}

err5:
	delete [] pchStrings;

err4:
	delete [] pulLists;

err3:
	delete [] pTags;

err2:
	fclose(file);

err1:
	return;
}

void Template::vScanData()
{
	TEMPLATE_HEAD *pHead;

	if (!data) {
		goto err1;
	}

	pHead = (TEMPLATE_HEAD*)data;

	if (pHead->lMagic != TEMPLATE_MAGIC) {
		goto err1;
	}
	if (pHead->lVersion != TEMPLATE_VERSION) {
		goto err1;
	}

	mpValueTags = (TAG_PAIR*)(data + sizeof(TEMPLATE_HEAD));
	mpListTags = (TAG_PAIR*)((char*)mpValueTags + pHead->lListTagsBegin);
	mpStringTags = (TAG_PAIR*)((char*)mpValueTags + pHead->lStringTagsBegin);
	mplLists = (long*)((char*)mpValueTags + pHead->lListsBegin);
	mpchStrings = ((char*)mpValueTags + pHead->lStringsBegin);

	// calculate the size of the various tag lists
	mnValueTags = mpListTags - mpValueTags;
	mnListTags = mpStringTags - mpListTags;
	mnStringTags = (TAG_PAIR*)mplLists - mpStringTags;

	assert(mnValueTags >= 0);
	assert(mnListTags >= 0);
	assert(mnStringTags >= 0);

	return;

err1:
	//
	// error return
	//
	mpValueTags = NULL;
	mpListTags = NULL;
	mpStringTags = NULL;
	mplLists = NULL;
	mpchStrings = NULL;

	mnValueTags = 0;
	mnListTags = 0;
	mnStringTags = 0;

	return;
}
