// UNIT.H -- generic unit definitions
//
// DJM

#ifndef _UNIT_
#define _UNIT_
#include "scimain.h"
#include "mapnode.h"
#include "drawobj.h"
#include "status.h"

#define MAX_LEVEL_UPGRADE		6	//the maximum that 'level' can be upgraded to
#define MAX_UPGRADES_UPGRADE	3	//the maximum that any individual upgrade can be upgraded to

typedef enum 
{
	SPEED_NORMAL,
	SPEED_CLOAKED,
	SPEED_ADRENALINE,
	SPEED_MOBBING
} SPEED_ENUMTYPE;

#define DEFAULT_SPEED_NORMAL_JUSTINCASE 2

class SequentialMovement;
class SequentialCommands;
class Item;
class FLAGENTRY;
//class Walker;

class FLAGENTRY
{
public:
	int value;
	FLAGENTRY * next;
};


#define MAXPLAYERCOUNT 8
#define NONPLAYERINDEX MAXPLAYERCOUNT
#define ALLPLAYERINDICES (MAXPLAYERCOUNT+1)

typedef enum _DIRECTION {
	D_SOUTH,			//0
	D_SOUTHWEST,		//1
	D_WEST,				//2
	D_NORTHWEST,		//3
	D_NORTH,			//4
	D_NORTHEAST,		//5
	D_EAST,				//6
	D_SOUTHEAST,		//7
	D_NOCHANGE,			//8
} DIRECTION;


//All the actions that need bitmaps for any unit type(vampire or peasant)
//
typedef enum _ACTIONS {
	//								VAMPIRE							PEASANT
	ACT_WALK,				//0    Walk(cloaked)				  standard Walk
	ACT_RUN,				//1    Run(uncloaked)				  Run Mobbing
	ACT_ATTACK,				//2    attack						  attack
	ACT_REACT,				//3    taking damage				  taking damage??
	ACT_FEED,				//4    feed on peasants				  fed upon by vampires
	ACT_DIE,				//5	   Die						      Die
	ACT_FIDGET_CLOAKED,		//6    Fidgeter						  N/A
	ACT_FIDGET_UNCLOAKED,	//7    Fidgeter						  Fidgter
	ACT_CAST,				//8    cast a spell					  N/A
	ACT_SPECIAL,			//9	   ???							  ???
} ACTIONS;
	

typedef enum _UNITTYPE {
	UT_VAMPIRE,
	UT_PEASANT,
	UT_GYPSY,
	UT_TURK,
	UT_SOLDIER,
	UT_VAMPIREHUNTER,
} UNITTYPE;

typedef enum _BEHAVIOR {
	BV_GUARD,
	BV_ASSAULT,
	BV_HARASS,
	BV_PATROL,
	BV_BODYGUARD,
	BV_SCOUT,
} BEHAVIOR;


typedef enum DEFAULT_ATTACK {
	DA_HANDTOHAND,
	DA_SPELL,
} DEFAULT_ATTACK;

typedef enum TAG_UPGRADE_NAME {
	UPGRADE_Level,
	UPGRADE_Attack,
	UPGRADE_Defend,		
	UPGRADE_Detection,	
	UPGRADE_Sight,
	UPGRADE_Speed,		
	UPGRADE_Spells,
	UPGRADE_Stealth,		
	UPGRADE_COUNT,
	UPGRADE_NONE
} UPGRADE_NAME;

#include "player.h"

#define		UNIT_FLAG_WAITING		0x00000001
#define		UNIT_FLAG_INFOG			0x00000002

#define		UNIT_HASPOWERSTONE	1

typedef enum TYPE_SETTING_LMH
{
	SETTING_LOW,
	SETTING_MEDIUM,
	SETTING_HIGH
} SETTING_LMH;

typedef struct 
{
	int count;
	int attack;
	int defense;
	int relativehealth;
	int level;
} TOTALSTATS;

class GuardCoordinator;
class MobCoordinator;

#ifndef _INVENTORY_H_
#include "inventory.h"
#endif 

class INTLIST
{
public:
	INTLIST();
	~INTLIST();
	void AddToEnd(int nValue);
	void RemoveAll();
	int GetSize();
	int GetItem(int nIndex);
private:
	int mnCount;
	int * anList;
	int mnAllocated;
};

class Unit : public DrawObj, public HasInventory {
	friend class VampireDecoy;
	friend class Vampire;
	friend class Human;
	friend class Missile;
	friend class Spell;
protected:
	int mnLastRetreatDirection;
	int FindDirectionToOffset(int x, int y);
	virtual void FindRetreatLocation(int nDistance, int * pnReturnX, int * pnReturnY);
	virtual Bitmap * GetHighlightBitmap();
	virtual void vInternalCommand_Attack(Unit * poTargetUnit);

	short mnFog;
	Unit(Player *poOwner, MapPlane *poPlane, MapNode *poNode, int nTemplateID);
	BOOL mbInvisible;
	WORD mwSpyFlags;
	int mnCyclesBeforeCast;
	int mnAlertCyclesRemaining;
	int mnAdrenalineCycleTicker;
	virtual void vProcessAdrenaline();
	void SetSpeed(SPEED_ENUMTYPE eSpeed);
	virtual void SetSpeedAdrenalineBoost() {}
	void SetSpy(Player * poPlayer);
	void ClearSpy(Player * poPlayer);

public:
	virtual BOOL bAbleToPerformCommand(Command & oCommand, BOOL bWithSound = FALSE) { return FALSE; }

	BOOL bInFog();
	void SetInFog();
	void SetNoFog();
	~Unit();
	void ClearFrozen();
	void SetFrozen();
	void vSetAlpha(BYTE jAlpha) {mjAlpha = jAlpha; if(mpoShadow) {mpoShadow->vSetAlpha(jAlpha);} }

	void ClearInvisible();
	void SetInvisible();
	BOOL IsInvisible() { return mbInvisible; }

	BOOL AlertClan(int nPlayerNumber, BOOL bWithSound = FALSE);
	void FireAlertClan(int nPlayerNumber);

	void PlayClassSound(int nSoundName, int nWait, int nWithin, int nMax);

	void DecrementCycleCounts();
	BOOL bNotEngagedWithAttacker(Unit * poUnitAttacker);
	int GetPronunciationOfClanForPlayer(int nPlayerNumber);
	// CLASS FUNCTIONS FOR UNIT
	static void vInitStatics();
	static void SwapPositions(Unit & unit1, Unit & unit2);
	static void SetAttribute(Unit * poThis, ATTR_CODE mnSetAttributeID, int setting);
	static HRESULT Create(Unit ** ppoUnit, Player *poOwner, MapPlane *poPlane, MapNode *poNode, int nTemplateID);
	static BYTE mjFullRed;
	static BYTE mjFullGreen;
	static BYTE mjFullBlue;
	static BYTE mjEmptyRed;
	static BYTE mjEmptyGreen;
	static BYTE mjEmptyBlue;

	// COMMON FUNCTIONS FOR UNIT
	long lBitmapNormal();
	long lBitmapRunning();
	long lBitmapReacting();
	long lBitmapAttacking();
	long lBitmapFeeding();
	long lBitmapCastingSpell();
	long lBitmapDying();
	long lBitmapPortrait();
	long lBitmapSoloPortrait();
	ENUM_SEX eGetSex();
	BOOL GetFlag(int nFlag);
	void SetFlag(int nFlag);
	void ClearFlag(int nFlag);
	virtual BOOL bCanKillVampireLord() { return FALSE; }
	virtual BOOL bIsHuman() = 0;
	virtual BOOL bCloaked() {return FALSE;}
	virtual BOOL bIsVampireLord() { return FALSE; }
	virtual BOOL bIsUpgradeInProgress() { return FALSE; }
	virtual SETTING_LMH GetAggression()  { return SETTING_LOW; }
	virtual SETTING_LMH GetTenacity()  { return SETTING_LOW; }
	virtual void GainPowerFromStone(Item * poItem, FLOAT fPower) {};
	virtual void GainPowerFromOwnedStone(Item * poItem) {};
	virtual int GetAnxiety() { return 0; }
	virtual void vIncreaseAnxiety(float nAmount) { return; }
	virtual void vSetAnxiety(int nAmount) { return; }
	
	void vOverride(float * pfVar, long lTag);
	void vOverride(int * pnVar, long lTag);

	virtual void vSetMobbing(BOOL bVal, MobCoordinator * poMobCoordinator = NULL);
	virtual BOOL vCanBeEnthralled() { return TRUE; }
	BOOL bIsMobLeader();
	BOOL bIsMobMember();
	BOOL bIsMemberOfThisMob(MobCoordinator * poMobCoordinator);
	
	long lDistToSpace(int nX, int nY);
	int  nGetX() { return mpoCurNode->mnXPos; };
	int  nGetY() { return mpoCurNode->mnYPos; };
	int	 nCurrentSpell() {return mnCurrentSpell;}
	void vSetPos(WORD wX, WORD wY);
	void vSetPixelPos(int nX, int nY);
	void Message(Unit * poFrom, DWORD dwMessage);
	void GetStatusIndicatorOffset(int * px, int * py);
	
	void vChangeFacing(int nCurX, int nCurY, int nNewX, int nNewY);
	void vFidget();
	DIRECTION	eDirection(int nCurX, int nCurY, int nNewX, int nNewY);
	DEFAULT_ATTACK eDefaultAttack() {return meDefaultAttack;}
	void		vSetDefaultAttack(DEFAULT_ATTACK eAtt);
	void		vCue();
	void		vSetAction(ACTIONS eAction);	//always set meAction through this call!!!! 
	ACTIONS		eAction() {return meAction;}

	BOOL bStuck();
	BOOL bIsInside();
	BOOL bAlive() { return mbAlive;}
	BOOL bIsCloaked() { return mbCloaked; }
	BOOL bIsVampire() {	return meType == UT_VAMPIRE; }
	BOOL bIsLocalVampire() {return ( (meType == UT_VAMPIRE) && (game->poLocalPlayer() == mpoOwner) );}
	BOOL bMoving(void) {return mbMoving;}

	UNITTYPE eUnitType() {return meType;}
	
	Player	*poOwner() {return mpoOwner;}

	int nPixelX() { return mlPixelX; }
	int nPixelY() { return mlPixelY; }

	int GetPower() { return (int)mfPower; }
	int GetMaxPower() { return (int)mfMaxPower; }

	short nLevel() { return mnLevel; }

	// RELATIVE HEALTH
	virtual short vGetRelativeHealth(Player * poPlayer) = 0;
	virtual void vUpdateRelativeHealth(Player * poPlayer) = 0;

	const char *GetName() {return mpoTemplate->strGetString(TAG_NAME, "Unknown");}

	// OVERRIDES
	void vDraw();
	void vDoit() {return;};
	virtual void vMoveOneCycle();
	void vCalcRoute();
	void vCalcRoute(int nOffsetX, int nOffsetY);
	Boolean HandleEvent(sciEvent & oEvent);

	// NON VIRTUAL FUNCTIONS WITH V PREFIX
	void vAddAttacker(Unit *poUnit);
	void vRemoveAttacker(Unit *poUnit);
	
	virtual void vCommandNone();


	// NEW VIRTUAL FUNCTIONS
	virtual void vCommand(Command &oCommand, BOOL bInternal = FALSE);
	virtual FLOAT vDamage(FLOAT nDamage, Unit *poUnitFrom = NULL);
	virtual FLOAT vDrainPower(FLOAT fDamage);
	void IncreasePower(FLOAT fPower);

	virtual int GetUpgradeSight() = 0;
	virtual int GetUpgradeAttack() = 0;
	virtual int GetUpgradeDefend() = 0;
	virtual int GetUpgradeSpeed() = 0;
	virtual int GetUpgradeStealth() = 0;
	virtual int GetUpgradeDetection() = 0;
	virtual int GetUpgradeSpells() = 0;
	virtual int GetUpgradeLevel() = 0;
	virtual int vGetVisibleRange();
	
	void vSetActionLoop();	//calculates and sets loop based on current action and direction

protected:
	virtual void vPowerToHealth();
	virtual void vThink();
	virtual void vScan();
	virtual void vSetLevel(int nLevel);
	virtual void vUpdateAttributes();
	virtual void vAttack();

	// PEASANT SPECIFIC
public:
	void CommandSequenceClear();

protected:
	void SelectNewRandomBehavior();


// OTHER FUNCTIONS
public:
	FLAGENTRY * mpnFlags;

	// debugging stuff -- checksum and text dumping
	long lGetChecksum(void);
	void vDumpSyncText(FILE *fileDump);
	
	Unit *poGetAttackee();
	Unit *poGetFirstAttacker();
	Unit *poGetNextAttacker();
	BOOL bSpaceVisible(int nX, int nY);

	BOOL CheckRoute(MapNode * poGoal, MapNode ** ppoDest);

	void vDrag(sciEvent &oEvent);
	MapNode *poGetNextMoveNode();
	MapNode *poGetNextMoveNodeWithOffset();		

	void vCheckVisibility (void);

	void vSaveMap(FILE *file, WORD wVersion);	
	void vLoadMap(FILE *file, WORD wVersion);

	BYTE mjPlayerLink;		//what player # this unit is linked to
 	
	//map editing mode specific stuff
	void vCopySelf(sciEvent &oEvent);	//makes copy of self and inserts into node nearest event
	void vInsertSelf(sciEvent &oEvent);	//inserts self into node nearest event
	void vMapShift(MapNode *poTo);		//essentially moves unit to new node

	// for guarding
	void vSetGuardedUnit(Unit *poUnit);

	void SetThrallOwner(Unit * poOwnerUnit, int nCycles);

public:
	void Sequence_WaitShort();
	void Sequence_WaitMedium();
	void Sequence_WaitLong();
	void Sequence_WalkToBuildingOutside(Building * pBuilding);
	void Sequence_WanderMethodically(int nPauseFreq = 0, int nDirectionX = -10000, int nDirectionY = -10000);
	void Sequence_WanderWithoutRest(int nDistance);
	void Sequence_WalkToBuildingInside(Building * pBuilding);
	void Sequence_Clear();
	void AcquireNewBuilding(Building ** ppOutBuilding, Building * pInOldBuilding);

	virtual Template * vGetClanTemplate() = 0;
	virtual vClan * vGetClan() = 0;

protected:
//	Walker		*mpoWalker;
	ACTIONS				meAction;			//current action(walking, running, casting, mobbing, etc)
	DIRECTION			meDirection;		//the current direction we are going/facing
	DEFAULT_ATTACK		meDefaultAttack;	//default attack for the unit

	virtual void CallForHelp();

	// waypoint movement
	SequentialMovement * mpoMovement;
	SequentialCommands*	mpoSequencer;
	
	void AddPatrolWaypoint( int nX, int nY );

	// for being guarded
	void vAddGuard(Unit * poUnit);
	void vRemoveGuard(Unit * poUnit);
	void GuardStop();
	void ClearThought();
	void Patrol();

	MapNode *poPopOneNode();
	void vMoveToNode(MapNode *poNode);
	void vSetAttackee(Unit *poUnit);
	void SetNode(MapNode * poNode);

	void SetAttacker(Unit * poAttacker, int nCycles);
	Unit * mpoAttacker;
	int mnUnderAttackCycles;							// REMAINING CYCLES

	virtual void vRegeneratePower();
	void Retreat();
	BOOL bShouldRetreat();
	BOOL bStayAndFight();
	BOOL bAttackEnemyUnit(Unit * poUnit);

	void vMoveToNextNode();
	void vMoveToNextNode(MapNode ** pReturnMapNode);
	void vMoveToNextNodeWithOffset();

	Player		*mpoOwner;
	UNITTYPE	meType;
	
	House		* mpoHouse;
	DrawObj		*mpoShadow;

	FLOAT		mfPower;
	FLOAT		mfMaxPower;
	short		mnLevel;

	static short mnRelHealthPercent;

	BEHAVIOR	meBehavior;

	SETTING_LMH	mTenacity;
	SETTING_LMH	mAggression;
	SETTING_LMH	mConversion;

	Command		moCurrentCommand;

	PlayScript	moAttackScript;
	PlayScript	moSightScript;
	PlayScript	moIdleScript;

	MapNode		*mpoLastNode;	// the one we left
	//MapNode		*mpoTempNode;	// the one we're moving into

	long		mlPixelX;	// pixel-accurate position
	long		mlPixelY;

	// movement-related
	short		mnSpeed;	// given in inverse units -- cycles/square
	short		mnPartialMove; // how many cycles in this move have we done?
	BOOL		mbBetweenPoints; // are we moving between points?
	BOOL		mbMoving;		// are we moving?
	int			mnStoppedCycles;
	int			mnTempBlockedCycles;

	// attack-related
	short		mnLastTargetX;
	short		mnLastTargetY;
	short		mnAttackDist;
	short		mnAttackRange;
	short		mnAttackSpeed;
	int			mnLastAttackCycle;
	int			mnDeathCycle;
	BOOL		mbAlive;
	FLOAT		mfFeedRate;		// AMOUNT TO FEED PER CYCLE
	short		mnVisibleRange;

	// detection related
	short		mnVampireRange[3];
	inline void	CheckNodeForVisibleUnits(MapNode * poNode, int nDistance);
	void		CheckNodeThatHasUnits(MapNode * poNode, int nDistance);
	BOOL		OutGunnedByVisibleUnits(Unit * poUnit);
	inline void	MarkUnit(Unit * poUnit);

	// thought related
	int			mnLastThinkCycle;
	short		mnThinkSpeed;

	// power conversion
	int 		mnLastPowerConvCycle;
	short		mnPowerConvSpeed;
	short		mnHealthPerPower;

	int			mnLastPowerRegenCycle;
	short		mnPowerRegenSpeed;
	short		mnPowerRegenPercent;

	// conversion
	short		mnConvertCycleCount;	// attackee
	short		mnMinConvHealthPercent;
	short		mnConvertCost;

	// wait related
	int			mnStartWaitCycle;
	DWORD		mdwFlags;

	// cloaking
	BOOL		mbCloaked;

	// attack-related
	//Unit*				mpoAttackee;
	int					mnAttackee;
	Unit*				mpoThrallOwner;
	int					mnThrallCyclesRemaining;
	RefList<Unit>		moAttackers;
	INTLIST				moVisibleUnits;
	TOTALSTATS			mnVisibleTotal[MAXPLAYERCOUNT + 1];
	int					mnCurrentSpell;

	// routing-related
	RefList<MapNode>	moCurrentRoute; // reverse order (end-to-begin)

	// guard-related

	// class sounds
	static int mclassstaticLastPlayedClassSound;
	static int mclassstaticClassSounds;

public:
	GuardCoordinator*	mpoGuardCoordinator;	// IF GUARDED
	MobCoordinator*		mpoMobCoordinator;
	Unit*				mpoGuardedUnit;			// IF GUARDING
	int IsGuarded()		{ return (mpoGuardCoordinator != NULL); };
	int IsPatrolling()	{ return (mpoMovement != NULL); };
	virtual BOOL vIsReadyToEnterMob() { return FALSE; }


};

#endif