#include "scimain.h"
#include "mapnode.h"
#include "mapplane.h"
#include "unit.h"
#include "building.h"
#include "command.h"
#include "objectdb.h"
#include "path.h"
#include "template.h"
#include "mapedit.h"
#include "guard.h"
#include "movement.h"
#include "procs.h"
#include "math.h"
#include "intplane.h"
#include "vampire.h"
#include "item.h"
#include "missile.h"
#include "mob.h"
#include "splash.h"
#include "soundlist.h"
#include "NonPlayerClass.h"
#include "spell.h"

extern Missile * CreateMissile(Unit * poFromUnit, MapNode * poSourceNode, MapNode * poTargetNode, MISSILE_TYPE missile, int nWait, BOOL bSolid, float fPercent = 0.0, double X = 10.0, double Y = 10.0, int nFallOff = 2, int nDirection = 1);

BOOL Vampire::bAbleToPerformCommand(Command & oCommand, BOOL bWithSound)
{
	switch (oCommand.meType)
	{
	case CM_CONVERT:
		if (GetPowerCostToConvert() > GetPowerExceedingEmergencyLevel())
		{
			// TBD -- PLAY SOUND
			return FALSE;
		}
		if (!bCanAttack())
		{
			// CANNOT DO THAT WHILE HOLDING A POWER STONE
			if (bWithSound)
			{
				if (bIsLocalVampire() && !moCurrentCommand.mbInternal)
				{
					pApp->pSoundManager->EmitFromObject(this, SOUND_RefuseAttackHoldingAPowerStone);
				}
			}
			return FALSE;
		}
		break;
	case CM_CAST:
		{
			FLOAT fRequired;
			UINT  nRequiredLevel=1;

			if (mnCyclesBeforeCast) return FALSE;

			Spell * poSpell = game->GetSpell(mnCurrentSpell);
			assert(poSpell);
			fRequired = poSpell->GetAdjustedSpellCost(this);
			nRequiredLevel = poSpell->GetRequiredLevel();
			//GetSpellCostAndLevel(mnCurrentSpell, &fRequired, &nRequiredLevel);
			//AdjustSpellCost(&fRequired);
			
			if (GetPowerExceedingEmergencyLevel() < fRequired)
			{
				// SOUND: REFUSE TO CAST -- NOT ENOUGH POWER
				if (bWithSound)
					pApp->pSoundManager->EmitFromObject(this, SOUND_NoCastLowPower1);
				return FALSE;
			}
			if ((UINT)mnLevel < nRequiredLevel)
			{
				// SOUND: REFUSE TO CAST -- LEVEL NOT HIGH ENOUGH
				// MAYBE "I DO NOT UNDERSTAND"
				if (bWithSound)
					pApp->pSoundManager->EmitFromObject(this, SOUND_NoCastDueToLevel);
				return FALSE;
			}
			// CHECK FOR FRIENDLIES IN THE BLAST ZONE
			if (mnCurrentSpell == SPELL_DEATHFROMABOVE)
			{
				int x=0,y=0;
				MapNode * poNodeEpicenter = oCommand.poGetTargetNode();
				if (!poNodeEpicenter) return FALSE;
				BOOL bCancel = FALSE;
				UINT nDimension = poSpell->GetDamageRange(this);

				// ADJUST TO ODD VALUE IF NOT ZERO
				if (nDimension && (nDimension % 2 == 0)) nDimension++;

				int nMaxRange = nDimension / 2;

				int nStartX = max(0, poNodeEpicenter->mnXPos - nMaxRange);
				int nStartY = max(0, poNodeEpicenter->mnYPos - nMaxRange);
				int nEndX = min(gpoMap->nGetWidth()-1, poNodeEpicenter->mnXPos + nMaxRange);
				int nEndY = min(gpoMap->nGetHeight()-1, poNodeEpicenter->mnYPos + nMaxRange);

				for (int nX = nStartX; nX<nEndX; nX++)
				{
					for(int nY = nStartY; nY<nEndY; nY++)
					{
						MapNode * poNode = gpoMap->poGetNode(nX, nY);
						Unit * poUnit = poNode->mpoUnit;
						if (poUnit)
						{
							if (poUnit->mpoOwner == mpoOwner)
							{
								bCancel = TRUE;
								break;
							}
						}						
					}
				}
				if (bCancel)
				{
					return FALSE;
				}
			}
		}
		// FALL THROUGH
	case CM_FEED:
	case CM_ATTACK:
		if (!bCanAttack())
		{
			// CANNOT DO THAT WHILE HOLDING A POWER STONE
			if (bWithSound)
			{
				if (bIsLocalVampire() && !moCurrentCommand.mbInternal)
				{
					pApp->pSoundManager->EmitFromObject(this, SOUND_RefuseAttackHoldingAPowerStone);
				}
			}
			return FALSE;
		}
		break;
	default:
		break;
	}
	return TRUE;
}

Vampire::Vampire(Player *poOwner, MapPlane *poPlane, MapNode *poNode, int nTemplateID):
	Unit(poOwner, poPlane, poNode, nTemplateID),mdwUpgradesPending(0)
{
	mnTracerReset = 0;
	mnCurrentSpell = 2;
	mAggression = SETTING_HIGH;
	mTenacity = SETTING_HIGH;

	mLastUpgrade = (UPGRADE_NAME)0;

	// RESET ALL UPGRADES
	manUpgrade[UPGRADE_Sight]		= 0;
	manUpgrade[UPGRADE_Attack]		= 0;
	manUpgrade[UPGRADE_Defend]		= 0;
	manUpgrade[UPGRADE_Speed]		= 0;
	manUpgrade[UPGRADE_Stealth]		= 0;
	manUpgrade[UPGRADE_Detection]	= 0;
	manUpgrade[UPGRADE_Spells]		= 0;

	manDesired[UPGRADE_Sight]		= 0;
	manDesired[UPGRADE_Attack]		= 0;
	manDesired[UPGRADE_Defend]		= 0;
	manDesired[UPGRADE_Speed]		= 0;
	manDesired[UPGRADE_Stealth]		= 0;
	manDesired[UPGRADE_Detection]	= 0;
	manDesired[UPGRADE_Spells]		= 0;
	
	mfUpgradeProgress = 0.0;
	mUpgradeName = UPGRADE_NONE;
	mbUpgrading = FALSE;
	
	mnVisibleRange = mnLevel + manUpgrade[UPGRADE_Sight] + 8;
	
	mnCelCastDelay = 0;

	mnVampireRange[0] = mnVisibleRange * 50 / 100;
	mnVampireRange[1] = mnVisibleRange * 75 / 100;
	mnVampireRange[2] = mnVisibleRange * 100 / 100;
	//status/level related
	memset(manApparentHealth, 0, MAX_PLAYERS * sizeof(int));	//init array
	memset(manApparentLevel, 0, MAX_PLAYERS * sizeof(int));		//init array

	//set up the crown(head) of the vampire lord
	mpoCrown = NULL;
	int nCrown = mpoTemplate->lGetValue(TAG_Bcwn, 0);
	if (nCrown) {
		mpoCrown = (Bitmap*)pApp->pResourceManager->LoadResource(RES_BITMAP, nCrown);
	}


	// SET THE DEFAULT DISGUISE
	mnBitmapCloaked = mpoTemplate->lGetValue(TAG_Bclk, nBitmapID());

	if (mpoOwner) 
	{
		assert(mpoOwner->mpoClan);
		mpoClan = mpoOwner->mpoClan;
	}
	// call this after clan is set - pb
	moStatus.vSetup(this, (sciPlane*)poPlane);

	if (mpoOwner)
	{
		if (mpoOwner->mbHuman && mpoOwner->mbLocal) 
		{
			vSetMarked(mpoOwner->mnPlayer, vGetRelativeHealth(mpoOwner), mnLevel);
			moStatus.vShow();
		}
	}



	//for map creation/loading/saving only
	mjPlayerLink = 0;		// this gets modified during map editing
	vSetLevel(mnLevel);
	manDesired[UPGRADE_Level]		= mnLevel;

	mfPower = mfMaxPower;
	mfHealth = mfMaxHealth;
	SetSpeed(SPEED_NORMAL);

	// get my special palette
	mpwTruePalette = mpoClan->pwGetClanPalette(mpoOwner->nColorIndex());

	mbCloaked = TRUE;
	mbDesiredCloakState = TRUE;
	if (mbCloaked) 
	{
		vCloak();
	} 
	else 
	{
		vUncloak();
	}

}

Vampire::~Vampire() 
{
	if (mpoCrown) {
		pApp->pResourceManager->FreeResource(mpoCrown);
		mpoCrown = NULL;
	}
}

void Vampire::FireAnxiety(int nAmount)
{
	// FOR THE RANGE, CALL Unit::vIncreaseAnxiety(nAmount) 
	// FOR ALL HUMAN UNITS
	// MAX ANXIETY BLAST RANGE
	int nRange = vGetClanTemplate()->lGetValue(TAG_MABR, 10);
	RECT r, r1,r2;
	r1.left = nGetX() - nRange;
	r1.right = nGetX() + nRange;
	r1.top = nGetY() - nRange;
	r1.bottom = nGetY() + nRange;
	r2.left = 0;
	r2.top = 0;
	r2.right = gpoMap->nGetWidth()-1;
	r2.bottom = gpoMap->nGetHeight()-1;

	if (!IntersectRect(&r, &r1, &r2)) return;
	
	for (int i=r.left; i<=r.right; i++)
	{
		for (int j=r.top; j<=r.bottom; j++)
		{
			Unit * poUnit = gpoMap->poGetNode(i, j)->mpoUnit;
			if (poUnit && poUnit->bIsHuman())
			{
				if (!poUnit->mnThrallCyclesRemaining)
				{
					poUnit->vIncreaseAnxiety((float)nAmount);
				}
			}
		}
	}
}

BOOL Vampire::bCanAttack()
{
	return !GetFlag(UNIT_HASPOWERSTONE);
}

BOOL Vampire::bCloaked()
{
	return mbCloaked;
}

void Vampire::vUncloak()
{
	if (mbCloaked == TRUE)
	{
		// if there is no uncloaked bitmap, the current one won't change
		vSetBitmap(mpoTemplate->lGetValue(TAG_Bnor, nBitmapID()));
		mpwSpecialPalette = mpwTruePalette;

		//we're actually changing modes
		if (eAction() == ACT_WALK)
			vSetAction(ACT_RUN);	//we were moving when we Cloaked so keep moving 
		else
			vSetAction(ACT_FIDGET_UNCLOAKED);	//we were doing something else when we cloaked so just stand there
	}

	mbCloaked = FALSE;
	if (mnAdrenalineCycleTicker > 0)
		SetSpeed(SPEED_ADRENALINE);
	else
		SetSpeed(SPEED_NORMAL);
}

void Vampire::vCloak()
{
	mbCloaked = TRUE;
	SetSpeed(SPEED_CLOAKED);

	if (bAlive()) {

		//set the appropiate bitmap
		if (mpoOwner != game->poLocalPlayer())
		{
			if (bMarked(game->poLocalPlayer()->mnPlayer))
			{
				vSetBitmap(mpoTemplate->lGetValue(TAG_Bnor, nBitmapID()));
				mpwSpecialPalette = mpwTruePalette;
			} else {
				vSetBitmap(mnBitmapCloaked);
				mpwSpecialPalette = NULL;
			}
		} else {
			mpwSpecialPalette = mpwTruePalette;
		}


		if (eAction() == ACT_RUN)
			vSetAction(ACT_WALK);	//we were moving when we Cloaked so keep moving 
		else
			vSetAction(ACT_FIDGET_CLOAKED);	//we were doing something else when we cloaked so just stand there
	}

	// if there is no cloaked bitmap, the current one won't change
//	if (bAlive() && mpoOwner != game->poLocalPlayer()) 
//	{
//		vSetBitmap(mnBitmapCloaked);
//	}	
}

void Vampire::vAttack()
{
	int nCurCycle = game->nGameCycle();
	if (nCurCycle - mnLastAttackCycle < mnAttackSpeed) 
	{
		return;
	}
	mnLastAttackCycle = nCurCycle;

	if (!mnAttackee) 
	{
		return;
	}
	Unit * poAttackee = (Unit*)goObjectDB.poGetObject(mnAttackee);
	if (!poAttackee) return;
	poAttackee->SetAttacker(this, 20);
	int nAttackBonus = 0;
	int nDefendBonus = 0;

	if (poAttackee->bNotEngagedWithAttacker(this))
	{
		nAttackBonus = 1;
		nDefendBonus = -1;
	}
	int nMinimum = (int)(((mnLevel + nAttackBonus + GetUpgradeAttack()) - (poAttackee->nLevel() + nDefendBonus + poAttackee->GetUpgradeDefend())) * mpoClan->GetXMinDamage());
	int nMaximum = max(10, (int)(((mnLevel + nAttackBonus + GetUpgradeAttack()) * mpoClan->GetXMaxDamageAttack()) - ((poAttackee->nLevel() + nDefendBonus + poAttackee->GetUpgradeDefend()) * poAttackee->vGetClan()->GetXMaxDamageDefend())));
	int nDamage = nMinimum + GlobalRand(nMaximum - nMinimum);
	vDebugOut("min%d max%d damage%d", nMinimum, nMaximum, nDamage);

	if (poAttackee->bIsVampire())
	{
		int nSize = poAttackee->poItemList()->GetSize();
		if (nSize)
		{
			while(nSize)
			{
				nSize--;

				Item * poItem = poAttackee->poItemList()->GetItem(nSize);

				// TRY TO STEAL EACH ITEM BUT ONLY ONE ITEM
				if (poItem->bCanSteal())
				{
					poAttackee->Give(this, poItem);
					break;  // ITEM STOLEN SO CONTINUE
				}
			}
		}
	}

	if (nDamage > 0)
	{
		FireAnxiety(mpoClan->GetAnxietyBlastForAttacking());
		if (mpoOwner == game->poLocalPlayer())		// HIT
		{
			vPlayRandomSound(TAG_Satk, FALSE);
		}
		poAttackee->vPlayRandomSound(TAG_Sdm1, FALSE);
		poAttackee->vDamage((float)nDamage, this);	
	}
	else
	{
		if (mpoOwner == game->poLocalPlayer())		// MISS
		{
			vPlayRandomSound(TAG_Smis, FALSE);
		}
	}
}


void Vampire::vFeed()
{
	/*
	int nCurCycle = game->nGameCycle();

	if (nCurCycle - mnLastAttackCycle < mnAttackSpeed) 
	{
		return;
	}

	mnLastAttackCycle = nCurCycle;
	*/
	assert(mnAttackee);
	Unit * poAttackee = (Unit*)goObjectDB.poGetObject(mnAttackee);
	if (!poAttackee)
	{
		vCommandNone();
		return;
	}
	if (moCurrentCommand.mnState == 0)
	{
		moCurrentCommand.mnState = 1;
		pApp->pSoundManager->EmitFromObject(this, 7120);
		mnCyclesUntilNextSound = 20;
		FireAnxiety(mpoClan->GetAnxietyBlastForFeeding());
	}
	else if (mnCyclesUntilNextSound == 0)
	{
		switch(moCurrentCommand.mnState)
		{
		case 1:
			mnCyclesUntilNextSound = 17;
			pApp->pSoundManager->EmitFromObject(this, 7123);
			break;
		case 2:
			mnCyclesUntilNextSound = 22;
			pApp->pSoundManager->EmitFromObject(this, 7121);
			break;
		case 3:
			mnCyclesUntilNextSound = 18;
			pApp->pSoundManager->EmitFromObject(this, 7124);
			break;
		case 4:
			mnCyclesUntilNextSound = 18;
			pApp->pSoundManager->EmitFromObject(this, 7122);
			break;
		case 5:
			mnCyclesUntilNextSound = 11;
			pApp->pSoundManager->EmitFromObject(this, 7125);
			break;
		case 6:
			mnCyclesUntilNextSound = 20;
			pApp->pSoundManager->EmitFromObject(this, 7120);
			break;
		}
		moCurrentCommand.mnState++;
		if (moCurrentCommand.mnState == 7)
			moCurrentCommand.mnState = 1;
	}
	else
	{
		mnCyclesUntilNextSound--;
	}

	// CONTINUE FEEDING?
	if (poAttackee->mfHealth > poAttackee->mfMaxHealth * mpoClan->GetPercentMinimumFeedingHealth())
	{
		FLOAT nRequestedAmount = min(mfMaxPower - mfPower / (FLOAT)(2*mnLevel), mfFeedRate);

		// IF THIS WILL TAKE ATTACKEE BELOW ACCEPTABLE LIMIT
		if (poAttackee->mfHealth - nRequestedAmount < mpoClan->GetPercentMinimumFeedingHealth())
		{
			nRequestedAmount = poAttackee->mfHealth - mpoClan->GetPercentMinimumFeedingHealth();
		}

		if (nRequestedAmount > 0.0)
		{
			/*  TBD -- feeding sound
			if (mpoOwner == game->poLocalPlayer()) 
			{
				vPlayRandomSound(TAG_Satk);
			}*/

			FLOAT fAcquired = poAttackee->vDamage(nRequestedAmount, this);

			// CONVERT HEALTH TO POWER
			//
			mfPower = min(mfMaxPower, mfPower + (2 * fAcquired * mnLevel));

			moStatus.vUpdateHealth(vGetRelativeHealth(mpoOwner)); 
		}
	}
}

int Vampire::vConvert()
{
	assert(mnAttackee);
	Unit * poAttackee = (Unit*)goObjectDB.poGetObject(mnAttackee);
	if (!poAttackee)
	{
		vCommandNone();
		return FALSE;
	}
	poAttackee->mnConvertCycleCount--;

	if (mnCyclesUntilNextSound == 0)
	{
		switch(moCurrentCommand.mnState)
		{
		case 0:
			mnCyclesUntilNextSound = 20;
			pApp->pSoundManager->EmitFromObject(poAttackee, 7423);
			pApp->pSoundManager->EmitFromObject(this, 7120);
			break;
		case 1:
			mnCyclesUntilNextSound = 17;
			pApp->pSoundManager->EmitFromObject(this, 7123);
			break;
		case 2:
			mnCyclesUntilNextSound = 22;
			pApp->pSoundManager->EmitFromObject(this, 7121);
			break;
		case 3:
			mnCyclesUntilNextSound = 18;
			pApp->pSoundManager->EmitFromObject(this, 7124);
			break;
		case 4:
			mnCyclesUntilNextSound = 18;
			pApp->pSoundManager->EmitFromObject(this, 7122);
			break;
		case 5:
			mnCyclesUntilNextSound = 11;
			pApp->pSoundManager->EmitFromObject(this, 7125);
			break;
		}
		moCurrentCommand.mnState++;
		if (moCurrentCommand.mnState == 6)
			moCurrentCommand.mnState = 0;
	}
	else
	{
		mnCyclesUntilNextSound--;
	}


	// CONTINUE CONVERSION
	FLOAT fMinHealth = max((float)1.0, poAttackee->mfMaxHealth * mpoClan->GetPercentHealthConversionFloor());	

	if (poAttackee->mfHealth > fMinHealth)
	{
		FLOAT nRequestedAmount = min(poAttackee->mfHealth - fMinHealth, mfFeedRate);

		if (nRequestedAmount > 0)
		{
			/* TBD -- CONVERSION SOUND
			if (mpoOwner == game->poLocalPlayer()) 
			{
				///vPlayRandomSound(TAG_Satk);
			}
			*/
			FLOAT nAcquired = poAttackee->vDamage(nRequestedAmount, this);

			// 
			// DON'T DO ANYTHING WITH IT.
			// 
			moStatus.vUpdateHealth(vGetRelativeHealth(mpoOwner));
		}
	}

	if (poAttackee->mnConvertCycleCount == 0)
	{
		// PERFORM THE UNIT CONVERSION
		MapNode * poNode = poAttackee->mpoCurNode;
		poAttackee->mpoCurNode = NULL;
		poNode->mpoUnit = NULL;
		mpoPlane->vRemoveFromDrawnList(poAttackee);
		Boolean bWasSelected = poAttackee->bIsSelected();
		if (bWasSelected)
		{
			mpoPlane->mpoMap->vRemoveSelection(poAttackee);
			mpoPlane->mpoMap->vCommitSelection();
		}
		

		Vampire * poNewVampire = NULL;
		//new Vampire(mpoOwner, mpoPlane, poNode, mpoClan->GetTemplateIdForSexAndLevel(SEX_MALE, 1));
		NonPlayerClass * poNpc = (NonPlayerClass * )poAttackee->vGetClan();

		if (SUCCEEDED(poNpc->vCreateConvert(&poNewVampire, this, poAttackee, poNode, mpoPlane)))
		{
			delete poAttackee;
			poAttackee = NULL;

			FireAnxiety(mpoClan->GetAnxietyBlastForConverting());

//			poNewVampire->mpoClan = mpoClan;
//			poNewVampire->vSetLevel(1);
//			poNewVampire->mpoOwner = mpoOwner;

			// SET HEALTH AND POWER ACCORDING TO CONVERTING VAMPIRE
			poNewVampire->mfHealth = poNewVampire->mfMaxHealth * (float)GetNewConvertHealth() / (float)100.0;
			poNewVampire->mfPower = poNewVampire->mfMaxPower * (float)GetNewConvertPower() / (float)100.0;

			poNewVampire->vCommandNone();
			poNewVampire->CommandSequenceClear();

			pApp->pSoundManager->EmitFromObject(poNewVampire, SOUND_MoanImmortality);

			//mpoOwner->vAddUnit(poNewVampire);
			
	//		poNewVampire->vCloak();
	//		poNewVampire->vSetBitmap(poNewVampire->nBitmapID());
	//		mpoAttackee->vCloak();

			if (bWasSelected) {
				mpoPlane->mpoMap->vAddSelection(poNewVampire);
				mpoPlane->mpoMap->vCommitSelection();
			}

			poNewVampire->moStatus.vUpdateHealth(poNewVampire->vGetRelativeHealth(mpoOwner));
			poNewVampire->moStatus.vShow();
	
			return TRUE;
		}
		else
		{
			// FAILED SO RESTORE
			poNode->mpoUnit = poAttackee;
			poAttackee->mpoCurNode = poNode;
			if (bWasSelected) 
			{
				mpoPlane->mpoMap->vAddSelection(poAttackee);
				mpoPlane->mpoMap->vCommitSelection();
			}
		}
	}	
	return FALSE;
}

FLOAT Vampire::vDamage(FLOAT fDamage, Unit * poUnitFrom)
{
	FLOAT fReturnDifference = mfHealth;

	mfHealth = max(0, mfHealth - fDamage);
	if (bIsVampireLord())
	{
		if (!poUnitFrom->bCanKillVampireLord())
		{
			mfHealth = max(1, mfHealth - fDamage);
		}
	}
	fReturnDifference = fReturnDifference - mfHealth;
	
	if (mfHealth == 0) 
	{
		mnDeathCycle = game->nGameCycle();
		mbAlive = FALSE;
		mpoCurNode->mpoUnit = NULL;
		vSetAction(ACT_DIE);
//		vSetBitmap(mpoTemplate->lGetValue(TAG_Bded, nBitmapID()));
		DropAll(mpoCurNode);
		vSetLayer(DP_ITEMS);
		mpoCurNode->vAddObjToFront(this);

		Missile * poMissile = new Missile(MISSILE_TIMERFADEDRAWOBJ, gpoMap->mpoPlane, this, mpoCurNode, mpoCurNode, 6019, 0, FALSE, 1.18f, NULL, 10.0, 10.0, 2, 0);
		poMissile->SetDrawObj(nGetID());
		poMissile->SetCyclesRemaining(200);

		vCommandNone();
		mbSelectable = FALSE;
		if (mbSelected) 
		{
			gpoMap->vRemoveSelection(this);
			gpoMap->vCommitSelection();
			vMarkUpdate();
		}

//		if (mpoOwner == game->poLocalPlayer()) 
//		{
		vPlayRandomSound(TAG_Sdth);
//		}
		
		//milestone win/lose screen
		if (bIsVampireLord())
		{
			if (mpoOwner == game->poLocalPlayer())
				LoseScreen *poLose = new LoseScreen();
			else
				WinScreen *poWin = new WinScreen();
		}

		// BLAST ANXIETY
		if (poUnitFrom && poUnitFrom->bIsVampire())
		{
			Vampire * poVampire = (Vampire *)poUnitFrom;
			poVampire->FireAnxiety(poVampire->mpoClan->GetAnxietyBlastForKilling());
		}
		
		// REMOVE UNIT FROM PLAYERS LIST
		mpoOwner->vDeleteUnit(this);
	} 
	else 
	{
		vSetAction(ACT_REACT);
		moStatus.vUpdateHealth(vGetRelativeHealth(mpoOwner)); //right now peasants even get updated

		if (mpoOwner == game->poLocalPlayer()) 
		{
			vPlayRandomSound(TAG_Sdm1, FALSE);
		}
	}

	return fReturnDifference;
}

/*
void Vampire::vScan()
{
	// CREATE LIST OF OBJECTS WITHIN VISIBLE RANGE
	int nCenterX = nGetX();
	int nCenterY = nGetY();

	int nLeft = max(0, nCenterX - mnVisibleRange);
	int nRight = min(gpoMap->nGetWidth() - 1, nCenterX + mnVisibleRange);
	int nTop = max(0, nCenterY - mnVisibleRange);
	int nBottom = min(gpoMap->nGetHeight() - 1, nCenterY + mnVisibleRange);

	moVisibleUnits.RemoveAll();

	for (int i=0; i < NPCCOUNTINDEX; i++)
	{
		mnVisibleTotal[i].attack = 0;
		mnVisibleTotal[i].defense = 0;
		mnVisibleTotal[i].level = 0;
		mnVisibleTotal[i].count = 0;
		mnVisibleTotal[i].relativehealth = 0;
	}

	int nPlayer;
	if (mpoOwner)
	{
		nPlayer = mpoOwner->mnPlayer;
	}
	else
	{
		nPlayer = NPCCOUNTINDEX-1;
	}

	// THIS
	mnVisibleTotal[nPlayer].count++;

	// ACCUMULATE TOTALS
	// TBD - JUST USE VISIBLE LEVELS 
	mnVisibleTotal[nPlayer].level += mnLevel; 
	mnVisibleTotal[nPlayer].attack += manUpgrade[UPGRADE_Attack];
	mnVisibleTotal[nPlayer].defense += manUpgrade[UPGRADE_Defend];
	mnVisibleTotal[nPlayer].relativehealth += nRelativeHealth();	

	// 
	//  TBD - BETTER SCAN ALGORITHM.  ALSO, INCLUDE VISIBILITY,
	//			LINE OF SIGHT,
	//
	int nRange = 1;
	while (nRange < mnVisibleRange)
	{
		MapNode * poNode;

		int y = nTop;
		int l = max(nLeft, nCenterX - nRange);
		int r = min(nRight, nCenterX + nRange);
		while(y <= nBottom)
		{	
			poNode = gpoMap->poGetNode(l, y);
			CheckNodeForVisibleUnits(poNode, nRange);
			poNode = gpoMap->poGetNode(r, y);
			CheckNodeForVisibleUnits(poNode, nRange);
			y++;
		}
		int x = nLeft + 1;
		int b = min(nBottom, nCenterY + nRange);
		int t = max(nTop, nCenterY - nRange);
		while (x < nRight)
		{
			poNode = gpoMap->poGetNode(x, b);
			CheckNodeForVisibleUnits(poNode, nRange);
			poNode = gpoMap->poGetNode(x, t);
			CheckNodeForVisibleUnits(poNode, nRange);
			x++;
		}
		nRange++;
	}
}
*/


void Vampire::vPowerToHealth()
{
	// HOW MUCH HEALTH IS NEEDED
	FLOAT fHealthNeeded = mfMaxHealth - mfHealth;

	if (fHealthNeeded > 0.0)	   	// HEALTH NEEDED
	{
		if (mfPower > 0.0)		// POWER IS AVAILABLE - ANY SHOULD BE USED FOR HEALING
		{
			// HOW MUCH POWER IS NEEDED
			assert(mpoClan->GetHealthPerPower() > 0);

			// USE UP TO <PERCENTAGE> OF CURRENT POWER
			FLOAT fRequestPower = min(mfPower * mpoClan->GetPercentPowerToHealth(), fHealthNeeded / mpoClan->GetHealthPerPower());

			assert(fRequestPower >= 0.0);

			//  CONVERT POWER TO HEALTH
			mfPower = mfPower - fRequestPower;
			mfHealth = mfHealth + fRequestPower * mpoClan->GetHealthPerPower();
			moStatus.vUpdateHealth(vGetRelativeHealth(mpoOwner));
		}
	}
}

void Vampire::vRegeneratePower()
{
	// POWER IS NEEDED
	if (mfPower < mfMaxPower)
	{
	
		mfPower = min(mfMaxPower, mfPower + mfMaxPower * GetPowerRegenPercent());
		moStatus.vUpdateHealth(vGetRelativeHealth(mpoOwner));
	}	
}

/*int Vampire::FindDirectionToOffset(int x, int y)
{
	assert(x!=0 || y!=0);  // NOT SAME NODE
	double dX = x;
	double dY = y;
	if (y == 0)
	{
		if (x > 0)
			return 0;
		else
			return 4;
	}
	else if (x == 0)
	{
		if (y > 0)
			return 6;
		else
			return 2;
	}
	double dAngle = atan(fabs((double)dY / (double)dX));

	// ADJUST dAngle
	if (y < 0)
	{
		if (x > 0)
		{
			
		}
		else
		{
			dAngle = PI - dAngle;
		}
	}
	else
	{
		if (x > 0)
		{
			dAngle = 2*PI - dAngle;
		}
		else
		{
			dAngle = PI + dAngle;
		}
	}
	dAngle = dAngle + 2*PI / 16;
	if (dAngle> 2*PI) dAngle = dAngle - 2*PI;
	int nReturn = (int)(dAngle / (2*PI / 8));
	assert(nReturn >= 0 && nReturn < 8);
	return nReturn;
}

*/
#define ADJUSTTOPERCENT(n, p) {n=(n)*(p);}
void Vampire::FindRetreatLocation(int nDistance, int * pnReturnX, int * pnReturnY)
{

	int aOffset[] = {	 1,  0, 
						 1, -1,
						 0, -1,
						-1, -1, 
						-1,  0,
						-1,  1,
						 0,  1,
						 1,  1 };


	double afDirectionScore[8] = { 1, 1, 1, 1, 1, 1, 1, 1 };

	//321	
	//4d0
	//567

	// ADJUST FOR GAME MAP SURROUNDINGS
	if (nGetX() == 0)
	{
		afDirectionScore[3] = 0;
		afDirectionScore[4] = 0;
		afDirectionScore[5] = 0;
	}
	else if (nGetX() == gpoMap->nGetWidth() - 1)
	{
		afDirectionScore[1] = 0;
		afDirectionScore[0] = 0;
		afDirectionScore[7] = 0;
	}

	if (nGetY() == 0)
	{
		afDirectionScore[1] = 0;
		afDirectionScore[2] = 0;
		afDirectionScore[3] = 0;
	}
	else if (nGetY() == gpoMap->nGetHeight() - 1)
	{
		afDirectionScore[7] = 0;
		afDirectionScore[6] = 0;
		afDirectionScore[5] = 0;
	}
	

	int nX = nGetX();
	int nY = nGetY();
	int j;
	for (j = 0; j<8; j++)
	{
		if (afDirectionScore[j] == 1)
		{
			MapNode * poNode = gpoMap->poGetNode(nX + aOffset[j*2], nY + aOffset[j*2 + 1]);
			TERRAINTYPE t = poNode->eTerrainType();
			switch (t)
			{
			case TT_CLEAR:
				break;
			case TT_OBSTACLE:
				afDirectionScore[j] = 0;
				break;
			default:
				afDirectionScore[j] = 0.5;
				break;
			}
		}
	}

	// ADJUST FOR GAME MAP POSITION
	if ((double)nGetX() > (0.9 * ((double)gpoMap->nGetWidth()-1)))
	{
		ADJUSTTOPERCENT(afDirectionScore[2], 0.07);
		ADJUSTTOPERCENT(afDirectionScore[1], 0.03);
		ADJUSTTOPERCENT(afDirectionScore[0], 0.015);
		ADJUSTTOPERCENT(afDirectionScore[7], 0.03);
		ADJUSTTOPERCENT(afDirectionScore[6], 0.07);
	}
	else if ((double)nGetX() < (0.1* ((double)gpoMap->nGetWidth()-1)))
	{
		ADJUSTTOPERCENT(afDirectionScore[6], 0.07);
		ADJUSTTOPERCENT(afDirectionScore[5], 0.03);
		ADJUSTTOPERCENT(afDirectionScore[4], 0.015);
		ADJUSTTOPERCENT(afDirectionScore[3], 0.03);
		ADJUSTTOPERCENT(afDirectionScore[2], 0.07);
	}
	if ((double)nGetY() < (0.1 * ((double)gpoMap->nGetHeight()-1)))
	{
		ADJUSTTOPERCENT(afDirectionScore[0], 0.07);
		ADJUSTTOPERCENT(afDirectionScore[1], 0.03);
		ADJUSTTOPERCENT(afDirectionScore[2], 0.015);
		ADJUSTTOPERCENT(afDirectionScore[3], 0.03);
		ADJUSTTOPERCENT(afDirectionScore[4], 0.07);
	}
	else if ((double)nGetY() > (0.9 * ((double)gpoMap->nGetHeight()-1)))
	{
		ADJUSTTOPERCENT(afDirectionScore[4], 0.07);
		ADJUSTTOPERCENT(afDirectionScore[5], 0.03);
		ADJUSTTOPERCENT(afDirectionScore[6], 0.015);
		ADJUSTTOPERCENT(afDirectionScore[7], 0.03);
		ADJUSTTOPERCENT(afDirectionScore[0], 0.07);
	}

	int i = moVisibleUnits.GetSize();
	while(i)
	{
		i--;
		Unit * poUnit = (Unit*)goObjectDB.poGetObject(moVisibleUnits.GetItem(i));
		if (poUnit == NULL) continue;
		if ((poUnit == this) || !poUnit->bAlive()) continue;

		int nDistance = poUnit->lDistToSpace(nGetX(), nGetY());
		int nDirection = FindDirectionToOffset(poUnit->nGetX() - nGetX(), poUnit->nGetY() - nGetY());
		if (nDistance == 1)
		{
			// THIS DIRECTION SCORE SHOULD BE MULTIPLIED BY 0
			afDirectionScore[nDirection] = 0.0;
			if (poUnit->bIsMobMember() || poUnit->bIsVampire())
			{
				// ADJACENT DIRECTIONS SHOULD BE MULTIPLIED BY 0.5
				int nOtherDirection;

				nOtherDirection = nDirection + 1;
				if (nOtherDirection == 8) nOtherDirection = 0;
				afDirectionScore[nOtherDirection] = afDirectionScore[nOtherDirection] * 0.5 / (1 + poUnit->bIsVampire());

				nOtherDirection = nDirection - 1;
				if (nOtherDirection == -1) nOtherDirection = 7;
				afDirectionScore[nOtherDirection] = afDirectionScore[nOtherDirection] * 0.5 / (1 + poUnit->bIsVampire());
			}
			continue;
		}
		else if (nDistance <= 4)
		{
			// THIS DIRECTION SCORE SHOULD BE MULTIPLIED BY 0.25
			afDirectionScore[nDirection] = afDirectionScore[nDirection] * 0.25;
			if (poUnit->bIsMobMember() || poUnit->bIsVampire())
			{
				// ADJACENT DIRECTIONS SHOULD BE MULTIPLIED BY 0.75
				int nOtherDirection;

				nOtherDirection = nDirection + 1;
				if (nOtherDirection == 8) nOtherDirection = 0;
				afDirectionScore[nOtherDirection] = afDirectionScore[nOtherDirection] * 0.75;

				nOtherDirection = nDirection - 1;
				if (nOtherDirection == -1) nOtherDirection = 7;
				afDirectionScore[nOtherDirection] = afDirectionScore[nOtherDirection] * 0.75;
			}
			continue;
		}
		else
		{
			// THIS DIRECTION SCORE SHOULD BE MULTIPLIED BY 0.5 * nDistance / (1+2*poUnitEnemy->bIsVampire())
			afDirectionScore[nDirection] = afDirectionScore[nDirection] * 0.5 * (nDistance / mnVisibleRange) / 2*poUnit->bIsVampire(); 
			continue;
		}
	}
	double dMax = -1;
	int nDirection = -1;
	int nCountOfMaxScores;
	nCountOfMaxScores = 0;
	
	for (i=0; i<8; i++)
	{
		if (afDirectionScore[i] == dMax)
		{
			nCountOfMaxScores++;
		} 
		else if (afDirectionScore[i] > dMax)
		{
			nDirection = i;
			nCountOfMaxScores = 1;
			dMax = afDirectionScore[i];
		}
	}
	if (mnLastRetreatDirection != -1)
	{
		// DIFFERENT DIRECTION
		if (nDirection != mnLastRetreatDirection)
		{
			// NO IMPROVEMENT SO USE SAME DIRECTION
			if (afDirectionScore[mnLastRetreatDirection] >= afDirectionScore[nDirection])
			{
				nDirection = mnLastRetreatDirection;
			}
			else
			{
				// IMPROVEMENT SO SELECT FURTHEST COANGULAR ONE (UNLESS THERE's JUST ONE)
				if (nCountOfMaxScores>1)
				{
					// int nPickOne = GlobalRand(nCountOfMaxScores);
					int nIn = mnLastRetreatDirection + 1;
					int nOut = mnLastRetreatDirection - 1;
					i = 8;
					while (i)
					{
						if (nIn == 8) nIn = 0;
						if (nOut == -1) nIn = 7;
						i--;
						if (afDirectionScore[nIn] == dMax)
						{
							nDirection = nIn;
							break;
						}
						i--;
						if (afDirectionScore[nOut] == dMax)
						{
							nDirection = nOut;
							break;
						}
						nOut--;
						nIn++;
					}
				}
			}
		}
	}
	mnLastRetreatDirection = nDirection;
	switch(nDirection)
	{
		case -1:
			*pnReturnX = 0;
			*pnReturnY = 0;
			break;
		default:
			*pnReturnX = aOffset[nDirection*2];
			*pnReturnY = aOffset[nDirection*2 + 1];
			break;
	}
}

void Vampire::vGameCycle()
{
	// we'll assume we're not moving until otherwise contradicted (in vMoveOneCycle)
	mbMoving = FALSE;

	if (mpoThrallOwner)
	{
		mnThrallCyclesRemaining--;
		if (mnThrallCyclesRemaining)
		{
			return;
		}
		mpoThrallOwner = NULL;
	}

	if (mnUnderAttackCycles)
	{
		mnUnderAttackCycles--;
	}
	if (mnCyclesBeforeCast)
	{
		mnCyclesBeforeCast--;
	}

	if (!bAlive())
	{
		vCommandNone();
		return;
	}

	if (bIsVampire())		// LIVING VAMPIRE
	{
		vRegeneratePower();
		vPowerToHealth();
		ProcessUpgrades();
		DecrementCycleCounts();
	} 
		
	vScan();

	// if he's en-route to a square, he's going to move no matter what
	if (mbBetweenPoints) 
	{
		vMoveOneCycle();
		return;
	}

	vThink();

	// we're at a square -- move, stay or fight?
	switch (moCurrentCommand.meType) {
	case CM_NONE:
		{
			if (IsGuarded())
			{
				mpoGuardCoordinator->Idle();
			}
			return;
		}
		break;
	case CM_GUARD_MOVEINTOPOSITION:

		// AT DESTINATION?
		if (mpoCurNode->mnXPos == moCurrentCommand.mnX && mpoCurNode->mnYPos == moCurrentCommand.mnY) 
		{
			// POST A GUARD_READY MESSAGE TO GUARDED UNIT
			vCommandNone();
			if (mpoGuardedUnit)
			{
				mpoGuardedUnit->Message(this, GUARD_READY);
			}
			
			return;
		}
		vMoveToNextNode();

		// TBD - check bad start position / blocked / send busy

			/*			
			
			if (pNode == NULL)
			{
				vCommandNone();
				mpoGuardedUnit->Message(this, GUARD_BUSY);
			}
		
			*/
		
		break;
	case CM_CAST:
		{
			// IF CLOAKED TAKE ONE CYCLE TO UNCLOAK
			
			if (mbCloaked && !bSpellCanCastWhileCloaked((SPELL_TYPE)mnCurrentSpell))
			{
				// CHECK FOR ABILITY TO PERFORM COMMAND BEFORE UNCLOAKING
				if (bAbleToPerformCommand(moCurrentCommand))
				{
					vUncloak();
				}
				else
				{
					vCommandNone();
				}
				return;
			}
			else
			{
				// CHECK FOR ABILITY TO PERFORM COMMAND BEFORE PERFORMING ACTION
				if (moCurrentCommand.mnState || bAbleToPerformCommand(moCurrentCommand))
				{
					if (moCurrentCommand.mnState == 0)
					{
						moCurrentCommand.mnState = 1;
					}
					UINT unRange;
					unRange = 0;
					Spell * poSpell = game->GetSpell(mnCurrentSpell);
					unRange = poSpell->GetRange(this);
					// GetSpellRange(mnCurrentSpell, &unRange);

					int nTargetX;
					int nTargetY;

					DrawObj *poTarget = NULL;

					// check for bad target
					if (moCurrentCommand.mbTargetUnit) 
					{
						poTarget = goObjectDB.poGetObject(moCurrentCommand.mnUnit);

						// if the target is gone or dead, we're done
						if (!poTarget || !poTarget->bAlive()) {
							vCommandNone();
							return;
						}
					}
					moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);

					if (unRange > 0)
					{
						MapNode * poTargetNode = moCurrentCommand.poGetTargetNode();
						if (unRange < (UINT)lDistToSpace(nTargetX, nTargetY))
						{
							// MOVE CLOSER

							// TARGET MOVED
							if (nTargetX != mnLastTargetX || nTargetY != mnLastTargetY) 
							{
								vCalcRoute();
							}
					
							vMoveToNextNode();
							return;
						}
					}

					if (meAction != ACT_CAST) 
					{
						//1st we need to turn in direction we're going to cast
						int nTargetX, nTargetY;
						moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);
						vChangeFacing(mpoCurNode->mnXPos, mpoCurNode->mnYPos, nTargetX, nTargetY);
//						DIRECTION eDir = eDirection(mpoCurNode->mnXPos, mpoCurNode->mnYPos, nTargetX, nTargetY);
//						vDebugOut("dir:%d\n",(int)eDir);
//						if (eDir != D_NOCHANGE)
//						{
//							meDirection = eDir;
//						}

						vSetAction(ACT_CAST);
						mnCelCastDelay = game->GetSpell(mnCurrentSpell)->lCelCastDelay();
						return;
					}
					else //we're in our 'casting' animation. 
					{
						if (mnCel < mnCelCastDelay)	// we want to see certain number of cels of the animation before spell actually starts
							return;
					}
					
					if (moCurrentCommand.mnState == 1 && mnCel == mnCelCastDelay)
					{
						// CAST THE SPELL
						moCurrentCommand.mnState = 2;						
						mnCyclesBeforeCast = poSpell->GetCastResetAfterSpell();
						vCast();
					}
					else if (moCurrentCommand.mnState == 2 && (mnCel == nGetNumCels()-1))
					{
						vCommandNone();
					}
				}
				else
					vCommandNone();
			}
		}
		break;
	case CM_FEED:
		{
			int nTargetX;
			int nTargetY;

			Unit * poTarget;

			// check for bad target
			if (moCurrentCommand.mbTargetUnit) 
			{
				poTarget = (Unit *)goObjectDB.poGetObject(moCurrentCommand.mnUnit);

				// if the target is gone or dead, we're done
				if (!poTarget || !poTarget->bAlive()) {
					vCommandNone();
					return;
				}
			}
			else
			{
				return;
			}
			
			moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);

			// are we close enough to uncloak and dash? Then do so.
			int nTargetDist = lDistToSpace(nTargetX, nTargetY);

			if (nTargetDist <= GetRangeToDecloak()) 
			{
				poTarget->SetThrallOwner(this, 50);
				vUncloak();
			}

			// are we close enough to attack? Then attack, don't move
			if (nTargetDist <= mnAttackRange) 
			{
				// IS FOOD STRONG ENOUGH TO BE FED UPON?
				if (mfPower < mfMaxPower  && poTarget->mfHealth > (float)poTarget->mfMaxHealth * mpoClan->GetPercentMinimumFeedingHealth())
				{
					Unit * poUnit = (Unit *) poTarget;
					if ((poUnit->mpoThrallOwner == this)||(poUnit->mpoThrallOwner == NULL))
					{
						poUnit->SetThrallOwner(this, 50);
						if (meAction != ACT_FEED) {
							//set attacker animation
							int nTargetPosX;
							int nTargetPosY;
							moCurrentCommand.vGetTargetPos(&nTargetPosX, &nTargetPosY);
							vChangeFacing(mpoCurNode->mnXPos, mpoCurNode->mnYPos, nTargetPosX, nTargetPosY);
							vSetAction(ACT_FEED);
							//now set attackee animation
							poTarget->vSetAction(ACT_FEED);
						}
						vFeed();
					}
				}
				else
				{
					vCommandNone();
				}
			}
			else 
			{
				// move closer

				// if the target has moved, recalculate the path
				if (nTargetX != mnLastTargetX || nTargetY != mnLastTargetY) {
					vCalcRoute();
				}

				// so now we move.
				vMoveToNextNode();
			}
			break;
		}
	case CM_CONVERT:
		{
			int nTargetX;
			int nTargetY;

			DrawObj *poTarget;

			// CHECK FOR BAD TARGET
			if (moCurrentCommand.mbTargetUnit) 
			{
				poTarget = goObjectDB.poGetObject(moCurrentCommand.mnUnit);

				// if the target is gone or dead, we're done
				if (!poTarget || !poTarget->bAlive()) 
				{
					vCommandNone();
					return;
				}
			}
			
			moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);

			// CHECK DISTANCE
			int nTargetDist = lDistToSpace(nTargetX, nTargetY);

			Unit * poUnit = (Unit *) poTarget;

			// CLOSE ENOUGH TO DECLOAK?
			if (nTargetDist <= GetRangeToDecloak()) 
			{
				// IF UNIT IS HUMAN
				if (poUnit->bIsHuman()&&((poUnit->mpoThrallOwner == NULL)||(poUnit->mpoThrallOwner == this)))
				{
					if (mbCloaked)
					{
						// CHECK FOR ABILITY TO PERFORM COMMAND BEFORE UNCLOAKING
						if (bAbleToPerformCommand(moCurrentCommand))
						{
							if (poUnit->mpoThrallOwner != this)
							{
								poUnit->SetThrallOwner(NULL, 50);
							}
							else
							{
								poUnit->SetThrallOwner(this, 50);
							}
							vUncloak();
							return;
						}
						else
						{
							// CANCEL THE COMMAND
							vCommandNone();
							return;
						}
					}
					// OTHERWISE FALL THROUGH
				}
				// UNIT NOT HUMAN OR UNIT IS BUSY
				else
				{
					vCommandNone();
					return;
				}
			}
			
			// WITHIN RANGE FOR ATTACK?
			if (nTargetDist <= mnAttackRange) 
			{
				// NOT A VAMPIRE OR OTHER CREATURE
				// NOT OWNED BY SOMEONE ELSE
				
				//if ((poUnit->bIsHuman())&&((poUnit->mpoThrallOwner == NULL)||(poUnit->mpoThrallOwner == this)))
				
				// NOT OWNED BY ANYONE INCLUDING SELF
				if (poUnit->mpoThrallOwner == NULL)
				{
					// RESET THE CONVERSION CYCLE
					poUnit->mnConvertCycleCount = GetConversionSpeed();

					// PAY POWER UP FRONT
					if (bAbleToPerformCommand(moCurrentCommand))
					{
						mfPower = mfPower - GetPowerCostToConvert();
						moStatus.vUpdateHealth(vGetRelativeHealth(mpoOwner));
					}
					// NOT ENOUGH POWER
					else  
					{
						// STUN THE UNIT AND DISOWN THE UNIT
						poUnit->SetThrallOwner(NULL, 50);
						vCommandNone();
						return;
					}
				}
				// STUN THE UNIT
				poUnit->SetThrallOwner(this, 50);
			
				// ABLE TO THRALL?
				if (poUnit->mpoThrallOwner == this)
				{

					//set up animations
					if (meAction != ACT_FEED) {
						//set attacker animation
						int nTargetPosX;
						int nTargetPosY;
						moCurrentCommand.vGetTargetPos(&nTargetPosX, &nTargetPosY);
						vChangeFacing(mpoCurNode->mnXPos, mpoCurNode->mnYPos, nTargetX, nTargetY);
						vSetAction(ACT_FEED);
						//now set attackee animation
						poUnit->vSetAction(ACT_FEED);
						mnCyclesUntilNextSound = 0;
					}

					if (vConvert())
					{
						// CONVERSION COMPLETED
						vCommandNone();
						return;
					}
				}
				// UNABLE TO THRALL SO ATTACK
				else
				{
					// ISSUE THIS AS A NON-INTERNAL COMMAND
					// SO TENACITY IS ADJUSTED
					Command oCommand;
					oCommand.meType = CM_ATTACK;
					oCommand.mbTargetUnit = TRUE;
					oCommand.mnUnit = poUnit->nGetID();
					vCommand(oCommand);
				}
/*				}
				else
				{
					vCommandNone();
				}*/
			}
			else 
			{
				// move closer

				// if the target has moved, recalculate the path
				if (nTargetX != mnLastTargetX || nTargetY != mnLastTargetY) {
					vCalcRoute();
				}

				// so now we move.
				vMoveToNextNode();
			}
		}
		break;

	case CM_ATTACK:
		{
			int nTargetX, nTargetY;
			Unit *poTarget;

			// check for bad target
			if (moCurrentCommand.mbTargetUnit) 
			{
				poTarget = (Unit *)goObjectDB.poGetObject(moCurrentCommand.mnUnit);

				// if the target is gone or dead, we're done
				if (!poTarget || !poTarget->bAlive()) 
				{
					vCommandNone();
					return;
				}
			}
			
			moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);

			// are we close enough to uncloak and dash? Then do so.
			int nTargetDist = lDistToSpace(nTargetX, nTargetY);

			if (!bCanAttack())
			{
				// CANNOT DO THAT WHILE HOLDING A POWER STONE
				if (bIsLocalVampire() && !moCurrentCommand.mbInternal)
				{
					pApp->pSoundManager->EmitFromObject(this, SOUND_RefuseAttackHoldingAPowerStone);
				}
				vCommandNone();
				return;
			}

			if (nTargetDist <= GetRangeToDecloak()) 
			{
				vUncloak();
			}

			// THIS IS NOT 0 IF INTERNAL COMMAND
			// SO IF INTERNALLY ISSUED COMMAND (IE. NOT A DIRECT COMMAND)
			// CONSIDER WHETHER TO CONTINUE THE ATTACK
			if (moCurrentCommand.mbInternal)
			{
				// CONTINUE THE ATTACK?
				// IF NOT UNDER ATTACK
				// AND AGGRESSION SAYS LEAVE
				if (!mnUnderAttackCycles && !bAttackEnemyUnit(poTarget))	
				{
					// STOP THE ATTACK
					vCommandNone();					
					return;
				}
			}

			// are we close enough to attack? Then attack, don't move
			if (nTargetDist <= mnAttackRange) 
			{
				//Unit * poUnit = (Unit *) poTarget;
				//if ((poUnit->mpoThrallOwner == this)||(poUnit->mpoThrallOwner == NULL))
				//{
					//poUnit->SetThrallOwner(this, 50);
				//}

				vChangeFacing(mpoCurNode->mnXPos, mpoCurNode->mnYPos, nTargetX, nTargetY);
				vSetAction(ACT_ATTACK);
				vAttack();
			} 
			else 
			{
				// move closer

				// if the target has moved, recalculate the path
				if (nTargetX != mnLastTargetX || nTargetY != mnLastTargetY) 
				{
					vCalcRoute();
				}

				// so now we move.
				vMoveToNextNode();
			}
			break;
		}
		
	case CM_GUARD_MOVETOVERIFY:
	case CM_GUARD_MOVEINTERNAL:
		if (mpoCurNode->mnXPos == moCurrentCommand.mnX && mpoCurNode->mnYPos == moCurrentCommand.mnY) 
		{
			if (IsPatrolling())
			{
				mpoMovement->ProceedToNext();
				Patrol();
			}
			else
			{
				if (CM_GUARD_MOVETOVERIFY == moCurrentCommand.meType)
				{
					assert(mpoGuardedUnit);

					vCommandNone();

					if (mpoGuardedUnit)
					{
						mpoGuardedUnit->Message(this, GUARD_VERIFIED);
					}
				}
				else
				{
					vCommandNone();
				}
			}
			return;
		}
		// move
		vMoveToNextNode();

		break;
	case CM_GIVE:
		{
			int nTargetX, nTargetY;
			Unit *poTarget;

			if (moCurrentCommand.mbTargetUnit) 
			{
				poTarget = (Unit *)goObjectDB.poGetObject(moCurrentCommand.mnUnit);
				
				if (!poTarget) 
				{
					vCommandNone();
					return;
				}
			}
			
			moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);

			int nTargetDist = lDistToSpace(nTargetX, nTargetY);

			if (nTargetDist <= 1) 
			{
/*				if (moCurrentCommand.mnNewValue == 3) //tw - commented these out
				{
					while (moItems.GetSize())
					{
						Give(poTarget, moItems.GetItem(0));
					}
				}
				else
				{
*/					if (moItems.GetSize() > moCurrentCommand.mnNewValue)
					{
						Give(poTarget, moItems.GetItem(moCurrentCommand.mnNewValue));
					}
//				}
				vCommandNone();
				return;
			} 
			else 
			{
				// MOVE CLOSER
				if (nTargetX != mnLastTargetX || nTargetY != mnLastTargetY) 
				{
					vCalcRoute();
				}
				vMoveToNextNode();
			}
			break;
		}
		break;
	case CM_DROP:
		{
			int nTargetX, nTargetY;
			
			moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);

			int nTargetDist = lDistToSpace(nTargetX, nTargetY);

			if (nTargetDist <= 1) 
			{
				if (moCurrentCommand.mnNewValue == 3)
				{
					while (moItems.GetSize())
					{
						Drop(moCurrentCommand.poGetTargetNode(), moItems.GetItem(0));
					}
				}
				else
				{
					if (moItems.GetSize() > moCurrentCommand.mnNewValue)
					{
						Drop(moCurrentCommand.poGetTargetNode(), moItems.GetItem(moCurrentCommand.mnNewValue));
					}
				}
				vCommandNone();
				return;
			} 
			else 
			{
				// MOVE CLOSER
				if (nTargetX != mnLastTargetX || nTargetY != mnLastTargetY) 
				{
					vCalcRoute();
				}
				vMoveToNextNode();
			}
			break;
		}
		break;

	case CM_TAKE:
		{
			int nTargetX, nTargetY;
			Item *poTarget;

			if (moCurrentCommand.mbTargetUnit) 
			{
				poTarget = (Item *)goObjectDB.poGetObject(moCurrentCommand.mnUnit);
				
				if (!poTarget) 
				{
					vCommandNone();
					return;
				}
			}
			
			moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);

			int nTargetDist = lDistToSpace(nTargetX, nTargetY);

			if (nTargetDist <= 0) 
			{
				Take(poTarget->poNode(), poTarget);
				vCommandNone();
				return;
			} 
			else 
			{
				if (nTargetX != mnLastTargetX || nTargetY != mnLastTargetY) 
				{
					// STOP THE TAKE
					vCommandNone();
					return;
				}
				vMoveToNextNode();
			}
			break;
		}
		break;
	case CM_RETREAT:
		//	1) REMAIN UNCLOAKED DURING RETREAT
		if (!mnAdrenalineCycleTicker)
		{
			// POP ADRENALINE IF NOT ALREADY
			// THIS WILL UNCLOAK IF NEEDED	
			mnAdrenalineCycleTicker = mpoClan->GetAdrenalineDuration();
			SetSpeedAdrenalineBoost();
		}

		// MAKE SURE WE ARE UNCLOAKED
		if (mbCloaked)
		{
			vUncloak();
			// TBD -- SHOULD WE DELAY ONE CYCLE
		}
		
		// DETERMINE WHERE TO GO
		BOOL bDone;
		bDone = FALSE;
		int nX;
		int nY;

		if (moCurrentCommand.mnState == STATE_COMMAND_NONE)
		while (!bDone)
		{
			switch (moCurrentCommand.mnState)
			{
			case STATE_COMMAND_NONE:
				{
					int nUnitCount = mpoOwner->GetLiveFriendlyUnitCount();
					if (nUnitCount == 1)
					{
						//	2) if last vampire 
						//		run away from enemy
						moCurrentCommand.mnState = STATE_RETREAT_RUNAWAYFROMENEMY;
						// TBD -- FIND DIRECTION AND PLACE TO RUN
					}
					else
					{
						Vampire * poVampire = mpoOwner->FindFriendlyUnitToRetreatTowards(this);
						// THIS SHOULD ALWAYS FIND A VAMPIRE
						assert(poVampire);
						moCurrentCommand.mnState = STATE_RETREAT_TOUNIT;
						moCurrentCommand.mbTargetUnit = TRUE;
						moCurrentCommand.mnUnit = poVampire->nGetID();
					}
				}			
				break;
			case STATE_RETREAT_RUNAWAYFROMENEMY:
				// TBD - FIND MOST UNPOPULATED DIRECTION FROM ENEMY -- THEN RUN!!!!
				{
					nX = 1;
					nY = 1;
					FindRetreatLocation(1, &nX, &nY);
					//nX = nX * 4;
					//nY = nY * 4;
					
					moCurrentCommand.mnX = nGetX() + nX;
					moCurrentCommand.mnY = nGetY() + nY;
					moCurrentCommand.mnX = max(0, moCurrentCommand.mnX);
					moCurrentCommand.mnX = min(gpoMap->nGetWidth()-1, moCurrentCommand.mnX);

					moCurrentCommand.mnY = max(0, moCurrentCommand.mnY);
					moCurrentCommand.mnY = min(gpoMap->nGetHeight()-1, moCurrentCommand.mnY);
				}
				bDone = TRUE;
				break;	
			case STATE_RETREAT_TOUNIT:
				bDone = TRUE;
				break;	
			}
		}

		BOOL bCancel;
		bCancel = FALSE;

		// CONTINUE TOWARDS DESTINATION UNTIL THERE ARE NO ENEMY IN VISIBLE RANGE
		//  OR REGAINED ENOUGH HEALTH TO JOIN FIGHT
		{
			int nCount = 0;
			int nNumPlayers = game->nGetPlayerCount();
			int nThisPlayer = mpoOwner->mnPlayer;
			int i;

			// COUNT ENEMY UNITS
			for (i=0;i<nThisPlayer;i++)
			{
				nCount += mnVisibleTotal[i].count;
			}
			for (i=nThisPlayer+1;i<nNumPlayers;i++)
			{
				nCount += mnVisibleTotal[i].count;
			}
			nCount += mnVisibleTotal[NONPLAYERINDEX].count;
			if (nCount == 0 || bStayAndFight())
			{
				vCommandNone();
				return;
			}
		}

		// STATE_RETREAT_TOUNIT
		if (moCurrentCommand.mbTargetUnit)
		{
			int nTargetX, nTargetY;
			Unit *poTarget;

			poTarget = (Unit *)goObjectDB.poGetObject(moCurrentCommand.mnUnit);
			
			if (!poTarget || !poTarget->bAlive())
			{
				// OUR TARGET HAS BEEN DELETED
				moCurrentCommand.mnState = STATE_COMMAND_NONE;
				return;
			}
			
			moCurrentCommand.vGetTargetPos(&nTargetX, &nTargetY);

			int nTargetDist = lDistToSpace(nTargetX, nTargetY);

			if (nTargetDist <= 2)
			{
				// CLOSE ENOUGH
				vCommandNone();
				return;
			} 
			else 
			{
				// MOVE CLOSER
				if (nTargetX != mnLastTargetX || nTargetY != mnLastTargetY) 
				{
					vCalcRoute();
				}
				vMoveToNextNode();
			}
		}
		else	// STATE_RETREAT_RUNAWAYFROMENEMY
		{
			if (mpoCurNode->mnXPos == moCurrentCommand.mnX && mpoCurNode->mnYPos == moCurrentCommand.mnY) 
			{
				moCurrentCommand.mnState = STATE_COMMAND_NONE;
				//vCommandNone();
				return;
			}
			vMoveToNextNode();
			break;
		}

		//	7) continue retreating until no visible enemy or enough health to continue attack

		break;
	case CM_MOVE:

		// are we at the destination? Then kill this move stuff
		if (mpoCurNode->mnXPos == moCurrentCommand.mnX && mpoCurNode->mnYPos == moCurrentCommand.mnY) {
			if (IsPatrolling())
			{
				mpoMovement->ProceedToNext();
				Patrol();
			}
			else
			{
				vCommandNone();
			}
			return;
		}
		if (IsGuarded())
		{
			assert(mpoGuardCoordinator);
			{
				BOOL bInterior = (moCurrentCommand.mnUnit == (ID_NONE - 1));
				mpoGuardCoordinator->Move(gpoMap->poGetNode(moCurrentCommand.mnX, moCurrentCommand.mnY), bInterior);
			}
			vCommandNone();
			return;
		}
		else
		{
			// move
			vMoveToNextNode();
		}
		break;
	case CM_WAIT_500MS:
		{
			if (!(mdwFlags & UNIT_FLAG_WAITING))
			{
//				vDebugOut("Started Waiting...\n");
				mdwFlags |= UNIT_FLAG_WAITING;
				mnStartWaitCycle = game->nGameCycle();
			}

			int nCurCycle = game->nGameCycle();
#define CYCLES_500MS	10
			if ((nCurCycle - mnStartWaitCycle) >= CYCLES_500MS) 
			{
//				vDebugOut("Finished Waiting...\n");
				mdwFlags = mdwFlags & (~(DWORD)UNIT_FLAG_WAITING );
				vCommandNone();
				return;
			}
		}
		break;

	default:
		break;
	} //switch
}


void Vampire::vCommand(Command &oCommand, BOOL bInternal)
{
	// dead units don't accept commands
	if (!mbAlive) {
		return;
	}

	if (!bInternal)
	{
		if (mpoPlane->mpoMap->GetMode() == UIMODE_ENTERCOMMANDS)
		{
			if (!mpoSequencer)
			{
				mpoSequencer = new SequentialCommands(*this);
			}
			mpoSequencer->Add(&oCommand);
			return;
		}
	}


	// unowned units can't accept outside commands
	if (!bInternal && !mpoOwner) {
		return;
	}

	// we don't accept commands from other players, either
	if (!bInternal && mpoOwner->mnPlayer != oCommand.mnPlayer) {
		return;
	}

	// TRACK INTERNAL COMMAND FOR LATER REFERENCE
	oCommand.mbInternal = bInternal;

	switch(oCommand.meType) {
	case CM_ATTR:
		if ((oCommand.mnAttributeID >= ATTR_NEWUPGRADE_00)&&(oCommand.mnAttributeID < ATTR_NEWUPGRADE_TOPPLUSONE))
		{
			UPGRADE_NAME upgrade = (UPGRADE_NAME)((int)oCommand.mnAttributeID - (int)ATTR_NEWUPGRADE_00);

			if (manDesired[upgrade] > oCommand.mnNewValue)
			{
				// IF UPGRADING THIS UPGRADE
				if (mbUpgrading && mUpgradeName == upgrade)
				{
					if (oCommand.mnNewValue <= manUpgrade[upgrade])	
					{
						// CANCEL THE CURRENT UPGRADE
						mbUpgrading = FALSE;
						mUpgradeName = UPGRADE_NONE;
						mfUpgradeProgress = 0.0;
						manDesired[upgrade] = manUpgrade[upgrade];
						if (mbSelected)
							gpoInterface->vUpdateButtonState(upgrade);
						break;
					}
				}
			}
			if (manUpgrade[upgrade] >= oCommand.mnNewValue)
			{
				manDesired[upgrade] = manUpgrade[upgrade];
				if (mbSelected)
					gpoInterface->vUpdateButtonState(upgrade);
				break;
			}
			manDesired[upgrade] = oCommand.mnNewValue;
			if (mbSelected)
				gpoInterface->vUpdateButtonState(upgrade);
			break;
			//NetworkUpgradeReceived(upgrade);
		}
		else
		{
			switch(oCommand.mnAttributeID)
			{
			case ATTR_AGGRESSION:
				mAggression = (SETTING_LMH) oCommand.mnNewValue;
				if (mbSelected)
					gpoInterface->vUpdateButtonState(ATTR_AGGRESSION);
				break;
			case ATTR_TENACITY:
				mTenacity = (SETTING_LMH) oCommand.mnNewValue;
				if (mbSelected)
					gpoInterface->vUpdateButtonState(ATTR_TENACITY);
				break;
			case ATTR_HEALTH:
				mfHealth = (float) oCommand.mnNewValue;
				break;
			case ATTR_CONVERSION:
				mConversion = (SETTING_LMH) oCommand.mnNewValue;
				break;
			case ATTR_READYSPELL:
				mnCurrentSpell = oCommand.mnNewValue;
				Spell * poSpell;
				poSpell = game->GetSpell(mnCurrentSpell);
				if (poSpell->bNoTargetNeeded())
				{
					poSpell->CastFromUnit(this, NULL, mpoCurNode);
				}
				else
				{
					switch(mnCurrentSpell)
					{
					case SPELL_DEATHFROMABOVE:
						{
							pApp->pResourceManager->PreloadResource(RES_SOUND, 7301);
							pApp->pResourceManager->PreloadResource(RES_SOUND, 7302);
							pApp->pResourceManager->PreloadResource(RES_BITMAP, 67700);
						}
						break;
					case SPELL_LIGHTNINGBOLT:
						{
							pApp->pResourceManager->PreloadResource(RES_SOUND, 7303);
						}
						break;
					}
				}
				break;
			case ATTR_DEFAULT_ATTACK:
				vSetDefaultAttack((DEFAULT_ATTACK)oCommand.mnNewValue);
				break;
			}
		}
		break;

	case CM_GUARD_MOVEINTOPOSITION:
		moCurrentCommand = oCommand;
		vCalcRoute();
		break;
	case CM_GUARD_MOVEINTERNAL:				// GENERAL MOVE COMMAND
	case CM_GUARD_MOVETOVERIFY:
		// the old route is certainly dead
		moCurrentCommand = oCommand;
		vCalcRoute();
		break;

	case CM_PATROL_WAYPOINT:
		// create waypoint
		// continue waypoint thought
		AddPatrolWaypoint(oCommand.mnX, oCommand.mnY);
		Patrol();
		break;

	case CM_DROP:
	case CM_GIVE:
	case CM_TAKE:
		{
			moCurrentCommand = oCommand;
			vCalcRoute();
		}
		break;

	case CM_CAST:
		{
			GuardStop();

			// CAST SPELL mnNewValue on location of target
			// SETS UP SPELL TO BE CAST IN GAMECYCLE

			if (meAction == ACT_CAST && mnCel < mnCelCastDelay)
				return; //throw away any new cast commands until we're through animating enough of the current spell

			moCurrentCommand = oCommand;
			//moCurrentCommand.mnNewValue = mnCurrentSpell;
		}
		break;
	case CM_RETREAT:
		{
			moCurrentCommand = oCommand;
			//mnLastRetreatDirection = -1;
		}
		break;

	case CM_MOVE:							// GENERAL MOVE COMMAND
		// the old route is certainly dead
		if (!bInternal)
		{
			ClearThought();
		}
		GuardStop();
		moCurrentCommand = oCommand;
		vCalcRoute();
		break;

	case CM_FORMATION:						// FORMATION RELATIVE TO UNIT
		// determine formation unit
		// determine formation offset
		// store formation offset
		// store formation unit
		
		break;


	case CM_GUARD:							// GUARD A UNIT
		{
			moCurrentCommand = oCommand;
			Unit *poObj = (Unit *)moCurrentCommand.poGetTarget();
			assert(poObj->eClass() == OC_UNIT);
			vSetGuardedUnit((Unit*)poObj);
			//vCalcRoute(nGetX() - poObj->nGetX(), nGetY() - poObj->nGetY());
		}
		break;


	case CM_ATTACK:							// DESIRE TO ATTACK
		{
			GuardStop();

			moCurrentCommand = oCommand;

			// ADJUST TENACITY SO THAT DIRECT ORDER CAN ENSUE
			if (!bInternal)		// DIRECT ORDER
			{
				while (!bStayAndFight())	// GOING TO RUN?
				{
					// ADJUST TENACITY
					mTenacity = (SETTING_LMH)((int)mTenacity + 1);		

					// IF THIS FIRES, LOOK AT Vampire::bStayAndFight()
					assert(mTenacity <= SETTING_HIGH);	
				}
			}

			Unit * poUnit = (Unit *)moCurrentCommand.poGetTarget();
			assert(poUnit->eClass() == OC_UNIT);
			if (poUnit->bAlive())
			{
				vSetAttackee(poUnit);
				vCalcRoute();
			}

		}
		break;

	case CM_CONVERT:
		{
			GuardStop();
			moCurrentCommand = oCommand;

			Unit * poUnit = (Unit *) moCurrentCommand.poGetTarget();

			assert(poUnit->eClass() == OC_UNIT);
			assert(poUnit->bAlive());
			assert(!poUnit->poOwner());

			vSetAttackee(poUnit);

			vCalcRoute();
		}
		break;

	case CM_FEED:
		{
			mnCyclesUntilNextSound = -1;

			GuardStop();
			moCurrentCommand = oCommand;

			Unit * poUnit = (Unit *) moCurrentCommand.poGetTarget();

			assert(poUnit->eClass() == OC_UNIT);
			assert(poUnit->bAlive());
			assert(!poUnit->poOwner());

			vSetAttackee(poUnit);
			vCalcRoute();
		}
		break;
	case CM_CLOAK:
		if (oCommand.mnNewValue)
		{
			mbDesiredCloakState = TRUE;
			vCloak();
		}
		else
		{
			mbDesiredCloakState = FALSE;
			vUncloak();
		}
		break;
	case CM_WAIT_500MS:
		moCurrentCommand = oCommand;
		break;

	default:
		// TBD -- handle other commands
		assert(0); // command not handled
		break;
	}
}


// Assumes player 0 is either:
//		SINGLE PLAYER GAME - the human player, players 1..MAX_PLAYERS are computer controlled
//		NET GAME           - the one who started game, 1..MAX_PLAYERS are human or computer controlled
void Vampire::vSetMarked (int nPlayerNumber, int nApparentHealth, int nApparentLevel) 
{
	assert((nPlayerNumber >= 0) && (nPlayerNumber < MAX_PLAYERS));

	manApparentHealth[nPlayerNumber] = nApparentHealth;	
	manApparentLevel[nPlayerNumber] = nApparentLevel;
	
	if (game->poLocalPlayer()->mnPlayer == nPlayerNumber)
	{
		vSetBitmap(mpoTemplate->lGetValue(TAG_Bnor, nBitmapID()));
		mpwSpecialPalette = mpwTruePalette;
	}

	//We have an unmarked ENEMY vampire that is selected by someone and just became marked,
	//so we need to update status indicator correctly
	if (mbSelected && (mpoOwner != game->poLocalPlayer()) )
	{
//		moStatus.vUpdateLevel(nApparentLevel); //not needed now
		moStatus.vUpdateHealth(nApparentHealth);
	}
}

void Vampire::vGetMarked (int nPlayerNumber, int * pnApparentHealth, int * pnApparentLevel)
{
	assert((nPlayerNumber >= 0) && (nPlayerNumber < MAX_PLAYERS));
	assert(pnApparentHealth && pnApparentLevel);
	assert(bMarked(nPlayerNumber));
	*pnApparentHealth = manApparentHealth[nPlayerNumber];
	*pnApparentLevel = manApparentLevel[nPlayerNumber];
}

void Vampire::vClearMarked (int nPlayerNumber) 
{
	assert((nPlayerNumber >= 0) && (nPlayerNumber < MAX_PLAYERS)); 
	manApparentLevel[nPlayerNumber] = 0;
}

BOOL Vampire::bMarked(int nPlayerNumber)
{
	if ((nPlayerNumber >= 0) && (nPlayerNumber < MAX_PLAYERS)) 
	{
		return (manApparentLevel[nPlayerNumber] != 0);
	}
	return FALSE;
}

void Vampire::vSetLevel(int nLevel)
{
	assert(nLevel<8);
	assert(mnLevel>=0);

	nLevel = max(0, min(nLevel, 8));
	mnLevel = nLevel;
	manUpgrade[UPGRADE_Level] = nLevel;

	vUpdateAttributes();

	if (mpoOwner)
	{
		mpoOwner->SetTopLevel(nLevel);
	}
}

// TBD - only recalculate attributes when dependencies change
void Vampire::vUpdateAttributes()
{
	mnAttackSpeed = mpoClan->GetBaseAttackSpeed() - (mnLevel + (manUpgrade[UPGRADE_Speed] * 2));

	// TBD - PULL FROM COMMON PEASANT CLAN NORMAL SPEED
	if (mbCloaked) 
		SetSpeed(SPEED_CLOAKED);
	else
		SetSpeed(SPEED_NORMAL);
	// TBD - do not hard code these.
	assert(mnLevel<8);
	assert(mnLevel>0);

	mfMaxPower = GetPowerForLevel(mnLevel);
	mfMaxHealth = GetHealthForLevel(mnLevel);

	mnVisibleRange = vGetVisibleRange();
	
}

int Vampire::vGetVisibleRange()
{
	if (bInFog())
		return 5 + manUpgrade[UPGRADE_Sight];  //mpoClan->GetFogSight()
	else
		return mnLevel + manUpgrade[UPGRADE_Sight] + mpoClan->GetBaseSight();
}

float Vampire::GetPowerForLevel(int nLevel)
{
	return (float) (25 * nLevel * nLevel + 75);
}

float Vampire::GetHealthForLevel(int nLevel)
{
	return (float) (20 * nLevel * nLevel + 50);
}

BOOL Vampire::bIsVampireLord()
{
	// paulb - ADJUST IF NEEDED - TBD
	if (mpoOwner)
	{
		return (mnLevel >= mpoOwner->GetTopLevel());
	}
	return FALSE;
}

void Vampire::SetTenacity(SETTING_LMH setting) 
{
	Command *poCommand = new Command;
	poCommand->meType = CM_ATTR;
	poCommand->mnAttributeID = ATTR_TENACITY;
	poCommand->mnNewValue = (int)setting;
	game->poLocalPlayer()->vPostCommand(poCommand);
}

void Vampire::SetAggression(SETTING_LMH setting) 
{ 
	Command *poCommand = new Command;
	poCommand->meType = CM_ATTR;
	poCommand->mnAttributeID = ATTR_AGGRESSION;
	poCommand->mnNewValue = (int)setting;
	game->poLocalPlayer()->vPostCommand(poCommand);
}

void Vampire::UpgradeUpgrade(UPGRADE_NAME UpgradeName, int nNewValue) 
{
	// DO NOT CALL NEEDLESSLY
	assert(bCanUpgradeUpgrade(UpgradeName, nNewValue));

	// POST A COMMAND TO CHANGE THE ATTRIBUTE
	Unit::SetAttribute(this, (ATTR_CODE)((int)UpgradeName + (int)ATTR_NEWUPGRADE_00), nNewValue);
	mdwUpgradesPending = mdwUpgradesPending | (0x00000001 << ((int)UpgradeName));
}

void Vampire::GroupUpgradeUpgrade(UPGRADE_NAME UpgradeName, int nNewValue) 
{
	// TBD - DOES THIS NEED TO FLAG mdwUpgradesPending?
	//
	// THIS MIGHT BE NECESSARY TO PREVENT MULTIPLE
	// CALLS DURING THE NAME NET TURN.  HOWEVER,
	// THIS MIGHT ALREADY BE HANDLED (I THINK IT IS
	// ACTUALLY) BY Vampire::ProcessUpgrades
	//

	// POST A COMMAND TO CHANGE THE ATTRIBUTE
	Unit::SetAttribute(NULL, (ATTR_CODE)((int)UpgradeName + (int)ATTR_NEWUPGRADE_00), nNewValue);
}


BOOL Vampire::bCanUpgradeUpgrade(UPGRADE_NAME upgrade, int nNewValue)
{	
	// THIS DOES NOT HAVE ONE PENDING
	if (!(mdwUpgradesPending & (0x00000001 << ((int)upgrade))))
	{
		// IF UPGRADE DOES NOT EQUAL IT
		if (manUpgrade[upgrade] <= nNewValue)
		{
			if (upgrade == UPGRADE_Level)
			{
				// ASSERTION : SHOULD NOT EVER EXCEED!
				assert(mpoOwner->GetUnitCountOfLevel(nNewValue) <= mpoClan->GetUnitsPerLevel(nNewValue, mpoOwner->GetTopLevel()));
				if (mpoOwner->GetUnitCountOfLevel(nNewValue) == mpoClan->GetUnitsPerLevel(nNewValue, mpoOwner->GetTopLevel()))
				{
					return FALSE;
				}
				else
				{
					// CHECK FOR ALL STEPS IN BETWEEN ON LEVEL UPGRADES
					if (nNewValue > (manUpgrade[upgrade] + 1))
					{
						// RECURSIVE
						if (!bCanUpgradeUpgrade(upgrade, nNewValue - 1))
						{
							return FALSE;
						}
					}
				}
			}
			else if (nNewValue > 2)
			{
				return FALSE;
			}
			return TRUE;			
		}
	}
	return FALSE;
}

void Vampire::NetworkUpgradeReceived(UPGRADE_NAME upgrade) 	
{ 
	DWORD dwUpgradeFlag = 0x00000001 << ((int)upgrade);
	assert(mdwUpgradesPending & dwUpgradeFlag);
	mdwUpgradesPending = mdwUpgradesPending & (~dwUpgradeFlag);
}

BOOL Vampire::IsOnSameTeamAs(Player * poPlayer)
{
	// TBD -- IMPLEMENT ALLY FOR 3+ PLAYERS
	// THIS WILL BE OKAY FOR NOW 
	return (poPlayer == mpoOwner);
}

FLOAT Vampire::GetPowerExceedingEmergencyLevel()
{
	return max(0, mfPower - mfMaxPower * mpoClan->GetPowerSaveForEmergency());
}


void Vampire::ProcessUpgrades()
{
	if (!mbUpgrading)
	{
		for(int j=0; j<(int)UPGRADE_COUNT; j++)
		{
			// THIS IS FOR UPGRADE CYCLING
			int i = (int)mLastUpgrade + j + 1;
			if (i >= (int)UPGRADE_COUNT)
			{
				i = i - UPGRADE_COUNT;
			}
			if (manUpgrade[i] < manDesired[i])
			{
				FLOAT fRequiredPower = GetRequiredPowerToUpgrade((UPGRADE_NAME)i);

				// IF ENOUGH POWER
				if (fRequiredPower < GetPowerExceedingEmergencyLevel())
				{
					if (bCanUpgradeUpgrade((UPGRADE_NAME)i, manUpgrade[i] + 1))
					{
						// SETUP THE UPGRADE PROCESS
						mbUpgrading = TRUE;
						mfUpgradeProgress = 0.0;
#define PROCESSUPGRADECYCLERESOLUTION 1
						mfUpgradeProgressPerCycle = (float)100.0 / ((float)GetRequiredTimeToUpgrade((UPGRADE_NAME)i) * (float)20.0);
						mUpgradeName = (UPGRADE_NAME)i;
						mLastUpgrade = mUpgradeName;

						// BILL IMMEDIATELY
						mfPower = mfPower - fRequiredPower;
						moStatus.vUpdateHealth(vGetRelativeHealth(mpoOwner));
						break;
					}
					// CANNOT UPGRADE 
					else  
					{
						// RESET DESIRED UPGRADE STATUS TO CURRENT SETTING
						manDesired[i] = manUpgrade[i];
						if (mbSelected)
						{
							gpoInterface->vUpdateButtonState((UPGRADE_NAME)i);
						}
					}
				}
			}
		}
	}
	if (mbUpgrading)
	{
		// CONTINUE UPGRADING
		mfUpgradeProgress = mfUpgradeProgress + mfUpgradeProgressPerCycle;
		if (mfUpgradeProgress >= 100.0)
		{
			mfUpgradeProgress = 0.0;
			manUpgrade[mUpgradeName] = manUpgrade[mUpgradeName] + 1;
			mbUpgrading = FALSE;

			if (mUpgradeName == UPGRADE_Level)
			{
				vSetLevel(manUpgrade[mUpgradeName]);
				// 100% HEALTH WITH UPGRADE
				mfHealth = mfMaxHealth;
				
				//Update level changes for local players only.
				//Enemy vampires that change levels in another players view will need to get 
				//trapped in Unit::CheckNodeForVisibleUnits
				if (bIsLocalVampire())
				{
//					moStatus.vUpdateLevel(mnLevel); //not needed now
					moStatus.vUpdateHealth(vGetRelativeHealth(mpoOwner));
				}
			}
			else
			{
				vSetLevel(mnLevel);
			}
			if (mbSelected)
			{
				gpoInterface->vUpdateButtonState(mUpgradeName);
			}
		}
	}
}


void Vampire::GetUpgradeInProgress(UPGRADE_NAME * pnUpgradeName, int * pnUpgradeProgress) 
{ 
	if (mbUpgrading)
	{
		*pnUpgradeName = mUpgradeName; 
		*pnUpgradeProgress = (int)mfUpgradeProgress;
	}
	else
	{
		*pnUpgradeName = UPGRADE_NONE;
		*pnUpgradeProgress = 0;
	}
}

FLOAT Vampire::GetRequiredPowerToUpgrade(UPGRADE_NAME upgrade)
{
	FLOAT fPercent = 0.0;
	assert(mpoClan);
	if (upgrade == UPGRADE_Level)	// LEVEL
	{
		fPercent = mpoClan->GetPercentPowerToUpLevel();
	}
	else							// SKILL
	{
		fPercent = mpoClan->GetPercentPowerToUpSkillFrom(manUpgrade[upgrade]);
	}
	return mfMaxPower * fPercent;
}

int Vampire::GetRequiredTimeToUpgrade(UPGRADE_NAME upgrade)
{
	assert(mpoClan);
	if (upgrade == UPGRADE_Level)	// LEVEL
	{
		return mpoClan->GetTimeToUpLevel();
	}
	else							// SKILL
	{
		return mpoClan->GetTimeToUpSkillFrom(manUpgrade[upgrade]);
	}
}

void Vampire::vCommandNone()
{
	if (meAction == ACT_WALK || meAction == ACT_RUN || meAction == ACT_FEED)
	{
		vFidget();
//		if (mbCloaked)
//			vSetAction(ACT_FIDGET_CLOAKED);
//		else
//			vSetAction(ACT_FIDGET_UNCLOAKED);
	}

	moCurrentCommand.meType = CM_NONE;
	mdwFlags = mdwFlags & (~(DWORD)UNIT_FLAG_WAITING );
	moCurrentRoute.RemoveAll();
	if (((bCloaked() == 0)&&(mbDesiredCloakState != 0))||((bCloaked() != 0)&&(mbDesiredCloakState == 0)))
	{
		if (mbDesiredCloakState)
		{
			vCloak();
		}
		else
		{
			vUncloak();
		}
	}

	if (mnAttackee) 
	{
		Unit * poAttackee = (Unit*)goObjectDB.poGetObject(mnAttackee);
		if (poAttackee)
		{
			if (poAttackee->bAlive())
			{
				poAttackee->vFidget();
			}
			poAttackee->vRemoveAttacker(this);
		}
		mnAttackee = NULL;
	}
	//vCloak();
}


int Vampire::GetNewConvertHealth()
{
	return mpoClan->GetNewConvertHealth();
}

int Vampire::GetNewConvertPower() 
{
	return mpoClan->GetNewConvertPower();
}

int Vampire::GetConversionSpeed()
{
	return mpoClan->GetConversionSpeed();
}

FLOAT Vampire::GetPowerRegenPercent()
{
	return mpoClan->GetPowerRegenPercent();
}

int Vampire::GetRangeToDecloak()
{
	return mpoClan->GetRangeToDecloak();
}

int Vampire::GetPowerCostToConvert()
{
	return mpoClan->GetPowerCostToConvert();
}

void Vampire::vDraw() 
{
	//CALL THE SUPER FIRST
	Unit::vDraw();

	//if (!bIsLocalVampire() && bIsInside() && !(((Building*)mpoCurNode->poGetCover())->bIsOpen()) )
	if (!bIsLocalVampire() && !mbVisible) {
		return; //don't bother drawing enemy's moStatus(or crown) if they're in a closed building
	}

	//Now draw the crown(head) if it exists
	if (mpoCrown)
	{
		//only draw crown if unit is marked
		if (mpoOwner && bMarked(game->poLocalPlayer()->mnPlayer) && mbAlive) 
		{
			RECT rCrownRect;
			SetRect(&rCrownRect, mrCurrent.left, mrCurrent.top, mrCurrent.right, mrCurrent.bottom);
			mpoCrown->SetLoop(mnLoop);
			mpoCrown->SetCel(mnCel);
			mpoCrown->vSetFlip(mbFlip);
			mpoCrown->vDraw(nPixelX() - mpoPlane->nMapOrgPixelX(), nPixelY() - mpoPlane->nMapOrgPixelY(), mpoPlane, mjAlpha, 0, 0, &rCrownRect);
		}
	}
	
	if (mpoOwner && bMarked(game->poLocalPlayer()->mnPlayer) && mbAlive) 
	{
		//
		moStatus.vDraw(nPixelX() - mpoPlane->nMapOrgPixelX(), 
					   nPixelY() - mpoPlane->nMapOrgPixelY() );
	}

}

void Vampire::GainPowerFromOwnedStone(Item * poItem)
{
	GainPowerFromStone(poItem, GetOwnedPowerGain());
}


void Vampire::GainPowerFromStone(Item * poItem, FLOAT fPower)
{
	// GAINS UP TO fPower FROM poItem POWER STONE
	FLOAT fPowerDesired = mfMaxPower - mfPower;

	// ADJUST TO CLAN/UNIT RANGES 
	fPower = mfMaxPower * (GetMinPowerGain() + fPower * (GetMaxPowerGain() - GetMinPowerGain()));

	fPowerDesired = min(fPower, fPowerDesired);

	mfPower = mfPower + poItem->DrainPower(fPowerDesired);
}

// RETURN RATIO OF OPG TO MPG
FLOAT Vampire::GetOwnedPowerGain()
{
	return (FLOAT)mpoTemplate->lGetValue(TAG_PoPg, 20) / (FLOAT)mpoTemplate->lGetValue(TAG_PMPg, 10);
}

// RETURN MAX POWER GAIN PER CYCLE
FLOAT Vampire::GetMaxPowerGain()
{
	return (FLOAT)mpoTemplate->lGetValue(TAG_PMPg, 10) / ((FLOAT)20 * (FLOAT)100);
}

// RETURN MIN POWER GAIN PER CYCLE
FLOAT Vampire::GetMinPowerGain()
{
	return (FLOAT)mpoTemplate->lGetValue(TAG_PmPg, 0) / ((FLOAT)20 * (FLOAT)100);
}

void Vampire::vProcessAdrenaline()
{
	assert(mnAdrenalineCycleTicker != 0);  // FUNCTION NEEDLESSLY CALLED
	if (mnAdrenalineCycleTicker < 0)
	{
		// countdown until -AdrenalinResetTime
		mnAdrenalineCycleTicker--;

		if (mnAdrenalineCycleTicker == -vGetClan()->GetAdrenalineResetTime())
		{
			// ADRENALIN WORN OFF
			mnAdrenalineCycleTicker = 0;
			// SET SPEED TO NORMAL
			SetSpeed(SPEED_NORMAL);
		}
	}
	else  // ADRENALIN STILL WORKING
	{
		mnAdrenalineCycleTicker--;

		// SKIP ZERO SINCE IT HAS SPECIAL MEANING
		if (mnAdrenalineCycleTicker == 0)
			mnAdrenalineCycleTicker = -1;

		// REMAIN UNCLOAKED DURING ADRENALIN
		if (!mbCloaked)
			vUncloak();
	}
}

void Vampire::AdjustSpellCost(FLOAT * pfCost)
{
	int nSpells = GetUpgradeSpells();

	if (nSpells == 0)
	{
		return;
	}
	else if (nSpells == 1)
	{
		*pfCost = *pfCost * mpoClan->GetSpellCostCoefficient1();
	} 
	else if (nSpells == 2)
	{
		*pfCost = *pfCost * mpoClan->GetSpellCostCoefficient2();
	}
}

BOOL Vampire::bSpellCanCastWhileCloaked(SPELL_TYPE eSpell)
{
	switch (eSpell)
	{
		case SPELL_CALL:
		case SPELL_INVISIBILITY:
			return TRUE;
		case SPELL_DEATHFROMABOVE:
		case SPELL_FORCELEECH:
		case SPELL_LIGHTNINGBOLT:
		case SPELL_DOOMSPIRAL:
		case SPELL_DEATHRAIN:
		case SPELL_CALLFOG:
		case SPELL_SWAP:
		case SPELL_PHANTOMS:
		case SPELL_EYEOFTHEDRAGON:
		case SPELL_MINDCLOUD:
		case SPELL_TALECSNARE:
		case SPELL_BRAINWORM:
		case SPELL_VEILOFMIST:
		case SPELL_NECROMANCE:
		case SPELL_GHOSTTOUCH:
		case SPELL_DISPERSEMOB:
			break;
	}
	return FALSE;
}

void Vampire::vCast()
{
	FLOAT fRequired;
	UINT  nRequiredLevel=1;
	UINT  unRange;

	Spell * poSpell = game->GetSpell(mnCurrentSpell);
	assert(poSpell);
	fRequired = poSpell->GetAdjustedSpellCost(this);
	nRequiredLevel = poSpell->GetRequiredLevel();

	//GetSpellCostAndLevel(mnCurrentSpell, &fRequired, &nRequiredLevel);
	//AdjustSpellCost(&fRequired);
	
	if (GetPowerExceedingEmergencyLevel() < fRequired)
	{
		// SOUND: REFUSE TO CAST -- NOT ENOUGH POWER
		pApp->pSoundManager->EmitFromObject(this, SOUND_NoCastLowPower1);
		return;
	}
	if ((UINT)mnLevel < nRequiredLevel)
	{
		// SOUND: REFUSE TO CAST -- LEVEL NOT HIGH ENOUGH
		// MAYBE "I DO NOT UNDERSTAND"
		pApp->pSoundManager->EmitFromObject(this, SOUND_NoCastDueToLevel);
		return;
	}
	//GetSpellRange(mnCurrentSpell, &unRange);
	unRange = poSpell->GetRange(this);
	if (unRange > 0)
	{
		MapNode * poTargetNode = moCurrentCommand.poGetTargetNode();
		if (unRange < (UINT)lDistToSpace(poTargetNode->mnXPos, poTargetNode->mnYPos))
		{
			// SOUND: THAT IS TOO FAR
			pApp->pSoundManager->EmitFromObject(this, SOUND_NoCastDueToRange);
			return;
		}
	}

	// DEDUCT POWER -- ABOUT TO CAST
	mfPower = mfPower - fRequired;
	Unit * poUnit = NULL;
	if (moCurrentCommand.mbTargetUnit) 
	{
		poUnit = (Unit*)goObjectDB.poGetObject(moCurrentCommand.mnUnit);
	}
	poSpell->CastFromUnit(this, poUnit, moCurrentCommand.poGetTargetNode());
}


void Vampire::vFireMissile(MapNode * poTargetNode, MISSILE_TYPE missile)
{
	MapNode * poSourceNode = poNode();

	CreateMissile(this, poSourceNode, poTargetNode, missile, 0, TRUE);
}

Missile *  CreateMissile(Unit * poFromUnit, MapNode * poSourceNode, MapNode * poTargetNode, MISSILE_TYPE missile, int nWait, BOOL bSolid, float fPercent, double X, double Y, int nFallOff, int nDirection)
{
	Missile * poMissile = NULL;
	switch (missile)
	{
	case MISSILE_DEATHFROMABOVE:
		{
			poMissile = new Missile(missile, gpoMap->mpoPlane, poFromUnit, poSourceNode, poTargetNode, 6000, nWait, bSolid, fPercent);
			poMissile->SetImpact(IMPACT_DEATHFROMABOVE);
		}
		break;
	case MISSILE_SMOKE:
		{
			poMissile = new Missile(missile, gpoMap->mpoPlane, poFromUnit, poSourceNode, poTargetNode, 6001, nWait, bSolid, fPercent);
		}
		break;
	case MISSILE_FOG:
		{
			poMissile = new Missile(MISSILE_TRACER, gpoMap->mpoPlane, poFromUnit, poSourceNode, poTargetNode, 6007, 10 + GlobalRand(5), bSolid, fPercent, NULL, X, Y, nFallOff, nDirection);
			poMissile->SetObstructView(TRUE);
		}
		break;
	case MISSILE_LINGERINGSMOKE:
		{
			poMissile = new Missile(missile, gpoMap->mpoPlane, poFromUnit, poSourceNode, poTargetNode, 6002+GlobalRand(3), nWait, bSolid, fPercent, NULL, X, Y, nFallOff, nDirection);
		}
		break;
	case MISSILE_DRIFTINGSMOKE:
		poMissile = new Missile(missile, gpoMap->mpoPlane, poFromUnit, poSourceNode, poTargetNode, 6002+GlobalRand(3), nWait, bSolid, fPercent, NULL, X, Y, nFallOff, nDirection);
		break;
	case MISSILE_TRACER:
		{
			poMissile = new Missile(missile, gpoMap->mpoPlane, poFromUnit, poSourceNode, poTargetNode, 6005, nWait, bSolid, fPercent, NULL, X, Y, nFallOff, nDirection);
		}
		break;
	default:
		assert(0); // MUST HAVE ADDED A NEW MISSILE TYPE
		break;
	}
	return poMissile;
}




// For adding to the map.
// The unit object being dragged around makes a copy of itself and inserts it in into the
// node nearest the event
void Vampire::vCopySelf(sciEvent &oEvent) {
	MapNode *poNode = mpoPlane->mpoMap->poNodeFromEvent(oEvent);
	if (poNode) {

		// If self is in the clipboard and a unit is in spot where we want a copy to go,
		// then we can't allow it as only 1 unit per node is allowed. This condition can and will
		// occur when you add a unit with the right mouse button, don't move it, and click the 
		// button again.
		//		OR
		// we right click on a unit that already exits (which is not equal to our self)
		if ( (gpoMapEdit->mpoMapNode->mpoUnit) && (poNode->mpoUnit) ||
			 (this != poNode->mpoUnit) ) {
			vDebugOut("can't add 2 vampires in same space\n");
		} else {
			// clear self out of node the copy is suppose to go into and put back onto clipboard
			mpoCurNode->mpoUnit = NULL;					//clear self out of node
			gpoMapEdit->mpoMapNode->mpoUnit = this;		//put onto clipboard
			mpoCurNode = gpoMapEdit->mpoMapNode;		//update back reference

			// now create copy and place into node
			Vampire *poCopy = new Vampire(mpoOwner, mpoPlane, poNode, nTemplateID());	
			poCopy->mjPlayerLink = mjPlayerLink;
		}
	}
}

// For adding to the map.
// The unit object being dragged around inserts ITSELF in the node nearest its event
void Vampire::vInsertSelf(sciEvent &oEvent) {
	MapNode *poNode = mpoPlane->mpoMap->poNodeFromEvent(oEvent);
	if (poNode) {

		// if we're on the clipboard
		if (gpoMapEdit->mpoMapNode->mpoUnit) {
			if (poNode->mpoUnit == NULL) {
				//most likely vDrag will have already inserted us into the node but if it didn't
				//we need to catch it here.
				mpoCurNode->mpoUnit = NULL;
				poNode->mpoUnit = this;
				vSetPos(poNode->mnXPos, poNode->mnYPos); //this call results in a MarkUpdate
//				mjPlayerLink = 0; //now done in mapedit.cpp
			} else {
				vDebugOut("can't add 2 vampires in same space\n"); //TBD - add some real error message here - TW
				delete this;
			}
		}
		
	} else {
		// we clicked somewhere not on the game map so get rid of self
		delete this;
	}

}

Template * Vampire::vGetClanTemplate()
{
	if (!mpoClan) return NULL;
	return mpoClan->GetTemplate();
}

vClan * Vampire::vGetClan()
{
	return mpoClan;
}

void Vampire::SetBitmapCloaked(int nBitmap)
{
	mnBitmapCloaked = nBitmap;
}

short Vampire::vGetRelativeHealth(Player * poPlayer)
{
	vUpdateRelativeHealth(poPlayer);

	int nPlayer;
	if (poPlayer)
	{
		nPlayer = poPlayer->mnPlayer;
	}
	else
	{
		nPlayer = NONPLAYERINDEX;
	}
	return manRelativeHealth[nPlayer];
}

void Vampire::vUpdateRelativeHealth(Player * poPlayer)
{
	// WHICH PLAYER 
	int nPlayer;
	if (poPlayer)
	{
		nPlayer = poPlayer->mnPlayer;
	}
	else
	{
		nPlayer = NONPLAYERINDEX;
	}

	float fValue;

	// IF ON THE SAME TEAM
	if ((poPlayer == mpoOwner)
		// OR IF UNCLOAKED
		||(!bCloaked())
		// OR PLAYER IS VAMPIRE TEAM AND THIS VAMPIRE WAS MARKED BY THAT TEAM
		||(poPlayer && manApparentLevel[nPlayer] != 0))
	{
		// MAKE IT ALL VISIBLE
		float fHealth = (mfHealth / mfMaxHealth) * 100;
		float fPower = (mfPower / mfMaxPower) * 100;

		//only local vampires get the relative health formula
		if (fHealth < fPower) 
		{
			// Health is lower so add it to 1/x of Power
			fValue = ((float)mnRelHealthPercent / 100) * fPower + fHealth;
		}
		else 
		{
			// Power is lower(or equal) so add it to 1/x of Health
			fValue = ((float)mnRelHealthPercent / 100) * fHealth + fPower;
		}
		manRelativeHealth[nPlayer] = min(100, (short)fValue);
	} 
	else
	{
		//all non-locals just report a percentage of their health
		fValue = (mfHealth / mfMaxHealth) * 100;
		manRelativeHealth[nPlayer] = (short)fValue;
	}
}

void Vampire::vMoveOneCycle()
{
	if (!mbCloaked && !mbInvisible)
	{
		mnTracerReset++;
		if (mnTracerReset == 3)
		{
			mnTracerReset = 0;
			CreateMissile(this, NULL, NULL, MISSILE_TRACER, 10, FALSE, 0, nPixelX(), nPixelY(), 1, 0);
		}
	}
	Unit::vMoveOneCycle();
}

void Vampire::SetSpeedAdrenalineBoost()
{
	assert(mbCloaked == 0);
	mnSpeed = (int)((double)mnSpeed * mpoClan->GetAdrenalineSpeedBoost());
}

void Vampire::CallForHelp()
{
	// RUN THROUGH THE LIST OF FRIENDLIES AND ASK THE CLOSEST ONES (WITHIN 20* UNITS)
	int nRange = mpoClan->GetRangeCallForHelp();
	mpoOwner->CallForHelpFromFriendlies(this, nRange);
}

// REQUESTS HELP FROM THIS VAMPIRE FOR *poVampireNeedsHelp
int Vampire::RequestHelp(Vampire * poVampireNeedsHelp)
{
	assert(poVampireNeedsHelp);

	// IF NOT UNDER ATTACK AND NO CURRENT COMMAND
	if (!mnUnderAttackCycles && (moCurrentCommand.meType == CM_NONE || (moCurrentCommand.meType == CM_MOVE && moCurrentCommand.mbInternal == TRUE)))
	{
		// IF HELPEE IS UNDER ATTACK
		if (poVampireNeedsHelp->mnUnderAttackCycles)
		{
			// ATTACK THE ATTACKER
			Unit * poUnit = poVampireNeedsHelp->mpoAttacker;
			if (poUnit)
			{
				vInternalCommand_Attack(poUnit);
			}
			return 1;
		}
		// IF HELPEE IS ATTACKING SOMEONE... 
		else if (poVampireNeedsHelp->moCurrentCommand.meType == CM_ATTACK)
		{
			// ATTACK THE TARGET
			Unit * poUnit = 
				(Unit*)goObjectDB.poGetObject(poVampireNeedsHelp->moCurrentCommand.mnUnit);
			if (poUnit)
			{
				vInternalCommand_Attack(poUnit);
			}
			return 1;
		}
		// NO PROBLEMS YET... JUST APPROACH
		else if (poVampireNeedsHelp->moCurrentCommand.meType == CM_NONE)
		{
			// MOVE TO ME WITHIN 1 OR 2 SPACES
			Command oCommand;

			oCommand.meType = CM_MOVE;
			oCommand.mnUnit = poVampireNeedsHelp->nGetID();
			oCommand.mbTargetUnit = TRUE;
			vCommand(oCommand, TRUE);
			return 1;
		}
		// TBD -- IF BLOCKED DURING A CALL TO HELP, OR IF WITHIN RANGE,
		//			FIRE LONG RANGE SPELL
	}
	return 0;  // DO NOT COUNT THIS VAMPIRE
}


//need to overide for additional 'crown' check
Boolean Vampire::bOnMe(sciEvent &oEvent)
{
	if (mpoCrown)
	{

		if ((oEvent.x >= mrCurrent.left) &&
			(oEvent.x <= mrCurrent.right) &&
			(oEvent.y >= mrCurrent.top) &&
			(oEvent.y <= mrCurrent.bottom)) {

			POINT op;
			op.x = oEvent.x - mrCurrent.left;
			op.y = oEvent.y - mrCurrent.top;
			mpoBitmap->SetLoop(mnLoop);	//make sure to do this so the 'shared' bitmap is currently where we need it to be
			mpoCrown->SetLoop(mnLoop);
			if (mpoBitmap->ColorOfPoint(op) != 0 || mpoCrown->ColorOfPoint(op) != 0) {
				return TRUE;
			} else {
				return FALSE;
			}

		} else {
			return FALSE;
		}

	} else {
		//no crown so do standard check
		return DrawObj::bOnMe(oEvent);
	}
}






VampireDecoy::VampireDecoy(Player *poOwner, MapPlane *poPlane, MapNode *poNode, int nTemplateID):
	Vampire(poOwner, poPlane, poNode, nTemplateID)
{
	mnUnitParentID = 0;
	mnVisibleRange = 2;
}

void VampireDecoy::SetUnitParentID(int nUnitID)
{
	Vampire * poTarget = (Vampire *)goObjectDB.poGetObject(nUnitID);
	assert(poTarget);
	mnUnitParentID = nUnitID;
}

VampireDecoy::~VampireDecoy()
{
}

void VampireDecoy::vGameCycle()
{
	if (!mbAlive) return;
	Vampire * poParentUnit = (Vampire *)goObjectDB.poGetObject(mnUnitParentID);

	if (!poParentUnit || mnUnderAttackCycles || poParentUnit->lDistToSpace(nGetX(), nGetY()) > poParentUnit->vGetVisibleRange() || !poParentUnit->bAlive())
	{
		if (mbSelected) 
		{
			gpoMap->vRemoveSelection(this);
			gpoMap->vCommitSelection();
			vMarkUpdate();
		}
		pApp->pSoundManager->EmitFromPosition(nPixelX(), nPixelY(), SOUND_POP);
		DropAll(mpoCurNode);
		mpoOwner->vDeleteUnit(this);
		mpoOwner = NULL;
		vSetVisibility(FALSE);
		mbAlive = FALSE;
		delete this;
		return;
	}
	switch (moCurrentCommand.meType)
	{
	case CM_NONE:
	case CM_CLOAK:
	case CM_MOVE:
	case CM_GUARD:
		Vampire::vGameCycle();
		return;
	default:
		break;
	}
}

void VampireDecoy::vCommand(Command &oCommand, BOOL bInternal)
{
	if (!mbAlive) {
		return;
	}

	// unowned units can't accept outside commands
	if (!bInternal && !mpoOwner)
	{
		return;
	}

	// TRACK INTERNAL COMMAND FOR LATER REFERENCE
	oCommand.mbInternal = bInternal;

	switch(oCommand.meType) {
	case CM_ATTR:
		if ((oCommand.mnAttributeID >= ATTR_NEWUPGRADE_00)&&(oCommand.mnAttributeID < ATTR_NEWUPGRADE_TOPPLUSONE))
		{
			UPGRADE_NAME upgrade = (UPGRADE_NAME)((int)oCommand.mnAttributeID - (int)ATTR_NEWUPGRADE_00);

			if (manDesired[upgrade] > oCommand.mnNewValue)
			{
				// IF UPGRADING THIS UPGRADE
				if (mbUpgrading && mUpgradeName == upgrade)
				{
					if (oCommand.mnNewValue <= manUpgrade[upgrade])	
					{
						// CANCEL THE CURRENT UPGRADE
						mbUpgrading = FALSE;
						mUpgradeName = UPGRADE_NONE;
						mfUpgradeProgress = 0.0;
						manDesired[upgrade] = manUpgrade[upgrade];
						if (mbSelected)
							gpoInterface->vUpdateButtonState(upgrade);
						break;
					}
				}
			}
			if (manUpgrade[upgrade] >= oCommand.mnNewValue)
			{
				manDesired[upgrade] = manUpgrade[upgrade];
				if (mbSelected)
					gpoInterface->vUpdateButtonState(upgrade);
				break;
			}
			manDesired[upgrade] = oCommand.mnNewValue;
			if (mbSelected)
				gpoInterface->vUpdateButtonState(upgrade);
			break;
			//NetworkUpgradeReceived(upgrade);
		}
		else
		{
			switch(oCommand.mnAttributeID)
			{
			case ATTR_AGGRESSION:
				mAggression = (SETTING_LMH) oCommand.mnNewValue;
				if (mbSelected)
					gpoInterface->vUpdateButtonState(ATTR_AGGRESSION);
				break;
			case ATTR_TENACITY:
				mTenacity = (SETTING_LMH) oCommand.mnNewValue;
				if (mbSelected)
					gpoInterface->vUpdateButtonState(ATTR_TENACITY);
				break;
			case ATTR_HEALTH:
				mfHealth = (float) oCommand.mnNewValue;
				break;
			case ATTR_CONVERSION:
				mConversion = (SETTING_LMH) oCommand.mnNewValue;
				break;
			case ATTR_READYSPELL:
				mnCurrentSpell = oCommand.mnNewValue;
				break;
			case ATTR_DEFAULT_ATTACK:
				vSetDefaultAttack((DEFAULT_ATTACK)oCommand.mnNewValue);
				break;
			}
		}
		break;

	case CM_GUARD_MOVEINTOPOSITION:
		moCurrentCommand = oCommand;
		vCalcRoute();
		break;
	case CM_GUARD_MOVEINTERNAL:				// GENERAL MOVE COMMAND
	case CM_GUARD_MOVETOVERIFY:
		// the old route is certainly dead
		moCurrentCommand = oCommand;
		vCalcRoute();
		break;

	case CM_PATROL_WAYPOINT:
		// create waypoint
		// continue waypoint thought
		AddPatrolWaypoint(oCommand.mnX, oCommand.mnY);
		Patrol();
		break;

	case CM_MOVE:							// GENERAL MOVE COMMAND
		// the old route is certainly dead
		if (!bInternal)
		{
			ClearThought();
		}
		GuardStop();
		moCurrentCommand = oCommand;
		vCalcRoute();
		break;

	case CM_GUARD:							// GUARD A UNIT
		{
			moCurrentCommand = oCommand;
			Unit *poObj = (Unit *)moCurrentCommand.poGetTarget();
			assert(poObj->eClass() == OC_UNIT);
			vSetGuardedUnit((Unit*)poObj);
			//vCalcRoute(nGetX() - poObj->nGetX(), nGetY() - poObj->nGetY());
		}
		break;

	case CM_CLOAK:
		if (oCommand.mnNewValue)
		{
			mbDesiredCloakState = TRUE;
			vCloak();
		}
		else
		{
			mbDesiredCloakState = FALSE;
			vUncloak();
		}
		break;

	default:
		break;
	}
}

void VampireDecoy::vCast()
{
	//
}

FLOAT VampireDecoy::vDamage(FLOAT fDamage, Unit * poUnitFrom)
{
	mnUnderAttackCycles = 20;

	return 0.0;
}

void VampireDecoy::vScan()
{
	// CREATE LIST OF OBJECTS WITHIN VISIBLE RANGE
	int nCenterX = nGetX();
	int nCenterY = nGetY();
	int nVisibleRange = vGetVisibleRange();

	int nLeft = max(0, nCenterX - nVisibleRange);
	int nRight = min(gpoMap->nGetWidth()-1, nCenterX + nVisibleRange);
	int nTop = max(0, nCenterY - nVisibleRange);
	int nBottom = min(gpoMap->nGetHeight()-1, nCenterY + nVisibleRange);

	moVisibleUnits.RemoveAll();

	for (int i=0; i < ALLPLAYERINDICES; i++)
	{
		mnVisibleTotal[i].attack = 0;
		mnVisibleTotal[i].defense = 0;
		mnVisibleTotal[i].level = 0;
		mnVisibleTotal[i].count = 0;
		mnVisibleTotal[i].relativehealth = 0;
	}

	int nPlayer;
	if (mpoOwner)
	{
		nPlayer = mpoOwner->mnPlayer;
	}
	else
	{
		nPlayer = NONPLAYERINDEX;
	}

	// THIS
	mnVisibleTotal[nPlayer].count++;

	// ACCUMULATE TOTALS
	// TBD - JUST USE VISIBLE LEVELS 
	mnVisibleTotal[nPlayer].level += mnLevel; 
	mnVisibleTotal[nPlayer].attack += GetUpgradeAttack();
	mnVisibleTotal[nPlayer].defense += GetUpgradeDefend();
	mnVisibleTotal[nPlayer].relativehealth += vGetRelativeHealth(mpoOwner);

	// 
	//  TBD - BETTER SCAN ALGORITHM.  ALSO, INCLUDE VISIBILITY,
	//			LINE OF SIGHT,
	//
#if 0
	int nRange = 1;
	while (nRange < nVisibleRange)
	{
		MapNode * poNode;

		int y = nTop;
		int l = max(nLeft, nCenterX - nRange);
		int r = min(nRight, nCenterX + nRange);
		while(y <= nBottom)
		{	
			poNode = gpoMap->poGetNode(l, y);
			CheckNodeForVisibleUnits(poNode, nRange);
			poNode = gpoMap->poGetNode(r, y);
			CheckNodeForVisibleUnits(poNode, nRange);
			y++;
		}
		int x = nLeft + 1;
		int b = min(nBottom, nCenterY + nRange);
		int t = max(nTop, nCenterY - nRange);
		while (x < nRight)
		{
			poNode = gpoMap->poGetNode(x, b);
			CheckNodeForVisibleUnits(poNode, nRange);
			poNode = gpoMap->poGetNode(x, t);
			CheckNodeForVisibleUnits(poNode, nRange);
			x++;
		}
		nRange++;
	}
#endif
	// we fudge a little on the squaring so as to get a more pleasing circle
	int nSquareRange = nVisibleRange * nVisibleRange + nVisibleRange;
	MapNode *poNode;
	int j;
	int nRange;
	int nVisibility;
	int nLocalMask;
	if (mpoOwner && mpoOwner == game->poLocalPlayer()) 
	{
		nLocalMask = 0xFFFFFFFF;
	} 
	else if (mwSpyFlags & ((WORD)1 << game->poLocalPlayer()->mnPlayer))
	{
		nLocalMask = 0xFFFFFFFF;
	} 
	else
	{
		nLocalMask = 0x0;
	}

	//
	// we do a 1/8th triangle of the full range area
	// then use rotations and reflections to check the other nodes
	for (i = -nVisibleRange; i <= 0; ++i) {
		for (j = i; j <= 0; ++j) {
			int nSquare = i * i + j * j;
			if (nSquare <= nSquareRange) {
				double fRange = sqrt((double)nSquare);
				nRange = (int)fRange;

#if VIS_FUZZY_BORDER
				nVisibility = (int)(((nVisibleRange + 0.5) - fRange) * (255.0 / VIS_FUZZY_BORDER) + 0.5);
				nVisibility = min(255, max(0, nVisibility));
#else
				nVisibility = 255;
#endif
				// this sets non-local players' visibility to zero without
				// using an extra if -- test is above, outside the loop
				nVisibility &= nLocalMask;

				// do all 8 reflected combinations
				poNode = gpoMap->poGetNodeRC(nCenterX + i, nCenterY + j);
				if (poNode) {
					CheckNodeForVisibleUnits(poNode, nRange);
					///poNode->vSetVisibility((BYTE)nVisibility);
				}
				if (i != j) {
					poNode = gpoMap->poGetNodeRC(nCenterX + j, nCenterY + i);
					if (poNode) {
						CheckNodeForVisibleUnits(poNode, nRange);
						///poNode->vSetVisibility((BYTE)nVisibility);
					}
				}

				if (j) {
					// don't bother doing -j if j == 0
					poNode = gpoMap->poGetNodeRC(nCenterX + i, nCenterY - j);
					if (poNode) {
						CheckNodeForVisibleUnits(poNode, nRange);
					///	poNode->vSetVisibility((BYTE)nVisibility);
					}
					if (i != -j) {
						poNode = gpoMap->poGetNodeRC(nCenterX - j, nCenterY + i);
						if (poNode) {
							CheckNodeForVisibleUnits(poNode, nRange);
						///	poNode->vSetVisibility((BYTE)nVisibility);
						}
					}
				}
				if (i) {
					// don't bother doing -i if i == 0
					poNode = gpoMap->poGetNodeRC(nCenterX - i, nCenterY + j);
					if (poNode) {
						CheckNodeForVisibleUnits(poNode, nRange);
						///poNode->vSetVisibility((BYTE)nVisibility);
					}
					if (i != -j) {
						poNode = gpoMap->poGetNodeRC(nCenterX + j, nCenterY - i);
						if (poNode) {
							CheckNodeForVisibleUnits(poNode, nRange);
						///	poNode->vSetVisibility((BYTE)nVisibility);
						}
					}
					if (j) {
						// don't bother doing -j if j == 0
						poNode = gpoMap->poGetNodeRC(nCenterX - i, nCenterY - j);
						if (poNode) {
							CheckNodeForVisibleUnits(poNode, nRange);
						///	poNode->vSetVisibility((BYTE)nVisibility);
						}
						if (i != j) {
							poNode = gpoMap->poGetNodeRC(nCenterX - j, nCenterY - i);
							if (poNode) {
								CheckNodeForVisibleUnits(poNode, nRange);
							//	poNode->vSetVisibility((BYTE)nVisibility);
							}
						}
					}
				}
			}
		}
	}
}

void VampireDecoy::CheckNodeForVisibleUnits(MapNode * poNode, int nDistance)
{
	assert(poNode);
	if (poNode->mpoUnit)
	{
		CheckNodeThatHasUnits(poNode, nDistance);
	}
}

