#include "os.h"

#include "app.h"
#include "sounder.h"
#include "timer.h"
#include "palette.h"
#include "util.h"
#include "resman.h"
#include "prf.h"
#include "unihand.h"
#include "inifile.h"
#include "sciplane.h"
#include "game.h"
#include "random.h"

#include <stdlib.h>
#include <string.h>
#include <time.h>

extern char szIniFilename[];
extern char *szAppName;
extern int numBytesAllocated;
extern BOOL hdVersion;
extern int maxMemory;
extern char cdDriveLetter;
extern char versionText[32];

VApplication::VApplication()
{
	IniFile *sierraINF;

	iReturnResult = 0;
	oLocalRandom.vSeed(time(NULL));
	oGlobalRandom.vSeed(time(NULL));
	bDone = FALSE;
	pWindow = NULL;
	pCurrentPalette = NULL;
	game = NULL;
	pSounder = NULL;
	pResourceManager = NULL;
	listOfResourceManagers = new List<ResourceManager>();
	mpoFrameBuf = NULL;

	char szIniPath[MAX_PATH];
	char szExePath[MAX_PATH];
	int nIndex = ::GetModuleFileName(NULL, szExePath, MAX_PATH);
	if (nIndex > 0) {
		while ((nIndex > 0) && (szExePath[nIndex] != '\\'))
			--nIndex;
		szExePath[nIndex] = '\\';
		szExePath[nIndex + 1] = '\0';
	} else {
		szExePath[0] = 0;
	}
	strcpy(szIniPath, szExePath);
	strcat(szIniPath, szIniFilename);
	pIniFile = new IniFile(szIniPath, "Settings");

	if (cdDriveLetter)
		pIniFile->WriteValue("CD Drive", cdDriveLetter);

	// Get PatchVersion string.
	strcpy(szIniPath, szExePath);
	strcat(szIniPath, "sierra.inf");
	sierraINF = new IniFile(szIniPath, "Ident");
	sierraINF->ReadString("PatchVersion", "0.0.0.0", versionText, sizeof(versionText));
	delete sierraINF;
}

VApplication::~VApplication()
{
	if (pIniFile) {
		delete pIniFile;
		pIniFile = NULL;
	}
	if (listOfResourceManagers) {
		delete listOfResourceManagers;
		listOfResourceManagers = NULL;
	}
}

int VApplication::Run()
{
	pTimer = new Timer;
	game = new Game();

	pSounder = NULL; // new Sounder;
	pSoundManager = new SoundManagerDirectSound;
	
	pWindow = new Window;
	//pWindow->MakeWindow();

	char szEntry[6];

	pApp->pIniFile->ReadString("DirectDraw", "No", szEntry, 5);
	if (szEntry[0] == 'n' || szEntry[0] == 'N') {
		mpoFrameBuf = new FrameBuffer(640, 480, 16, FB_WINDOW);
	} else {
		mpoFrameBuf = new FrameBuffer(640, 480, 16, FB_FULLSCREEN);
	}

	pSoundManager->PreInit();
	BOOL bOkay = FALSE;
	pSoundManager->Init(&bOkay);

	pResourceManager = new ResourceManager;

#ifdef NDEBUG
	BOOL bError = FALSE;
	try
	{
		game->Play();
	}
	catch(...)
	{
		bError = TRUE;
	}
#else
	game->Play();
#endif

   if (pSoundManager) 
   {
      pSoundManager->ShutDown();
      delete pSoundManager;
      pSoundManager = NULL;
   }
	if (pSounder) {
		delete pSounder;
		pSounder = NULL;
	}
	if (pWindow) {
		delete pWindow;
		pWindow = NULL;
	}
	if (game) {
		delete game;
		game = NULL;
	}
	if (pResourceManager) {
		delete pResourceManager;
		pResourceManager = NULL;
	}
	if (pTimer) {
		delete pTimer;
		pTimer = NULL;
	}
#ifdef NDEBUG
	if (bError)
	{
		MessageBox(0, "Internal Program Error", szAppName, MB_ICONERROR);
	}
#endif

	return iReturnResult;
}

void VApplication::Idle()
{
	if (pSounder && !pSounder->IsSoundPlaying())
		pSounder->EndSound();
}

void VApplication::Exit()
{
	bDone = TRUE;
}

void VApplication::PurgeResources(int bytesCanUse)
{
	ResourceManager *poResMan;
	ResourceManager **ppoResMan;

	if (bytesCanUse == -1) {
		bytesCanUse = maxMemory;
	}
	ppoResMan = listOfResourceManagers->GetBegin();
	while (ppoResMan != listOfResourceManagers->GetEnd() && (numBytesAllocated > bytesCanUse)) {
		poResMan = *ppoResMan++;
		poResMan->PurgeResources(bytesCanUse);
	}
}
